Package: tidytlg
Title: Create TLGs using the 'tidyverse'
Version: 0.12.0
Language: en-US
Authors@R: 
     c(person(given = "Nicholas",
           family = "Masel",
           role = "aut",
           email = "nmasel@its.jnj.com"),
       person(given = "Steven",
           family = "Haesendonckx",
           role = "aut",
           email = "shaesen2@its.jnj.com"),
       person(given = "Pelagia",
           family = "Alexandra Papadopoulou",
           role = "aut",
           email = "ppapadop@its.jnj.com"),
       person(given = "Sheng-Wei",
           family = "Wang",
           role = "aut",
           email = "swang69@its.jnj.com"),
       person(given = "Eli",
           family = "Miller",
           role = "aut",
           email = "eli.miller@atorusresearch.com",
           comment = c(ORCID = "0000-0002-2127-9456")),
       person(given = "Nathan",
           family = "Kosiba",
           email = "nkosiba@its.jnj.com",
           role = "aut",
           comment = c(ORCID = "0000-0001-5359-4234")),
       person(given = "Aidan",
           family = "Ceney",
           role = "aut",
           email = "aceney@its.jnj.com",
           comment = c(ORCID = "0000-0001-8313-487X")),
       person(given = "Janssen R&D",
           role = c("cph", "fnd")),
       person(given = "David",
           family = "Hugh-Jones",
           role = "cph",
           comment = "Author of included 'huxtable' library"),
        person(given = "Konrad",
            family = "Pagacz",
            role = c("aut", "cre"),
            email = "kpagacz@its.jnj.com")
     )
Description: Generate tables, listings, and graphs (TLG) using 'tidyverse'.
  Tables can be created functionally, using a standard TLG process, or by
  specifying table and column metadata to create generic analysis summaries.
  The 'envsetup' package can also be leveraged to create environments for table
  creation.
License: Apache License 2.0
URL: https://pharmaverse.github.io/tidytlg/main/,
        https://github.com/pharmaverse/tidytlg
BugReports: https://github.com/pharmaverse/tidytlg/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: testthat (>= 2.1.0), knitr (>= 1.23), rmarkdown (>= 2.10),
        renv (>= 0.13.2), shiny (>= 1.3.2), kableExtra (>= 1.3.4),
        haven (>= 2.4.1), usethis (>= 1.6.3), withr (>= 2.3.0)
Imports: cli (>= 3.6.0), dplyr (>= 1.1.0), tibble (>= 2.1.3), magrittr
        (>= 1.5), rlang (>= 0.4.10), tidyr (>= 1.0.0), stats (>=
        3.6.0), stringr (>= 1.4.0), forcats (>= 0.5.1), purrr (>=
        0.3.4), huxtable (>= 5.1.0), assertthat (>= 0.2.1), glue (>=
        1.4.2), crayon (>= 1.4.1), methods, readxl (>= 1.3.1),
        cellranger (>= 1.1.0), png (>= 0.1-7), ggplot2 (>= 3.3.2),
        rstudioapi (>= 0.13)
Depends: R (>= 4.1.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-21 10:22:49 UTC; kpagacz
Author: Nicholas Masel [aut],
  Steven Haesendonckx [aut],
  Pelagia Alexandra Papadopoulou [aut],
  Sheng-Wei Wang [aut],
  Eli Miller [aut] (ORCID: <https://orcid.org/0000-0002-2127-9456>),
  Nathan Kosiba [aut] (ORCID: <https://orcid.org/0000-0001-5359-4234>),
  Aidan Ceney [aut] (ORCID: <https://orcid.org/0000-0001-8313-487X>),
  Janssen R&D [cph, fnd],
  David Hugh-Jones [cph] (Author of included 'huxtable' library),
  Konrad Pagacz [aut, cre]
Maintainer: Konrad Pagacz <kpagacz@its.jnj.com>
Repository: CRAN
Date/Publication: 2026-04-21 13:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 19:18:10 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(numeral_formatter,"function")
S3method(numeral_formatter,character)
S3method(numeral_formatter,default)
S3method(numeral_formatter,numeric)
S3method(print,statlist)
S3method(print,tidytlg.freq)
S3method(print,tidytlg.nested_freq)
S3method(print,tidytlg.univar)
export(add_bottom_borders)
export(add_format)
export(add_indent)
export(add_newrows)
export(bind_table)
export(char2factor)
export(col_borders)
export(freq)
export(generate_results)
export(gentlg)
export(get_file_name)
export(insert_empty_rows)
export(nested_freq)
export(no_borders)
export(replace_na_with_blank)
export(rmdpstitle)
export(roundSAS)
export(row_border)
export(single_border)
export(spanning_borders)
export(spanning_headers)
export(statlist)
export(tidytlg_metadata)
export(tidytlg_titles)
export(tlgsetup)
export(univar)
importFrom(cellranger,cell_cols)
importFrom(crayon,bold)
importFrom(crayon,underline)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,cur_group_id)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,ends_with)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_at)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,row_number)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(forcats,fct_expand)
importFrom(forcats,fct_recode)
importFrom(forcats,fct_relevel)
importFrom(ggplot2,ggsave)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(huxtable,everywhere)
importFrom(magrittr,"%>%")
importFrom(magrittr,extract2)
importFrom(methods,formalArgs)
importFrom(png,readPNG)
importFrom(purrr,compact)
importFrom(purrr,discard)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dfr)
importFrom(purrr,map_lgl)
importFrom(purrr,pmap)
importFrom(purrr,walk)
importFrom(readxl,read_excel)
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,as_name)
importFrom(rlang,as_string)
importFrom(rlang,enexpr)
importFrom(rlang,enexprs)
importFrom(rlang,eval_tidy)
importFrom(rlang,expr)
importFrom(rlang,exprs)
importFrom(rlang,is_call)
importFrom(rlang,is_named)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(rstudioapi,getSourceEditorContext)
importFrom(rstudioapi,isAvailable)
importFrom(stats,lag)
importFrom(stats,setNames)
importFrom(stringr,regex)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(stringr,str_replace)
importFrom(stringr,str_split)
importFrom(stringr,str_to_lower)
importFrom(tibble,add_row)
importFrom(tibble,tibble)
importFrom(tibble,tibble_row)
importFrom(tibble,tribble)
importFrom(tidyr,complete)
importFrom(tidyr,crossing)
importFrom(tidyr,fill)
importFrom(tidyr,nest)
importFrom(tidyr,nesting)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,unnest)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,tail)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# tidytlg <img src='man/figures/logo.png' align="right" height="200" style="float:right; height:200px;" />

<!-- badges: start -->

<!-- <img src="man/figures/logo.png" align="right" height = "13"/> -->
[![CRAN
status](https://www.r-pkg.org/badges/version/tidytlg)](https://CRAN.R-project.org/package=tidytlg)
<!-- badges: end -->

The goal of tidytlg is to generate table, listings, and graphs (TLG)
using Tidyverse. This can be achieved multiple ways with this package.

- Functional method: build a custom script for each TLG
- Metadata method: build a generic script that utilizes column and table
  metadata to produce each TLG result

## Installation

### Development version

``` r
# install.packages("devtools")
devtools::install_github("pharmaverse/tidytlg")
```

## Functional method example

``` r
library(dplyr)
library(tidytlg)

# Note cdisc_adsl is built into the package for use
ittpop <- cdisc_adsl %>%
  filter(ITTFL == "Y")

# frequency of Intend-to-Treat patients by planned treatment
tbl1 <- freq(ittpop,
             rowvar = "ITTFL",
             statlist = statlist("n"),
             colvar = "TRT01P",
             rowtext = "Analysis Set:  Intend-to-Treat Population",
             subset = ITTFL == "Y")

# N, MEAN (SD), MEDIAN, RANGE, IQ Range of age by planned treatment
tbl2 <- univar(ittpop,
               rowvar = "AGE",
               colvar = "TRT01P",
               row_header = "Age (Years)")

# frequency of Race by planned treatment
tbl3 <- freq(ittpop,
             rowvar =  "RACE",
             statlist = statlist(c("N", "n (x.x%)")),
             colvar = "TRT01P",
             row_header = "Race, n(%)")

# combine results together
tbl <- bind_table(tbl1, tbl2, tbl3)

# conver to hux object -----------------------------------------------------------------
gentlg(huxme       = tbl ,
       orientation = "landscape",
       file        = "DEMO",
       title       = "Custom Method",
       footers     = "Produced with tidytlg",
       colspan     = list(c("", "", "Xanomeline", "Xanomeline")),
       colheader   = c("", "Placebo", "High", "Low"),
       wcol        = .30)
```

## Metadata method example

``` r
library(dplyr)
library(tidytlg)

adsl <- cdisc_adsl

table_metadata <- tibble::tribble(
  ~anbr,~func,    ~df,     ~rowvar,            ~rowtext,   ~row_header,                    ~statlist,         ~subset,
  1,   "freq", "adsl",     "ITTFL", "Analysis set: itt",            NA,                statlist("n"),  "ITTFL == 'Y'",
  2, "univar", "adsl",       "AGE",                  NA, "Age (Years)",                           NA,              NA,
  3,   "freq", "adsl",      "RACE",                  NA,  "Race, n(%)", statlist(c("N", "n (x.x%)")),              NA
) %>%
  mutate(colvar  = "TRT01PN")

tbl <- generate_results(table_metadata, 
                        column_metadata_file = system.file("extdata/column_metadata.xlsx", package = "tidytlg"),
                        tbltype = "type1") 

# conver to hux object -----------------------------------------------------------------
tblid <- "Table01"

gentlg(huxme       = tbl,
       orientation = "landscape",
       file        = tblid,
       title_file = system.file("extdata/titles.xls", package = "tidytlg"),
       wcol        = .30)
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE------------------------------------------------------------
library(dplyr)
library(tidytlg)

tbl <- tibble::tribble(
  ~label, ~col1, ~col2, ~col3,
  "Analysis Set: ITT", "86", "84", "84",
  "Age (Years)", NA, NA, NA,
  "N", "86", "84", "84"
)

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO1",
  title = "Basic tbl without formatting",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)

## -----------------------------------------------------------------------------
tbl <- tbl %>%
  mutate(newrows = case_when(
    label == "Age (Years)" ~ 1,
    TRUE ~ 0
  ))

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO2",
  title = "Adding the variable of newrows",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)

## -----------------------------------------------------------------------------
tbl <- tbl %>%
  mutate(indentme = case_when(
    label == "N" ~ 1,
    TRUE ~ 0
  ))

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO3",
  title = "Adding the variable of indentme",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)

## -----------------------------------------------------------------------------
tbl <- tbl %>%
  mutate(boldme = case_when(
    label == "Age (Years)" ~ 1,
    TRUE ~ 0
  ))

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO4",
  title = "Adding the variable of boldme",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)

## -----------------------------------------------------------------------------
tbl <- cdisc_adsl %>%
  univar(
    colvar = "TRT01PN",
    rowvar = "AGE",
    statlist = statlist(c("N", "MEANSD")),
    decimal = 0,
    row_header = "Age (Years)"
  ) %>%
  mutate(anbr = "01")

knitr::kable(tbl)

## -----------------------------------------------------------------------------
tbl <- cdisc_adsl %>%
  univar(
    colvar = "TRT01PN",
    rowvar = "AGE",
    statlist = statlist(c("N", "MEANSD")),
    decimal = 0,
    row_header = "Age (Years)"
  ) %>%
  mutate(anbr = "01") %>%
  add_format()

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  tlf = "Table",
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO5",
  title = "Using row_type to set up indentation",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "figures"
)

working_dir <- tempdir()

## ----echo=FALSE, out.width = "600px"------------------------------------------
knitr::include_graphics("titles.PNG")

## ----echo=FALSE, out.width = "500px"------------------------------------------
knitr::include_graphics("column_metadata.PNG")

## ----echo=FALSE, out.width = "600px", out.height = "60px"---------------------
knitr::include_graphics("column_header.PNG")

## ----message=FALSE------------------------------------------------------------
# Prep Environment -------------------------------------------------------------
library(dplyr)
library(haven)
library(tidytlg)

# read adsl from PhUSE test data factory
testdata <- "https://github.com/phuse-org/TestDataFactory/raw/refs/heads/main/Updated/TDF_ADaM/adsl.xpt"
adsl <- read_xpt(url(testdata))

## ----message=FALSE------------------------------------------------------------
# Process Data -----------------------------------------------------------------
adsl <- adsl %>%
  filter(ITTFL == "Y") %>%
  mutate(
    SEX = factor(
      SEX,
      levels = c("M", "F", "U"), labels = c("Male", "Female", "Unknown")
    )
  ) %>%
  tlgsetup(
    var = "TRT01PN",
    column_metadata_file = system.file(
      "extdata/column_metadata.xlsx",
      package = "tidytlg"
    ),
    tbltype = "type3"
  )

## ----message=FALSE------------------------------------------------------------
# Generate Results -------------------------------------------------------------

## Analysis set row
t1 <- adsl %>%
  freq(
    colvar = "colnbr",
    rowvar = "ITTFL",
    statlist = statlist("n"),
    subset = ITTFL == "Y",
    rowtext = "Analysis set: ITT"
  )

## Univariate summary for AGE
t2 <- adsl %>%
  univar(
    colvar = "colnbr",
    rowvar = "AGE",
    statlist = statlist(c("N", "MEANSD", "MEDIAN", "RANGE", "IQRANGE")),
    decimal = 0,
    row_header = "Age, years"
  )

## Count (percentages) for SEX
t3 <- adsl %>%
  freq(
    colvar = "colnbr",
    rowvar = "SEX",
    statlist = statlist(c("N", "n (x.x%)")),
    row_header = "Gender"
  )

## -----------------------------------------------------------------------------
# Format Results ---------------------------------------------------------------

tbl <- bind_table(t1, t2, t3,
  column_metadata_file = system.file(
    "extdata/column_metadata.xlsx",
    package = "tidytlg"
  ),
  tbltype = "type3"
)

## -----------------------------------------------------------------------------
knitr::kable(tbl)

## -----------------------------------------------------------------------------
tblid <- "Table01"

gentlg(
  huxme = tbl,
  opath = file.path(working_dir),
  file = tblid,
  orientation = "landscape",
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)[[1]]

## -----------------------------------------------------------------------------
gentlg(
  huxme = tbl,
  format = "HTML",
  print.hux = FALSE,
  file = tblid,
  orientation = "landscape",
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)[[1]]

## -----------------------------------------------------------------------------
# Prep Environment -------------------------------------------------------------
library(dplyr)
library(haven)
library(tidytlg)

adsl <- cdisc_adsl
adae <- cdisc_adae

# Process Data -----------------------------------------------------------------
adsl <- adsl %>%
  filter(SAFFL == "Y") %>%
  select(USUBJID, SAFFL, TRT01AN, TRT01A)

adae <- adae %>%
  filter(SAFFL == "Y" & TRTEMFL == "Y") %>%
  mutate(
    BSPT = paste(AEBODSYS, "[", AEDECOD, "]"),
    SAEFL = if_else(AESER == "Y", "Yes", "No"),
    DTHFL = if_else(AEOUT == "FATAL", "Yes", "No")
  ) %>%
  select(USUBJID, ASTDY, TRTA, BSPT, AETERM, SAEFL, DTHFL)

tbl <- inner_join(adsl, adae, by = "USUBJID") %>%
  arrange(TRT01AN, USUBJID, ASTDY) %>%
  select(TRT01A, USUBJID, ASTDY, TRTA, BSPT, AETERM, SAEFL, DTHFL) %>%
  filter(USUBJID %in% c("01-701-1015", "01-701-1023"))

# Output Results ---------------------------------------------------------------
gentlg(
  huxme = tbl,
  tlf = "l",
  format = "HTML",
  print.hux = FALSE,
  orientation = "landscape",
  file = "Listing01",
  title = "Listing of Adverse Events",
  idvars = c("TRT01A", "USUBJID"),
  wcol = 0.15,
  colheader = c(
    "Treatment Group",
    "Subject ID",
    "Study Day of AE",
    "Treatment Period",
    "Body System [Preferred Term]",
    "Verbatim Term",
    "Serious",
    "Fatal"
  )
)[[1]]

## -----------------------------------------------------------------------------
# Prep Environment -------------------------------------------------------------
library(dplyr)
library(haven)
library(ggplot2)
library(tidytlg)

# read adsl from PhUSE test data factory
testdata <- "https://github.com/phuse-org/TestDataFactory/raw/refs/heads/main/Updated/TDF_ADaM/adsl.xpt"
adsl <- read_xpt(url(testdata))

tblid <- "Graph01"

# Process Data -------------------------------------------------------
adsl <- adsl %>%
  filter(ITTFL == "Y") %>%
  select(USUBJID, ITTFL, TRT01PN, TRT01P, AGE, SEX, HEIGHTBL, WEIGHTBL) %>%
  mutate(SEX = factor(SEX, levels = c("M", "F"), labels = c("Male", "Female")))

# Generate Results -------------------------------------------------------------

plot <- ggplot(data = adsl, aes(x = HEIGHTBL, y = WEIGHTBL)) +
  geom_point() +
  labs(
    x = "Baseline Height (cm)",
    y = "Baseline Weight (kg)"
  ) +
  facet_wrap(~SEX, nrow = 1)

# create png file
png(
  file.path(working_dir, paste0(tblid, ".png")),
  width = 2800, height = 1300, res = 300, type = "cairo"
)

plot

dev.off()

# Output Results ---------------------------------------------------------------

gentlg(
  tlf = "g",
  plotnames = file.path(
    system.file("extdata", package = "tidytlg"), paste0(tblid, ".png")
  ),
  plotwidth = 10,
  plotheight = 5,
  orientation = "landscape",
  opath = file.path(working_dir), ,
  file = tblid,
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)

## ----echo=FALSE, out.width = "750px"------------------------------------------
knitr::include_graphics("graph01.PNG")

## -----------------------------------------------------------------------------
library(dplyr)
library(haven)
library(tidytlg)

# read adsl from PhUSE test data factory
testdata <- "https://github.com/phuse-org/TestDataFactory/raw/refs/heads/main/Updated/TDF_ADaM/adsl.xpt"
adsl <- read_xpt(url(testdata))

# Process data
adsl <- adsl %>%
  filter(ITTFL == "Y") %>%
  mutate(
    SEX = factor(
      SEX,
      levels = c("M", "F", "U"), labels = c("Male", "Female", "Unknown")
    )
  )

# define table metadata
table_metadata <- tibble::tribble(
  ~func, ~df, ~rowvar, ~decimal, ~rowtext, ~row_header, ~statlist, ~subset,
  "freq",
  "adsl", "ITTFL", NA, "Analysis set: ITT", NA, statlist("n"), "ITTFL == 'Y'",
  "univar", "adsl", "AGE", 0, NA, "Age (Years)", NA, NA,
  "freq", "adsl", "SEX", NA, NA, "Gender", statlist(c("N", "n (x.x%)")), NA
) %>%
  mutate(colvar = "TRT01PN")

# Generate results
tbl <- generate_results(table_metadata,
  column_metadata_file = system.file(
    "extdata/column_metadata.xlsx",
    package = "tidytlg"
  ),
  tbltype = "type3"
)

# Output results
tblid <- "Table01"

gentlg(
  huxme = tbl,
  format = "HTML",
  print.hux = FALSE,
  file = tblid,
  orientation = "landscape",
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)[[1]]

## ----message=FALSE------------------------------------------------------------
library(dplyr)
library(haven)
library(tidytlg)

# read adsl from PhUSE test data factory
testdata <- "https://github.com/phuse-org/TestDataFactory/raw/refs/heads/main/Updated/TDF_ADaM/adsl.xpt"
adsl <- read_xpt(url(testdata))

# Process data
adsl <- adsl %>%
  filter(ITTFL == "Y") %>%
  mutate(SEX = factor(SEX, levels = c("M", "F"), labels = c("Male", "Female")))

# define table metadata
table_metadata <- tibble::tribble(
  ~func, ~df, ~rowvar, ~decimal, ~rowtext,
  ~row_header, ~statlist, ~subset, ~tablebyvar,
  "univar", "adsl", "AGE", 0, NA, "Age (Years)", NA, NA, "SEX",
  "freq", "adsl", "RACE", NA, NA,
  "Race", statlist(c("N", "n (x.x%)")), NA, "SEX"
) %>%
  mutate(colvar = "TRT01PN")

# Generate results
tbl <- generate_results(table_metadata,
  column_metadata_file = system.file(
    "extdata/column_metadata.xlsx",
    package = "tidytlg"
  ),
  tbltype = "type3"
)

# Output results
tblid <- "Table01"

gentlg(
  huxme = tbl,
  format = "HTML",
  print.hux = FALSE,
  file = tblid,
  orientation = "landscape",
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)[[1]]

#' check_req_arg
#'
#' @param arg required argument to check
#'
#' @return TRUE/FALSE based on if argument is NULL
#' @noRd
check_req_arg <- function(arg) {
  arg <- arg[[1]]
  !is.null(arg)
}

#' check_is_statlist
#'
#' @param arg required argument to check
#'
#' @return TRUE/FALSE if is statlist
#' @noRd
check_is_statlist <- function(arg) {
  class(arg)[1] == "statlist"
}

#' check_is_nested_rowvar
#'
#' @param arg required argument to check
#'
#' @return TRUE/FALSE if is rowvar is correct
#' @noRd
check_is_nested_rowvar <- function(arg) {
  if (!stringr::str_detect(arg, "[a-zA-Z0-9]+\\*[a-zA-Z0-9]+")) {
    return(FALSE)
  }
  if (stringr::str_detect(
    arg,
    "[a-zA-Z0-9]+\\*[a-zA-Z0-9]+\\*[a-zA-Z0-9]+\\*[a-zA-Z0-9]+"
  )) {
    return(FALSE)
  }
  stringr::str_detect(arg, "^[a-zA-Z0-9]+\\*([a-zA-Z0-9]+\\**)+$")
}


#' check_is_data_frame
#'
#' @param arg argument to check
#'
#' @return TRUE/FALSE based on if argument is a dataframe
#' @noRd
check_is_data_frame <- function(arg) {
  arg <- arg[[1]]
  is.data.frame(arg)
}

#' check_is_numeric
#'
#' @param arg argument to check
#'
#' @return TRUE/FALSE based on if argument is a numeric
#' @noRd
check_is_numeric <- function(arg) {
  arg <- arg[[1]]
  is.numeric(arg)
}

#' check_is_boolean
#'
#' @param arg argument to check
#'
#' @return TRUE/FALSE based on if argument is a boolean
#' @noRd
check_is_boolean <- function(arg) {
  arg <- arg[[1]]
  is.logical(arg)
}

#' check_is_character
#'
#' @param arg argument to check
#'
#' @return TRUE/FALSE based on if argument is a character
#' @noRd
check_is_character <- function(arg) {
  arg <- arg[[1]]
  is.character(arg)
}


#' check_arg_exists
#'
#' @param arg required argument to check
#'
#' @return TRUE/FALSE based on if argument exists in df
#' @noRd
check_arg_exists <- function(arg, df) {
  arg <- arg[[1]]
  if (length(arg) == 1) {
    if (arg %>% stringr::str_detect("\\*")) {
      arg <- arg %>%
        stringr::str_split("\\*") %>%
        base::unlist()
    }
  }
  if (!all(arg %in% names(df))) {
    return(FALSE)
  }
  TRUE
}
#' check_arg_exists
#'
#' @param arg required argument to check
#'
#' @return TRUE/FALSE based on if argument exists in df
#' @noRd
check_file <- function(arg) {
  arg <- arg[[1]]
  if (is.null(arg)) {
    TRUE
  } else {
    file.exists(arg)
  }
}


#' check_words
#'
#' @param ... permissible words in the list
#' @param values the list to check for specific words
#'
#' @noRd
check_words <- function(..., values) {
  accepted_words <- unlist(c(...))
  expr <- expr(function(values) values %in% !!accepted_words)
  make_function(body = expr, env = parent.frame())()
}

make_function <- function(args = pairlist(), body, env = parent.frame()) {
  eval(call("function", args, body), env)
}


#' `check_statlist_univar`
#'
#' @param vec statlist argument to `univar`
#'
#' @return TRUE/FALSE based on if stat in accepted values
#' @noRd
check_statlist_univar <- function(statlist_values) {
  accepted_stats <- c(
    "N", "SUM", "MEAN", "GeoMEAN", "SD", "SE", "CV", "GSD", "GSE",
    "MEANSD", "MEANSE", "MEDIAN", "MIN", "MAX", "RANGE", "Q1", "Q3",
    "IQRANGE", "MEDRANGE", "MEDIQRANGE", "MEAN_CI", "GeoMEAN_CI"
  )
  statlist_values[!(statlist_values %in% accepted_stats)]
}

#' check_statlist_freq
#'
#' @param vec statlist argument to freq and nested_freq
#'
#' @return TRUE/FALSE based on if stat in accepted values
#' @noRd
check_statlist_freq <- function(statlist_values) {
  accepted_stats <- c(
    "n", "n (x.x)", "n (x.x%)", "n/N", "n/N (x.x)",
    "n/N (x.x%)", "N"
  )
  statlist_values[!(statlist_values %in% accepted_stats)]
}

#' check_cutoff_stat
#'
#' @param cutoff_stat argument from freq/nested_freq
#'
#' @return TRUE/FALSE based on if stat in accepted values
#' @noRd
check_cutoff_stat <- function(func, cutoff_stat) {
  if (cutoff_stat %in% c("pct", "n")) {
    return()
  }
  stop("Cutoff stat ", cutoff_stat,
    " for function ", func, "\n\n",
    call. = FALSE
  )
}


#' check_arg_tlf
#'
#' @param arg tlg argument from gentlg
#'
#' @return TRUE/FALSE based on if arg in accepted values
#' @noRd
check_arg_tlf <- function(arg) {
  func <- check_words("t", "l", "f", "g")
  do.call(func, list(substr(tolower(arg), 1, 1)))
}

#' check_arg_format
#'
#' @param arg tlg argument from gentlg
#'
#' @return TRUE/FALSE based on if arg in accepted values
#' @noRd
check_arg_format <- function(arg) {
  func <- check_words("rtf", "html")
  do.call(func, list(tolower(arg)))
}

#' check_names_column_metadata
#'
#' @param column_metadata column metadata from generate_results()
#'
#' @return TRUE/FALSE based on if names are in accepted values
#' @noRd
check_names_column_metadata <- function(column_metadata) {
  all(c("coldef", "decode") %in% names(column_metadata))
}

#' check_any_na
#'
#' @param vec vector to check
#'
#' @return TRUE/FALSE based on if NA values are found
#' @noRd
check_any_na <- function(vec) {
  !any(is.na(vec))
}

#' check_all_na
#'
#' @param vec vector to check
#'
#' @return TRUE/FALSE based on if vector is all NA values
#' @noRd
check_all_na <- function(vec) {
  !all(is.na(vec))
}

#' check_geo_est
#'
#' @param rowvar row variable to check for geometric estimability
#'
#' @return TRUE/FALSE based on if geometric stats are estimable
#' @noRd
check_geo_est <- function(rowvar, df) {
  !any(df[[rowvar[[1]]]] < 0)
}

#' check_decimal
#'
#' @param decimal
#'
#' @return TRUE/FALSE if number is greater than or equal to 0
#' @noRd
check_decimal <- function(decimal) {
  0 <= decimal
}


#' check_nested_vars
#'
#' @param nested_vars argument that has nested vars to check
#'
#' @return TRUE/FALSE if argument is structured correctly
#' @noRd
check_nested_vars <- function(nested_vars) {
  stringr::str_detect(nested_vars, "^([:alnum:]+\\*)([:alnum:]+\\*)?[:alnum:]+$")
}

#' check_in_df
#'
#' @param func argument base function name
#' @param arglist  argument to check
#' @param df_field which dataframe to test
#' @param args_req_check required argument to check
#'
#' @return NULL
#' Throws error if arg does not exist in df
#' @noRd
check_in_df <- function(func,
                        arglist,
                        df_field = "df",
                        args_req_check =
                          c(
                            "colvar", "rowvar",
                            "rowbyvar", "tablebyvar",
                            "precisionby", "precisionon",
                            "groupby", "tableby", "idvars", "var"
                          )) {
  args_req_check <- args_req_check[args_req_check %in% names(arglist)]

  tests <- tidyr::tibble(args = "arglist", el = args_req_check)
  tests <- tests %>% dplyr::mutate(
    test = "check_arg_exists",
    level = "stop", msg = paste0(
      el, " '", arglist[el],
      "' does not exist in ", df_field, " or is all NA for function: ", func
    ),
    req_df = TRUE
  )
  check_args(func, arglist, tests, field = df_field)
}

#' check_nested_rowvar
#'
#' @param func argument base function name
#' @param arglist required argument to check
#' @param df_field required df to check
#'
#' @return NULL
#' Throws error if arg does not exist in df
#' @noRd
check_nested_rowvar <- function(func, arglist, df_field = "df") {
  tests <- tidyr::tibble(args = "arglist", el = "rowvar")
  tests <- tests %>% dplyr::mutate(
    test = "check_is_nested_rowvar",
    level = "stop",
    msg = "Argument rowvar to function nested_freq is incorrectly formatted.",
    req_df = FALSE
  )
  check_args(func, arglist, tests, field = df_field)
}

#' check_in_df_bind_table
#'
#' @param func argument base function name
#' @param dfs list of dfs to check
#' @param arglist required argument to check
#'
#' @return NULL
#' Throws error if arg does not exist in df
#' @noRd
check_in_df_bind_table <- function(func, dfs, arglist) {
  args_req_check <- c("rowbyvar", "tablebyvar")
  args_req_check <- args_req_check[args_req_check %in% names(arglist)]

  count <- 1
  for (i in dfs) {
    arglist_tmp <- append(arglist, list(i))
    names(arglist_tmp)[length(arglist_tmp)] <- "df"
    tests <- tidyr::tibble(args = "arglist", el = args_req_check)
    tests <- tests %>% dplyr::mutate(
      test = "check_arg_exists",
      level = "stop", msg = paste0(
        el, " '", arglist[el],
        "' does not exist in df ", count, " or is all NA for function: ", func
      ),
      req_df = TRUE
    )
    check_args(func, arglist_tmp, tests)
    count <- count + 1
  }
}

#' check_req_fields
#'
#' @param func argument base function name
#' @param arglist required argument to check
#' @param req_fields fields in df that are required
#'
#' @return NULL
#' Throws error if arg does not exist in df
#' @noRd
check_req_fields <- function(func, arglist, req_fields) {
  df <- arglist[["df"]]
  missing <- req_fields[!(req_fields %in% names(df))]
  if (length(missing) == 0) {
    return()
  }

  stop("Reqiured fields ", paste0(missing, colapse = ", "),
    " for function ", func, "\n\n",
    call. = FALSE
  )
}

#' check_cutoff_format
#'
#' @param func that is being checked
#' @param cutoff to check
#'
#' @return TRUE/FALSE if is cutoff format is correct
#' @noRd
check_cutoff_format <- function(func, cutoff) {
  if (is.null(cutoff)) {
    return()
  }
  if (stringr::str_detect(
    cutoff, "^([a-zA-Z0-9]+|[a-zA-Z0-9]+ >= [a-zA-Z0-9]+( (&|\\|) [a-zA-Z0-9]+ >= [a-zA-Z0-9]+)*)$"
  )) {
    return()
  }
  stop("cutoff for function ", func,
    " is incorrectly formated\n\n",
    call. = FALSE
  )
}


#' check_req_fields
#'
#' @param func argument base function name
#' @param arglist required argument to check
#' @param req_fields fields in df that are required
#'
#' @return NULL
#' Throws error if arg does not exist in df
#' @noRd
check_req_fields <- function(func, arglist, req_fields) {
  df <- arglist[["df"]]
  missing <- req_fields[!(req_fields %in% names(df))]
  if (length(missing) == 0) {
    return()
  }

  stop("Reqiured fields ", paste0(missing, colapse = ", "),
    " for function ", func, "\n\n",
    call. = FALSE
  )
}

#' check_req_vars
#'
#' @param func argument base function name
#' @param arglist required argument to check
#' @param lst_req required arguments for function
#'
#' @return NULL
#' Throws error if required arg is missing
#' @noRd
check_req_vars <- function(func, arglist, lst_req) {
  tests <- tidyr::tibble(args = "arglist", el = lst_req)
  tests <- tests %>% dplyr::mutate(
    test = "check_req_arg",
    level = "stop", msg = paste0(
      "Argument ",
      el, " to function ", func,
      " is required but no value has been supplied"
    ),
    req_df = FALSE
  )
  check_args(func, arglist, tests)
}

#' check_file_exists
#'
#' @param func argument base function name
#' @param arglist required argument to check
#' @param file required arguments for function
#'
#' @return NULL
#' Throws error file missing
#' @noRd
check_file_exists <- function(func, arglist, file) {
  tests <- tidyr::tibble(args = "arglist", el = file)
  tests <- tests %>% dplyr::mutate(
    test = "check_file",
    level = "stop", msg = paste0(
      "Argument ",
      file, " to function ", func,
      " file does not exist."
    ),
    req_df = FALSE
  )
  check_args(func, arglist, tests)
}


#' check_var_types
#'
#' @param func argument base function name
#' @param arglist required argument to check
#' @param lst_types required arguments types
#'
#' @return NULL
#' Throws error if arg is incorrect types
#' @noRd
check_var_types <- function(func, arglist, lst_types) {
  tests_for_types <- list(
    "data.frame" = "check_is_data_frame",
    "numeric" = "check_is_numeric",
    "character" = "check_is_character",
    "boolean" = "check_is_boolean"
  )
  tests <- tidyr::tibble(args = "arglist", el = names(lst_types))
  tests <- tests %>% dplyr::mutate(
    test = tests_for_types[lst_types],
    level = "stop", msg = paste0(
      "Argument ",
      el, " to function ", func,
      " should be type ", lst_types
    ),
    req_df = FALSE
  )
  check_args(func, arglist, tests)
}


#' check_var_duplicates
#'
#' @param func argument base function name
#' @param arglist required argument to check
#'
#' @return NULL
#' Throws error if arg reference the same value
#' @noRd
check_var_duplicates <- function(func, arglist) {
  args_req_check <- c(
    "colvar", "rowvar", "rowbyvar",
    "tablebyvar", "groupby", "tableby"
  )
  args_req_check <- arglist[names(arglist) %in% args_req_check]

  if (length(args_req_check) < 2) {
    return()
  }

  # Breaks apart nested rowvar
  if (!is.null(arglist[["rowvar"]])) {
    if (arglist[["rowvar"]] %>% stringr::str_detect("\\*")) {
      row_var_list <- arglist[["rowvar"]] %>%
        stringr::str_split("\\*") %>%
        base::unlist()
      names(row_var_list) <- paste0("nested rowvar ", seq_along(row_var_list))
      args_req_check <- args_req_check[-(which(names(args_req_check) == "rowvar"))]
      args_req_check <- append(args_req_check, row_var_list)
    }
  }
  df_dupes <- as.data.frame(args_req_check)
  lst_dupes <- unlist(args_req_check)

  df_dupes <- df_dupes %>%
    dplyr::rowwise() %>%
    dplyr::mutate(
      CHECK = length(args_req_check) == length(unique(dplyr::c_across(cols = everything())))
    )

  if (df_dupes$CHECK[1]) {
    return()
  }

  dup_count <- df_dupes %>%
    t() %>%
    base::as.data.frame() %>%
    dplyr::count(V1) %>%
    dplyr::filter(n > 1)
  first <- TRUE
  for (i in dup_count$V1) {
    if (first) {
      msg <- paste0(
        paste0(names(lst_dupes[lst_dupes == i]), collapse = ", "),
        " all have value ", i, " for function ", func
      )
      first <- FALSE
    } else {
      msg <- paste0(msg, "\n", paste0(names(lst_dupes[lst_dupes == i]),
        collapse = ", " # nolint indentation_linter
      ), " all have value ", i, " for function ", func)
    }
  }

  stop(paste0(msg, "\n\n"), call. = FALSE)
}

#' check_statlist
#'
#' @param func argument base function name
#' @param astatlist statlist to test
#'
#' @return NULL
#' Throws error if statlist contains incorrect stat or stats
#' @noRd
check_statlist <- function(func, astatlist) {
  if (is.null(astatlist)) {
    msg <- paste0(
      "statlist for function ", func,
      " is empty\nIf you would like to see a full list of included stats ",
      "type `?", func, "` in the console"
    )
  } else {
    if (func == "univar") {
      errs <- check_statlist_univar(astatlist)
    } else {
      errs <- check_statlist_freq(astatlist)
    }

    if (length(errs) == 0) {
      return()
    }

    msg <- paste0(
      "The following values are included in statlist",
      " to function ", func, " but are not allowed: ",
      paste0(errs, collapse = ", "),
      " if you would like to see a full list of included stats ",
      "type `?", func, "` in the console"
    )
  }
  stop(paste0(msg, "\n\n"), call. = FALSE)
}


#' check_statlist_n
#' Throws error if N is used with no other args used by freq functions
#' @param func argument base function name
#' @param astatlist statlist to test
#'
#' @return NULL
#' @noRd
check_statlist_n <- function(func, astatlist) {
  if (!("N" %in% astatlist)) {
    return()
  }
  if ("N" %in% astatlist && length(astatlist) != 1) {
    return()
  }

  msg <- paste0("Stat N must be used with another stat ex. statlist(c('N','n (x.x%)')) for function ", func)

  stop(msg, call. = FALSE)
}

#' check_freq_cutoff
#' @param df freq dataframe
#'
#' @return NULL
#' Throws error if cutoff results in empty df, called by freq
#' @noRd
check_freq_cutoff <- function(df) {
  if (nrow(df) == 0) {
    stop("cutoff resulted in an empty table for function freq\n\n",
      call. = FALSE
    )
  } else {
    invisible(NULL)
  }
}

#' check_geo_stats
#' @param arglist arguments to check
#'
#' @return NULL
#' Throws error if geo stats can not be calculated and are present
#' @noRd
check_geo_stats <- function(arglist) {
  # add optional checks for special cases
  if (!any(
    arglist[["statlist"]][["stats"]] %in%
      c("GeoMEAN", "GSD", "GSE", "GeoMEAN_CI")
  )) {
    return()
  }

  tests <- tibble(
    args = "arglist", el = "rowvar", test = "check_geo_est",
    level = "stop",
    msg = (paste0(
      el, " argument to ", "univar",
      " contain negative values, geometric stats are not estimable"
    )), req_df = TRUE
  )

  # run tier two tests
  check_args("univar", arglist, tests)
}

#' check_alpha
#' @param alpha argument for univar
#'
#' @return NULL
#' Throws error if alpha is not between 0 and 1
#' @noRd
check_alpha <- function(alpha) {
  if (0 <= alpha && alpha <= 1.0) {
    return()
  }
  stop("alpha must be a number in between 0 and 1 for function univar\n\n",
    call. = FALSE
  )
}


#' check_opath
#'
#' @param opath argument to check if they exist
#'
#' @return NULL
#' Throws error if opath does not exist
#' @noRd
check_opath <- function(opath) {
  if (file.exists(opath)) {
    return()
  }
  stop(paste0("opath '", opath, "' does not exist for function gentlg", "\n\n"),
    call. = FALSE
  )
}

#' check_wcol
#' @param huxme dataframe to check
#' @param wcol argument to check if they exist
#'
#' @return NULL
#' Throws error if `wcol` is not the correct length
#' @noRd
check_wcol <- function(huxme, wcol) {
  lst_to_remove <- c(
    "anbr", "roworder", "boldme", "indentme", "newrows", "newpage",
    "rowvar", "row_type", "nested_level", "group_level"
  )
  huxme_output <- huxme %>% dplyr::select(!c(
    any_of(c("func", lst_to_remove)),
    ends_with("_ord")
  ))

  if (length(wcol) == 1 || length(wcol) == length(huxme_output)) {
    return()
  }
  stop(paste0("wcol's length must be 1 or the length of final output", "\n\n"),
    call. = FALSE
  )
}

#' check_plotnames
#' Throws error if `plotnames` do not exist
#' @param plotnames plotname to check
#'
#' @return NULL
#' @noRd
check_plotnames <- function(plotnames) {
  if (is.null(plotnames)) {
    return()
  }
  if (all(file.exists(plotnames))) {
    return()
  }
  stop(paste0(
    "plotnames '", plotnames,
    "' does not exist for function gentlg", "\n\n"
  ), call. = FALSE)
}

#' check_orientation
#'
#' @param orientation argument to check if they exist
#'
#' @return NULL
#' Throws error if orientation is incorrect
#' @noRd
check_orientation <- function(orientation) {
  if (orientation %in% c("landscape", "portrait")) {
    return()
  }
  stop(paste0(
    "orientation '", orientation,
    "' is not either 'landscape' or 'portrait' for function gentlg",
    "\n\n"
  ), call. = FALSE)
}

#' check_dfs_formats
#'
#' @param dfs to check for format issues
#'
#' @return NULL
#' Throws error if any df has a missing required field
#' @noRd
check_dfs_formats <- function(dfs) {
  index_missing <- c()
  count <- 1
  for (i in dfs) {
    if (!all((c("row_type", "group_level") %in% names(i)))) {
      index_missing <- c(index_missing, count)
    }
    count <- count + 1
  }
  if (length(index_missing) == 0) {
    return()
  }
  stop(paste0(
    "dfs ", paste0(index_missing, collapse = ", "),
    " are missing the required fields for function bind_table",
    "\n\n"
  ), call. = FALSE)
}

#' check_dfs
#'
#' @param dfs to check if provided
#'
#' @return NULL
#' Throws error if any df has a missing required field
#' @noRd
check_dfs <- function(dfs) {
  if (length(dfs) >= 1) {
    return()
  }
  stop("dfs should be provided for function bind_table\n\n", call. = FALSE)
}

#' check_column_metadata_file
#'
#' @param column_metadata_file meta data file to check
#'
#' @return NULL
#' Throws error if file does not exist then throws another error if file exists but has incorrect version
#' @noRd
check_column_metadata_file <- function(column_metadata_file) {
  if (!file.exists(column_metadata_file)) {
    stop(paste0(
      "column_metadata_file '", column_metadata_file,
      "' does not exist for function generate_results", "\n\n"
    ), call. = FALSE)
  }
  check_column_metadata(readxl::read_excel(column_metadata_file))
}

#' check_column_metadata
#'
#' @param column_metadata to check for format issues
#'
#' @return NULL
#' Throws error if any column_metadata has a missing required field
#' @noRd
check_column_metadata <- function(column_metadata) {
  if (is.null(column_metadata)) {
    stop(paste0(
      "column_metadata is missing for function generate_results",
      "\n\n"
    ), call. = FALSE)
  }

  if (!all(c("tbltype", "coldef", "decode") %in% names(column_metadata))) {
    cheked_fields <- c("tbltype", "coldef", "decode", "span1")
    needed_fields <- cheked_fields[!cheked_fields %in% names(column_metadata)]
    stop(paste0(
      "column_metadata requries fields '", paste0(needed_fields, collapse = "',
                                                  '"),
      "' for function generate_results",
      "\n\n"
    ), call. = FALSE)
  }
}

#' check_table_metadata
#'
#' @param table_metadata to check for format issues
#' @param env to check for values
#'
#' @return NULL
#' Throws error if any column_metadata has a missing required field
#' @noRd
check_table_metadata <- function(table_metadata, env) {
  if (is.null(table_metadata)) {
    stop(paste0(
      "column_metadata is missing for function generate_results",
      "\n\n"
    ), call. = FALSE)
  }

  if (!all(c("func", "df", "colvar", "rowvar") %in% names(table_metadata))) {
    check_fields <- c("func", "df", "colvar", "rowvar")
    needed_fields <- check_fields[!check_fields %in% names(table_metadata)]
    stop(paste0(
      "table_metadata requries fields '", paste0(needed_fields, collapse = "',
                                                 '"),
      "' for function generate_results",
      "\n\n"
    ), call. = FALSE)
  }
  for (i in table_metadata[["func"]]) {
    if (!(i %in% c("nested_freq", "freq", "univar"))) {
      stop(paste0(
        "table_metadata field 'func' can either be 'freq',
        'nested_freq', 'univar' for function generate_results",
        "\n\n"
      ), call. = FALSE)
    }
  }

  for (i in table_metadata[["df"]]) {
    if (!exists(i, envir = env)) {
      stop(paste0(
        "table_metadata field 'df' points to data frame that does not exist for function generate_results",
        "\n\n"
      ), call. = FALSE)
    }
  }
}

#' check_elements
#'
#' @param source_func name of function (for warning and error clarification)
#' @param args list of a arguments from function
#' @param el argument to be checked
#' @param func function to be used to check the argument
#' @param level stop/warning level of function to be used if issues are found
#' @param msg message to be evaluated if test fails
#' @param req_df does test require the df argument to test
#' @param df_field name of df from args that will be checked
#'
#' @return list of errors and warnings generated
#' @noRd
check_elements <- function(source_func, args, el,
                           func, level, msg, req_df, df_field) {
  element <- as_string(el)
  vec <- args[element]

  # get the stats out of statlist
  if (el == "statlist") {
    vec <- vec$statlist[["stats"]]
  }

  # unlist things so we can run them through the do.call
  if (c(el) %in% c("statlist", "cutoff_stat", "variable_names")) {
    vec <- unlist(vec)
  }

  field_to_grab <- df_field
  if (req_df) {
    passed_vec <- list(vec, args[[field_to_grab]])
  } else {
    passed_vec <- list(vec)
  }

  # run test functions
  failures <- vec %>%
    discard(~ do.call(func, passed_vec)) %>%
    unique()


  # create output messages
  if (length(failures) > 0) {
    if (is.language(msg)) {
      string <- eval(msg)
    } else if (!is.na(msg)) {
      string <- msg
    } else {
      string <- paste0(
        "Argument ", element,
        " to function ", source_func,
        " fails ", func, " check \n"
      )
    }
  } else {
    string <- NULL
  }

  # assign output depending on level
  if (level == "stop") {
    error_string <- string
    warning_string <- NULL
  } else if (level == "warning") {
    error_string <- NULL
    warning_string <- string
  }

  list(warning = warning_string, error = error_string)
}


#' check_args
#'
#' @param func function that is being checked
#' @param arglist list of arguments to be used
#' @param tests tibble of tests to be used
#' @param field dataframe name for testing
#' @noRd
check_args <- function(func, arglist, tests, field = "df") {
  messages <- pmap(
    tests,
    ~ check_elements(
      func, get(..1), sym(..2), ..3, ..4, ..5, ..6, field
    )
  )

  # errors
  errors <- map(messages, "error") %>%
    compact() %>%
    paste0(., collapse = "\n\n")
  if (errors != "") {
    stop(paste0(errors, "\n\n"), call. = FALSE)
  }

  # warnings
  warnings <- map(messages, "warning") %>%
    compact() %>%
    paste0(., collapse = "\n\n")
  if (warnings != "") {
    warning(paste0(warnings, "\n\n"), call. = FALSE)
  }
}

#' check_univar
#'
#' @param arglist arguments to univar function
#'
#' @noRd
check_univar <- function(arglist) {
  if (!check_is_statlist(arglist[["statlist"]])) {
    stop("statlist must be of type statlist EX. statlist(c('N'))")
  }

  check_req_vars("univar", arglist, c("df", "colvar", "rowvar"))

  var_type_check <- c(
    "df" = "data.frame", "wide" = "boolean",
    "colvar" = "character", "rowvar" = "character",
    "rowbyvar" = "character", "decimal" = "numeric",
    "precisionby" = "character", "precisionon" = "character",
    "rowtext" = "character", "row_header" = "character",
    ".keep" = "boolean", " .ord" = "boolean",
    "alpha" = "numeric"
  )


  var_type_check <- var_type_check[names(var_type_check) %in% names(arglist)]
  check_var_types("univar", arglist, var_type_check)
  # run stats tests
  check_in_df("univar", arglist)
  check_var_duplicates("univar", arglist)
  check_statlist("univar", arglist[["statlist"]]$stats)
  check_geo_stats(arglist)
  check_alpha(arglist[["alpha"]])
}

#' check_freq
#'
#' @param arglist arguments to freq function
#'
#' @noRd
check_freq <- function(arglist) {
  if (!check_is_statlist(arglist[["statlist"]])) {
    stop("statlist must be of type statlist EX. statlist(c('n'))")
  }

  check_req_vars("freq", arglist, c("df", "colvar", "rowvar"))

  var_type_check <- c(
    "df" = "data.frame", "denom_df" = "data.frame",
    "colvar" = "character", "rowvar" = "character",
    "rowbyvar" = "character", "decimal" = "numeric",
    "cutoff_stat" = "character", "descending_by" = "character",
    "display_missing" = "boolean", "pad" = "boolean",
    "rowtext" = "character", "row_header" = "character",
    ".keep" = "boolean", ".ord" = "boolean",
    "nested" = "boolean"
  )
  var_type_check <- var_type_check[names(var_type_check) %in% names(arglist)]
  check_var_types("freq", arglist, var_type_check)
  # run stats tests
  check_in_df("freq", arglist)
  if (!is.null(arglist["denom_df"])) {
    check_in_df("freq", arglist,
      df_field = "denom_df",
      args_req_check = c("colvar")
    )
  }
  check_var_duplicates("freq", arglist)
  check_statlist("freq", arglist[["statlist"]]$stats)
  check_statlist_n("freq", arglist[["statlist"]]$stats)
  check_cutoff_stat("freq", arglist[["cutoff_stat"]])
  check_cutoff_format("freq", arglist[["cutoff"]])
}
#' check_nested_freq
#'
#' @param arglist arguments to nested_freq function
#'
#' @noRd
check_nested_freq <- function(arglist) {
  if (!check_is_statlist(arglist[["statlist"]])) {
    stop("statlist must be of type statlist EX. statlist(c('N'))")
  }
  # statlist is passed through as a vector so need to evaluate
  arglist[["statlist"]] <- eval_tidy(arglist[["statlist"]])


  check_req_vars("nested_freq", arglist, c("df", "colvar", "rowvar"))

  var_type_check <- c(
    "df" = "data.frame", "denom_df" = "data.frame",
    "colvar" = "character", "rowvar" = "character",
    "rowbyvar" = "character", "decimal" = "numeric",
    "cutoff_stat" = "character", "descending_by" = "character",
    "display_missing" = "boolean", "pad" = "boolean",
    "rowtext" = "character", "row_header" = "character",
    ".keep" = "boolean", ".ord" = "boolean",
    "nested" = "boolean"
  )
  var_type_check <- var_type_check[names(var_type_check) %in% names(arglist)]
  check_var_types("nested_freq", arglist, var_type_check)
  check_nested_rowvar("nested_freq", arglist)
  check_in_df("nested_freq", arglist)
  check_var_duplicates("nested_freq", arglist)
  check_statlist("nested_freq", arglist[["statlist"]]$stats)
  check_statlist_n("nested_freq", arglist[["statlist"]]$stats)
  check_cutoff_stat("nested_freq", arglist[["cutoff_stat"]])
  check_cutoff_format("nested_freq", arglist[["cutoff"]])
}

#' check_metadata
#'
#' @param arglist arguments to generate_results function
#'
#' @noRd
check_generate_results <- function(arglist) {
  check_req_vars("generate_results", arglist, c("table_metadata"))

  check_table_metadata(arglist[["table_metadata"]], arglist[["env"]])

  if (!is.null(arglist[["column_metadata_file"]])) {
    check_column_metadata_file(arglist[["column_metadata_file"]])
  } else {
    check_column_metadata(arglist[["column_metadata"]])
  }
}

#' check_gentlg
#'
#' @param arglist arguments to gentlg function
#'
#' @noRd
check_gentlg <- function(arglist) {
  check_req_vars("gentlg", arglist, c("tlf", "format"))


  # build tier two tests
  tier_two_tests <- tribble(
    ~arglist, ~el, ~func, ~issue, ~msg, ~use_df,
    "arglist", "tlf", "check_arg_tlf", "stop",
    quote(paste0(
      "The following values are included in argument ", element,
      " to function ", source_func, " but are not allowed: ", all_fails,
      "\nIf you would like to see guidance on this argument ",
      "type `?gentlg` in the console"
    )), FALSE,
    "arglist", "format", "check_arg_format", "stop",
    quote(paste0(
      "The following values are included in argument ", element,
      " to function ", source_func, " but are not allowed: ", all_fails,
      "\nIf you would like to see guidance on this argument ",
      "type `?gentlg` in the console"
    )), FALSE
  )
  # run tests
  check_args("gentlg", arglist, tier_two_tests)
  check_opath(arglist[["opath"]])
  check_plotnames(arglist[["plotnames"]])
  check_in_df("gentlg", arglist, df_field = "huxme")
  if (substr(arglist[["tlf"]], 1, 1) %in% c("t", "T", "L", "l")) {
    check_wcol(arglist[["huxme"]], arglist[["wcol"]])
  }
  check_orientation(arglist[["orientation"]])
}

#' check_add_format
#'
#' @param arglist arguments to gentlg function
#'
#' @noRd
check_add_format <- function(arglist) {
  check_req_vars("add_format", arglist, c("df"))

  var_type_check <- c(
    "df" = "data.frame", "tableby" = "character",
    "groupby" = "character", ".keep" = "boolean"
  )

  var_type_check <- var_type_check[names(var_type_check) %in% names(arglist)]
  check_var_types("add_format", arglist, var_type_check)
  check_in_df("add_format", arglist)
  check_var_duplicates("add_format", arglist)
  check_req_fields("add_format", arglist, c("anbr"))
}

#' check_add_format
#'
#' @param arglist arguments to gentlg function
#'
#' @noRd
check_tlgsetup <- function(arglist) {
  check_req_vars("check_tlgsetup", arglist, c("df"))

  var_type_check <- c(
    "df" = "data.frame", "var" = "character",
    "column_metadata_file" = "character", "column_metadata" = "data.frame",
    "tbltype" = "character"
  )

  var_type_check <- var_type_check[names(var_type_check) %in% names(arglist)]
  check_var_types("check_tlgsetup", arglist, var_type_check)
  check_in_df("check_tlgsetup", arglist)
  check_file_exists("check_tlgsetup", arglist, "column_metadata_file")
}

check_bind_table <- function(dfs, arglist) {
  if (!is.null(arglist[["add_count"]]) && arglist[["add_count"]]) {
    check_req_vars("bind_table", arglist, c("colvar"))
  }
  check_dfs(dfs)
  var_type_check <- c(
    "tablebyvar" = "character", "prefix" = "character",
    "groupby" = "character", ".keep" = "boolean",
    "add_count" = "boolean", "add_format" = "boolean",
    "column_metadata_file" = "character", "column_metadata" = "data.frame",
    "tbltype" = "character", "rowbyvar" = "character",
    "colvar" = "character"
  )

  var_type_check <- var_type_check[names(var_type_check) %in% names(arglist)]
  check_var_types("bind_table", arglist, var_type_check)
  check_var_duplicates("bind_table", arglist)
  check_file_exists("bind_table", arglist, "column_metadata_file")
}

mtcars_test <- mtcars
mtcars_test$negnum <- range(-3, 2, 5)
mtcars_test$allna <- as.numeric(NA)
mtcars_test$somena <- rep(c(NA, 4))
mtcars_test$str <- "string"
mtcars_test$USUBJID <- row.names(mtcars_test)
test_eval <- "am"
test_that("checks for univar function correctly", {
  # df
  expect_visible(univar(mtcars_test, colvar = "gear", rowvar = "wt"))
  expect_visible(univar(mtcars_test, colvar = test_eval, rowvar = "wt"))
  expect_error(
    univar(c(1, 2), colvar = "gear", rowvar = "wt"), "Argument df to function univar should be type data.frame"
  )
  # rowvar
  expect_visible(univar(mtcars_test, colvar = "gear", rowvar = "somena"))
  expect_error(univar(mtcars_test, colvar = "gear", rowvar = "allna"))
  # geometric estimable check
  expect_error(univar(mtcars_test, colvar = "gear", rowvar = "negnum", statlist = statlist(c("GSD", "GSE"))))
  # wide bool check
  expect_visible(univar(mtcars_test, colvar = "gear", rowvar = "wt", wide = TRUE))
  expect_error(univar(mtcars_test, colvar = "gear", rowvar = "wt", wide = "str"))
  # check statlist
  expect_error(univar(mtcars_test, colvar = "gear", rowvar = "wt", statlist = statlist(c("NOT A STAT"))))
})

test_that("checks for freq function correctly", {
  # df
  expect_visible(freq(mtcars_test, colvar = "gear", rowvar = "cyl"))
  expect_visible(freq(mtcars_test, colvar = "gear", rowvar = test_eval))
  expect_error(freq(c(1, 2), colvar = "gear", rowvar = "cyl"), "")
})

test_that("checks for nested_freq function correctly", {
  # df
  expect_visible(nested_freq(mtcars_test, colvar = "gear", rowvar = "cyl*carb"))
  expect_visible(nested_freq(mtcars_test, colvar = test_eval, rowvar = "cyl*carb"))
  expect_error(
    nested_freq(c(1, 2), colvar = "gear", rowvar = "cyl*carb"),
    "Argument df to function nested_freq should be type data.frame"
  )
  # rowvar nesting
  expect_visible(nested_freq(mtcars_test, colvar = "gear", rowvar = "cyl*carb"))
  expect_visible(nested_freq(mtcars_test, colvar = "gear", rowvar = "cyl*carb*am"))
  expect_error(
    nested_freq(mtcars_test, colvar = "gear", rowvar = "cyl"),
    "Argument rowvar to function nested_freq is incorrectly formatted."
  )
  expect_error(
    nested_freq(mtcars_test, colvar = "gear", rowvar = "cyl*am*vs*disp"),
    "Argument rowvar to function nested_freq is incorrectly formatted."
  )
})

test_that("check updates to check gentlg", {
  basefreq <- freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", statlist = statlist(c("n")))

  suppressWarnings(
    expect_error(
      gentlg(
        huxme = basefreq,
        file = "vignettes/validation/test_code/test_code_output/test",
        title = "Test",
        format = "RTF",
        opath = "ERROR",
        colheader = c("Test", "One", "Two", "The")
      ),
      "opath 'ERROR' does not exist for function gentlg"
    )
  )

  expect_error(
    gentlg(
      huxme = basefreq,
      file = "vignettes/validation/test_code/test_code_output/test",
      title = "Test",
      format = "ERROR",
      colheader = c("Test", "One", "Two", "The")
    )
  )

  suppressWarnings(
    expect_error(
      gentlg(
        huxme = basefreq,
        file = "vignettes/validation/test_code/test_code_output/test",
        title = "Test",
        format = "RTF",
        wcol = c(1, 1),
        colheader = NA
      )
    ),
    "wcol's length must be 1 or the length of final output"
  )

  suppressWarnings(
    expect_error(
      gentlg(
        huxme = basefreq,
        tlf = "Figure",
        file = "vignettes/validation/test_code/test_code_output/test",
        title = "Test",
        format = "RTF",
        plotnames = "ERROR",
        colheader = NA
      ),
      "plotnames 'ERROR' does not exist for function gentlg"
    )
  )

  expect_error(
    gentlg(
      huxme = basefreq,
      tlf = "Figure",
      file = "vignettes/validation/test_code/test_code_output/test",
      title = "Test",
      format = "RTF",
      idvars = "ERROR",
      colheader = NA
    ),
    "idvars 'ERROR' does not exist in huxme or is all NA for function: gentlg"
  )

  expect_error(
    gentlg(
      huxme = basefreq,
      file = "vignettes/validation/test_code/test_code_output/test",
      title = "Test",
      format = "RTF",
      orientation = "ERROR"
    ),
    "orientation 'ERROR' is not either 'landscape' or 'portrait' for function gentlg"
  )
})

test_that("test freq update", {
  expect_error(
    freq(cdisc_adsl, rowvar = "RACE", colvar = NA, statlist = statlist(c("n"))),
    "colvar to function freq should be type character"
  )

  expect_error(
    suppressWarnings(freq(cdisc_adsl, rowvar = "RACE", colvar = "", statlist = statlist(c("n")))),
    "colvar '' does not exist in df or is all NA for function: freq"
  )

  expect_error(
    suppressWarnings(freq(cdisc_adsl, rowvar = "RACE", colvar = "MISSING", statlist = statlist(c("n")))),
    "colvar 'MISSING' does not exist in df or is all NA for function: freq"
  )

  expect_error(
    suppressWarnings(freq(cdisc_adsl, rowvar = NA, colvar = "RACE", statlist = statlist(c("n")))),
    "Argument rowvar to function freq should be type character"
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "", colvar = "RACE", statlist = statlist(c("n"))),
    "rowvar '' does not exist in df or is all NA for function: freq"
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "MISSING", colvar = "RACE", statlist = statlist(c("n"))),
    "rowvar 'MISSING' does not exist in df or is all NA for function: freq"
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "RACE", colvar = "TRT01P", tablebyvar = "MISSING", statlist = statlist(c("n"))),
    "tablebyvar 'MISSING' does not exist in df or is all NA for function: freq"
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "RACE", colvar = "TRT01P", rowbyvar = "MISSING", statlist = statlist(c("n"))),
    "rowbyvar 'MISSING' does not exist in df or is all NA for function: freq"
  )

  expect_error(freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", statlist = statlist(c("N"))))

  expect_error(freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", statlist = statlist(c())))
  expect_error(freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", statlist = statlist(c("ERROR"))))

  expect_error(
    freq(cdisc_adsl, rowvar = "ITTFL", colvar = "ITTFL"),
    paste0(
      paste0(c("colvar", "rowvar"), collapse = ", "), " all have value ", "ITTFL", " for function ", "freq", "\n\n"
    )
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", rowbyvar = "ITTFL"),
    paste0(
      paste0(c("rowvar", "rowbyvar"), collapse = ", "), " all have value ", "ITTFL", " for function ", "freq", "\n\n"
    )
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", tablebyvar = "ITTFL"),
    paste0(
      paste0(c("tablebyvar", "rowvar"), collapse = ", "), " all have value ", "ITTFL", " for function ", "freq", "\n\n"
    )
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", rowbyvar = "TRT01PN"),
    paste0(
      paste0(c("colvar", "rowbyvar"), collapse = ", "), " all have value ", "TRT01PN", " for function ", "freq", "\n\n"
    )
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", rowbyvar = "TRT01PN"),
    paste0(
      paste0(c("colvar", "rowbyvar"), collapse = ", "), " all have value ", "TRT01PN", " for function ", "freq", "\n\n"
    )
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", tablebyvar = "TRT01PN"),
    paste0(
      paste0(c("colvar", "tablebyvar"), collapse = ", "),
      " all have value ",
      "TRT01PN",
      " for function ",
      "freq",
      "\n\n"
    )
  )

  expect_error(
    freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", rowbyvar = "SEX", tablebyvar = "SEX"),
    paste0(
      paste0(c("tablebyvar", "rowbyvar"), collapse = ", "), " all have value ", "SEX", " for function ", "freq", "\n\n"
    )
  )
})

#' @editor Aidan Ceney
#' @editDate 2022-03-02
test_that("Test univar new checks", {
  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = NULL,
      rowvar = "AVAL",
      statlist = statlist(c("N", "MEAN_CI", "MIN", "MAX")),
      alpha = .10
    ),
    "Argument colvar to function univar is required but no value has been supplied"
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = NULL,
      statlist = statlist(c("N", "MEAN_CI", "MIN", "MAX")),
      alpha = .10
    ),
    "Argument rowvar to function univar is required but no value has been supplied"
  )

  expect_error( # Supress addtional warning
    suppressWarnings({
      univar(
        cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
        colvar = "MISSING",
        rowvar = "AVAL",
        statlist = statlist(c("N", "MEAN_CI", "MIN", "MAX")),
        alpha = .10
      )
    }), "colvar 'MISSING' does not exist in df or is all NA for function: univar"
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "MISSING",
      statlist = statlist(c("N", "MEAN_CI", "MIN", "MAX")),
      alpha = .10
    ),
    "rowvar 'MISSING' does not exist in df or is all NA for function: univar"
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      tablebyvar = "MISSING",
      statlist = statlist(c("N", "MEAN_CI", "MIN", "MAX")),
      alpha = .10
    ),
    "tablebyvar 'MISSING' does not exist in df or is all NA for function: univar"
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      rowbyvar = "MISSING",
      statlist = statlist(c("N", "MEAN_CI", "MIN", "MAX")),
      alpha = .10
    ),
    "rowbyvar 'MISSING' does not exist in df or is all NA for function: univar"
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      statlist = statlist(c()),
      alpha = .10
    )
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      statlist = statlist(c("")),
      alpha = .10
    )
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      statlist = statlist(c("ERROR")),
      alpha = .10
    )
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      tablebyvar = "TRTA",
      alpha = .10
    ),
    "colvar, tablebyvar all have value TRTA for function univar"
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      rowbyvar = "TRTA",
      alpha = .10
    ),
    "colvar, rowbyvar all have value TRTA for function univar"
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      rowbyvar = "SEX",
      tablebyvar = "SEX",
      alpha = .10
    ),
    "tablebyvar, rowbyvar all have value SEX for function univar"
  )

  expect_error(
    univar(cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")), colvar = "TRTA", rowvar = "AVAL", alpha = 1.5),
    "alpha must be a number in between 0 and 1 for function univar"
  )

  expect_error(
    univar(cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")), colvar = "TRTA", rowvar = "AVAL", alpha = -.5),
    "alpha must be a number in between 0 and 1 for function univar"
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA", rowvar = "AVAL", precisionby = "SEX"
    )
  )

  expect_error(
    univar(
      cdisc_advs %>% filter(PARAMCD %in% c("HEIGHT", "WEIGHT")),
      colvar = "TRTA",
      rowvar = "AVAL",
      precisionon = "MISSING"
    ),
    "precisionon 'MISSING' does not exist in df or is all NA for function: univar"
  )
})

test_that("new check errors nested freq", {
  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*RACE", colvar = NA, statlist = statlist(c("n"))),
    "Argument colvar to function nested_freq should be type character"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "RACE", colvar = "", statlist = statlist(c("n"))),
    "Argument rowvar to function nested_freq is incorrectly formatted."
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*RACE", colvar = "MISSING", statlist = statlist(c("n"))),
    "colvar 'MISSING' does not exist in df or is all NA for function: nested_freq"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = NA, colvar = "RACE", statlist = statlist(c("n"))),
    "Argument rowvar to function nested_freq should be type character"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "", colvar = "RACE", statlist = statlist(c("n"))),
    "Argument rowvar to function nested_freq is incorrectly formatted."
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "MISSING", colvar = "RACE", statlist = statlist(c("n"))),
    "Argument rowvar to function nested_freq is incorrectly formatted."
  )

  expect_error(
    nested_freq(
      cdisc_adsl,
      rowvar = "STUDYID*RACE", colvar = "TRT01P", tablebyvar = "MISSING", statlist = statlist(c("n"))
    ),
    "tablebyvar 'MISSING' does not exist in df or is all NA for function: nested_freq"
  )

  expect_error(
    nested_freq(
      cdisc_adsl,
      rowvar = "STUDYID*RACE", colvar = "TRT01P", rowbyvar = "MISSING", statlist = statlist(c("n"))
    ),
    "rowbyvar 'MISSING' does not exist in df or is all NA for function: nested_freq"
  )

  expect_error(nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", statlist = statlist(c("N"))))
  expect_error(nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", statlist = statlist(c())))
  expect_error(nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", statlist = statlist(c("ERROR"))))

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", cutoff = 2, cutoff_stat = "ERROR"),
    "Cutoff stat ERROR for function nested_freq"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "ITTFL"),
    "colvar, nested rowvar 2 all have value ITTFL for function nested_freq"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", rowbyvar = "ITTFL"),
    "rowbyvar, nested rowvar 2 all have value ITTFL for function nested_freq"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", tablebyvar = "ITTFL"),
    "tablebyvar, nested rowvar 2 all have value ITTFL for function nested_freq"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", rowbyvar = "TRT01PN"),
    "colvar, rowbyvar all have value TRT01PN for function nested_freq"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", rowbyvar = "TRT01PN"),
    "colvar, rowbyvar all have value TRT01PN for function nested_freq"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", tablebyvar = "TRT01PN"),
    "tablebyvar all have value TRT01PN for function nested_freq"
  )

  expect_error(
    nested_freq(cdisc_adsl, rowvar = "STUDYID*ITTFL", colvar = "TRT01PN", rowbyvar = "SEX", tablebyvar = "SEX"),
    "tablebyvar, rowbyvar all have value SEX for function nested_freq"
  )
})

test_that("Errors for check generate results", {
  expect_error(generate_results(NULL))

  expect_error(generate_results(table_metadata))

  expect_error(generate_results(tibble::tribble(~Test, ~tbltype, "TEST", "Test")))

  table_metadata <- tibble::tribble(
    ~anbr, ~func, ~tbltype, ~df, ~colvar, ~rowvar, 1, "freq", "Test", "Missing", "TRT01PN", "ITTFL"
  )
  col_metadata <- tibble::tribble(
    ~tbltype, ~coldef, ~decode, ~span1, "Test", "Xanomeline High Dose", "High Dose", "Xanomeline"
  )
  expect_error(
    generate_results(
      tibble::tribble(~anbr, ~func, ~tbltype, ~df, ~colvar, ~rowvar, 1, "freq", "Test", "Missing", "TRT01PN", "ITTFL"),
      column_metadata = col_metadata,
      tbltype = "Test"
    )
  )

  table_metadata <- tibble::tribble(
    ~anbr, ~func, ~tbltype, ~df, ~colvar, ~rowvar, 1, "ERROR", "Test", "cdisc_adsl", "TRT01PN", "ITTFL"
  )
  col_metadata <- tibble::tribble(
    ~tbltype, ~coldef, ~decode, ~span1, "Test", "Xanomeline High Dose", "High Dose", "Xanomeline"
  )
  expect_error(
    generate_results(
      tibble::tribble(
        ~anbr, ~func, ~tbltype, ~df, ~colvar, ~rowvar, 1, "ERROR", "Test", "cdisc_adsl", "TRT01PN", "ITTFL"
      ),
      column_metadata = col_metadata,
      tbltype = "Test"
    )
  )
})

test_that("Test add_format", {
  basefreq <- freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN", statlist = statlist(c("n")))

  expect_error(
    basefreq %>% add_format(tableby = ""), "tableby '' does not exist in df or is all NA for function: add_format"
  )
  expect_error(
    basefreq %>% add_format(tableby = NA), "Argument tableby to function add_format should be type character"
  )
  expect_error(
    basefreq %>% add_format(tableby = "MISSING"),
    "tableby 'MISSING' does not exist in df or is all NA for function: add_format"
  )
  expect_error(
    basefreq %>% add_format(groupby = ""), "groupby '' does not exist in df or is all NA for function: add_format"
  )
  expect_error(
    basefreq %>% add_format(groupby = NA), "Argument groupby to function add_format should be type character"
  )
  expect_error(
    basefreq %>% add_format(groupby = "MISSING"),
    "groupby 'MISSING' does not exist in df or is all NA for function: add_format"
  )

  expect_error(basefreq %>% dplyr::select(-row_type) %>% add_format(), "Reqiured fields anbr,  for function add_format")
})

test_that("Sparse dfs pass check_wcol works as expected with sparse dfs", {
  a_freq <- freq(cdisc_adsl, rowvar = "ITTFL", colvar = "TRT01PN") %>% select(-c("row_type", "group_level"))

  expect_equal(check_wcol(a_freq, c(1, 1, 1, 1)), NULL)
})

#' Replace leading whitespaces with left indentation RTF markup
#'
#' @details
#' The following function receives a string 'x' and returns a modified string
#' where every leading whitespace is replaced with 90 twips (0.0625 inches) left indentation RTF markup.
#' If the input does not start with a whitespace, the string is returned as is.
#'
#' @param x `character(1)` a string to replace the leading whitespaces.
#'
#' @examples
#' tidytlg:::replace_lead_whitespaces_ind("    this is x")
#' # [1] "\intbl\li360\fi0 this is x"
#' tidytlg:::replace_lead_whitespaces_ind("this is x")
#' # [1] "this is x"
#'
#' @return `character(1)` RTF markup with leading whitespaces replaced.
#' @keywords internal
replace_lead_whitespaces_ind <- function(x) {
  # get number of leading whitespaces
  num_whitespaces <- attr(regexpr("^\\s*", x), "match.length")
  # 2 whitespaces represent an indentation of 0.125 inches = 180 twips
  # e.g. 4 whitespaces represent a left-indentation of 0.25 inches = 360 twips
  num_twips <- num_whitespaces * 90
  if (num_twips > 0) {
    raw_rtf_markup <- paste0("\\intbl\\li", num_twips, "\\fi0")
    x <- paste0(raw_rtf_markup, " ", trimws(x, "left"))
  }
  x
}

df <- data.frame(label = c("boy", "girl"), name = c("Bob", "Lily"), age = c(12, 15))

test_that("custom alignments work", {
  # `alignments` must be a list of named lists
  expect_error(gentlg(huxme = df, print.hux = FALSE, alignments = 1), "`alignments` must be a list")

  expect_error(gentlg(huxme = df, print.hux = FALSE, alignments = list(1)), "Each item of `alignments` must be a list")

  # Apply alignments to one data frame
  hux_table <- gentlg(
    huxme = df,
    print.hux = FALSE,
    alignments = list(
      list(
        row = 1:4, col = 2, value = "left"
      ), # Column `name` to the left
      list(row = 3, col = 3, value = "right") # Cell `12` to the right
    )
  )[[
    1
  ]]

  align_property <- huxtable::align(hux_table)
  expect_equal(align_property[4, 2], "left")
  expect_equal(align_property[3, 3], "right")

  # Apply alignments to two data frames
  hux_tables <- gentlg(
    huxme = list(df, df),
    print.hux = FALSE,
    alignments = list( # Column `name` to the left for the first data frame
      list(list(row = 1:4, col = 2, value = "left")), # Cell `12` to the right for the second data frame
      list(list(row = 3, col = 3, value = "right"))
    )
  )

  align_property_1 <- huxtable::align(hux_tables[[1]])
  expect_equal(align_property_1[4, 2], "left")
  expect_equal(align_property_1[3, 3], "center")

  align_property_2 <- huxtable::align(hux_tables[[2]])
  expect_equal(align_property_2[4, 2], "center")
  expect_equal(align_property_2[3, 3], "right")
})

test_that("replace_lead_whitespaces_ind() is replacing whitespaces", {
  # case when there are 2 leading whitespaces, should insert 180 twips
  expect_no_error(res <- gentlg(
    huxme = df,
    print.hux = FALSE,
    colheader = c("  Gender", "Name", "Age")
  ))
  res <- as.character(res[[1]][2, 1])
  expect_equal(res, "\\keepn\\trhdr \\intbl\\li180\\fi0 Gender")

  # case when there are no leading whitespaces, should leave it as is
  expect_no_error(res <- gentlg(
    huxme = df,
    print.hux = FALSE,
    colheader = c("Gender", "Name", "Age")
  ))
  res <- as.character(res[[1]][2, 1])
  expect_equal(res, "\\keepn\\trhdr Gender")
})

test_that("gentlg() sets the right colwidths when passing a combination of vectors and numeric values", {
  wcol <- list(c(0.5, 0.3, 0.2), c(0.25))
  expect_no_error(hux_tables <- gentlg(
    huxme = list(df, df),
    wcol = wcol,
    print.hux = FALSE
  ))

  for (i in seq_along(wcol)) {
    ht <- hux_tables[[i]]
    num_cols <- ncol(ht)
    actual_colwidths <- as.numeric(huxtable::col_width(ht))
    if (length(wcol[[i]]) == 1) {
      expected_colwidths <- c(wcol[[i]], rep((1 - wcol[[i]]) / (num_cols - 1), num_cols - 1))
    } else {
      expected_colwidths <- wcol[[i]]
    }
    expect_equal(actual_colwidths, expected_colwidths)
  }
})

test_that("gentlg() sets the right colwidths when passing all colwidths explicitly", {
  wcol <- list(c(0.5, 0.3, 0.2), c(0.25, 0.5, 0.25))
  expect_no_error(hux_tables <- gentlg(
    huxme = list(df, df),
    wcol = wcol,
    print.hux = FALSE
  ))

  for (i in seq_along(wcol)) {
    ht <- hux_tables[[i]]
    num_cols <- ncol(ht)
    expected_colwidths <- wcol[[i]]
    actual_colwidths <- as.numeric(huxtable::col_width(ht))
    expect_equal(expected_colwidths, actual_colwidths)
  }
})

test_that("gentlg() validates that if hux is a single data.frame, wcol cannot be a list", {
  wcol <- list(c(0.5), c(0.4, 0.4, 0.2))
  expect_error(
    hux_tables <- gentlg(
      huxme = df,
      wcol = wcol,
      print.hux = FALSE
    ),
    "\\'wcol\\' appears to be"
  )
})

test_that("gentlg() validates each element in wcol has the correct length", {
  wcol <- list(c(0.5, 0.3), c(0.4, 0.4, 0.2))
  expect_error(
    hux_tables <- gentlg(
      huxme = list(df, df),
      wcol = wcol,
      print.hux = FALSE
    ),
    "wcol\\'s length must be 1 or the length of final output"
  )
})

test_that("gentlg() validates that the sum of colwidths is 1", {
  wcol <- list(c(0.5, 0.2, 0.5), c(0.4, 0.4, 0.2))
  expect_error(
    hux_tables <- gentlg(
      huxme = list(df, df),
      wcol = wcol,
      print.hux = FALSE
    ),
    "wcol not defined properly"
  )
})

test_that("gentlg() validates hux length equals wcol length if wcol is a list", {
  wcol <- list(c(0.5), c(0.4, 0.4, 0.2))
  expect_error(
    hux_tables <- gentlg(
      huxme = list(df, df, df),
      wcol = wcol,
      print.hux = FALSE
    ),
    "Arguments \\'wcol\\' and \\'huxme\\' must have the same length."
  )
})

test_that("gentlg() saves footnotes as one row or multiple rows depending on 'footers_one_row'", {
  footers <- c("Footer 1", "Footer 2", "Footer 3")
  ht1 <- gentlg(huxme = df, print.hux = FALSE)[[1]]
  ht2 <- gentlg(huxme = df, print.hux = FALSE, footers = footers)[[1]]
  ht3 <- gentlg(huxme = df, print.hux = FALSE, footers = footers, footers_one_row = TRUE)[[1]]
  # check the overal number of rows
  expect_equal(nrow(ht2), nrow(ht1) + length(footers))
  expect_equal(nrow(ht3), nrow(ht1) + 1)
  # check the footers content
  ht2_footers <- as.data.frame(ht2[5:7, 1])[, 1]
  ht3_footers <- as.data.frame(ht3[5, 1])[, 1]
  expect_equal(ht2_footers, c("\\line Footer 1", "Footer 2", "Footer 3"))
  expect_equal(ht3_footers, "\\line Footer 1 \\line Footer 2 \\line Footer 3")
})

#' Spanning headers for outputs
#'
#' This will create the list object to be passed to `gentlg()` You can create as
#' many spanning headers as you like, just add variables prefixed with span to
#' the column metadata.
#'
#' @param column_metadata dataframe containing the column metadata that is
#'   passed to `tlgsetup()` (see `tlgsetup()` for details)
#'
#' @return List of character vectors containing column headers for an output.
#' @export
#'
#' @examples
#' column_metadata <-
#'   tibble::tribble(
#'     ~tbltype, ~coldef, ~decode,                ~span1,
#'     "type1",  "0",     "Placebo",              "",
#'     "type1",  "54",    "Low Dose",             "Xanomeline",
#'     "type1",  "81",    "High Dose",            "Xanomeline",
#'     "type1",  "54+81", "Total Xanomeline",     ""
#'   )
#'
#' spanning_headers(column_metadata)
spanning_headers <- function(column_metadata) {
  span_element <- function(x) {
    c("", stringr::str_replace_na(x, ""))
  }

  not_empty <- function(x) {
    !all(is.na(x) | is.null(x) | x == "")
  }

  span <- column_metadata %>%
    dplyr::select_if(not_empty) %>%
    dplyr::select(dplyr::starts_with("span")) %>%
    lapply(span_element)

  if (length(span) == 0) (span <- NULL)

  span
}

#' @title spanning_headers() requirements
#' @section Last updated by:
#' Nicholas Masel
#' @section Last update date:
#' 26-OCT-2020
#' @section Category:
#' Format Results

library(dplyr)

# Requirements --------------------------------------------------------------------------------
#' T1. Accounts for ...
#' T1.1 NA.
#' T1.2 ''.
#' T2. Accounts for ...
#' T2.1 one spanning header.
#' T2.2 two spanning headers.
#' T2.3 three spanning headers.


# Tests ---------------------------------------------------------------------------------------
test_that("T1.1 NA.", {
  column_metadata <- tibble::tribble(
    ~tbltype, ~coldef, ~decode, ~span1, "type1", "0", "Placebo", NA, "type1", "1", "Low", NA, "type1", "2", "High", NA
  )

  expect_equal(spanning_headers(column_metadata), NULL)
})

test_that("T1.2 ''.", {
  column_metadata <- tibble::tribble(
    ~tbltype, ~coldef, ~decode, ~span1, "type1", "0", "Placebo", "", "type1", "1", "Low", "", "type1", "2", "High", ""
  )

  expect_equal(spanning_headers(column_metadata), NULL)
})

test_that("T2.1 one spanning header.", {
  column_metadata <- tibble::tribble(
    ~tbltype,
    ~coldef,
    ~decode,
    ~span1,
    "type1",
    "0",
    "Placebo",
    NA,
    "type1",
    "1",
    "Low",
    "Active",
    "type1",
    "2",
    "High",
    "Active"
  )

  expect_equal(spanning_headers(column_metadata), list(span1 = c("", "", "Active", "Active")))
})

test_that("T2.2 two spanning headers.", {
  column_metadata <- tibble::tribble(
    ~tbltype,
    ~coldef,
    ~decode,
    ~span1,
    ~span2,
    "type1",
    "0",
    "Placebo",
    NA,
    "Header 2",
    "type1",
    "1",
    "Low",
    "Active",
    "Header 2",
    "type1",
    "2",
    "High",
    "Active",
    "Header 2"
  )

  expect_equal(
    spanning_headers(column_metadata),
    list(span1 = c("", "", "Active", "Active"), span2 = c("", "Header 2", "Header 2", "Header 2"))
  )
})

test_that("T2.3 three spanning headers.", {
  column_metadata <- tibble::tribble(
    ~tbltype,
    ~coldef,
    ~decode,
    ~span1,
    ~span2,
    ~span3,
    "type1",
    "0",
    "Placebo",
    NA,
    "Header 2",
    NA,
    "type1",
    "1",
    "Low",
    "Active",
    "Header 2",
    NA,
    "type1",
    "2",
    "High",
    "Active",
    "Header 2",
    "Header 3"
  )

  expect_equal(
    spanning_headers(column_metadata),
    list(
      span1 = c("", "", "Active", "Active"),
      span2 = c("", "Header 2", "Header 2", "Header 2"),
      span3 = c("", "", "", "Header 3")
    )
  )
})

#' Descriptive statistics
#'
#' Univariate statistics for a variables by treatment and/or group.
#'
#' @param df (required) dataframe containing records to summarize by treatment.
#' @param colvar (required) character vector of the treatment variable within
#'   the dataframe.
#' @param tablebyvar (optional) repeat entire table by variable within `df`.
#' @param rowvar (required) character vector of variable to summarize within the
#'   dataframe.
#' @param rowbyvar (optional) repeat `rowvar` by variable within `df`.
#' @param statlist (optional) `statlist` object of stats to keep (default =
#'   `statlist(c("N", "MEANSD", "MEDIAN", "RANGE", "IQRANGE"))`).
#' @param decimal (optional) decimal precision root level, when using
#'   `presisionby` this will be used as the base decimal cap (default = 1).
#' @param precisionby (optional) vector of by variable(s) to use when
#'   calculating parameter based precision.
#' @param precisionon (optional) variable to use when calculating parameter
#'   based precision. If `precisionby` is specified but not `precisionon`
#'   this will default to `rowvar`.
#' @param wide (optional) logical indicating to convert labels to column and
#'   columns to labels (default = `FALSE`).
#' @param alpha (optional) alpha level for 2-sided confidence interval (default
#'   = 0.05).
#' @param rowtext (optional) A text string to replace the `label` value on the
#'   table. Useful for tables with a single row.
#' @param row_header (optional) A row to add as a header for the table.
#' @param .keep (optional) Should the `rowbyvar` and `tablebyvar` be
#'   output in the table. If `FALSE`, `rowbyvar` will still be output
#'   in the `label` column. (default = `TRUE`).
#' @param .ord Should the ordering columns be output with the table? This is
#'   useful if a table needs to be merged or reordered in any way after build.
#' @param ... (optional) Named arguments to be included as columns on the table.
#'
#' @return dataframe of results
#' @export
#'
#' @examples
#' adsl <-
#'   structure(
#'     list(
#'       USUBJID = c(
#'         "DEMO-101", "DEMO-102", "DEMO-103", "DEMO-104",
#'         "DEMO-105", "DEMO-106"
#'       ),
#'       AGE = c(59, 51, 57, 65, 21, 80),
#'       SEX = c("F", "M", "F", "M", "F", "M"),
#'       WEIGHTBL = c(83.6, 75, 84, 90, 65, 70),
#'       colnbr = structure(
#'         c(1L, 3L, 2L, 2L, 3L, 1L),
#'         .Label = c("Placebo", "Low", "High"),
#'         class = "factor"
#'       )
#'     ),
#'     row.names = c(NA, 6L),
#'     class = "data.frame"
#'   )
#'
#' # N, Mean(SD), Median, Range, IQ Range for a rowvar by colvar
#' univar(adsl,
#'   colvar = "colnbr",
#'   rowvar = "AGE"
#' )
#'
#' # N and Mean for a rowvar by colvar
#' univar(adsl,
#'   colvar   = "colnbr",
#'   rowvar   = "AGE",
#'   statlist = statlist(c("N", "MEAN"))
#' )
#'
#' # N and Mean for a rowvar by colvar and a by variable
#' univar(adsl,
#'   colvar   = "colnbr",
#'   rowvar   = "AGE",
#'   rowbyvar = "SEX",
#'   statlist = statlist(c("N", "MEAN"))
#' )
#'
#' # Below illustrates how make the same calls to univar() as above, using table
#' # and column metadata # along with generate_results().
#'
#' column_metadata <- tibble::tribble(
#'   ~tbltype, ~coldef, ~decode,
#'   "type1", "0", "Placebo",
#'   "type1", "54", "Low",
#'   "type1", "81", "High"
#' )
#'
#' # N, Mean(SD), Median, Range, IQ Range for a rowvar by colvar
#' table_metadata <- tibble::tribble(
#'   ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar,
#'   "1", "univar", "cdisc_adae", "AGE", "type1", "TRTA"
#' )
#'
#' generate_results(table_metadata,
#'   column_metadata = column_metadata,
#'   tbltype = "type1"
#' )
#'
#'
#' # N and Mean for a rowvar by colvar
#' table_metadata <- tibble::tribble(
#'   ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar, ~statlist,
#'   "1", "univar", "cdisc_adae", "AGE", "type1", "TRTA",
#'   statlist(c("N", "MEAN"))
#' )
#'
#' generate_results(table_metadata,
#'   column_metadata = column_metadata,
#'   tbltype = "type1"
#' )
#'
#'
#' # N and Mean for a rowvar by colvar and a by variable
#' table_metadata <- tibble::tribble(
#'   ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar, ~statlist, ~by,
#'   "1", "univar", "cdisc_adae", "AGE", "type1", "TRTA",
#'   statlist(c("N", "MEAN")), "SEX"
#' )
#'
#' generate_results(table_metadata,
#'   column_metadata = column_metadata,
#'   tbltype = "type1"
#' )
univar <- function(df,
                   colvar = NULL,
                   tablebyvar = NULL,
                   rowvar = NULL,
                   rowbyvar = NULL,
                   statlist = getOption("tidytlg.univar.statlist.default"),
                   decimal = 1,
                   precisionby = NULL,
                   precisionon = NULL,
                   wide = FALSE,
                   alpha = 0.05,
                   rowtext = NULL,
                   row_header = NULL,
                   .keep = TRUE,
                   .ord = FALSE,
                   ...) {
  # check all the arguments being passed in except ...
  arglist <- list()
  args_to_chk <- names(formals())[names(formals()) != "..."]
  purrr::walk(
    args_to_chk,
    .f = function(x) {
      arglist[[x]] <<- eval(rlang::sym(x))
    }
  )
  check_univar(arglist)

  # get just the stats out of statlist for use in function
  statlist <- statlist[["stats"]]

  # build precision data for later
  if (!is.null(precisionby) ||
    !is.null(precisionon)) { # nolint indentation_linter
    if (!all(c(precisionby) %in% (c(tablebyvar, rowbyvar)))) {
      stop(
        "All values of argument `precisionby` must be a part of either
          `tablebyvar` or `rowbyvar` arguments",
        call. = FALSE
      )
    }
    if (is.null(precisionon)) {
      precisionon <- rowvar
    }
    precision_data <-
      make_precision_data(df, decimal, precisionby, precisionon)
  } else {
    precision_data <- NA
  }

  univars <- df %>%
    # Derive the statistics for the table. Logic in this function will determine
    # what needs to be done based on statlist. Results in a long dataframe with
    # one value per row containing all necessary statistics
    derive_univar(colvar, tablebyvar, rowbyvar, rowvar, statlist, alpha) %>%
    # Numeric values are rounded in this function.
    round_univar(statlist, decimal, precisionby, precision_data) %>%
    # Cells that contain multiple statistics (i.e. MEANSD) are merged here.
    # Still a long dataframe with one row per value but merged statistics
    # are present but their individual components (i.e. SD) are not.
    arrange_univar(statlist, colvar, tablebyvar, rowbyvar, alpha) %>%
    # The final table is pivoted here.
    pivot_univar(
      wide = wide,
      statlist = statlist,
      tablebyvar = tablebyvar,
      colvar = colvar,
      rowbyvar = rowbyvar,
      rowtext = rowtext,
      row_header = row_header,
      rowvar = rowvar,
      .keep = .keep,
      .ord = .ord,
      ...
    )

  attr(univars, "colvar") <- colvar
  attr(univars, "tablebyvar") <- tablebyvar
  attr(univars, "rowbyvar") <- rowbyvar
  attr(univars, "rowvar") <- rowvar
  attr(univars, "statlist") <- statlist
  attr(univars, "decimal") <- decimal
  attr(univars, "precisionby") <- precisionby
  attr(univars, "wide") <- wide
  attr(univars, "alpha") <- alpha
  attr(univars, "rowtext") <- rowtext
  attr(univars, "row_header") <- row_header

  structure(univars,
    class = c("tidytlg.univar", class(univars))
  )
}

#' Derive statistics for univar table
#'
#' @return A long dataframe with all the stats needed to display or arrange
#'
#' @noRd
derive_univar <-
  function(df,
           colvar,
           tablebyvar,
           rowbyvar,
           rowvar,
           statlist,
           alpha) {
    name_levels <- unique(replace_statlist(statlist))
    name_levels <- name_levels[name_levels != "tval"]

    suppressWarnings(
      df %>%
        # Drop any NA values in df before derivation
        tidyr::drop_na(.data[[rowvar]]) %>%
        dplyr::group_by_at((c(colvar, tablebyvar, rowbyvar))) %>%
        # Get summaries to derive. See get summaries for the interface between what
        # the user passes and what is derived
        dplyr::summarise(
          rowvar = rowvar,
          !!!get_summaries(statlist, rowvar, alpha)
        ) %>%
        dplyr::ungroup() %>%
        # Flip the columns from the summaries into a long dataframe
        pivot_longer(cols = any_of(replace_statlist(statlist))) %>%
        # remove tval if there
        filter(name != "tval") %>%
        # make the name a factor
        mutate(name = factor(name, levels = name_levels)) %>%
        # Logic for completing the factors in the data.frame. This could be replaced
        # by a preprocessing call similar to char2factor
        complete(!!!syms(c(colvar, tablebyvar, rowbyvar, "name")),
          fill = list(rowvar = rowvar, value = NaN)
        ) %>%
        mutate(
          across(
            .cols = any_of(c(rowbyvar, tablebyvar)),
            .fns = function(var) {
              if (is.factor(var)) {
                unclass(var)
              } else {
                var2 <- as.factor(var)
                unclass(var2)
              }
            },
            .names = "{.col}_ord"
          )
        )
    )
  }

#' Get number of decimals
#'
#' @return a vector of how many decimals there are
#'
#' @noRd
get_decimals <- function(v) {
  v %>%
    replace_na(0) %>%
    map(function(n) {
      if (str_detect(as.character(n), "\\.")) {
        nchar(str_split(as.character(n), "\\.", simplify = TRUE))[, 2]
      } else {
        0
      }
    }) %>%
    unlist()
}

#' Get precision from data
#'
#' @return a dataframe with precision by variables and data precision values
#'
#' @noRd
make_precision_data <-
  function(df, decimal, precisionby, precisionon) {
    if (!is.null(precisionby)) {
      df %>%
        group_by(!!!syms(precisionby)) %>%
        summarise(decimal = min(max(get_decimals(
          !!!syms(precisionon)
        )), decimal)) %>%
        ungroup() %>%
        complete(!!!syms(c(precisionby)), fill = list(decimal = decimal)) %>%
        select(!!!syms(c(precisionby, "decimal"))) %>%
        unique()
    } else {
      df %>%
        summarise(decimal = min(max(get_decimals(
          !!!syms(precisionon)
        )), decimal)) %>%
        select("decimal") %>%
        unique()
    }
  }

#' Round values from derived dataframe
#'
#' @return returns update of dataframe replacing missing values
#'
#' @noRd
update_missing <- function(df) {
  missing_update <- c(
    "N" = "0",
    "SUM" = "-",
    "MEAN" = "-",
    "GeoMEAN" = "-",
    "SD" = "-",
    "SE" = "-",
    "CV" = "-",
    "GSD" = "-",
    "GSE" = "-",
    "MEANSD" = "-",
    "MEANSE" = "-",
    "MEDIAN" = "-",
    "MIN" = "-",
    "MAX" = "-",
    "RANGE" = "(-; -)",
    "Q1" = "-",
    "Q3" = "-",
    "IQRANGE" = "(-; -)",
    "MEDRANGE" = "(-; -)",
    "MEDIQRANGE" = "(-; -)",
    "MEAN_CI" = "- (-; -)",
    "GeoMEAN_CI" = "- (-; -)"
  )

  missing_base <- c(
    "N" = "-",
    "SUM" = "-",
    "MEAN" = "-",
    "GeoMEAN" = "-",
    "SD" = "-",
    "SE" = "-",
    "CV" = "-",
    "GSD" = "-",
    "GSE" = "-",
    "MEANSD" = "- (-)",
    "MEANSE" = "- (-)",
    "MEDIAN" = "-",
    "MIN" = "-",
    "MAX" = "-",
    "RANGE" = "(-; -)",
    "Q1" = "-",
    "Q3" = "-",
    "IQRANGE" = "(-; -)",
    "MEDRANGE" = "(-; -)",
    "MEDIQRANGE" = "(-; -)",
    "MEAN_CI" = "- (-; -)",
    "GeoMEAN_CI" = "- (-; -)"
  )
  ret <- df |>
    group_by(name) |>
    mutate(
      value = ifelse(value == missing_base[[name[1]]],
        missing_update[[name[1]]], value
      ),
      value = ifelse(
        value %>% stringr::str_detect(" ") &
          missing_update[[name[1]]] %>% str_detect(" ") &
          value %>% stringr::str_detect("$[0-9]+(\\.[0-9]+)* (\\(-\\)|\\(-; -\\))"),
        paste0(
          value %>% stringr::str_extract("$[0-9]+(\\.[0-9]+)*"),
          missing_update[[name[1]]] %>% stringr::str_extract(" .+")
        ),
        value
      )
    ) |>
    ungroup()
  ret
}

#' Round values from derived dataframe
#'
#' @return A long dataframe with rounded values.
#'
#' @noRd
round_univar <-
  function(df,
           statlist,
           decimal,
           precisionby,
           precision_data) {
    # The default lengths
    dg <- getOption("tidytlg.precision.extra")

    if (!is.null(precision_data)) {
      dvec <- dg %>%
        crossing(precision_data) %>%
        mutate(prec = if_else(stat == "N", 0, extra + decimal))

      if (!is.null(precisionby)) {
        df %>%
          left_join(dvec, by = c(precisionby, "name" = "stat")) %>%
          rowwise() %>%
          mutate(value = unname(roundSAS(
            value,
            digits = prec,
            as_char = TRUE,
            na_char = "-"
          ))) %>%
          ungroup()
      } else {
        df %>%
          left_join(dvec, by = c("name" = "stat")) %>%
          rowwise() %>%
          mutate(value = unname(roundSAS(
            value,
            digits = prec,
            as_char = TRUE,
            na_char = "-"
          ))) %>%
          ungroup()
      }
    } else {
      dvec <- dg %>%
        mutate(prec = if_else(stat == "N", 0, extra + decimal))

      df %>%
        left_join(dvec, by = c("name" = "stat")) %>%
        rowwise() %>%
        mutate(value = unname(roundSAS(
          value,
          digits = prec,
          as_char = TRUE,
          na_char = "-"
        ))) %>%
        ungroup()
    }
  }

#' Arrange values for merged stats
#'
#' Some stats are composed of different stats (MEANSD)
#'
#' @return A dataframe where values are arranged with respect to
#' merged statistics
#'
#' @noRd
arrange_univar <-
  function(df,
           statlist,
           colvar,
           tablebyvar,
           rowbyvar,
           alpha) {
    # The data.frame used to turn the stat into stat labels. Options are defaulted
    # in tidytlg.R.  Confidence Interval is updated if alpha is not 0.05
    stat_labels <- getOption("tidytlg.stat_labels")
    if (alpha != 0.05) {
      pctile <- 100 - (alpha * 100)
      stat_labels[stat_labels$stat == "MEAN_CI", "label"] <-
        paste0("Mean (", as.character(pctile), "% C.I.)")
      stat_labels[stat_labels$stat == "GeoMEAN_CI", "label"] <-
        paste0("Geometric Mean (", as.character(pctile), "% C.I.)")
    }
    # The stats that are composed of multiple statistics
    merged_stats <-
      statlist[!(statlist %in% replace_statlist(statlist))]
    # The stats that are used to derive other stats, but not displayed
    dropped_vars <-
      replace_statlist(statlist)[!(replace_statlist(statlist) %in% statlist)]

    # Expressions for how the results will be displayed for merged stats
    arrange_expr <- exprs(
      MEANSD = paste0(.data$MEAN, " (", .data$SD, ")"),
      MEANSE = paste0(.data$MEAN, " (", .data$SE, ")"),
      RANGE = paste0("(", .data$MIN, "; ", .data$MAX, ")"),
      IQRANGE = paste0("(", .data$Q1, "; ", .data$Q3, ")"),
      MEDRANGE = paste0(.data$MEDIAN, " (", .data$MIN, "; ", .data$MAX, ")"),
      MEDIQRANGE = paste0(.data$MEDIAN, " (", .data$Q1, "; ", .data$Q3, ")"),
      MEAN_CI = paste0(
        .data$MEAN, " (", .data$LCL_MEAN, "; ",
        .data$UCL_MEAN, ")"
      ),
      GeoMEAN_CI = paste0(
        .data$GeoMEAN,
        " (",
        .data$LCL_GeoMEAN,
        "; ",
        .data$UCL_GeoMEAN,
        ")"
      )
    )[merged_stats]

    # process merged stats if there are any
    if (length(merged_stats) && nrow(df) > 0) {
      df %>%
        # Temporarily pivot the table to allow for easy arranging
        pivot_wider(
          id_cols = c(any_of(
            c("rowvar", colvar, tablebyvar, rowbyvar)
          ), ends_with("_ord")),
          names_from = "name",
          values_from = "value"
        ) %>%
        # Arrange the stats that need to be arranged
        dplyr::mutate(!!!arrange_expr) %>%
        # Only select the merged statistics
        select(c(all_of(
          c("rowvar", !!colvar, !!tablebyvar, !!rowbyvar, merged_stats)
        ), ends_with("_ord"))) %>%
        # Pivot the merged statistics dataframe to a long dataframe
        pivot_longer(cols = c(any_of(merged_stats))) %>%
        # The . is the merged stats and df is the original data.frame. Now it contains
        # The merged stats and the original stats
        bind_rows(df) %>%
        # Drop the stats that aren't being displayed
        filter(!(name %in% dropped_vars)) %>%
        rowwise() %>%
        # Change original names to labels to display
        mutate(label = ifelse(
          as.character(name) %in% stat_labels$stat,
          stat_labels[stat_labels$stat == as.character(name), "label"][[1]],
          as.character(name)
        )) %>%
        update_missing()
    } else {
      # if there are no merged stats we just need to make the label column
      df %>%
        # Drop the stats that aren't being displayed
        filter(!(name %in% dropped_vars)) %>%
        rowwise() %>%
        # Change original names to labels to display
        mutate(label = ifelse(
          as.character(name) %in% stat_labels$stat,
          stat_labels[stat_labels$stat == as.character(name), "label"][[1]],
          as.character(name)
        )) %>%
        update_missing()
    }
  }

#' Pivot univar table
#'
#' @return The final table that is ready for column headers and formatting
#'
#' @noRd
pivot_univar <-
  function(df,
           wide,
           statlist,
           tablebyvar,
           colvar,
           rowbyvar,
           rowtext,
           row_header,
           rowvar,
           .keep,
           .ord,
           ...) {
    ord_columns <- names(df)[endsWith(names(df), "_ord")]
    if (.keep) {
      drop_columns <- "rowvar"
    } else {
      drop_columns <- c("rowvar", rowbyvar, tablebyvar)
    }
    if (!.ord) {
      drop_columns <- c(
        drop_columns,
        ord_columns
      )
    }

    if (!wide) {
      df2 <- df %>%
        arrange(
          !!!syms(c(colvar, tablebyvar, rowbyvar)), "rowvar",
          match(df$name, statlist)
        ) %>%
        pivot_wider(id_cols = any_of(
          c(
            colvar,
            tablebyvar,
            rowbyvar,
            "rowvar",
            "label",
            ord_columns
          )
        ), names_from = all_of(c(colvar))) %>%
        mutate(row_type = ifelse(label == "N", "N", "VALUE")) %>%
        mutate(
          across(
            .cols = -ends_with("_ord"),
            .fns = ~ as.character(.x)
          ),
          across(
            .cols = -ends_with("_ord"),
            .fns = ~ replace_na_with_blank(.x)
          )
        ) %>%
        byvar_merge_univar(
          rowvar = rowvar,
          rowbyvar = rowbyvar,
          tablebyvar = tablebyvar,
          rename_col = "label",
          row_header = row_header
        )
    } else {
      df2 <- df %>%
        arrange(
          !!!syms(c(colvar, tablebyvar, rowbyvar)),
          match(df$name, statlist)
        ) %>%
        pivot_wider(
          id_cols = c(any_of(c(
            colvar, tablebyvar, rowbyvar
          )), ends_with("_ord")),
          names_from = "label"
        ) %>%
        mutate(row_type = "VALUE") %>%
        mutate(
          across(
            .cols = -ends_with("_ord"),
            .fns = ~ as.character(.x)
          ),
          across(
            .cols = -ends_with("_ord"),
            .fns = ~ replace_na_with_blank(.x)
          )
        ) %>%
        byvar_merge_univar(
          rowvar = NULL,
          rowbyvar = rowbyvar,
          tablebyvar = tablebyvar,
          rename_col = colvar,
          row_header = row_header
        )
    }
    df2 %>%
      rowtext_univar(rowtext, rowvar, rowbyvar, tablebyvar, row_header) %>%
      # adding on the group_level variable for indentation
      add_group_level(rowbyvar) %>%
      select(-any_of(drop_columns)) %>%
      mutate(
        !!!list(...),
        across(
          .cols = -ends_with("_ord"),
          .fns = ~ replace_na_with_blank(.)
        )
      )
  }

#' Return the summaries that need to be calculated
#'
#' @return A list of expressions to evaluate on the data.
#'
#' @noRd
get_summaries <- function(statlist, rowvar, alpha) {
  # Replace statlist with all needed summaries to compute
  statlist <- replace_statlist(statlist)

  summaries <- exprs(
    N = dplyr::n(),
    SUM = base::sum(.data[[rowvar]]),
    MEAN = base::mean(.data[[rowvar]]),
    SD = stats::sd(.data[[rowvar]]),
    SE = .data$SD / base::sqrt(.data$N),
    CV = (.data$SD / .data$MEAN) * 100,
    MEDIAN = stats::median(.data[[rowvar]]),
    MIN = ifelse(all(is.na(.data[[rowvar]])), NA_real_,
      base::min(.data[[rowvar]])
    ),
    MAX = ifelse(all(is.na(.data[[rowvar]])), NA_real_,
      base::max(.data[[rowvar]])
    ),
    Q1 = stats::quantile(.data[[rowvar]], 0.25, type = 2),
    Q3 = stats::quantile(.data[[rowvar]], 0.75, type = 2),
    GeoMEAN = ifelse(any(.data[[rowvar]] < 0), NA_real_, exp(base::mean(log(
      .data[[rowvar]][.data[[rowvar]] > 0]
    )))),
    GSD = ifelse(any(.data[[rowvar]] < 0), NA_real_, exp(stats::sd(log(
      .data[[rowvar]][.data[[rowvar]] > 0]
    )))),
    GSE = ifelse(any(.data[[rowvar]] < 0), NA_real_, exp(stats::sd(log(
      .data[[rowvar]][.data[[rowvar]] > 0]
    )) / sqrt(length(
      .data[[rowvar]][.data[[rowvar]] > 0]
    )))),
    tval = ifelse(.data$N > 1, stats::qt(1 - (alpha / 2),
      df = (.data$N - 1) # nolint indentation_linter
    ), NA_real_),
    LCL_MEAN = .data$MEAN - .data$tval * .data$SE,
    UCL_MEAN = .data$MEAN + .data$tval * .data$SE,
    LCL_GeoMEAN = ifelse(
      any(.data[[rowvar]] < 0),
      NA_real_,
      exp(
        base::mean(log(.data[[rowvar]][.data[[rowvar]] > 0])) -
          .data$tval * stats::sd(log(.data[[rowvar]][.data[[rowvar]] > 0])) /
            sqrt(length(.data[[rowvar]][.data[[rowvar]] > 0]))
      )
    ),
    UCL_GeoMEAN = ifelse(
      any(.data[[rowvar]] < 0),
      NA_real_,
      exp(
        base::mean(log(.data[[rowvar]][.data[[rowvar]] > 0])) +
          .data$tval * stats::sd(log(.data[[rowvar]][.data[[rowvar]] > 0])) /
            sqrt(length(.data[[rowvar]][.data[[rowvar]] > 0]))
      )
    )
  )

  summaries[statlist][order(match(statlist, names(summaries)))]
}

#' Convert a user list of stats into stats that need to be derived
#'
#' @return A vector of stats that need to be derived. Converted from a list that
#'   may or may not include stats that are merged versions.
#'
#' @noRd
replace_statlist <- function(statlist) {
  replaced_stats <- c(
    "MEANSD", "MEANSE", "RANGE", "IQRANGE", "MEDRANGE",
    "MEDIQRANGE", "MEAN_CI", "GeoMEAN_CI", "SE", "CV",
    "LCL_MEAN", "UCL_MEAN", "LCL_GeoMEAN", "UCL_GeoMEAN"
  )
  new_stats <- list(
    MEANSD = c("MEAN", "SD", "N"),
    MEANSE = c("MEAN", "SE", "SD", "N"),
    RANGE = c("MIN", "MAX"),
    IQRANGE = c("Q1", "Q3"),
    MEDRANGE = c("MEDIAN", "MIN", "MAX"),
    MEDIQRANGE = c("MEDIAN", "Q1", "Q3"),
    MEAN_CI = c(
      "MEAN", "LCL_MEAN", "UCL_MEAN", "tval", "SE",
      "SD", "N"
    ),
    GeoMEAN_CI = c(
      "GeoMEAN", "LCL_GeoMEAN", "UCL_GeoMEAN",
      "tval", "N"
    ),
    SE = c("SE", "SD", "N"),
    CV = c("CV", "SD", "MEAN"),
    LCL_MEAN = c("LCL_MEAN", "MEAN", "tval", "SE", "SD", "N"),
    UCL_MEAN = c("UCL_MEAN", "MEAN", "tval", "SE", "SD", "N"),
    LCL_GeoMEAN = c("LCL_GeoMEAN", "tval", "N"),
    UCL_GeoMEAN = c("UCL_GeoMEAN", "tval", "N")
  )

  # Logic for replacing statlist
  present_replacements <- statlist[statlist %in% replaced_stats]
  present_replacements %>%
    map(~ new_stats[[.]]) %>%
    unlist() %>%
    c(statlist[!(statlist %in% replaced_stats)])
}

byvar_merge_univar <-
  function(df,
           rowvar,
           rowbyvar,
           tablebyvar,
           rename_col,
           row_header) {
    if (!is.null(row_header)) {
      df <- df %>%
        nest(nest = -any_of(c(
          rowbyvar,
          rowvar,
          tablebyvar,
          paste0(rowbyvar, "_ord"),
          paste0(tablebyvar, "_ord")
        ))) %>%
        rowwise() %>%
        mutate(nest = list(
          add_row(
            nest,
            !!rename_col := row_header,
            row_type = "HEADER",
            .before = 1
          )
        )) %>%
        unnest(nest)
    }

    if (is.null(rowbyvar)) {
      return(df)
    }

    if (length(rowbyvar) == 1) {
      df %>%
        nest(nest = -any_of(c(
          rowbyvar, rowvar, tablebyvar, paste0(rowbyvar, "_ord")
        ))) %>%
        rowwise() %>%
        mutate(nest = list(
          add_row(
            nest,
            !!rename_col := as.character(!!sym(rowbyvar)),
            row_type = "BY_HEADER1",
            .before = 1
          ) %>%
            mutate(across(
              .cols = -ends_with("_ord"),
              .fns = ~ replace_na_with_blank(.x)
            ))
        )) %>%
        unnest(nest)
    } else {
      df %>%
        nest(nest = -any_of(c(
          rowbyvar, rowvar, tablebyvar, paste0(rowbyvar, "_ord")
        ))) %>%
        rowwise() %>%
        mutate(nest = list(
          add_row(
            nest, !!rename_col := as.character(!!sym(rowbyvar[2])),
            row_type = "BY_HEADER2",
            .before = 1
          ) %>%
            mutate(across(
              .cols = -ends_with("_ord"),
              .fns = ~ replace_na_with_blank(.x)
            ))
        )) %>%
        unnest(nest) %>%
        nest(nest = -any_of(c(
          rowbyvar[1], rowvar, tablebyvar, paste0(rowbyvar[1], "_ord")
        ))) %>%
        rowwise() %>%
        mutate(nest = list(
          add_row(
            nest,
            !!rename_col := as.character(!!sym(rowbyvar[1])),
            row_type = "BY_HEADER1", !!paste0(rowbyvar[[2]], "_ord") := 0,
            .before = 1
          ) %>%
            mutate(across(
              .cols = -ends_with("_ord"),
              .fns = ~ replace_na_with_blank(.x)
            ))
        )) %>%
        unnest(nest)
    }
  }

rowtext_univar <-
  function(res,
           rowtext,
           rowvar,
           rowbyvar,
           tablebyvar,
           row_header) {
    if (is.null(rowtext)) {
      return(res)
    }

    res_group <- c(rowbyvar, tablebyvar)
    len_oner <-
      # length of single row table
      as.numeric(!is.null(row_header)) + as.numeric(!is.null(rowbyvar)) + 1
    res <- res %>% group_by_at(res_group)
    unq_labels <- unique(res$label)
    names(unq_labels) <- unq_labels
    if (is_named(rowtext)) {
      unq_labels[which(unq_labels %in% rowtext)] <-
        names(rowtext)[which(rowtext %in% unq_labels)]
    }

    suppressWarnings(res <-
      res %>% mutate(
        label = case_when(
          is_named(rowtext) ~ unq_labels[label],
          length(rowtext) == 1 &
            len_oner == n() & row_number() == n() ~ rowtext[1],
          TRUE ~ label
        )
      ))

    res <- res %>% ungroup()
  }

mtcars2 <- mtcars %>% rownames_to_column(var = "USUBJID")

t1 <- univar(mtcars2, colvar = "gear", rowvar = "drat", tablebyvar = "am", rowbyvar = "carb")

t2 <- univar(mtcars2, colvar = "gear", rowvar = "wt", tablebyvar = "am", rowbyvar = "carb") %>% mutate(anbr = "hi")

t3 <- univar(mtcars2, colvar = "gear", rowvar = "hp", tablebyvar = "am", rowbyvar = "carb") %>% mutate(anbr = "7")

tbl1 <- bind_table(t1, t2, t3, colvar = "gear", rowbyvar = "carb")

tbl2 <- bind_table(t1, t2, t3, colvar = "gear", tablebyvar = "am", rowbyvar = "carb")

test_that("anbr is assigned correctly", {
  expect_equal(tbl1$anbr, c(rep(1, nrow(t1)), rep(2, nrow(t2)), rep(7, nrow(t3))))
  expect_equal(tbl1$anbr, c(rep(1, nrow(t1)), rep(2, nrow(t2)), rep(7, nrow(t3))))
})

test_that("format variables are added", {
  expect_true(all(c("anbr", "indentme", "roworder", "newrows", "newpage") %in% names(tbl1)))
  expect_true(all(c("anbr", "indentme", "roworder", "newrows", "newpage") %in% names(tbl2)))
})

test_that("bind_table can handle non tidytlg tables", {
  adae <- cdisc_adae %>%
    filter(SAFFL == "Y", TRTEMFL == "Y") %>%
    rename(TRT01AN = TRTAN)

  adsl <- cdisc_adsl %>% filter(SAFFL == "Y")

  t1 <- adsl %>% freq(colvar = "TRT01AN", rowvar = "SAFFL", rowtext = "Analysis set: Safety", statlist = statlist("n"))

  t2 <- adae %>%
    freq(
      denom_df = adsl,
      colvar = "TRT01AN",
      rowvar = "TRTEMFL",
      rowtext = "Subjects with 1 or more AEs",
      statlist = statlist("n (x.x%)"),
      subset = TRTEMFL == "Y"
    )

  t3_1 <- data.frame(label = c("System organ class", "Preferred term"), row_type = "HEADER")

  t3 <- adae %>%
    nested_freq(
      denom_df = adsl,
      colvar = "TRT01AN",
      rowvar = "AEBODSYS*AEDECOD",
      subset = TRTEMFL == "Y",
      statlist = statlist("n (x.x%)")
    )

  expect_silent({
    bind_table(t1, t2, t3_1, t3, colvar = "TRT01AN", rowbyvar = "AEBODSYS")
  })
})

t1 <- univar(mtcars, colvar = "gear", rowvar = "wt")

t2 <- univar(
  mtcars,
  colvar = "gear",
  rowvar = "wt",
  decimal = 1,
  precisionon = "wt",
  statlist = statlist(
    c(
      "N",
      "SUM",
      "MEAN",
      "GeoMEAN",
      "SD",
      "SE",
      "CV",
      "GSD",
      "GSE",
      "MEANSD",
      "MEANSE",
      "MEDIAN",
      "MIN",
      "MAX",
      "RANGE",
      "Q1",
      "Q3",
      "IQRANGE",
      "MEDRANGE",
      "MEDIQRANGE",
      "MEAN_CI",
      "GeoMEAN_CI"
    )
  )
)
t3 <- univar(
  mtcars,
  colvar = "gear",
  rowvar = "wt",
  tablebyvar = "vs",
  statlist = statlist(
    c(
      "N",
      "SUM",
      "MEAN",
      "GeoMEAN",
      "SD",
      "SE",
      "CV",
      "GSD",
      "GSE",
      "MEANSD",
      "MEANSE",
      "MEDIAN",
      "MIN",
      "MAX",
      "RANGE",
      "Q1",
      "Q3",
      "IQRANGE",
      "MEDRANGE",
      "MEDIQRANGE",
      "MEAN_CI",
      "GeoMEAN_CI"
    )
  ),
  precisionby = c("vs"),
  decimal = 3
)

t4 <- univar(
  mtcars,
  colvar = "gear",
  rowvar = "wt",
  rowbyvar = "am",
  statlist = statlist(
    c(
      "N",
      "SUM",
      "MEAN",
      "GeoMEAN",
      "SD",
      "SE",
      "CV",
      "GSD",
      "GSE",
      "MEANSD",
      "MEANSE",
      "MEDIAN",
      "MIN",
      "MAX",
      "RANGE",
      "Q1",
      "Q3",
      "IQRANGE",
      "MEDRANGE",
      "MEDIQRANGE",
      "MEAN_CI",
      "GeoMEAN_CI"
    )
  )
)

t5 <- univar(
  mtcars,
  colvar = "gear",
  rowvar = "wt",
  rowbyvar = "am",
  tablebyvar = "vs",
  statlist = statlist(
    c(
      "N",
      "SUM",
      "MEAN",
      "GeoMEAN",
      "SD",
      "SE",
      "CV",
      "GSD",
      "GSE",
      "MEANSD",
      "MEANSE",
      "MEDIAN",
      "MIN",
      "MAX",
      "RANGE",
      "Q1",
      "Q3",
      "IQRANGE",
      "MEDRANGE",
      "MEDIQRANGE",
      "MEAN_CI",
      "GeoMEAN_CI"
    )
  )
)

t6 <- univar(mtcars, colvar = "gear", rowvar = "wt", wide = TRUE)

t7 <- univar(mtcars, colvar = "gear", rowvar = "wt", statlist = statlist(c("N", "SUM", "MEAN")), wide = TRUE)

t8 <- univar(mtcars, colvar = "gear", rowvar = "wt", tablebyvar = "vs", wide = TRUE)

t9 <- univar(mtcars, colvar = "gear", rowvar = "wt", rowbyvar = "am", wide = TRUE)

t10 <- univar(mtcars, colvar = "gear", rowvar = "wt", rowbyvar = "am", tablebyvar = "vs", wide = TRUE)

test_that("univar outputs are named as expected", {
  expect_setequal(names(t1), c("label", "row_type", "3", "4", "5", "group_level"))

  expect_setequal(names(t2), c("label", "row_type", "3", "4", "5", "group_level"))

  expect_setequal(names(t3), c("vs", "label", "row_type", "3", "4", "5", "group_level"))

  expect_setequal(names(t4), c("label", "3", "4", "5", "row_type", "am", "group_level"))

  expect_setequal(names(t5), c("vs", "label", "row_type", "3", "4", "5", "am", "group_level"))

  expect_setequal(names(t6), c("gear", "Mean (SD)", "Range", "IQ range", "Median", "N", "row_type", "group_level"))

  expect_setequal(names(t7), c("gear", "Mean", "N", "Sum", "row_type", "group_level"))

  expect_setequal(
    names(t8), c("gear", "vs", "Mean (SD)", "Range", "IQ range", "Median", "N", "row_type", "group_level")
  )

  expect_setequal(
    names(t9), c("gear", "am", "Mean (SD)", "Range", "IQ range", "Median", "N", "row_type", "group_level")
  )

  expect_setequal(
    names(t10), c("vs", "gear", "am", "Mean (SD)", "Range", "IQ range", "Median", "N", "row_type", "group_level")
  )
})

test_that("univar computes statistics as expected", {
  expect_equal(
    t2,
    tibble::tribble(
      ~label,
      ~`3`,
      ~`4`,
      ~`5`,
      ~row_type,
      ~group_level,
      "N",
      "15",
      "12",
      "5",
      "N",
      0,
      "Sum",
      "58.4",
      "31.4",
      "13.2",
      "VALUE",
      0,
      "Mean",
      "3.89",
      "2.62",
      "2.63",
      "VALUE",
      0,
      "Geometric Mean",
      "3.81",
      "2.54",
      "2.52",
      "VALUE",
      0,
      "Std. Dev.",
      "0.833",
      "0.633",
      "0.819",
      "VALUE",
      0,
      "Std. Error",
      "0.215",
      "0.183",
      "0.366",
      "VALUE",
      0,
      "CV (%)",
      "21.40",
      "24.18",
      "31.11",
      "VALUE",
      0,
      "Geometric Std. Dev.",
      "1.231",
      "1.292",
      "1.409",
      "VALUE",
      0,
      "Geometric Std. Error",
      "1.055",
      "1.077",
      "1.166",
      "VALUE",
      0,
      "Mean (SD)",
      "3.89 (0.833)",
      "2.62 (0.633)",
      "2.63 (0.819)",
      "VALUE",
      0,
      "Mean (SE)",
      "3.89 (0.215)",
      "2.62 (0.183)",
      "2.63 (0.366)",
      "VALUE",
      0,
      "Median",
      "3.73",
      "2.70",
      "2.77",
      "VALUE",
      0,
      "Minimum",
      "2.5",
      "1.6",
      "1.5",
      "VALUE",
      0,
      "Maximum",
      "5.4",
      "3.4",
      "3.6",
      "VALUE",
      0,
      "Range",
      "(2.5; 5.4)",
      "(1.6; 3.4)",
      "(1.5; 3.6)",
      "VALUE",
      0,
      "First quartile",
      "3.44",
      "2.07",
      "2.14",
      "VALUE",
      0,
      "Third quartile",
      "4.07",
      "3.17",
      "3.17",
      "VALUE",
      0,
      "IQ range",
      "(3.44; 4.07)",
      "(2.07; 3.17)",
      "(2.14; 3.17)",
      "VALUE",
      0,
      "Median (Range)",
      "3.73 (2.5; 5.4)",
      "2.70 (1.6; 3.4)",
      "2.77 (1.5; 3.6)",
      "VALUE",
      0,
      "Median (Q1; Q3)",
      "3.73 (3.44; 4.07)",
      "2.70 (2.07; 3.17)",
      "2.77 (2.14; 3.17)",
      "VALUE",
      0,
      "Mean (95% C.I.)",
      "3.89 (3.431; 4.354)",
      "2.62 (2.215; 3.019)",
      "2.63 (1.616; 3.649)",
      "VALUE",
      0,
      "Geometric Mean (95% C.I.)",
      "3.81 (3.399; 4.280)",
      "2.54 (2.161; 2.991)",
      "2.52 (1.646; 3.856)",
      "VALUE",
      0,
    ),
    ignore_attr = TRUE
  )
})

test_that("rowtext works as expected with univar", {
  expected <- tibble::tribble(
    ~label,
    ~X3,
    ~X4,
    ~X5,
    ~row_type,
    ~group_level,
    "Mpg Details",
    "",
    "",
    "",
    "HEADER",
    0,
    "N",
    "15",
    "12",
    "5",
    "N",
    0,
    "Mean (SD)",
    "16.11 (3.372)",
    "24.53 (5.277)",
    "21.38 (6.659)",
    "VALUE",
    0,
    "Median",
    "15.50",
    "22.80",
    "19.70",
    "VALUE",
    0,
    "Range",
    "(10.4; 21.5)",
    "(17.8; 33.9)",
    "(15.0; 30.4)",
    "VALUE",
    0,
    "IQ range",
    "(14.30; 18.70)",
    "(21.00; 28.85)",
    "(15.80; 26.00)",
    "VALUE",
    0
  )

  expect_equal(
    mtcars %>% univar(colvar = "gear", row_header = "Mpg Details", rowvar = "mpg"), expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~am,
    ~label,
    ~X3,
    ~X4,
    ~X5,
    ~row_type,
    ~group_level,
    "0",
    "0",
    "",
    "",
    "",
    "BY_HEADER1",
    0,
    "0",
    "Mpg Details",
    "",
    "",
    "",
    "HEADER",
    0,
    "0",
    "N",
    "15",
    "4",
    "0",
    "N",
    0,
    "0",
    "Mean (SD)",
    "16.11 (3.372)",
    "21.05 (3.070)",
    "-",
    "VALUE",
    0,
    "0",
    "Median",
    "15.50",
    "21.00",
    "-",
    "VALUE",
    0,
    "0",
    "Range",
    "(10.4; 21.5)",
    "(17.8; 24.4)",
    "(-; -)",
    "VALUE",
    0,
    "0",
    "IQ range",
    "(14.30; 18.70)",
    "(18.50; 23.60)",
    "(-; -)",
    "VALUE",
    0,
    "1",
    "1",
    "",
    "",
    "",
    "BY_HEADER1",
    0,
    "1",
    "Mpg Details",
    "",
    "",
    "",
    "HEADER",
    0,
    "1",
    "N",
    "0",
    "8",
    "5",
    "N",
    0,
    "1",
    "Mean (SD)",
    "-",
    "26.28 (5.414)",
    "21.38 (6.659)",
    "VALUE",
    0,
    "1",
    "Median",
    "-",
    "25.05",
    "19.70",
    "VALUE",
    0,
    "1",
    "Range",
    "(-; -)",
    "(21.0; 33.9)",
    "(15.0; 30.4)",
    "VALUE",
    0,
    "1",
    "IQ range",
    "(-; -)",
    "(21.20; 31.40)",
    "(15.80; 26.00)",
    "VALUE",
    0
  )

  expect_equal(
    mtcars %>% univar(colvar = "gear", row_header = "Mpg Details", rowvar = "mpg", rowbyvar = "am"),
    expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~label, ~X3, ~X4, ~X5, ~row_type, ~group_level, "Mpg mean", "16.11", "24.53", "21.38", "VALUE", 0
  )

  expect_equal(
    mtcars %>% univar(colvar = "gear", rowtext = "Mpg mean", rowvar = "mpg", statlist = statlist("MEAN")),
    expected,
    ignore_attr = TRUE
  )
})

test_that("univar can handle sparse data as expected", {
  mtcars2 <- mtcars %>%
    rownames_to_column(var = "USUBJID") %>%
    mutate(across(.col = -any_of("mpg"), .fns = as.factor)) %>%
    filter(FALSE)

  expected <- tibble::tribble(
    ~label,
    ~X4,
    ~X6,
    ~X8,
    ~row_type,
    ~group_level,
    "N",
    "0",
    "0",
    "0",
    "N",
    0,
    "Mean (SD)",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "Median",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "Range",
    "(-; -)",
    "(-; -)",
    "(-; -)",
    "VALUE",
    0,
    "IQ range",
    "(-; -)",
    "(-; -)",
    "(-; -)",
    "VALUE",
    0
  )

  expect_equal(univar(mtcars2, colvar = "cyl", rowvar = "mpg"), expected, ignore_attr = TRUE)
})

test_that("MEANSE statlist works as expected", {
  t1 <- mtcars %>% univar(colvar = "cyl", rowvar = "mpg", statlist = statlist(c("MEANSE")))
  expected <- tibble::tribble(
    ~label,
    ~X4,
    ~X6,
    ~X8,
    ~row_type,
    ~group_level,
    "Mean (SE)",
    "26.66 (1.360)",
    "19.74 (0.549)",
    "15.10 (0.684)",
    "VALUE",
    0
  )

  expect_equal(t1, expected, ignore_attr = TRUE)
})

newrows <- "newrows"
cast_to_char <- function(df) {
  lapply(df, as.character)
}

testthat::test_that("Does not insert rows if the newrows column does not exist", {
  i <- iris[1:10, ]
  i[] <- cast_to_char(i)

  actual <- insert_empty_rows(i)
  testthat::expect_identical(insert_empty_rows(i), i)
})

testthat::test_that("Does not insert rows if the newrows column is all 0s", {
  i <- subset(iris[1:10, ], select = -c(Species))
  i[] <- cast_to_char(i)
  i[[newrows]] <- rep(0, nrow(i))

  testthat::expect_identical(insert_empty_rows(i), i)
})

testthat::test_that("Inserts rows when there are 1s in the newrows column", {
  i <- subset(iris[1:10, ], select = -c(Species))
  i[] <- cast_to_char(i)
  i[[newrows]] <- c(0, 1, 0, 1, 1, 0, 0, 0, 0, 0)

  actual <- insert_empty_rows(i)
  expected_rownames <- as.integer(seq_len(nrow(actual)))
  testthat::expect_identical(attr(actual, "row.names", expected_rownames), expected_rownames)
})

testthat::test_that("Inserts an empty row when the 1 is in the last row of newrows", {
  i <- subset(iris[1:10, ], select = -c(Species))
  i[] <- cast_to_char(i)
  i[[newrows]] <- c(rep(0, 9), 1)

  actual <- insert_empty_rows(i)
  expected_rownames <- as.integer(seq_len(nrow(actual)))
  testthat::expect_identical(attr(actual, "row.names", expected_rownames), expected_rownames)
})

testthat::test_that("Inserts empty rows when there are 2 1s in the last two rows of newros", {
  i <- subset(iris[1:10, ], select = -c(Species))
  i[] <- cast_to_char(i)
  i[[newrows]] <- c(rep(0, 8), 1, 1)

  actual <- insert_empty_rows(i)
  expected_rownames <- as.integer(seq_len(nrow(actual)))
  testthat::expect_identical(attr(actual, "row.names", expected_rownames), expected_rownames)
})

testthat::test_that("Zeroes the newrows column", {
  i <- subset(iris[1:10, ], select = -c(Species))
  i[] <- cast_to_char(i)
  i[[newrows]] <- c(rep(0, 8), 1, 1)

  actual <- insert_empty_rows(i)
  testthat::expect_identical(actual$newrows, rep(0, nrow(actual)))
})

testthat::test_that("Preserves the formatting columns", {
  limit <- 9
  i <- subset(iris[1:limit, ], select = -c(Species))
  i[] <- cast_to_char(i)
  i[[newrows]] <- c(rep(0, limit - 1), 1)
  row_type <- sample(c("S", "T"), size = nrow(i), replace = TRUE)
  anbr <- sample(c("1"), size = nrow(i), replace = TRUE)
  indentme <- sample(c("Y", "N"), size = nrow(i), replace = TRUE)
  roworder <- seq_len(nrow(i))
  i$row_type <- row_type
  i$anbr <- anbr
  i$indentme <- indentme
  i$roworder <- roworder

  actual <- insert_empty_rows(i)
  testthat::expect_identical(actual$row_type, c(row_type[-length(row_type)], "EMPTY", row_type[length(row_type)]))
  testthat::expect_identical(actual$anbr, c(anbr, "1"))
  testthat::expect_identical(actual$indentme, c(indentme[-length(row_type)], "0", indentme[length(row_type)]))
  testthat::expect_identical(actual$roworder, seq_len(nrow(actual)))
})

testthat::test_that("Inserts empty rows when the input table has newrows", {
  output_directory <- "test_outputs/emptyrowsinsertions"
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    library(dplyr)
    library(tidytlg)

    # Note cdisc_adsl is built into the package for use
    ittpop <- cdisc_adsl %>% filter(ITTFL == "Y")

    # frequency of Intend-to-Treat patients by planned treatment
    tbl1 <- freq(
      ittpop,
      rowvar = "ITTFL",
      statlist = statlist("n"),
      colvar = "TRT01P",
      rowtext = "Analysis Set:  Intend-to-Treat Population",
      subset = ITTFL == "Y"
    )

    # N, MEAN (SD), MEDIAN, RANGE, IQ Range of age by planned treatment
    tbl2 <- univar(ittpop, rowvar = "AGE", colvar = "TRT01P", row_header = "Age (Years)")

    # frequency of Race by planned treatment
    tbl3 <- freq(
      ittpop,
      rowvar = "RACE", statlist = statlist(c("N", "n (x.x%)")), colvar = "TRT01P", row_header = "Race, n(%)"
    )

    # combine results together
    tbl <- bind_table(tbl1, tbl2, tbl3)

    # conver to hux object
    withr::with_dir(
      new = testthat::test_path(output_directory),
      code = {
        gentlg(
          huxme = tbl,
          orientation = "landscape",
          file = "inserts",
          title = "Custom Method",
          footers = "Produced with tidytlg",
          colspan = list(c("", "", "Xanomeline", "Xanomeline")),
          colheader = c("", "Placebo", "High", "Low"),
          wcol = .30
        )
      }
    )
  })
  expect_snapshot_file(test_path(sprintf("%s/inserts.rtf", output_directory)))
})

test_that("rowtext works as expected for univar tables", {
  expected <- tibble::tribble(
    ~label,
    ~X3,
    ~X4,
    ~X5,
    ~row_type,
    ~group_level,
    "Miles per Gal",
    "",
    "",
    "",
    "HEADER",
    0,
    "N",
    "15",
    "12",
    "5",
    "N",
    0,
    "Mean (SD)",
    "16.11 (3.372)",
    "24.53 (5.277)",
    "21.38 (6.659)",
    "VALUE",
    0,
    "Median",
    "15.50",
    "22.80",
    "19.70",
    "VALUE",
    0,
    "Range",
    "(10.4; 21.5)",
    "(17.8; 33.9)",
    "(15.0; 30.4)",
    "VALUE",
    0,
    "IQ range",
    "(14.30; 18.70)",
    "(21.00; 28.85)",
    "(15.80; 26.00)",
    "VALUE",
    0
  )

  expect_equal(
    mtcars %>% univar(colvar = "gear", rowvar = "mpg", row_header = "Miles per Gal"), expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~label, ~X3, ~X4, ~X5, ~row_type, ~group_level, "Mean label", "16.11", "24.53", "21.38", "VALUE", 0
  )

  expect_equal(
    mtcars %>% univar(colvar = "gear", rowvar = "mpg", rowtext = "Mean label", statlist = statlist("MEAN")),
    expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~cyl,
    ~label,
    ~X3,
    ~X4,
    ~X5,
    ~row_type,
    ~group_level,
    "4",
    "Multiple Mean Labels",
    "21.50",
    "26.93",
    "28.20",
    "VALUE",
    0,
    "6",
    "Multiple Mean Labels",
    "19.75",
    "19.75",
    "19.70",
    "VALUE",
    0,
    "8",
    "Multiple Mean Labels",
    "15.05",
    "-",
    "15.40",
    "VALUE",
    0
  )

  expect_equal(
    mtcars %>%
      univar(
        colvar = "gear",
        rowvar = "mpg",
        rowtext = "Multiple Mean Labels",
        tablebyvar = "cyl",
        statlist = statlist("MEAN")
      ),
    expected,
    ignore_attr = TRUE
  )
})

#' Removes no-op padding from an RTF markup
#'
#' @details
#' Removes no-op padding, which is padding that has a thickness of zero
#' from an RTF markup.
#'
#' @param rtf `character(1)` RTF markup.
#'
#' @return `character(1)` RTF markup with removed zero padding
#' @keywords internal
remove_zero_padding <- function(rtf) {
  noop_padding <- function(direction) {
    sprintf(r"{\\clpadf%s3\\clpad%s0 }", direction, direction)
  }
  for (direction in c("l", "r", "b", "t")) {
    rtf <- stringr::str_replace_all(rtf, noop_padding(direction), "")
  }
  rtf
}

testthat::test_that("Removes zero padding (real rtf)", {
  rtf_con <- file(testthat::test_path("test_data", "zero_padding", "input.rtf"))
  input_file <- paste(readLines(rtf_con), collapse = "\n")
  expected_con <- file(testthat::test_path("test_data", "zero_padding", "expected.rtf"))
  expected <- paste(readLines(expected_con), collapse = "\n")

  actual <- remove_zero_padding(input_file)
  testthat::expect_equal(actual, expected)
})

testthat::test_that("Removes zero padding", {
  example <- paste0(
    "\\clbrdrt\\clbrdrl\\clbrdrb\\clbrdrr\\clvertalb\\clpadfl3",
    "\\clpadl0 \\clpadft3\\clpadt60 \\clpadfb3\\clpadb0 \\clpadfr3\\clpadr60"
  )
  actual <- remove_zero_padding(example)
  expected <- "\\clbrdrt\\clbrdrl\\clbrdrb\\clbrdrr\\clvertalb\\clpadft3\\clpadt60 \\clpadfr3\\clpadr60"
  testthat::expect_equal(actual, expected)
})

#' @title Inserts empty rows into a data frame
#'
#' @details
#' `gentlg` allows for formatting the input table based on formatting columns
#' (see [gentlg()]). One of the formatting mechanisms is empty row insertion.
#' This function inserts the empty rows based on the `newrows` column in the
#' data frame. The new rows are inserted before the rows with value `1` in the
#' `newrows` column.
#'
#' @param huxme `data.frame` The input data frame.
#' @param newrows `integer` A numeric vector specifying where the new rows
#' should be inserted.
#'
#' @return A data frame with added new empty rows.
#' @export
#'
#' @examples
#' df <- iris[1:10, ]
#' df <- as.data.frame(apply(df, 2, as.character))
#' df$newrows <- c(0, 1, 0, 1, 1, 0, 0, 0, 0, 0)
#' insert_empty_rows(df)
insert_empty_rows <- function(huxme, newrows = huxme$newrows) {
  if (is.null(newrows)) {
    return(huxme)
  }
  assertthat::assert_that(is.data.frame(huxme))
  assertthat::assert_that(is.numeric(newrows))
  assertthat::assert_that(nrow(huxme) == length(newrows))

  format_columns <- c("row_type", "anbr", "indentme", "roworder", "newrows")
  data_columns <- names(huxme)[!names(huxme) %in% format_columns]

  emptyrow <- huxme[NA, ][1, ]
  emptyrow[, data_columns] <- ""

  has_roworder <- "roworder" %in% names(huxme)
  if (has_roworder) {
    emptyrow[, "roworder"] <- 0
  }
  if ("newrows" %in% names(huxme)) {
    emptyrow$newrows <- 0
  }
  if ("row_type" %in% names(huxme)) {
    emptyrow$row_type <- "EMPTY"
  }
  if ("indentme" %in% names(huxme)) {
    emptyrow[, "indentme"] <- 0
  }
  if ("newpage" %in% names(huxme)) {
    emptyrow[, "newpage"] <- 0
  }

  cum_sum <- cumsum(newrows)
  df_chunks <- split(huxme, cum_sum)
  if ("anbr" %in% names(huxme)) {
    df_chunks <- lapply(df_chunks, function(chunk) {
      emptyrow$anbr <- chunk$anbr[1]
      rbind(chunk, emptyrow)
    })
  } else {
    df_chunks <- lapply(df_chunks, function(chunk) {
      rbind(chunk, emptyrow)
    })
  }

  names(df_chunks) <- NULL
  merged_df <- do.call(rbind, df_chunks)
  merged_df <- merged_df[-nrow(merged_df), ]
  if (has_roworder) {
    merged_df$roworder <- seq_len(nrow(merged_df))
  }

  attr(merged_df, "row.names") <- as.integer(seq_len(nrow(merged_df)))
  if ("newrows" %in% names(huxme)) {
    merged_df$newrows <- 0
  }
  merged_df
}

#' Create a `statlist` interface for a table
#'
#' The `statlist` is the interface for the presentation of data in a `tidytlg`
#' table.
#'
#' @param stats (required) A character vector of statistics to display in the
#'   table.
#' @param ... (optional) Additional configuration for stats. See sections below
#'   for allowable arguments.
#'
#' @section `Statlists` for `freq()` and `nested_freq()`:
#' `freq()` `statlists` can be composed of `n` (count), `N` (denominator), and
#' `x.x` (percentage, formatted with or without a percent sign).
#' Denominators will include missing values if the 'display_missing'
#' argument is `TRUE`, otherwise they will be excluded. They can be arranged
#' in the following ways:
#' \itemize{
#'   \item{`n`}
#'   \item{`n/N`}
#'   \item{`n (x.x)`}
#'   \item{`n (x.x%)`}
#'   \item{`n/N (x.x)`}
#'   \item{`n/N (x.x%)`}
#' }
#'
#' The following other configurations are supported:
#' \itemize{
#'   \item{`denoms_by` - Controls what groupings of variables should define the
#'     denominator. Variables should be passed as a quoted vector}
#'   \item{`distinct` - A boolean value. Should the numerator reflect distinct
#'     `USUBJIDs` or event counts. Defaults to `TRUE` which captures distinct
#'     subjects.}
#'    \item{`distinct_by` - A character value used to select the variable that
#'    should be used to "distinct" the frequency tables. Defaults to `USUBJID`.}
#'   \item{`zero_denom` - The string to display when there are no records found
#'     in an entire denominator group. Defaults to `-`}
#'   \item{`zero_n` - The string to display when there are no records found for
#'     a numerator. Defaults to `0`.}
#' }
#'
#' @section `Statlists` for `univar` `statlists`:
#' \itemize{
#'   \item{`N`}
#'   \item{`SUM`}
#'   \item{`MEAN`}
#'   \item{`GeoMEAN`}
#'   \item{`SD`}
#'   \item{`SE`}
#'   \item{`CV`}
#'   \item{`GSD`}
#'   \item{`GSE`}
#'   \item{`MEANSD`}
#'   \item{`MEANSE`}
#'   \item{`MEDIAN`}
#'   \item{`MIN`}
#'   \item{`MAX`}
#'   \item{`RANGE`}
#'   \item{`Q1`}
#'   \item{`Q3`}
#'   \item{`IQRANGE`}
#'   \item{`MEDRANGE`}
#'   \item{`MEDIQRANGE`}
#'   \item{`MEAN_CI`}
#'   \item{`GeoMEAN_CI`}
#' }
#'
#'   where `GeoMEAN`: Geometric Mean, `CV`: Coefficient of Variation,
#'   `GSD`: Geometric standard deviation, `GSE`: Geometric standard error,
#'   `MEAN_CI`: Mean (95% C.I.), `GeoMEAN_CI`: Geometric Mean (95% C.I.).
#'   In calculating geometric statistics, if there
#'   are zero values in the inputs, zero values will be excluded before
#'   calculating geometric statistics.
#'
#' @return A `statlist` object that can be passed in the `statlist` argument of
#'   `freq`, `nested_freq`, or `univar`.
#'
#' @export
#'
#' @examples
#'
#' freq(
#'   mtcars,
#'   colvar = "gear",
#'   rowvar = "cyl",
#'   rowbyvar = "am",
#'   statlist = statlist("n/N (x.x)",
#'     distinct = FALSE,
#'     denoms_by = c("gear", "am"),
#'     zero_denom = "_0_"
#'   )
#' )
statlist <- function(stats, ...) {
  # Validate Here
  st <- list()
  st$stats <- stats

  structure(append(st, list(...)), class = c("statlist", "list"))
}

library(tibble)
library(haven)

test_that("descending by will sort by the column named passed, but will ", {
  mtcars2 <- mtcars %>%
    rownames_to_column(var = "USUBJID") %>%
    mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

  mtcars3 <- freq(mtcars2, colvar = "am", rowvar = "cyl", statlist = statlist("n (x.x%)"))
  expect_equal(mtcars3[["0"]], c("3 (15.8%)", "4 (21.1%)", "12 (63.2%)"), ignore_attr = TRUE)

  mtcars4 <- freq(mtcars2, colvar = "am", rowvar = "cyl", statlist = statlist("n (x.x%)"), descending_by = "0")
  expect_equal(mtcars4[["0"]], c("12 (63.2%)", "4 (21.1%)", "3 (15.8%)"), ignore_attr = TRUE)
})

test_that("sparse data is handled correctly", {
  mtcars2 <- mtcars %>%
    rownames_to_column(var = "USUBJID") %>%
    mutate(across(.cols = everything(), .fns = as.factor)) %>%
    filter(FALSE)

  expected <- tibble::tribble(
    ~label,
    ~X4,
    ~X6,
    ~X8,
    ~row_type,
    ~group_level,
    "3",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "4",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "5",
    "-",
    "-",
    "-",
    "VALUE",
    0
  )

  expect_equal(
    freq(mtcars2, colvar = "cyl", rowvar = "gear", statlist = statlist("n (x.x%)")), expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~label,
    ~X4,
    ~X6,
    ~X8,
    ~row_type,
    ~group_level,
    "3",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "4",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "5",
    "-",
    "-",
    "-",
    "VALUE",
    0
  )

  expect_equal(
    freq(mtcars2, colvar = "cyl", rowvar = "gear", statlist = statlist(c("n"))), expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~label,
    ~X4,
    ~X6,
    ~X8,
    ~row_type,
    ~group_level,
    ~am,
    "0",
    "",
    "",
    "",
    "BY_HEADER1",
    0,
    "0",
    "3",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "0",
    "4",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "0",
    "5",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "0",
    "1",
    "",
    "",
    "",
    "BY_HEADER1",
    0,
    "1",
    "3",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "1",
    "4",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "1",
    "5",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "1"
  )
  expected$am <- factor(expected$am)

  expect_equal(
    freq(mtcars2, colvar = "cyl", rowvar = "gear", rowbyvar = "am", statlist = statlist(c("n"))),
    expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~label,
    ~X4,
    ~X6,
    ~X8,
    ~row_type,
    ~group_level,
    ~am,
    "3",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "0",
    "4",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "0",
    "5",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "0",
    "3",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "1",
    "4",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "1",
    "5",
    "-",
    "-",
    "-",
    "VALUE",
    0,
    "1"
  )
  expected$am <- factor(expected$am)

  expect_equal(
    freq(mtcars2, colvar = "cyl", rowvar = "gear", tablebyvar = "am", statlist = statlist(c("n"))),
    expected,
    ignore_attr = TRUE
  )
})

library(tibble)
library(haven)

test_that("cutoff and cutoff_stat work as expected", {
  mtcars2 <- mtcars %>%
    rownames_to_column(var = "USUBJID") %>%
    mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

  mtcars3 <- freq(mtcars2, colvar = "am", rowvar = "cyl", cutoff = 25, statlist = statlist("n (x.x%)"))

  expect_equal(mtcars3[["0"]], c("3 (15.8%)", "12 (63.2%)"), ignore_attr = TRUE)
  expect_equal(mtcars3[["1"]], c("8 (61.5%)", "2 (15.4%)"), ignore_attr = TRUE)

  mtcars4 <- freq(mtcars2, colvar = "am", rowvar = "cyl", cutoff = "0 >= 25", statlist = statlist("n (x.x%)"))
  expect_equal(mtcars4[["0"]], "12 (63.2%)", ignore_attr = TRUE)

  mtcars5 <- freq(
    mtcars2,
    colvar = "am", rowvar = "cyl", cutoff_stat = "n", cutoff = "0 >= 10", statlist = statlist("n (x.x%)")
  )
  expect_equal(mtcars5[["0"]], "12 (63.2%)", ignore_attr = TRUE)
})

library(tibble)
library(haven)

test_that("rowtext will relabel the row if unnamed and only one row is passed", {
  mtcars2 <- mtcars %>% rownames_to_column(var = "USUBJID")

  mtcars3 <- freq(
    mtcars2,
    colvar = "am",
    rowvar = "cyl",
    statlist = statlist("n (x.x%)"),
    subset = cyl == 4,
    rowtext = "Cars with 4 cylinders"
  )

  expected_df <- tribble(
    ~label, ~`0`, ~`1`, ~row_type, ~group_level, "Cars with 4 cylinders", "3 (15.8%)", "8 (61.5%)", "HEADER", 0
  ) %>%
    structure(class = c("freqs", class(.)))

  expect_equal(mtcars3, expected_df, ignore_attr = TRUE)

  expected <- tibble::tribble(~label, ~X0, ~X1, ~row_type, ~group_level, "Cars with 0 cylinders", "0", "0", "HEADER", 0)

  expect_equal(
    freq(
      mtcars2,
      colvar = "am",
      rowvar = "cyl",
      statlist = statlist("n (x.x%)", distinct = TRUE),
      subset = cyl == 0,
      rowtext = "Cars with 0 cylinders"
    ),
    expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~label,
    ~X0,
    ~X1,
    ~row_type,
    ~group_level,
    "Cars with 6 cylinders",
    "4 (21.1%)",
    "3 (23.1%)",
    "VALUE",
    0,
    "Cars with 8 cylinders",
    "12 (63.2%)",
    "2 (15.4%)",
    "VALUE",
    0
  )

  expect_equal(
    freq(
      mtcars2,
      colvar = "am",
      rowvar = "cyl",
      statlist = statlist("n (x.x%)", distinct = TRUE),
      subset = cyl > 4,
      rowtext = c("Cars with 6 cylinders" = "6", "Cars with 8 cylinders" = "8")
    ),
    expected,
    ignore_attr = TRUE
  )
})

test_that("row_header will add a header row", {
  mtcars2 <- mtcars %>%
    rownames_to_column(var = "USUBJID") %>%
    mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

  mtcars3 <- freq(
    mtcars2,
    colvar = "am", rowvar = "cyl", statlist = statlist(c("N", "n (x.x%)")), row_header = "Cars cylinders"
  )

  expected_df <- tribble(
    ~label,
    ~`0`,
    ~`1`,
    ~row_type,
    ~group_level,
    "Cars cylinders",
    "",
    "",
    "HEADER",
    0,
    "N",
    "19",
    "13",
    "N",
    0,
    "4",
    "3 (15.8%)",
    "8 (61.5%)",
    "VALUE",
    0,
    "6",
    "4 (21.1%)",
    "3 (23.1%)",
    "VALUE",
    0,
    "8",
    "12 (63.2%)",
    "2 (15.4%)",
    "VALUE",
    0
  ) %>%
    structure(class = c("freqs", class(.)))

  expect_equal(mtcars3, expected_df, ignore_attr = TRUE)

  mtcars4 <- freq(
    mtcars2,
    colvar = "am", rowvar = "cyl", statlist = statlist("n (x.x%)"), row_header = "Cars cylinders"
  )

  expected_df <- tribble(
    ~label,
    ~`0`,
    ~`1`,
    ~row_type,
    ~group_level,
    "Cars cylinders",
    "",
    "",
    "HEADER",
    0,
    "4",
    "3 (15.8%)",
    "8 (61.5%)",
    "VALUE",
    0,
    "6",
    "4 (21.1%)",
    "3 (23.1%)",
    "VALUE",
    0,
    "8",
    "12 (63.2%)",
    "2 (15.4%)",
    "VALUE",
    0
  ) %>%
    structure(class = c("freqs", class(.)))

  expect_equal(mtcars4, expected_df, ignore_attr = TRUE)
})

library(dplyr)
library(haven)
library(tibble)

adsl <- cdisc_adsl %>% filter(TRT01P != "")

adae <- read_sas("test_data/adae.sas7bdat")

test_that("the cutoff argument in nested drops values with no subordiate items", {
  expected <- tibble::tribble(
    ~label,
    ~Placebo,
    ~Xanomeline.High.Dose,
    ~Xanomeline.Low.Dose,
    ~row_type,
    ~nested_level,
    ~group_level,
    ~SEX,
    "F",
    "2 (40.0%)",
    "3 (60.0%)",
    "1 (20.0%)",
    "NESTED",
    0,
    0,
    "F",
    "WHITE",
    "2 (40.0%)",
    "3 (60.0%)",
    "1 (20.0%)",
    "NESTED",
    1,
    0,
    "F",
    "M",
    "3 (60.0%)",
    "2 (40.0%)",
    "4 (80.0%)",
    "NESTED",
    0,
    0,
    "M",
    "WHITE",
    "3 (60.0%)",
    "2 (40.0%)",
    "4 (80.0%)",
    "NESTED",
    1,
    0,
    "M"
  )
  expected$SEX <- factor(expected$SEX)

  expect_equal(
    nested_freq(adsl, rowvar = "SEX*RACE", colvar = "TRT01P", statlist = statlist("n (x.x%)"), cutoff = 10),
    expected,
    ignore_attr = TRUE
  )

  expected <- tibble::tribble(
    ~label,
    ~Placebo,
    ~Xanomeline.High.Dose,
    ~Xanomeline.Low.Dose,
    ~row_type,
    ~AEBODSYS,
    ~anbr,
    ~indentme,
    ~roworder,
    ~newrows,
    ~newpage,
    "F",
    NA,
    NA,
    NA,
    "TABLE_BY_HEADER",
    NA,
    0,
    0,
    1L,
    1,
    0,
    "CARDIAC DISORDERS",
    "1 (50.0)",
    "0",
    "0",
    "NESTED",
    "CARDIAC DISORDERS",
    1,
    1,
    1L,
    1,
    0,
    "BUNDLE BRANCH BLOCK LEFT",
    "1 (50.0)",
    "0",
    "0",
    "NESTED",
    "CARDIAC DISORDERS",
    1,
    2,
    2L,
    0,
    0,
    "GASTROINTESTINAL DISORDERS",
    "2 (100.0)",
    "0",
    "0",
    "NESTED",
    "GASTROINTESTINAL DISORDERS",
    1,
    1,
    3L,
    0,
    0,
    "DIARRHOEA",
    "1 (50.0)",
    "0",
    "0",
    "NESTED",
    "GASTROINTESTINAL DISORDERS",
    1,
    2,
    4L,
    0,
    0,
    "HIATUS HERNIA",
    "1 (50.0)",
    "0",
    "0",
    "NESTED",
    "GASTROINTESTINAL DISORDERS",
    1,
    2,
    5L,
    0,
    0,
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    "1 (50.0)",
    "3 (100.0)",
    "0",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    1,
    6L,
    0,
    0,
    "APPLICATION SITE ERYTHEMA",
    "1 (50.0)",
    "2 (66.7)",
    "0",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    7L,
    0,
    0,
    "APPLICATION SITE IRRITATION",
    "0",
    "1 (33.3)",
    "0",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    8L,
    0,
    0,
    "APPLICATION SITE PAIN",
    "0",
    "1 (33.3)",
    "0",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    9L,
    0,
    0,
    "APPLICATION SITE PRURITUS",
    "1 (50.0)",
    "3 (100.0)",
    "0",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    10L,
    0,
    0,
    "APPLICATION SITE VESICLES",
    "0",
    "1 (33.3)",
    "0",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    11L,
    0,
    0,
    "FATIGUE",
    "0",
    "2 (66.7)",
    "0",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    12L,
    0,
    0,
    "INFECTIONS AND INFESTATIONS",
    "1 (50.0)",
    "0",
    "1 (100.0)",
    "NESTED",
    "INFECTIONS AND INFESTATIONS",
    1,
    1,
    13L,
    0,
    0,
    "CELLULITIS",
    "0",
    "0",
    "1 (100.0)",
    "NESTED",
    "INFECTIONS AND INFESTATIONS",
    1,
    2,
    14L,
    0,
    0,
    "LOCALISED INFECTION",
    "0",
    "0",
    "1 (100.0)",
    "NESTED",
    "INFECTIONS AND INFESTATIONS",
    1,
    2,
    15L,
    0,
    0,
    "UPPER RESPIRATORY TRACT INFECTION",
    "1 (50.0)",
    "0",
    "0",
    "NESTED",
    "INFECTIONS AND INFESTATIONS",
    1,
    2,
    16L,
    0,
    0,
    "MUSCULOSKELETAL AND CONNECTIVE TISSUE DISORDERS",
    "0",
    "0",
    "1 (100.0)",
    "NESTED",
    "MUSCULOSKELETAL AND CONNECTIVE TISSUE DISORDERS",
    1,
    1,
    17L,
    0,
    0,
    "ARTHRALGIA",
    "0",
    "0",
    "1 (100.0)",
    "NESTED",
    "MUSCULOSKELETAL AND CONNECTIVE TISSUE DISORDERS",
    1,
    2,
    18L,
    0,
    0,
    "RENAL AND URINARY DISORDERS",
    "0",
    "0",
    "1 (100.0)",
    "NESTED",
    "RENAL AND URINARY DISORDERS",
    1,
    1,
    19L,
    0,
    0,
    "MICTURITION URGENCY",
    "0",
    "0",
    "1 (100.0)",
    "NESTED",
    "RENAL AND URINARY DISORDERS",
    1,
    2,
    20L,
    0,
    0,
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    "0",
    "1 (33.3)",
    "1 (100.0)",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    1,
    21L,
    0,
    0,
    "ERYTHEMA",
    "0",
    "0",
    "1 (100.0)",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    2,
    22L,
    0,
    0,
    "PRURITUS",
    "0",
    "0",
    "1 (100.0)",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    2,
    23L,
    0,
    0,
    "RASH",
    "0",
    "1 (33.3)",
    "0",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    2,
    24L,
    0,
    0,
    "M",
    NA,
    NA,
    NA,
    "TABLE_BY_HEADER",
    NA,
    0,
    0,
    1L,
    0,
    1,
    "CARDIAC DISORDERS",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "CARDIAC DISORDERS",
    1,
    1,
    1L,
    1,
    0,
    "ATRIOVENTRICULAR BLOCK SECOND DEGREE",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "CARDIAC DISORDERS",
    1,
    2,
    2L,
    0,
    0,
    "EYE DISORDERS",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "EYE DISORDERS",
    1,
    1,
    3L,
    0,
    0,
    "EYE ALLERGY",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "EYE DISORDERS",
    1,
    2,
    4L,
    0,
    0,
    "EYE PRURITUS",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "EYE DISORDERS",
    1,
    2,
    5L,
    0,
    0,
    "EYE SWELLING",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "EYE DISORDERS",
    1,
    2,
    6L,
    0,
    0,
    "GASTROINTESTINAL DISORDERS",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "GASTROINTESTINAL DISORDERS",
    1,
    1,
    7L,
    0,
    0,
    "DYSPEPSIA",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "GASTROINTESTINAL DISORDERS",
    1,
    2,
    8L,
    0,
    0,
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    "1 (33.3)",
    "2 (100.0)",
    "3 (100.0)",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    1,
    9L,
    0,
    0,
    "APPLICATION SITE DERMATITIS",
    "0",
    "0",
    "1 (33.3)",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    10L,
    0,
    0,
    "APPLICATION SITE ERYTHEMA",
    "0",
    "2 (100.0)",
    "1 (33.3)",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    11L,
    0,
    0,
    "APPLICATION SITE IRRITATION",
    "0",
    "0",
    "1 (33.3)",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    12L,
    0,
    0,
    "APPLICATION SITE PRURITUS",
    "0",
    "2 (100.0)",
    "2 (66.7)",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    13L,
    0,
    0,
    "APPLICATION SITE URTICARIA",
    "0",
    "0",
    "1 (33.3)",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    14L,
    0,
    0,
    "APPLICATION SITE VESICLES",
    "0",
    "0",
    "1 (33.3)",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    15L,
    0,
    0,
    "FATIGUE",
    "0",
    "0",
    "1 (33.3)",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    16L,
    0,
    0,
    "PYREXIA",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    1,
    2,
    17L,
    0,
    0,
    "INFECTIONS AND INFESTATIONS",
    "1 (33.3)",
    "1 (50.0)",
    "0",
    "NESTED",
    "INFECTIONS AND INFESTATIONS",
    1,
    1,
    18L,
    0,
    0,
    "LOWER RESPIRATORY TRACT INFECTION",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "INFECTIONS AND INFESTATIONS",
    1,
    2,
    19L,
    0,
    0,
    "URINARY TRACT INFECTION",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "INFECTIONS AND INFESTATIONS",
    1,
    2,
    20L,
    0,
    0,
    "MUSCULOSKELETAL AND CONNECTIVE TISSUE DISORDERS",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "MUSCULOSKELETAL AND CONNECTIVE TISSUE DISORDERS",
    1,
    1,
    21L,
    0,
    0,
    "FLANK PAIN",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "MUSCULOSKELETAL AND CONNECTIVE TISSUE DISORDERS",
    1,
    2,
    22L,
    0,
    0,
    "PSYCHIATRIC DISORDERS",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "PSYCHIATRIC DISORDERS",
    1,
    1,
    23L,
    0,
    0,
    "DEPRESSED MOOD",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "PSYCHIATRIC DISORDERS",
    1,
    2,
    24L,
    0,
    0,
    "RENAL AND URINARY DISORDERS",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "RENAL AND URINARY DISORDERS",
    1,
    1,
    25L,
    0,
    0,
    "CALCULUS URETHRAL",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "RENAL AND URINARY DISORDERS",
    1,
    2,
    26L,
    0,
    0,
    "RESPIRATORY, THORACIC AND MEDIASTINAL DISORDERS",
    "2 (66.7)",
    "1 (50.0)",
    "1 (33.3)",
    "NESTED",
    "RESPIRATORY, THORACIC AND MEDIASTINAL DISORDERS",
    1,
    1,
    27L,
    0,
    0,
    "COUGH",
    "1 (33.3)",
    "0",
    "0",
    "NESTED",
    "RESPIRATORY, THORACIC AND MEDIASTINAL DISORDERS",
    1,
    2,
    28L,
    0,
    0,
    "EPISTAXIS",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "RESPIRATORY, THORACIC AND MEDIASTINAL DISORDERS",
    1,
    2,
    29L,
    0,
    0,
    "NASAL CONGESTION",
    "1 (33.3)",
    "0",
    "1 (33.3)",
    "NESTED",
    "RESPIRATORY, THORACIC AND MEDIASTINAL DISORDERS",
    1,
    2,
    30L,
    0,
    0,
    "PHARYNGOLARYNGEAL PAIN",
    "0",
    "0",
    "1 (33.3)",
    "NESTED",
    "RESPIRATORY, THORACIC AND MEDIASTINAL DISORDERS",
    1,
    2,
    31L,
    0,
    0,
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    "2 (66.7)",
    "1 (50.0)",
    "2 (66.7)",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    1,
    32L,
    0,
    0,
    "ACTINIC KERATOSIS",
    "0",
    "1 (50.0)",
    "0",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    2,
    33L,
    0,
    0,
    "ERYTHEMA",
    "1 (33.3)",
    "0",
    "2 (66.7)",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    2,
    34L,
    0,
    0,
    "PRURITUS",
    "1 (33.3)",
    "0",
    "1 (33.3)",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    2,
    35L,
    0,
    0,
    "PRURITUS GENERALISED",
    "0",
    "0",
    "1 (33.3)",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    2,
    36L,
    0,
    0,
    "URTICARIA",
    "0",
    "0",
    "1 (33.3)",
    "NESTED",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS",
    1,
    2,
    37L,
    0,
    0
  )
  expected$AEBODSYS <- factor(expected$AEBODSYS)

  expect_equal(
    nested_freq(
      cdisc_adae,
      rowvar = "AEBODSYS*AEDECOD",
      colvar = "TRTA",
      tablebyvar = "SEX",
      cutoff = 10,
      statlist = statlist("n (x.x)", distinct = TRUE, denoms_by = c("TRTA", "SEX"))
    ) %>%
      bind_table(tablebyvar = "SEX", colvar = "TRTA"),
    expected,
    ignore_attr = TRUE
  )
})

test_that("tlgsetup will factorize the colnbr", {
  mtcars2 <- mtcars %>% filter(cyl != 8)

  mtcars3 <- mtcars2 %>%
    tlgsetup(
      var = "cyl", column_metadata_file = test_path("test_data/column_metadata_mtcars.xlsx"), tbltype = "mtcars2"
    ) %>%
    freq(colvar = "colnbr", rowvar = "gear", statlist = statlist("n", distinct = FALSE), .keep = FALSE)

  expected <- tibble::tribble(
    ~label,
    ~col1,
    ~col2,
    ~col3,
    ~col4,
    ~col5,
    ~row_type,
    ~group_level,
    "3",
    "1",
    "2",
    "-",
    "2",
    "3",
    "VALUE",
    0,
    "4",
    "8",
    "4",
    "-",
    "4",
    "12",
    "VALUE",
    0,
    "5",
    "2",
    "1",
    "-",
    "1",
    "3",
    "VALUE",
    0
  )

  expect_equal(mtcars3, expected, ignore_attr = TRUE)

  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    withr::with_dir(
      new = test_path("test_outputs"),
      code = {
        gentlg(mtcars3, colheader = c("label", "col1", "col2", "col3", "col4", "col5"), file = "testtlgsetup")
      }
    )
  })

  expect_snapshot_file(test_path("test_outputs/testtlgsetup.rtf"))
})

library(tibble)
library(haven)

test_that("subset will filter records as expected", {
  mtcars2 <- mtcars %>%
    rownames_to_column(var = "USUBJID") %>%
    mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

  mtcars3 <- freq(mtcars2, colvar = "am", rowvar = "cyl", subset = gear == 4, statlist = statlist("n (x.x%)"))

  expect_equal(mtcars3[["0"]], c("2 (10.5%)", "2 (10.5%)", "0"), ignore_attr = TRUE)
})

#' Print tidytlg table to the R Console
#'
#' When inputting a variable to the console, print is implicitly called on the object.
#' This S3 method prints out a nice format for the user.
#' @export
#' @noRd
print.tidytlg.freq <- function(x, ...) {
  cat(paste0(c("Column Variables: ", attr(x, "colvar"), "\n")))

  if (!is.null(attr(x, "rowbyvar"))) {
    cat(paste0(c("Row By Variables Variables: ", attr(x, "rowbyvar"), "\n")))
  }

  if (!is.null(attr(x, "tablebyvar"))) {
    cat(paste0(c("Table By Variables: ", attr(x, "tablebyvar"), "\n")))
  }

  cat(paste0(c("Row Variable: ", attr(x, "rowvar"), "\n")))

  cat(paste0(c("Statistic Formatting: ", attr(x, "statlist"), "\n")))

  if (!is.null(attr(x, "cutoff"))) {
    cat(paste0(c(
      "Value Cutoffs: ", attr(x, "cutoff"), " by variable ",
      attr(x, "cutoff_stat"), "\n"
    )))
  }

  cat(paste0(c(
    "Descending By: ",
    attr(x, "descending_by") %||% "Factored Sorting", "\n"
  )))

  if (!is.null(attr(x, "rowtext"))) {
    cat(paste0(c("Row Label Values: ", attr(x, "rowtext"), "\n")))
  }

  if (!is.null(attr(x, "row_header"))) {
    cat(paste0(c("Row Header: ", attr(x, "row_header"), "\n")))
  }

  if (is.call(attr(x, "subset"))) {
    cat(paste0(c("Subset Logic: ", as_name(attr(x, "subset")[[1]]))))
  }

  if (!is.null(attr(x, "denoms_by"))) {
    cat(paste0(c("Denominators By Variables: ", attr(x, "denoms_by"), "\n")))
  }

  NextMethod("print")
}

#' @export
print.tidytlg.nested_freq <- function(x, ...) {
  NextMethod("print")
}

#' @export
print.tidytlg.univar <- function(x, ...) {
  cat(paste0(c("Column Variables: ", attr(x, "colvar"), "\n")))

  if (!is.null(attr(x, "rowbyvar"))) {
    cat(paste0(c("Row By Variables Variables: ", attr(x, "rowbyvar"), "\n")))
  }

  if (!is.null(attr(x, "tablebyvar"))) {
    cat(paste0(c("Table By Variables: ", attr(x, "tablebyvar"), "\n")))
  }

  cat(paste0(c("Row Variable: ", attr(x, "rowvar"), "\n")))

  cat(paste0(c("Statistic Formatting: ", attr(x, "statlist"), "\n")))

  cat(underline("Statistic Presentation"), "\n")
  cat(paste0(c("Decimal Precision: ", attr(x, "decimal"), "\n")))
  cat(paste0(c("Alpha for CI Intervals: ", attr(x, "alpha"), "\n")))
  cat(paste0(c(
    "Parameter Based PrecisionL ",
    attr(x, "precisionby") %||% "None", "\n"
  )))

  if (!is.null(attr(x, "rowtext"))) {
    cat(paste0(c("Row Label Values: ", attr(x, "rowtext"), "\n")))
  }

  if (!is.null(attr(x, "row_header"))) {
    cat(paste0(c("Row Header: ", attr(x, "row_header"), "\n")))
  }

  NextMethod("print")
}

#' @export
print.statlist <- function(x, ...) {
  cat(bold("tidytlg Statlist"), "\n")
  cat(underline("stats"), "\n")
  cat(paste0(" ", x$stats, sep = "\n", collapse = ""))
  if (length(x) > 1) {
    cat(underline("Configurations"), "\n")
    for (i in seq_len(length(x[-1]))) {
      cat(bold(names(x[-1])[i]), "-", paste0(x[-1][[i]], collapse = ","), "\n")
    }
  }
  invisible(x)
}

#' Add the `newrows` variable to the results dataframe.
#'
#' The `newrows` variable is used by `gentlg()` to define when to add a blank row
#' to the output. Data will be grouped by `anbr` and the variables passed into
#' the `tableby` and `groupby` parameters. `newrows` will be set to 1
#' for the first record in each group, except for the first row in the data.
#' The first row will always be set to 0.
#'
#' @param df dataframe of results. must contain the `anbr` variable that is
#' added by [add_format()].
#' @param tableby character vector containing table by variables used to
#' generate the results.
#' @param groupby character vector containing group by variables used to
#' generate the results.
#'
#' @return dataframe with the variable `newrows` and `roworder` added.
#' `newrows` is used by `gentlg` to insert line breaks.
#' @export
#'
#' @examples
#' # Example showing how newrows is set to one for each new anbr except
#' # the first
#' tbl <-
#'   structure(
#'     list(
#'       rowvar = c("RANDFL", "AGE", "AGE", "AGE", "AGE", "AGE"),
#'       anbr = c(1L, 2L, 2L, 2L, 2L, 2L),
#'       label = c(
#'         "Analysis set: Subjects Randomized", "Age (Years)", "N",
#'         "Mean (SD)", "Range", "IQ Range"
#'       ),
#'       row_type = c("COUNT", "UNIVAR", "UNIVAR", "UNIVAR", "UNIVAR", "UNIVAR")
#'     ),
#'     row.names = c(NA, -6L),
#'     class = c("tbl_df", "tbl", "data.frame")
#'   )
#'
#' add_newrows(tbl)
#'
#' # Example of use when you have results summarized by one or more variables
#' tbl2 <- tibble::tribble(
#'   ~anbr, ~SEX, ~label, ~row_type,
#'   "01", "F", "Sex : F", "TABLE_BY_HEADER",
#'   "01", "F", "<65", "VALUE",
#'   "01", "F", "65-80", "VALUE",
#'   "01", "F", ">80", "VALUE",
#'   "01", "M", "Sex : M", "TABLE_BY_HEADER",
#'   "01", "M", "<65", "VALUE",
#'   "01", "M", "65-80", "VALUE",
#'   "01", "M", ">80", "VALUE"
#' )
#'
#' add_newrows(tbl2, tableby = "SEX")
#'
#' tbl3 <- tibble::tribble(
#'   ~anbr, ~SEX, ~ETHNIC, ~label, ~row_type,
#'   "01", "F", NA, "Sex : F", "TABLE_BY_HEADER",
#'   "01", "F", "HISPANIC OR LATINO", "HISPANIC OR LATINO", "BY_HEADER1",
#'   "01", "F", "HISPANIC OR LATINO", "<65", "VALUE",
#'   "01", "F", "HISPANIC OR LATINO", ">80", "VALUE",
#'   "01", "F", "HISPANIC OR LATINO", "65-80", "VALUE",
#'   "01", "F", "NOT HISPANIC OR LATINO", "NOT HISPANIC OR LATINO", "BY_HEADER1",
#'   "01", "F", "NOT HISPANIC OR LATINO", "<65", "VALUE",
#'   "01", "F", "NOT HISPANIC OR LATINO", "65-80", "VALUE",
#'   "01", "F", "NOT HISPANIC OR LATINO", ">80", "VALUE",
#'   "01", "M", NA, "Sex : M", "TABLE_BY_HEADER",
#'   "01", "M", "HISPANIC OR LATINO", "HISPANIC OR LATINO", "BY_HEADER1",
#'   "01", "M", "HISPANIC OR LATINO", "<65", "VALUE",
#'   "01", "M", "HISPANIC OR LATINO", "65-80", "VALUE",
#'   "01", "M", "HISPANIC OR LATINO", ">80", "VALUE",
#'   "01", "M", "NOT HISPANIC OR LATINO", "NOT HISPANIC OR LATINO", "BY_HEADER1",
#'   "01", "M", "NOT HISPANIC OR LATINO", "<65", "VALUE",
#'   "01", "M", "NOT HISPANIC OR LATINO", "65-80", "VALUE",
#'   "01", "M", "NOT HISPANIC OR LATINO", ">80", "VALUE"
#' )
#'
#' add_newrows(tbl3, tableby = "SEX", groupby = "ETHNIC")
add_newrows <- function(df, tableby = NULL, groupby = NULL) {
  dfn <- df %>%
    dplyr::group_by_at(c(tableby, groupby, "anbr")) %>%
    dplyr::mutate(
      roworder = dplyr::row_number(),
      newrows =
        dplyr::case_when(
          .data$roworder == 1 ~ 1,
          row_type == "TABLE_BY_HEADER" ~ 1,
          TRUE ~ 0
        )
    ) %>%
    dplyr::ungroup() %>%
    dplyr::mutate(
      newrows = ifelse(dplyr::row_number() == 1, 0, .data$newrows),
      newrows = ifelse(lag(newrows, na = 0) == 1 &
        row_type == "BY_HEADER2", 0, newrows) # nolint indentation_linter
    )

  if ("nested_level" %in% names(dfn)) {
    dfn %>%
      dplyr::mutate(newrows = case_when(
        row_type == "NESTED" &
          nested_level == 0 ~ 1,
        TRUE ~ newrows
      ))
  } else {
    dfn
  }
}

#' Output a `tidytlg` table
#'
#' Generate and output a `huxtable` with desired properties
#' During this function call, the `huxtable` can be written to an RTF or
#' displayed in HTML. `gentlg` is vectorized, see parameter descriptions
#' to learn for which arguments.
#'
#' @author Steven Haesendonckx <shaesen2@@its.jnj.com>
#' @author Pelagia Alexandra Papadopoulou <ppapadop@@its.jnj.com>
#'
#' @param huxme (optional) For tables and listings, A list of input dataframes
#'   containing all columns of interest. For graphs, either `NULL` or
#'   a list of `ggplot` objects. Vectorized.
#' @param tlf (optional) String, representing the output choice. Choices are
#' `"Table"` `"Listing"` `"Figure"`. Abbreviations are allowed e.g. `"T"` for Table.
#' Strings can be either upper- or lowercase. Vectorized. (Default = `"Table"`)
#' @param format (optional) String, representing the output format. Choices are
#' `"rtf"` and `"html"`. Strings can be either upper- or lowercase.(Default = `"rtf"`)
#' @param colspan (optional) A list of character vectors representing the
#' spanning headers to be used for the table or listing. The first vector
#' represents the top spanning header, etc. Each vector should have a length
#' equal to the number of columns in the output data frame. A spanning header
#' is identified through the use of the same column name in adjacent elements.
#' Vectorized.
#' @param idvars (optional) Character vector defining the columns of a listing
#' where repeated values should be removed recursively. If `NULL` then
#' all column names are used in the algorithm. If `NA`, then the listing remains
#' as is.
#' @param plotnames (optional) Character vector containing the names of the PNG
#' files, with their extension to be incorporated for figure outputs.
#' The PNG files need to be located in the path defined by the
#' parameter `opath`.
#' @param plotwidth (optional) Numerical value that indicates the plot width in
#' cm for figure outputs. (Default = 6)
#' @param plotheight (optional) Numerical value that indicates the plot height
#' in cm for figure outputs. (Default = 5)
#' @param wcol (optional) Can be one of:
#' - a single numeric value that represents the width of the first column
#' - a numeric vector, specifying the widths of all columns in the final table or listing
#' - a list of numeric vectors (applicable when `huxme` is a list). Each element
#' can specify the widths of all columns or the width of the first column only\cr
#'
#' When a single numerical value is used, this will be taken as the column width
#' for the first column. The other columns will be equally spaced across the
#' remainder of the available space. Alternatively, a vector can be used to
#' represent the widths of all columns in the final output. The order of the
#' arguments needs to correspond to the order of the columns in the `huxme`
#' dataset, that are not part of the formatting algorithms
#' (e.g. `anbr`, `roworder`, `newpage`, `newrow`,
#' `indentme`, `boldme`, `by_value`, `by_order`).
#' The sum of the widths in the vector needs to be less or equal to one. When
#' `format="HTML"` `wcol` can take only one value, the width
#' of the first column. (Default = 0.45).
#' @param opath (optional) File path pointing to the output files
#' (including PNG files for graphs). (Default = ".").
#' @param orientation (optional) String: "portrait" or "landscape".
#' (Default = "portrait")
#' @param file (required) String. Output identifier.
#' File name will be adjusted to be lowercase and have `-` and `_` removed,
#' this will not affect table title.
#' @param title_file An Excel file that will be read in
#' with `readxl::read_excel()` to be used as the `title` and `footers` argument.
#' The use of `title` or `footers` will override the values passed by this
#' argument. The file should be either an `xls` or `xlsx` file with the columns
#' `TABLE ID`, `IDENTIFIER`, and `TEXT`. The file will be read in, subset to
#' where the `tblid` matches the `tlf` argument, and identifiers with 'title' or
#' 'footnote' will be used to populate the table.
#' @param title (required) String. Title of the output. Vectorized.
#' @param footers (optional) Character vector, containing strings of footnotes
#' to be included. Vectorized.
#' @param print.hux (optional) Logical, indicating whether the output should be
#' printed to RTF `('format' = "rtf")` or displayed
#' as HTML `('format' = "HTML")`.
#' (Default = `TRUE`). Note that RTF is written using `quick_rtf_jnj()`
#' function and that the HTML is displayed via the `huxtable::print_html`
#' function.
#' @param watermark (optional) String containing the desired watermark for
#' RTF outputs. Vectorized.
#' @param colheader (optional) Character vector that contains the column labels
#' for a table or listing. Default uses the column labels of `huxme`.
#' Vectorized.
#' @param pagenum (optional) Logical. When true page numbers are added on the
#' right side of the footer section in the format page `x/y`.
#' Vectorized. (Default = `FALSE`).
#' @param bottom_borders (optional) Matrix or `"old_format"`.
#' A matrix indicating where to add the bottom
#' borders. Vectorized. See [add_bottom_borders()] for more information.
#' If `"old_format"`, then borders are added to the `colspan` and `colheader`
#' rows. (Default = "old_format").
#' @param border_fns (optional) List. A list of functions that transform
#' the matrix passed to `bottom_borders`. Vectorized. See
#' [add_bottom_borders()] for more information.
#' @param alignments (optional) List of named lists. Vectorized.
#' (Default = `list()`) Used to specify individual column or cell alignments.
#' Each named list contains `row`, `col`, and `value`, which are passed to
#' [huxtable::set_align()] to set the alignments.
#' @param footers_one_row (optional) Logical. Whether to export the footers
#' as a single table row (Default = `FALSE`).
#'
#' @section `Huxme` Details:
#' For tables and listings, formatting of the output can be dictated through the
#' formatting columns
#' (`newrows`, `indentme`, `boldme`, `newpage`), present in the input dataframe.
#' The final `huxtable` will display all columns of the input dataframe, except
#' any recognized formatting or sorting columns.
#' For tables, the algorithm uses the column `label` as first column.
#' The remaining columns are treated as summary columns.
#' For graphs, you can pass a `ggplot` object directly
#' into `huxme` and `gentlg` will save a PNG with with `ggplot2::ggsave()`
#' and output an RTF.
#'
#' @return A list of formatted `huxtables` with desired
#' properties for output to an RTF or HTML.
#' @export
#'
#' @examples
#'
#' final <- data.frame(
#'   label = c(
#'     "Overall", "Safety Analysis Set",
#'     "Any Adverse event{\\super a}", "- Serious Adverse Event"
#'   ),
#'   Drug_A = c("", "40", "10 (25%)", "0"),
#'   Drug_B = c("", "40", "10 (25%)", "0"),
#'   anbr = c(1, 2, 3, 4),
#'   roworder = c(1, 1, 1, 1),
#'   boldme = c(1, 0, 0, 0),
#'   newrows = c(0, 0, 1, 0),
#'   indentme = c(0, 0, 0, 1),
#'   newpage = c(0, 0, 0, 0)
#' )
#'
#' # Produce output in rtf format
#' gentlg(
#'   huxme = final,
#'   wcol = c(0.70, 0.15, 0.15),
#'   file = "TSFAEX",
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   )
#' )
#'
#' # Pass in column headers instead of using variable name
#' gentlg(
#'   huxme = final,
#'   wcol = c(0.70, 0.15, 0.15),
#'   file = "TSFAEX",
#'   colheader = c("", "Drug A", "Drug B"),
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   )
#' )
#'
#' # Add spanning bottom borders under the cells in the second row
#' gentlg(
#'   huxme = final,
#'   wcol = c(0.70, 0.15, 0.15),
#'   file = "TSFAEX",
#'   colheader = c("", "Drug A", "Drug B"),
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   ),
#'   border_fns = list(spanning_borders(2))
#' )
#'
#' # Use a watermark
#' gentlg(
#'   huxme = final,
#'   wcol = c(0.70, 0.15, 0.15),
#'   file = "TSFAEX",
#'   colheader = c("", "Drug A", "Drug B"),
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   ),
#'   watermark = "Confidential"
#' )
#'
#' # Set alignments
#' gentlg(
#'   huxme = final,
#'   file = "TSFAEX",
#'   alignments = list(
#'     # Align the second column to the left
#'     list(row = 1:7, col = 2, value = "left"),
#'
#'     # Align cell "Drug: B" to the right
#'     list(row = 2, col = 3, value = "right")
#'   )
#' )
#'
#' final_2 <- data.frame(
#'   label = c(
#'     "Overall", "Safety Analysis Set",
#'     "Any Adverse event{\\super a}", "- Serious Adverse Event"
#'   ),
#'   Drug_A = c("", "40", "10 (25%)", "0"),
#'   Drug_B = c("", "40", "10 (25%)", "0")
#' )
#'
#' gentlg(
#'   huxme = list(final_2, final_2),
#'   wcol = list(c(0.70, 0.15, 0.15), c(0.5)),
#'   file = "TSFAEX",
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   )
#' )
#'
#' # Produce output in HTML format
#' hux <- gentlg(
#'   huxme = final,
#'   file = "TSFAEX",
#'   colheader = c("", "Drug A", "Drug B"),
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   ),
#'   watermark = "Confidential",
#'   format = "HTML",
#'   print.hux = FALSE
#' )
#'
#' # Export to HTML page
#' huxtable::quick_html(hux, file = "TSFAEX.html", open = FALSE)
#'
#' # clean up.
#' file.remove("TSFAEX.html", "tsfaex.rtf")
#' @references \url{https://github.com/hughjonesd/huxtable}
gentlg <- function(huxme = NULL,
                   tlf = "Table",
                   format = "rtf",
                   colspan = NULL,
                   idvars = NULL,
                   plotnames = NULL,
                   plotwidth = NULL,
                   plotheight = NULL,
                   wcol = 0.45,
                   orientation = "portrait",
                   opath = ".",
                   title_file = NULL,
                   file = NULL,
                   title = NULL,
                   footers = NULL,
                   print.hux = TRUE,
                   watermark = NULL,
                   colheader = NULL,
                   pagenum = FALSE,
                   bottom_borders = "old_format",
                   border_fns = list(),
                   alignments = list(),
                   footers_one_row = FALSE) {
  # Validate `alignments` here because of its complicated data structure
  stopifnot("`alignments` must be a list" = is.list(alignments))

  for (alignment in alignments) {
    stopifnot("Each item of `alignments` must be a list" = is.list(alignment))
  }

  # if wcol is a list, then huxme must be a list with same length,
  # and wcol[[i]] must be a length 1 vector or a vector with as many numeric values
  # as number of columns in huxme[[i]]
  if (is.list(wcol)) {
    assertthat::assert_that(is.list(huxme) && !is.data.frame(huxme),
      msg = paste0(
        "'wcol' appears to be a list while huxme is not a list of tables/listings. ",
        "If you intended 'wcol' to apply to the single output, convert it to a ",
        "vector, otherwise pass a non-data.frame list to 'huxme'."
      )
    )
    ## already know wcol is a list and huxme is a non-data.frame list
    assertthat::assert_that(length(huxme) == length(wcol),
      msg = "Arguments 'wcol' and 'huxme' must have the same length."
    )
  }

  adjfilename <- stringr::str_replace_all(
    stringr::str_to_lower(file),
    "(-|_)", ""
  )
  if (is.null(huxme)) {
    ht <- gentlg_single(
      huxme = NULL,
      tlf = tlf,
      format = format,
      colspan = colspan,
      idvars = idvars,
      plotnames = plotnames,
      plotwidth = plotwidth,
      plotheight = plotheight,
      wcol = wcol,
      orientation = orientation,
      opath = opath,
      title_file = title_file,
      file = file,
      title = title,
      footers = footers,
      print.hux = print.hux,
      watermark = watermark,
      colheader = colheader,
      pagenum = pagenum,
      bottom_borders = bottom_borders,
      border_fns = border_fns,
      alignments = alignments,
      footers_one_row = footers_one_row
    )

    if (print.hux == FALSE) {
      return(ht$ht)
    } else if (print.hux == TRUE && is_format_rtf(format)) {
      quick_rtf_jnj(
        list(ht$ht),
        file = paste(file.path(opath, adjfilename), ".rtf", sep = ""),
        pagenum = pagenum,
        portrait = tolower(orientation) == "portrait",
        watermark = list(watermark),
        nheader = 1 + ifelse(is.null(ht$colspan), 0, ht$colspan),
        tlf = tlf,
      )
      return(invisible(NULL))
    } else if (print.hux == TRUE && toupper(format) == "HTML") {
      huxtable::print_html(ht$ht)
      return(invisible(NULL))
    }
  }

  if (inherits(huxme, "data.frame") || inherits(huxme, "ggplot")) {
    huxme <- list(huxme)
  }

  # If we leave NULLs in the arguments
  # then the mapply won't run, so we
  # wrap the NULLs in a list.
  # The same goes for scalar arguments that
  # can be arrays.
  if (!is.list(title)) {
    title <- list(title)
  }
  if (!is.list(footers)) {
    footers <- list(footers)
  }
  if (!is.list(watermark)) {
    watermark <- list(watermark)
  }
  if (!is.list(colheader)) {
    colheader <- list(colheader)
  }
  if (!is.list(bottom_borders)) {
    bottom_borders <- list(bottom_borders)
  }
  assertthat::assert_that(is.list(border_fns))
  if (length(
    border_fns
  ) == 0 ||
    (length(border_fns) > 0 && !is.list(border_fns[[1]]))) {
    border_fns <- list(border_fns)
  }
  if (
    (is.list(colspan) && length(colspan) > 0 && !is.list(colspan[[1]])) ||
      is.null(colspan)
  ) {
    colspan <- list(colspan)
  }

  if (length(alignments) == 0 || !all(sapply(unlist(alignments, FALSE), is.list))) {
    alignments <- list(alignments)
  }

  if (!is.list(wcol)) {
    wcol <- list(wcol)
  }

  hts <- mapply(
    function(ht,
             colspan,
             title,
             footers,
             watermark,
             colheader,
             pagenum,
             bottom_borders,
             border_fns,
             alignments,
             index,
             wcol) {
      gentlg_single(
        huxme = ht,
        tlf = tlf,
        format = format,
        colspan = colspan,
        idvars = idvars,
        plotnames = plotnames,
        plotwidth = plotwidth,
        plotheight = plotheight,
        wcol = wcol,
        orientation = orientation,
        opath = opath,
        title_file = title_file,
        file = file,
        title = title,
        footers = footers,
        print.hux = FALSE,
        watermark = watermark,
        colheader = colheader,
        pagenum = pagenum,
        bottom_borders = bottom_borders,
        border_fns = border_fns,
        alignments = alignments,
        index_in_result = index,
        footers_one_row = footers_one_row
      )
    },
    ht = huxme,
    colspan = colspan,
    title = title,
    footers = footers,
    watermark = watermark,
    colheader = colheader,
    pagenum = pagenum,
    bottom_borders = bottom_borders,
    border_fns = border_fns,
    alignments = alignments,
    index = seq_along(huxme),
    wcol = wcol,
    SIMPLIFY = FALSE
  )

  if (print.hux == FALSE) {
    lapply(hts, function(ht) ht$ht)
  } else if (print.hux == TRUE && is_format_rtf(format)) {
    quick_rtf_jnj(lapply(hts, function(ht) ht$ht),
      file = paste(file.path(opath, adjfilename), ".rtf", sep = ""),
      pagenum = pagenum,
      portrait = tolower(orientation) == "portrait",
      watermark = watermark,
      nheader = 1 + as.numeric(lapply(hts, function(ht) length(ht$colspan))),
      tlf = tlf,
    )
  } else if (print.hux == TRUE && toupper(format) == "HTML") {
    lapply(hts, huxtable::print_html)
  }
}

library(tibble)
test_that("titles_file addes titles as expected", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    mtcars2 <- mtcars %>%
      rownames_to_column(var = "USUBJID") %>%
      mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

    tab <- freq(mtcars2, colvar = "cyl", rowvar = "am")

    withr::with_dir(
      new = test_path("test_outputs"),
      code = {
        gentlg(tab, file = "TSIDEM01", title_file = "titles.xls", colheader = c("label", "4", "6", "8"))
      }
    )

    withr::with_dir(
      new = test_path("test_outputs"),
      code = {
        gentlg(tab, file = "GSVIT01A", title_file = "titles.xls", colheader = c("label", "4", "6", "8"))
      }
    )
    expect_snapshot_file(test_path("test_outputs/gsvit01a.rtf"))
  })
})

library(tibble)

test_that("colheader works as expected for listings", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    mtcars2 <- mtcars %>%
      rownames_to_column(var = "USUBJID") %>%
      mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear)) %>%
      select("USUBJID", "mpg", "cyl", "disp", "hp")

    withr::with_dir(test_path("test_outputs"), {
      gentlg(mtcars2, file = "colheader1", tlf = "l")
    })

    mtcars3 <- mtcars2
    attr(mtcars3$mpg, "label") <- "Miles/(US) gallon"
    attr(mtcars3$cyl, "label") <- "Number of cylinders"
    attr(mtcars3$disp, "label") <- "Displacement (cu.in.)"
    attr(mtcars3$hp, "label") <- "Gross horsepower"

    withr::with_dir(test_path("test_outputs"), {
      gentlg(mtcars3, file = "colheader2", tlf = "l")
    })
  })

  expect_snapshot_file(test_path("test_outputs/colheader1.rtf"))
  expect_snapshot_file(test_path("test_outputs/colheader2.rtf"))
})

library(ggplot2)

test_that("gentlg will preserve aspect ratio of pngs", {
  png(filename = test_path("test_outputs/png1.png"), height = 325, width = 200, type = "cairo-png")
  plot(mtcars$cyl)
  if (dev.cur() != 1) dev.off()
  png(filename = test_path("test_outputs/png2.png"), height = 193, width = 538, type = "cairo-png")
  plot(mtcars$cyl)
  if (dev.cur() != 1) dev.off()

  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    withr::with_dir(test_path("test_outputs"), {
      gentlg(plotnames = "png1.png", tlf = "f", file = "png1")
      gentlg(plotnames = "png2.png", tlf = "f", file = "png2")
    })
  })
  skip_if(Sys.getenv("RENV_PATHS_CACHE") == "")
  expect_snapshot_file(test_path("test_outputs/png1.rtf"))
  expect_snapshot_file(test_path("test_outputs/png2.rtf"))
})

test_that("gentlg can take multiple plotnames arguments", {
  a_tempdir <- test_path("test_outputs")

  adsl <- cdisc_adsl %>% filter(SAFFL == "Y")

  tblid <- "test2plots"

  plot1 <- ggplot(data = adsl, aes(x = HEIGHTBL, y = WEIGHTBL)) +
    geom_point() +
    labs(x = "Baseline Height (cm)", y = "Baseline Weight (kg)")

  png(paste0(a_tempdir, "/plot1.png"), width = 2800, height = 1800, res = 300, type = "cairo")

  plot1

  if (dev.cur() != 1) dev.off()

  plot2 <- ggplot(data = adsl, aes(x = WEIGHTBL, y = HEIGHTBL)) +
    geom_point() +
    labs(x = "Baseline Weight (kg)", y = "Baseline Height (cm)")

  png(paste0(a_tempdir, "/plot2.png"), width = 2800, height = 1800, res = 300, type = "cairo")

  plot2

  if (dev.cur() != 1) dev.off()

  gentlg(
    tlf = "g",
    plotnames = paste0(a_tempdir, "/", c("plot1.png", "plot2.png")),
    opath = a_tempdir,
    plotwidth = 8,
    plotheight = 5,
    orientation = "landscape",
    file = tblid,
    title = "Scatter plot"
  )

  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    expect_snapshot_file(test_path("test_outputs/test2plots.rtf"))
  })
})

library(tibble)

test_that("multiple tables are printed to one rtf file", {
  output_directory <- "test_outputs/multiple_tables_in_one_file"

  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    mtcars2 <- mtcars %>%
      rownames_to_column(var = "USUBJID") %>%
      mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

    tab <- freq(mtcars2, colvar = "cyl", rowvar = "am") %>% select(-c("row_type", "group_level"))

    tab2 <- freq(
      mtcars2 %>%
        tlgsetup(
          var = "cyl",
          column_metadata_file = testthat::test_path("test_data/column_metadata_mtcars.xlsx"),
          tbltype = "mtcars2"
        ),
      colvar = "colnbr",
      rowvar = "am"
    ) %>%
      bind_table(
        column_metadata_file = testthat::test_path("test_data/column_metadata_mtcars.xlsx"), tbltype = "mtcars2"
      ) %>%
      select("label", starts_with("col"))

    withr::with_dir(
      new = testthat::test_path(output_directory),
      code = {
        gentlg(
          list(tab, tab2),
          file = "multipletabs",
          title = list("Title 1", "Title 2"),
          footers = list("Footer 1", "Footer 2")
        )
      }
    )
  })

  expect_snapshot_file(test_path(sprintf("%s/multipletabs.rtf", output_directory)))

  # Verify only one file was created
  files <- list.files(path = testthat::test_path(output_directory), pattern = "*.rtf")
  testthat::expect_identical(
    length(files), 1L,
    label = sprintf("Expected to find only one file. Found: %s", paste0(files, collapse = " "))
  )
})

library(tibble)

output_directory <- "test_outputs/borders"
testthat::test_that("default uses old format", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    tab <- iris[1:5, ]
    tab$label <- c("Label", rep("", 4))

    withr::with_dir(
      new = testthat::test_path(output_directory),
      code = {
        gentlg(
          tab,
          file = "oldformat", title = "Title1", colspan = list(c("", rep("Measures", 4), "")), footers = "Footer1"
        )
      }
    )
  })

  expect_snapshot_file(test_path(sprintf("%s/oldformat.rtf", output_directory)))
})

testthat::test_that("No borders generates rtf with no borders", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    tab <- iris[1:5, ]
    tab$label <- c("Label", rep("", 4))

    withr::with_dir(
      new = testthat::test_path(output_directory),
      code = {
        gentlg(
          tab,
          file = "noborders",
          title = "Title1",
          colspan = list(c("", rep("Measures", 4), "")),
          footers = "Footer1",
          border_fns = list(no_borders)
        )
      }
    )
  })

  expect_snapshot_file(test_path(sprintf("%s/noborders.rtf", output_directory)))
})

testthat::test_that("Spanning borders in the first row", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    tab <- iris[1:5, ]
    tab$label <- c("Label", rep("", 4))

    withr::with_dir(
      new = testthat::test_path(output_directory),
      code = {
        gentlg(
          tab,
          file = "spanningbordersrow1",
          title = "Title1",
          colspan = list(c("", rep("Measures", 4), "")),
          footers = "Footer1",
          border_fns = list(no_borders, spanning_borders(1))
        )
      }
    )
  })

  expect_snapshot_file(test_path(sprintf("%s/spanningbordersrow1.rtf", output_directory)))
})

testthat::test_that("Inserts separated borders in the second row", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    bmat2 <- matrix(c(0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 2, 1, 1, 1, 1, 1, 1), nrow = 3, ncol = 6, byrow = TRUE)

    tbl <- data.frame(
      label = "Subjects with ≥ 1 concomitant medication",
      col1 = "1 (100.0%)",
      col2 = "1 (100.0%)",
      col3 = "2 (100.0%)",
      col4 = "1 (100.0%)",
      col5 = "3 (100.0%)",
      row_type = "VALUE"
    )

    withr::with_dir(
      new = testthat::test_path(output_directory),
      code = {
        gentlg(
          huxme = tbl,
          orientation = "landscape",
          file = "stubborntest",
          title = "Summary of Concomitant Medications",
          colspan = list(
            c("", "Active Study Agent", "Active Study Agent", "Active Study Agent", "", ""),
            c("", "Treatment 1", "Treatment 2", "Combined", "Placebo", "Total")
          ),
          colheader = c(" Standardized medication name", "N=1", "N=1", "N=2", "N=1", "N=3"),
          wcol = .30,
          bottom_borders = bmat2,
          border_fns = list()
        )
      }
    )
  })

  expect_snapshot_file(test_path(sprintf("%s/stubborntest.rtf", output_directory)))
})

testthat::test_that("Inserts separated borders under the first row for a listing", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    adsl <- cdisc_adsl
    adae <- cdisc_adae

    adsl <- adsl %>%
      filter(SAFFL == "Y") %>%
      select(USUBJID, SAFFL, TRT01AN, TRT01A)

    adae <- adae %>%
      filter(SAFFL == "Y" & TRTEMFL == "Y") %>%
      mutate(
        BSPT = paste(AEBODSYS, "[", AEDECOD, "]"),
        SAEFL = if_else(AESER == "Y", "Yes", "No"),
        DTHFL = if_else(AEOUT == "FATAL", "Yes", "No")
      ) %>%
      select(USUBJID, ASTDY, TRTA, BSPT, AETERM, SAEFL, DTHFL)

    tbl <- inner_join(adsl, adae, by = "USUBJID") %>%
      arrange(TRT01AN, USUBJID, ASTDY) %>%
      select(TRT01A, USUBJID, ASTDY, TRTA, BSPT, AETERM, SAEFL, DTHFL) %>%
      filter(USUBJID %in% c("01-701-1015", "01-701-1023"))

    withr::with_dir(
      new = testthat::test_path(output_directory),
      code = {
        gentlg(
          huxme = tbl,
          tlf = "l",
          orientation = "landscape",
          file = "oldformatlisting",
          title = "Listing of Adverse Events",
          idvars = c("TRT01A", "USUBJID"),
          wcol = 0.15,
          colheader = c(
            "Treatment Group",
            "Subject ID",
            "Study Day of AE",
            "Treatment Period",
            "Body System [Preferred Term]",
            "Verbatim Term",
            "Serious",
            "Fatal"
          )
        )
      }
    )
  })

  expect_snapshot_file(test_path(sprintf("%s/oldformatlisting.rtf", output_directory)))
})

library(tibble)

test_that("colspan works as expected", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    mtcars2 <- mtcars %>%
      rownames_to_column(var = "USUBJID") %>%
      mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

    tab <- freq(mtcars2, colvar = "cyl", rowvar = "am") %>% select(-c("row_type", "group_level"))

    withr::with_dir(
      new = test_path("test_outputs"),
      code = {
        gentlg(tab, file = "colspan1", colspan = list(c("", "All", "All", "All"), c("", "Low", "High", "High")))
      }
    )

    tab <- freq(
      mtcars2 %>%
        tlgsetup(
          var = "cyl", column_metadata_file = test_path("test_data/column_metadata_mtcars.xlsx"), tbltype = "mtcars2"
        ),
      colvar = "colnbr",
      rowvar = "am"
    ) %>%
      bind_table(column_metadata_file = test_path("test_data/column_metadata_mtcars.xlsx"), tbltype = "mtcars2") %>%
      select("label", starts_with("col"))

    withr::with_dir(
      new = test_path("test_outputs"),
      code = {
        gentlg(tab, file = "colspan2")
      }
    )

    tab <- freq(mtcars2, colvar = "cyl", rowvar = "am") %>% select(-c("row_type", "group_level"))

    withr::with_dir(
      new = test_path("test_outputs"),
      code = {
        gentlg(tab, file = "colspan3", colspan = list(c("", "Low", "High", "High"), c("", "All", "All", "All")))
      }
    )
  })
  expect_snapshot_file(test_path("test_outputs/colspan1.rtf"))
  expect_snapshot_file(test_path("test_outputs/colspan2.rtf"))
  expect_snapshot_file(test_path("test_outputs/colspan3.rtf"))
})

library(tibble)

test_that("Page numbers are added on each page", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    mtcars2 <- mtcars %>%
      rownames_to_column(var = "USUBJID") %>%
      mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

    tab <- freq(mtcars2, colvar = "cyl", rowvar = "am") %>% select(-c("row_type", "group_level"))
    tab2 <- tab

    withr::with_dir(
      new = test_path("test_outputs"),
      code = {
        gentlg(list(tab, tab2), file = "pagenumbers", tlf = "Listing", pagenum = TRUE)
      }
    )
  })
  expect_snapshot_file(test_path("test_outputs/pagenumbers.rtf"))
})

test_that("The page number mentions the table", {
  withr::with_options(list(tidytlg.add_datetime = FALSE), {
    mtcars2 <- mtcars %>%
      rownames_to_column(var = "USUBJID") %>%
      mutate(am = factor(am), cyl = factor(cyl), gear = factor(gear))

    tab <- freq(mtcars2, colvar = "cyl", rowvar = "am") %>% select(-c("row_type", "group_level"))
    tab2 <- tab

    withr::with_dir(
      new = test_path("test_outputs"),
      code = {
        gentlg(list(tab, tab2), file = "pagenumberstable", pagenum = TRUE)
      }
    )
  })
  expect_snapshot_file(test_path("test_outputs/pagenumberstable.rtf"))
})

#' Setup data to support the specified column type
#'
#' `tlgsetup` is useful for preprocessing total columns and columns composed of
#' other columns. `tlgsetup` is called internally by `generate_results()`
#' and can be run manually for custom tables.
#'
#' @param df dataframe of records for analysis.
#' @param var character vector that identifies the numeric column/treatment
#'   variable.
#' @param column_metadata_file A file containing the column metadata. Read in
#'   with `readxl::read_excel()`. If a `column_metadata` dataframe is passed in
#'   too, this is ignored.
#' @param column_metadata A dataframe containing the column metadata. This will
#'   be used in place of `column_metadata_file`.
#' @param tbltype A value used to subset the `column_metadata`, both this
#'   and the file requirements are needed to bind the data to the table.
#'
#' @return dataframe with observations added to support the column type as well
#'   as the factor variable `colnbr` which is used as our new column summary
#'   variable. Regardless of if a `coldef` exists in data,
#'   the column will exist in the table.
#'
#' @examples
#' df <-
#'   tibble::tribble(
#'     ~TRT01AN, ~USUBJID,
#'     0, "A",
#'     54, "B",
#'     81, "C"
#'   )
#'
#' tlgsetup(df, "TRT01AN", column_metadata = column_metadata)
#'
#' # Using a dataframe of column metadata
#' column_metadata <-
#'   tibble::tribble(
#'     ~tbltype, ~coldef, ~decode,                ~span1,
#'     "type1",  "0",     "Placebo",              "",
#'     "type1",  "54",    "Low Dose",             "Xanomeline",
#'     "type1",  "81",    "High Dose",            "Xanomeline",
#'     "type1",  "54+81", "Total Xanomeline",     ""
#'   )
#'
#' tlgsetup(df, "TRT01AN", column_metadata = column_metadata)
#' @export
tlgsetup <-
  function(df,
           var,
           column_metadata_file = NULL,
           column_metadata = NULL,
           tbltype = NULL) {
    arglist <- list()
    args_to_chk <- names(formals())[names(formals()) != "..."]
    purrr::walk(
      args_to_chk,
      .f = function(x) {
        arglist[[x]] <<- eval(rlang::sym(x))
      }
    )
    check_tlgsetup(arglist)

    if ("colnbr" %in% names(df)) {
      message("tidytlg seems to have been run more than once. Remove 'colnbr'
            column and rerun if chnages to column_metadata.")
      return(df)
    }

    if (is.null(c(column_metadata_file, column_metadata))) {
      return(df)
    } else if (is.null(column_metadata)) {
      column_metadata <- readxl::read_excel(column_metadata_file, sheet = 1)
    }

    if (!is.null(tbltype)) {
      column_metadata <- column_metadata %>%
        # Per JCEV-16: Any non breaking space(utf8 160) should be collapsed into space
        # (utf8 32)
        mutate(
          tbltype = gsub("\u00A0", " ", tbltype)
        ) %>%
        filter(tbltype == !!tbltype)
    }

    rlang::check_required(df)
    rlang::check_required(var)
    rlang::check_required(column_metadata)

    if (inherits(df, "survfit")) {
      return(df)
    } else {
      # add rows to df based off column metadata
      split <- column_metadata %>%
        dplyr::mutate(colnbr = forcats::fct_reorder(paste0(
          "col",
          dplyr::row_number()
        ), dplyr::row_number())) %>%
        tidyr::separate_rows(coldef, sep = "\\+")

      if (!is.factor(df[[var]])) {
        df[[var]] <- as.character(df[[var]])
      }


      # temp fix to align attributes and prevent warning
      attr(split$coldef, "label") <- attr(df[[var]], "label")

      split %>%
        inner_join(df, by = c("coldef" = var), keep = TRUE, relationship = "many-to-many") %>%
        select(-coldef, -decode, -dplyr::starts_with("span")) %>%
        mutate(
          colnbr = fct_expand(colnbr, unique(as.character(split$colnbr)))
        )
    }
  }

#' Frequency counts and percentages
#'
#' Frequency counts and percentages for a variable by treatment and/or group.
#'
#' @param df (required) dataframe containing records to summarize by treatment.
#' @param denom_df (optional) dataframe used for population based denominators
#'  (default = `df`).
#' @param colvar (required) treatment variable within `df` to use to summarize
#' @param tablebyvar (optional) repeat entire table by variable within `df`
#' @param rowvar (required) character vector of variables to
#'  summarize within the dataframe.
#' @param rowbyvar (optional) repeat `rowvar` by variable within `df`
#' @param statlist (optional) `statlist` object of stats to keep of length
#'  1 or 2 specifying list of statistics and format desired
#'  (e.g `statlist(c("N", "n (x.x\%)"))`) (default = `statlist(c("n
#'  (x.x)"))`).
#' @param decimal (optional) decimal precision root level default (default = 1).
#' @param nested (optional) INTERNAL USE ONLY. The default should
#'  not be changed. Switch on when this function is called by
#'  `nested_freq()` so we will not include the by variables as
#'   part of the group denominators (default = `FALSE`).
#' @param cutoff (optional) percentage cutoff threshold. This can be passed as a
#'  numeric cutoff, in that case any rows with greater than or equal to that
#'  cutoff will be preserved, others will be dropped. To specify a single column
#'  to define the cutoff logic, pass a character value of the form
#' `<colName> >= <value>` and only that column will be used.
#' @param cutoff_stat (optional) The value to cutoff by, `n` or `pct.`
#'  (default = `'pct'`). Can be done with multiple columns
#'  by adding `&` or `|` ex. `col1 >= val1 & col2 >= val2`.
#' @param subset (optional) An R expression that will be passed to a
#'  [`dplyr::filter()`] function to subset the `data.frame`.
#'  This is performed on the numerator before any other
#'  derivations. Denominators must be preprocessed and passed
#'  through using `denom_df`.
#' @param descending_by (optional) The column or columns to sort descending
#'  counts. Can also provide a named list to do ascending order ex.
#'  `c("VarName1" = "asc", "VarName2" = "desc")` would sort by `VarName1` in
#'  ascending order and `VarName2` in descending order.
#'  In case of a tie in count or `descending_by` not provided,
#'  the columns will be sorted alphabetically.
#' @param display_missing (optional) Should the "missing" values be displayed?
#'  If missing values are displayed, denominators will include missing values.
#'  (default = `FALSE`).
#' @param rowtext (optional) A character vector used to rename the `label`
#'  column. If named, names will give the new level and values will be the
#'  replaced value. If unnamed, and the table has only one row, the `rowtext`
#'  will rename the label of the row. If the `rowtext` is unnamed,
#'  the table has no rows, and there is a subset,
#'  the table will be populated with zeros and the label will be the only row.
#' @param row_header (optional) A character vector to be added to the table.
#' @param .keep (optional) Should the `rowbyvar` and `tablebyvar` be
#'  output in the table. If `FALSE`, `rowbyvar` will still be
#'  output in the `label` column. (Default = `TRUE`).
#' @param .ord Should the ordering columns be output with the table? This is
#'  useful if a table needs to be merged or reordered in any way after build.
#' @param pad (optional) A boolean that controls if levels with zero records
#'  should be included in the final table. (default = `TRUE`).
#' @param ... (optional) Named arguments to be included as columns on the table.
#'
#' @section Sorting a 'freq' table:
#'
#'  By default, a frequency table is sorted based on the factor level of the
#'  `rowvar` variable. If the `rowvar` variable isn't a factor, it will be
#'  sorted alphabetically. This behavior can be modified in two ways, the first
#'  is the `char2factor()` function that offers a interface for discretization a
#'  variable based on a numeric variable, like `VISITN`. The second is based on
#'  the `descending_by` argument which will sort based on counts on a variable.
#'
#' @return A dataframe of results
#'
#' @importFrom rlang enexprs
#'
#' @export
#'
#' @examples
#' adsl <- data.frame(
#'   USUBJID = c("DEMO-101", "DEMO-102", "DEMO-103"),
#'   RACE = c("WHITE", "BLACK", "ASIAN"),
#'   SEX = c("F", "M", "F"),
#'   colnbr = factor(c("Placebo", "Low", "High"))
#' )
#'
#' # Unique subject count of a single variable
#' freq(adsl,
#'   colvar = "colnbr",
#'   rowvar = "RACE",
#'   statlist = statlist("n")
#' )
#'
#' # Unique subject count and percent of a single variable
#' freq(adsl,
#'   colvar = "colnbr",
#'   rowvar = "RACE",
#'   statlist = statlist(c("N", "n (x.x%)"))
#' )
#'
#' # Unique subject count of a variable by another variable
#' freq(adsl,
#'   colvar = "colnbr",
#'   rowvar = "RACE",
#'   rowbyvar = "SEX",
#'   statlist = statlist("n")
#' )
#'
#' # Unique subject count of a variable by another variable using colvar and
#' # group to define the denominator
#' freq(adsl,
#'   colvar = "colnbr",
#'   rowvar = "RACE",
#'   rowbyvar = "SEX",
#'   statlist = statlist("n (x.x%)", denoms_by = c("colnbr", "SEX"))
#' )
#'
#' # Cut records where count meets threshold for any column
#' freq(cdisc_adsl,
#'   rowvar = "ETHNIC",
#'   colvar = "TRT01P",
#'   statlist = statlist("n (x.x%)"),
#'   cutoff = "5",
#'   cutoff_stat = "n"
#' )
#'
#' # Cut records where count meets threshold for a specific column
#' freq(cdisc_adsl,
#'   rowvar = "ETHNIC",
#'   colvar = "TRT01P",
#'   statlist = statlist("n (x.x%)"),
#'   cutoff = "Placebo >= 3",
#'   cutoff_stat = "n"
#' )
#'
#' # Below illustrates how to make the same calls to freq() as above, using
#' # table and column metadata.
#'
#' # Unique subject count of a single variable
#' table_metadata <- tibble::tribble(
#'   ~anbr, ~func, ~df, ~rowvar, ~statlist, ~colvar,
#'   1, "freq", "cdisc_adsl", "ETHNIC", statlist("n"), "TRT01PN"
#' )
#'
#' generate_results(table_metadata,
#'   column_metadata = column_metadata,
#'   tbltype = "type1"
#' )
#'
#' # Unique subject count and percent of a single variable
#' table_metadata <- tibble::tribble(
#'   ~anbr, ~func, ~df, ~rowvar, ~statlist, ~colvar,
#'   "1", "freq", "cdisc_adsl", "ETHNIC", statlist(c("N", "n (x.x%)")), "TRT01PN"
#' )
#'
#' generate_results(table_metadata,
#'   column_metadata = column_metadata,
#'   tbltype = "type1"
#' )
#'
#' # Cut records where count meets threshold for any column
#' table_metadata <- tibble::tibble(
#'   anbr = "1", func = "freq", df = "cdisc_adsl", rowvar = "ETHNIC",
#'   statlist = statlist("n (x.x%)"), colvar = "TRT01PN", cutoff = 5,
#'   cutoff_stat = "n"
#' )
#'
#' generate_results(table_metadata,
#'   column_metadata = column_metadata,
#'   tbltype = "type1"
#' )
#'
#' # Cut records where count meets threshold for a specific column
#' table_metadata <- tibble::tibble(
#'   anbr = 1, func = "freq", df = "cdisc_adsl", rowvar = "ETHNIC",
#'   statlist = statlist("n (x.x%)"), colvar = "TRT01PN",
#'   cutoff = "col1 >= 3", cutoff_stat = "n"
#' )
#'
#' generate_results(table_metadata,
#'   column_metadata = column_metadata,
#'   tbltype = "type1"
#' )
freq <- function(df,
                 denom_df = df,
                 colvar = NULL,
                 tablebyvar = NULL,
                 rowvar = NULL,
                 rowbyvar = NULL,
                 statlist = getOption("tidytlg.freq.statlist.default"),
                 decimal = 1,
                 nested = FALSE,
                 cutoff = NULL,
                 cutoff_stat = "pct",
                 subset = TRUE,
                 descending_by = NULL,
                 display_missing = FALSE,
                 rowtext = NULL,
                 row_header = NULL,
                 .keep = TRUE,
                 .ord = FALSE,
                 pad = TRUE,
                 ...) {
  subset <- enexprs(subset)
  if (length(subset) > 0 && is.character(subset[[1]])) {
    subset <- as.list(parse(text = subset))
  }
  has_subset <- is_call(subset[[1]])
  rowtext <- expr(!!rowtext)

  # check all the arguments being passed in except ...
  arglist <- list()
  args_to_chk <- names(formals())[names(formals()) != "..."]
  walk(args_to_chk, .f = function(x) arglist[[x]] <<- eval(sym(x)))
  check_freq(arglist)

  if (!pad && is.null(cutoff)) {
    cutoff <- 1
    cutoff_stat <- "n"
  }

  # attempt to use colvar, tablebyvar, rowbyvar to define denominator
  # if all are not present in denom_df, use what is present and alert user to
  # what was used
  vars_in_denom_names <- names(denom_df) %in% c(colvar, tablebyvar, rowbyvar)
  vars_in_denom_df <- names(denom_df)[vars_in_denom_names]

  # If the statlist has a denoms_by config, use that. Otherwise default to
  # colvar, tablebyvar, and rowbyvar
  denoms_by <- statlist[["denoms_by"]] %||% vars_in_denom_df

  # Added to close JCEV-10
  denom_df <- denom_df %>%
    {
      if (!display_missing && identical(df, denom_df)) {
        filter(., !is.na(!!sym(rowvar)))
      } else {
        .
      }
    }

  if (getOption("tidytlg.denoms.message")) {
    message("denominator set using ", paste0(denoms_by, collapse = ", "),
      ". You can overwrite using the denom_by argument in statlist.\n",
      sep = ""
    )
  }

  colvar_cols <- levels(as.factor(df[[colvar]]))

  freqs <- df %>%
    # filter out based on subset
    filter(!!!subset) %>%
    # Derive counts and percents
    derive_freq(
      denom_df = denom_df,
      colvar = colvar,
      rowbyvar = rowbyvar,
      tablebyvar = tablebyvar,
      rowvar = rowvar,
      statlist = statlist,
      nested = nested,
      rowtext = rowtext,
      has_subset = has_subset,
      denoms_by = denoms_by,
      display_missing = display_missing
    ) %>%
    # Drop values based on 'cutoff' argument
    drop_freq(
      cutoff = cutoff,
      rowvar = rowvar,
      cutoff_stat = cutoff_stat,
      colvar = colvar,
      drop_by = c(tablebyvar, rowbyvar)
    ) %>%
    # round values
    round_freq(decimal = decimal) %>%
    # Arrange values from n and pct into values
    arrange_freq(statlist = statlist) %>%
    # merge in bigN row, tablebyvar, and rowbyvar
    byvar_merge_freq(
      tablebyvar = tablebyvar,
      rowbyvar = rowbyvar,
      colvar = colvar,
      rowvar = rowvar,
      has_big_n = "N" %in% statlist[["stats"]],
      row_header = row_header
    ) %>%
    # Pivot table
    pivot_freq(
      colvar = colvar,
      rowvar = rowvar,
      rowbyvar = rowbyvar,
      tablebyvar = tablebyvar,
      statlist = statlist,
      rowtext = rowtext,
      row_header = row_header,
      display_missing = display_missing,
      nested = nested,
      .keep = .keep,
      .ord = .ord,
      ... = ...
    ) %>%
    # Sort rows based on 'descending_by'
    sort_freq(
      rowvar = rowvar,
      descending_by = descending_by,
      tablebyvar = tablebyvar,
      rowbyvar = rowbyvar,
      colvar_cols = colvar_cols,
      colvar = colvar
    )


  attr(freqs, "colvar") <- colvar
  attr(freqs, "rowbyvar") <- rowbyvar
  attr(freqs, "tablebyvar") <- tablebyvar
  attr(freqs, "rowvar") <- rowvar
  attr(freqs, "statlist") <- statlist
  attr(freqs, "cutoff") <- cutoff
  attr(freqs, "cutoff_stat") <- cutoff_stat
  attr(freqs, "descending_by") <- descending_by
  attr(freqs, "rowtext") <- rowtext
  attr(freqs, "row_header") <- row_header
  attr(freqs, "subset") <- subset
  attr(freqs, "denoms_by") <- denoms_by

  structure(freqs,
    class = c("tidytlg.freq", class(freqs))
  )
}

#' Derive frequency
#'
#' @return A long dataframe with the columns: `colvar`, `rowbyvar`, `rowvar`, n,
#'   N, denom, and pct.
#'
#' @noRd
derive_freq <- function(df, denom_df, colvar, rowbyvar, tablebyvar, rowvar,
                        statlist, nested, rowtext, has_subset, denoms_by,
                        display_missing) {
  # Denom dataframe, same as bigN, except the name.
  denom <- denom_df %>%
    derive_count(denoms_by, statlist, "denom")

  # The dataframe with counts will contain the n, N, pct, and denoms.
  res <- df %>%
    derive_count(
      by = c(colvar, rowbyvar, tablebyvar, rowvar),
      statlist = statlist,
      count_name = "n"
    ) %>%
    left_join(denom, by = intersect(names(.), names(denom))) %>%
    mutate(
      pct = n / denom * 100
    )

  # Logic for determining if the row_type is VALUE, or if the rowtext will
  # turn this into a HEADER
  if (nested) {
    row_type_value <- "NESTED"
  } else if (!is.null(
    {{ rowtext }}
  ) &&
    !is_named({{ rowtext }}) &&
    nrow(res) == length(unique(res[[colvar]])) &&
    !("N" %in% statlist[["stats"]])) {
    row_type_value <- "HEADER"
  } else {
    row_type_value <- "VALUE"
  }


  df <- res %>%
    mutate(
      row_type = row_type_value
    )

  if (has_subset && nrow(df) == 0 && !is.null(rowtext)) {
    complete_logic <- list(0, 0, 0, rowtext, "HEADER") %>%
      setNames(c("n", "denom", "pct", rowvar, "row_type"))
    df <- df %>%
      mutate(!!sym(rowvar) := as.character(!!sym(rowvar))) %>%
      complete(!!sym(colvar), fill = complete_logic) %>%
      # Recalculate the denoms and pcts because they can be lost depending on
      # the structure of denoms_df and df
      select(-"denom") %>%
      left_join(denom, by = denoms_by) %>%
      mutate(
        pct = n / denom * 100
      )
  }

  df %>%
    mutate(
      across(
        .cols = any_of(c(rowvar, rowbyvar, tablebyvar)),
        .fns = ~ unclass(.),
        .names = "{.col}_ord"
      )
    )
}

#' Freq Workhorse
#'
#' This is where the actual counting is happening. Is generalized to take the
#' variables to count across and the name of the variable to save the count as.
#'
#' @return A dataframe with the counts of 'df' by 'by' with the name of
#'   'count_name'. Is made distinct if the statlist notes subject level counts.
#'
#' @noRd
derive_count <- function(df, by, statlist, count_name) {
  # A list object to fill the missing denom values. Here because unquoting a
  # variable to be used as a name in a list is messy
  complete_fill <- list(0)
  names(complete_fill) <- count_name

  # The logic for completing is different for nested and non-nested layers
  if (attr(df, "nested") %||% FALSE && count_name == "n") {
    # For nested layers, use 'nesting' on the by values, becuase we only want
    # the combinations that appear in the data.
    complete_logic <- exprs(
      nesting(
        !!!syms(by[(by %in% c(attr(df, "nested_level"), attr(df, "rowbyvar")))])
      ),
      !!!syms(by[!(by %in% c(attr(df, "nested_level"), attr(df, "rowbyvar")))])
    )
  } else {
    # Otherwise, get all the combinations even if they aren't present in the
    # data.
    complete_logic <- exprs(!!!syms(unname(by)))
  }

  df %>%
    # Convert variables we're counting across to factors if they aren't already
    mutate(across(.cols = all_of(unname(by)), as.factor)) %>%
    # If the data should be distinct, make it so, otherwise do nothing
    distinct_check(by, statlist) %>%
    # Group variables to count across and count them
    group_by(across(all_of(unname(by)))) %>%
    summarise(!!count_name := n()) %>%
    # Ungroup to make sure you're working with a clean dataframe
    ungroup() %>%
    # If there is any combination of factors that don't appear, make their count
    # 0 Don't complete here if its nested, logic for that is in nested_freq.R
    complete(., !!!complete_logic, fill = complete_fill)
}

#' Return a distinct dataframe based on statlist
#'
#' @return Depending on the statlist, either return the dataframe unmodified, or
#'   make the data.frame distinct based on 'distinct_by'.
#'
#' @noRd
distinct_check <- function(df, by, statlist) {
  distinct_by <- statlist$distinct_by %||% "USUBJID"

  # If there is a group count, make the count distinct. Logic taken from legacy
  # freq code.
  if (statlist[["distinct"]] %||% TRUE) {
    distinct(df, !!!syms(c(distinct_by, by)), .keep_all = TRUE)
  } else {
    df
  }
}

#' Drop values in the table based on a cutoff
#'
#' @return The table less the dropped values.
#'
#' @noRd
drop_freq <- function(df, cutoff, rowvar, cutoff_stat, colvar, drop_by) {
  # If there isn't a tablebyvar, run it through drop_values
  if (is.null(drop_by)) {
    df %>%
      drop_values(cutoff, rowvar, cutoff_stat, colvar, drop_by)
  } else {
    # If there is a tablebyvar, we need to run 'drop_values' for each value of
    # tablebyvar. So filter them one-by-one and run it through drop_values. Then
    # row_bind them together.

    drop_splits <- split(
      unique(df[, drop_by]),
      as.numeric(rownames(unique(df[, drop_by])))
    )

    map_dfr(
      drop_splits,
      function(x) {
        filter_logic <- unname(
          imap(
            x,
            ~ expr(!!sym(.y) == !!as.character(.x))
          )
        )
        df %>%
          filter(!!!filter_logic) %>%
          drop_values(cutoff, rowvar, cutoff_stat, colvar, drop_by)
      }
    )
  }
}

drop_values <- function(df, cutoff, rowvar, cutoff_stat, colvar, drop_by) {
  # If there is no cutoff, just return the dataframe
  if (is.null(cutoff)) {
    df
  } else { # If there is a cutoff:
    # If the cutoff specifies a column name:
    if (grepl(">=", cutoff)) {
      cutoff <- paste0(
        "`", cutoff |>
          stringr::str_replace_all("& ", "& `") |>
          stringr::str_replace_all("\\| ", "| `") |>
          stringr::str_replace_all(" >=", "` >=")
      )
      cutoff <- as.list(parse(text = cutoff))

      pivot_id_cols <- unname(c(drop_by, rowvar, "nested_level", "row_type"))
      kept_values <- suppressMessages(
        df |>
          pivot_wider(
            id_cols = any_of(pivot_id_cols),
            names_from = all_of(colvar),
            values_from = all_of(c(cutoff_stat)),
            values_fill = 0
          ) |>
          # Get a list of values from specified column that are >= to the cutoff
          filter(!!!cutoff) |>
          select(!!rowvar) |>
          unlist()
      )

      # If the column name doesn't specify a column name:
    } else {
      # Determine the values to keep
      kept_values <- df |>
        # Get a list of values that have a value >= the cutoff
        filter(!!sym(cutoff_stat) >= as.numeric(trimws(cutoff))) |>
        select(!!rowvar) |>
        unlist()
    }
    # Filter everything that wasn't found above.
    df %>%
      filter(.data[[rowvar]] %in% kept_values)
  }
}

#' Sort rows in long data.frame
#'
#' If no descending by is passed it defaults to sorting everything by the factor
#' alphabetically.
#'
#' @noRd
sort_freq <- function(df,
                      rowvar,
                      descending_by,
                      tablebyvar,
                      rowbyvar,
                      colvar_cols,
                      colvar) {
  if (is.null(descending_by)) {
    return(df %>% select(-any_of(paste0("n", colvar_cols))))
  }

  orig_order <- names(df)
  ord_cols <- c(rowvar, tablebyvar, rowbyvar)
  ord_cols <- paste0(ord_cols[!is.null(ord_cols)], "_ord")

  row_header_location <- which(df$row_type == "HEADER")

  if (length(row_header_location) != 0) {
    row_header_df <- df %>% slice(row_header_location)
    df <- df %>% slice(-row_header_location)
  }

  df_ord <- df %>% select(dplyr::contains(ord_cols))
  df <- df %>% select(-dplyr::contains(ord_cols))

  if ("nested_level" %in% orig_order) {
    for (i in 1:(length(rowvar) - 1)) {
      df <- df %>%
        mutate(
          across(
            .cols = "nested_level",
            function(x) x != (i - 1), .names = paste0("ntop", rowvar[i])
          )
        )
    }
  }

  df <- df %>% dplyr::arrange(label)

  if ("nested_level" %in% orig_order) {
    df <- df %>% dplyr::arrange(nested_level)
  }

  if (is_named(descending_by)) {
    df <- df %>% group_by(!!!syms(c(tablebyvar, rowbyvar)))
    for (i in rev(seq_len(length(descending_by)))) {
      sort_col <- paste0("n", names(descending_by)[[i]])
      type <- descending_by[[i]]
      if (type == "asc") {
        df <- df %>%
          mutate(
            dplyr::across(
              .cols = sort_col,
              function(x) {
                ifelse(row_type %>% stringr::str_detect("(HEADER|^N$)"), -x, x)
              }
            )
          ) %>%
          arrange(!!sym(sort_col))
      } else {
        df <- df %>% arrange(desc(!!sym(sort_col)))
      }
    }
  } else {
    df <- df %>% group_by(!!!syms(c(tablebyvar, rowbyvar)))
    for (i in rev(seq_len(length(descending_by)))) {
      df <- df %>% arrange(desc(!!sym(paste0("n", descending_by[i]))))
    }
  }

  if (length(rowvar) > 1) {
    sortvar <- c()
    for (i in 1:(length(rowvar) - 1)) {
      df <- df %>%
        mutate(
          sorter = ifelse(!(!!sym(paste0("ntop", rowvar[i]))), row_number(), NA)
        ) %>%
        group_by(
          !!!syms(c(tablebyvar, rowbyvar, rowvar[1:i]))
        ) %>%
        fill(
          sorter,
          .direction = "downup"
        ) %>%
        ungroup() %>%
        mutate(
          across(.cols = "sorter", function(x) x, .names = paste0("sort", rowvar[i]))
        ) %>%
        select(-sorter)
      sortvar <- c(
        sortvar,
        paste0("sort", rowvar[i]),
        paste0("ntop", rowvar[i])
      )
    }
    df <- df %>%
      arrange(!!!syms(sortvar)) %>%
      select(-any_of(sortvar))
  }

  df <- df %>%
    ungroup() %>%
    arrange(!!!syms(c(tablebyvar, rowbyvar))) %>%
    bind_cols(df_ord) %>%
    select(!!!syms(orig_order))

  if (length(row_header_location) != 0) {
    for (i in seq_len(nrow(row_header_df))) {
      df <- df %>%
        add_row(row_header_df %>% slice(i), .before = row_header_location[i])
    }
  }
  df |> select(-any_of(paste0("n", colvar_cols)))
}

#' Round values in a freq table
#'
#' @return Table with rounded 'pct'
#'
#' @noRd
round_freq <- function(df, decimal) {
  # format pct using roundSAS
  df |>
    mutate(pct = roundSAS(pct, digits = as.numeric(decimal), as_char = TRUE))
}

#' Arrange values from n and pct into results
#'
#' @return A table with combined results
#'
#' @noRd
arrange_freq <- function(df, statlist) {
  format_as <- list(
    n               = expr(as.character(.data$n)),
    `n (x.x)`       = expr(paste0(.data$n, " (", pct, ")")),
    `n (x.x%)`      = expr(paste0(.data$n, " (", pct, "%)")),
    `n/N`           = expr(paste0(.data$n, "/", .data$denom)),
    `n/N (x.x)`     = expr(paste0(.data$n, "/", .data$denom, " (", pct, ")")),
    `n/N (x.x%)`    = expr(paste0(.data$n, "/", .data$denom, " (", pct, "%)"))
  )
  lsformats <- statlist[["stats"]][statlist[["stats"]] != "N"]

  # Format N counts as character
  df <- df %>% mutate(N = as.character(denom))

  purrr::map_dfr(lsformats, function(format_) {
    df %>%
      mutate(
        # Evaluate the expression from format_as
        stat = format_,
        results = eval(format_as[[format_]]),
        # Replace results with '-' if n/N is 0 or NA
        results = case_when(
          .data$denom %in% c(0, NA) ~ statlist[["zero_denom"]] %||% "-",
          .data$n %in% c(0, NA) & grepl("/N", format_) ~
            str_replace(results, "\\s.*$", ""),
          .data$n %in% c(0, NA) & !grepl(format_, "/N") ~
            statlist[["zero_n"]] %||% "0",
          TRUE ~ results
        )
      )
  })
}

#' Pivot freq table
#'
#' @return The (nearly) final wide table for display
#'
#' @noRd

pivot_freq <- function(df, colvar, rowvar, rowbyvar, tablebyvar, statlist,
                       rowtext, row_header, display_missing = FALSE,
                       nested = FALSE, .keep = FALSE, .ord, ...) {
  ord_columns <- names(df)[endsWith(names(df), "_ord")]
  # The columns to output for the table. If .keep is true, add the rowbyvar
  output_columns <- c(
    "label",
    levels(df[[colvar]]),
    paste0("n", levels(df[[colvar]])),
    "row_type",
    "nested_level",
    "group_level"
  )
  if (.keep) {
    output_columns <- c(output_columns, rowbyvar, tablebyvar)
  }
  if (.ord) {
    output_columns <- c(output_columns, ord_columns)
  }

  missing_name <- "Missing"
  # Create the result dataframe and add "VALUE" as the row_type
  pivot_id_cols <- unname(
    c(
      rowbyvar, rowvar, tablebyvar, "nested_level", "row_type", ord_columns,
      "stat"
    )
  )
  res <- suppressMessages(
    df %>%
      pivot_wider(
        id_cols = any_of(pivot_id_cols),
        names_from = all_of(colvar),
        values_from = c("results"),
        values_fill = "0"
      ) %>%
      left_join(
        suppressWarnings(
          df %>% mutate(n = ifelse(row_type == "N", as.numeric(results), n))
        ) %>%
          pivot_wider(
            id_cols = any_of(pivot_id_cols),
            names_from = all_of(colvar),
            values_from = c("n"),
            values_fill = 0,
            names_prefix = "n"
          )
      ) %>%
      select(-stat) %>%
      mutate(
        # For all the na values of rowvar, rename them to missing.
        !!rowvar[1] := ifelse(is.na(!!sym(rowvar)),
          missing_name,
          as.character(!!sym(rowvar))
        )
      ) %>%
      { # drop out the missings unless display_missing is true
        if (display_missing) . else filter(., !!sym(rowvar[1]) != missing_name)
      }
  )

  if (!is.null(rowbyvar) && length(rowbyvar) == 1) {
    res <- res %>% tidyr::fill(!!sym(rowbyvar), .direction = "up")
  }
  if (nrow(res) != 0) {
    res <- res %>%
      mutate(
        across(
          .cols = paste0("n", levels(df[[colvar]])),
          function(x) {
            ifelse(
              row_type |>
                stringr::str_detect("HEADER"),
              max(x, na.rm = TRUE) + n() + 1 - row_number(), x
            )
          }
        )
      )
  }

  res <- res %>%
    rowtext_freq(
      rowtext,
      rowvar,
      rowbyvar,
      tablebyvar,
      row_header
    )
  # If your table has zero rows, just return the zero rows.
  if (nrow(res) == 0) {
    return(res)
  }

  # Bind the rowtextHeader table to add the rowtext header if it exists
  res %>%
    mutate(
      label = as.character(label),
      across(
        .cols = -any_of(c("nested_level", ord_columns)),
        .fns = ~ replace_na_with_blank(.x)
      )
    ) %>%
    add_meta_freq(df, colvar, tablebyvar) %>%
    # add on the group_level variable for indentation
    add_group_level(rowbyvar) %>%
    select(
      any_of(output_columns)
    ) %>%
    mutate(
      !!!list(...)
    )
}

#' Label rename for rowtext
#'
#' @return Either a named character vector with the name being the new label.
#'   or null if its not needed
#'
#' @noRd
rowtext_freq <- function(res, rowtext, rowvar, rowbyvar, tablebyvar,
                         row_header) {
  # If you have a rowtext, its unnamed, and there is only one row. Change
  # rowtext so it relevels the label
  res <- res %>% mutate(label = !!sym(rowvar))
  if (is.null(rowtext)) {
    return(res)
  } else if (is_named(rowtext)) {
    return(
      suppressWarnings(
        res %>%
          mutate(label = as.character(fct_recode(!!sym(rowvar), !!!rowtext)))
      )
    )
  }

  res_group <- c(rowbyvar, tablebyvar)
  len_oner <- as.numeric(!is.null(row_header)) +
    as.numeric(!is.null(rowbyvar)) + 1 # length of single row table
  res <- res %>%
    group_by_at(res_group)

  # if row text is named list it will refactor, if length one will replace the
  # single row, otherwise does nothing
  suppressWarnings(
    res <- res %>%
      mutate(label = case_when(
        length(rowtext) == 1 & len_oner == n() & row_number() == n() ~
          rowtext[1],
        TRUE ~ label
      ))
  )
  res <- res %>% ungroup()
}

#' Logic for merging in bigN and rowbyvar
#'
#' @param has_big_n boolean. Does the table have a big N in the statlist?
#'
#' @noRd
byvar_merge_freq <- function(df, tablebyvar, rowbyvar, colvar, rowvar, has_big_n,
                             row_header) {
  df |>
    big_n_merge(colvar, rowvar, rowbyvar, tablebyvar, has_big_n) |>
    row_header_merge(colvar, rowvar, rowbyvar, tablebyvar, row_header) |>
    rowbyvar_merge(colvar, rowvar, rowbyvar, tablebyvar)
}

#' Add metadata to a freq table
#'
#' This adds the 'denom' to columns for the bind_table function
#'
#' @noRd
add_meta_freq <- function(table, df, colvar, tablebyvar) {
  denom_ <- df %>%
    select(!!colvar, !!tablebyvar, denom) %>%
    filter(!is.na(denom)) %>%
    unique() %>%
    rename_with(
      ~ paste0("denom_", .),
      .cols = -all_of("denom")
    )

  attr(table, "denom") <- denom_

  table
}

#' Merge in bigN rows
#' @noRd
big_n_merge <- function(df, colvar, rowvar, rowbyvar, tablebyvar, has_big_n) {
  if (has_big_n) {
    df %>%
      var_merge(
        nestby = c(colvar, rowbyvar, tablebyvar),
        rowvar = rowvar,
        rowvar_name = "N",
        result_value = as.character(data_nest[["N"]][1]),
        row_type_value = "N"
      )
  } else {
    df
  }
}

row_header_merge <- function(df, colvar, rowvar, rowbyvar, tablebyvar,
                             row_header) {
  if (is.null(row_header)) {
    df
  } else {
    df %>%
      var_merge(
        nestby = c(colvar, rowbyvar, tablebyvar),
        rowvar = rowvar,
        rowvar_name = row_header,
        result_value = "",
        row_type_value = "HEADER"
      )
  }
}

#' Merge in rowbyvar rows
#'
#' Split up for logic when there are 0, 1, and 2 rowbyvars
#'
#' @noRd
rowbyvar_merge <- function(df, colvar, rowvar, rowbyvar, tablebyvar) {
  if (is.null(rowbyvar)) {
    df
  } else if (length(rowbyvar) == 1) {
    df %>%
      var_merge(
        nestby = c(colvar, rowbyvar, tablebyvar),
        rowvar = rowvar,
        rowvar_name = sym(rowbyvar),
        result_value = "",
        row_type_value = "BY_HEADER1"
      )
  } else {
    df %>%
      var_merge(
        nestby = c(colvar, rowbyvar[1], tablebyvar),
        rowvar = rowvar,
        rowvar_name = sym(rowbyvar[1]),
        result_value = "",
        row_type_value = "BY_HEADER1"
      ) %>%
      var_merge(
        nestby = c(colvar, rowbyvar, tablebyvar),
        rowvar = rowvar,
        rowvar_name = sym(rowbyvar[2]),
        result_value = "",
        row_type_value = "BY_HEADER2"
      )
  }
}

#' Workhorse for merging in a freq table
#' @noRd
var_merge <- function(df, nestby, rowvar, rowvar_name, result_value,
                      row_type_value) {
  result_value <- enexpr(result_value)
  ord_values_all <- names(df)[endsWith(names(df), "_ord")]
  ord_in_nest <- map_chr(str_split(ord_values_all, "_"), `[[`, 1) %in% nestby
  ord_values <- ord_values_all[ord_in_nest]
  ord_values_fill <- ord_values_all[!ord_in_nest]
  ord_values_fill <- setNames(
    as.list(rep(0, length(ord_values_fill))),
    ord_values_fill
  )

  a_nest <- c(nestby, ord_values)


  df <- df %>%
    nest(data_nest = -any_of(a_nest)) %>%
    rowwise() %>%
    mutate(
      data_nest = list(
        tibble(!!rowvar := !!rowvar_name,
          results = !!result_value,
          row_type = row_type_value,
          !!!ord_values_fill
        ) %>%
          bind_rows(data_nest)
      )
    ) %>%
    unnest(data_nest) %>%
    ungroup()
  df
}

library(tibble)

test_that("pad argument works as expected", {
  mtcars2 <- mtcars %>%
    rownames_to_column(var = "USUBJID") %>%
    mutate(cyl = as.factor(cyl))

  mtcars2 <- mtcars2 %>% filter(cyl != 8)

  t1 <- freq(mtcars2, colvar = "gear", rowvar = "cyl")
  t2 <- freq(mtcars2, colvar = "gear", rowvar = "cyl", pad = FALSE)
  t3 <- freq(mtcars2, colvar = "gear", rowvar = "cyl", rowbyvar = "am", pad = FALSE)

  expected <- tibble::tribble(
    ~label,
    ~X3,
    ~X4,
    ~X5,
    ~row_type,
    ~group_level,
    "4",
    "1 (33.3)",
    "8 (66.7)",
    "2 (66.7)",
    "VALUE",
    0,
    "6",
    "2 (66.7)",
    "4 (33.3)",
    "1 (33.3)",
    "VALUE",
    0,
    "8",
    "0",
    "0",
    "0",
    "VALUE",
    0
  )

  expect_equal(t1, expected, ignore_attr = TRUE)

  expected <- tibble::tribble(
    ~label,
    ~X3,
    ~X4,
    ~X5,
    ~row_type,
    ~group_level,
    "4",
    "1 (33.3)",
    "8 (66.7)",
    "2 (66.7)",
    "VALUE",
    0,
    "6",
    "2 (66.7)",
    "4 (33.3)",
    "1 (33.3)",
    "VALUE",
    0
  )

  expect_equal(t2, expected, ignore_attr = TRUE)

  expected <- tibble::tribble(
    ~label,
    ~X3,
    ~X4,
    ~X5,
    ~row_type,
    ~group_level,
    ~am,
    "0",
    "",
    "",
    "",
    "BY_HEADER1",
    0,
    "0",
    "4",
    "1 (33.3)",
    "2 (50.0)",
    "-",
    "VALUE",
    0,
    "0",
    "6",
    "2 (66.7)",
    "2 (50.0)",
    "-",
    "VALUE",
    0,
    "0",
    "1",
    "",
    "",
    "",
    "BY_HEADER1",
    0,
    "1",
    "4",
    "-",
    "6 (75.0)",
    "2 (66.7)",
    "VALUE",
    0,
    "1",
    "6",
    "-",
    "2 (25.0)",
    "1 (33.3)",
    "VALUE",
    0,
    "1"
  )

  expected$am <- factor(expected$am)

  expect_equal(t3, expected, ignore_attr = TRUE)
})

#' SAS rounding in R
#'
#' `roundSAS` is an alternative rounding function, ensuring that decimals equal or
#' bigger than 5 are rounded upwards to the nearest number and returned as
#' character vector.
#'
#' At the midpoint of a decimal place (e.g. 0.5, 1.5), the round function in R
#' rounds to the nearest even number (i.e. 0.5 is rounded to 0; 1.5 is rounded
#' to 2), whereas SAS rounds to the nearest number (i.e. 0.5 is rounded to 1;
#' 1.5 is rounded to 2). The `roundSAS` function is an alternative rounding
#' function for R that ensures rounding to the nearest number, as done in SAS.
#' `roundSAS` comes from this Stack Overflow
#' [post](https://stackoverflow.com/questions/12688717/round-up-from-5).
#'
#' @param x Numeric vector.
#' @param digits An integer specifying the number of decimal places to be
#'   displayed after rounding. Default is 0.
#' @param as_char logical value indicating conversion of rounded numerical
#'   vector to character vector; default is `FALSE`.
#' @param na_char A character string indicating missing value; if not specified,
#'   `"NA"` is created.
#'
#' @return character vector of rounded values
#' @export
#'
#' @examples
#' ### input data vector with midpoint decimals
#' x <- c(-2.5, -1.5, -0.5, 0.5, 1.5, 2.5)
#'
#' ### rounds to integer
#' roundSAS(x, digits = 0)
#'
#' ### input data vector with a missing value
#' y <- c(8.65, 8.75, NA, 9.85, 9.95)
#'
#' ### rounds to tenths and label the missing value with "NE"
#' roundSAS(y, digits = 1, as_char = TRUE, na_char = "NE")
#'
roundSAS <- function(x, # nolint object_name_linter
                     digits = 0,
                     as_char = FALSE,
                     na_char = NULL) {
  # perform SAS rounding ----------------------------------------------------
  posneg <- sign(x)
  z <- abs(x) * 10^digits
  z <- z + 0.5 + sqrt(.Machine$double.eps)
  z <- trunc(z)
  z <- (z / 10^digits)
  z <- ifelse(!is.na(z) & z > 0, z * posneg, z)

  # output rounded values ---------------------------------------------------
  if (as_char && is.null(na_char)) {
    ## convert rounded values to character vector
    formatC(z, format = "f", digits = digits)
  } else if (as_char && !is.null(na_char)) {
    ## convert to character vector and use na_char for missing value
    formatC(z, format = "f", digits = digits) %>%
      str_replace(" *(NA|NaN|NULL)", na_char)
  } else {
    ## return numeric vector of rounded values
    z
  }
}

#' @title roundSAS() requirements
#' @section Last updated by:
#' Sheng-Wei Wang
#' @section Last update date:
#' 13-APR-2021
#' @section Category:
#' Generate Results

library(dplyr)

# Requirements --------------------------------------------------------------------------------
#' T1. The function produces expected decimal rounding ...
#' T1.1 for rounding up midpoint values
#' T1.2 for rounding up midpoint values and convert to character
#' T2. The function creates expected missing value label ...
#' T2.1 when converting to character with missing values
#' T2.2 when converting to character and na_char is specified

test_that("T1.1 for rounding up midpoint values", {
  x <- c(-2.5, -1.5, -0.5, 0.5, 1.5, 2.5)

  expect_equal(roundSAS(x, digits = 0), c(-3, -2, -1, 1, 2, 3))
})

test_that("T1.2 for rounding negative values to 0", {
  x <- c(-2.5, -1.5, -0.0000000000000000005, 0.5, 1.5, 2.5)

  expect_equal(roundSAS(x, digits = 0), c(-3, -2, 0, 1, 2, 3))
})

test_that("T1.2 for rounding up midpoint values and convert to character", {
  x <- c(-2.5, -1.5, -0.5, 0.5, 1.5, 2.5)

  expect_equal(roundSAS(x, digits = 0, as_char = TRUE), c("-3", "-2", "-1", "1", "2", "3"))
})

test_that("T2.1 when converting to character with missing values", {
  y <- c(8.65, 8.75, NA, 9.85, 9.95)

  expect_equal(roundSAS(y, digits = 1, as_char = TRUE), c("8.7", "8.8", "NA", "9.9", "10.0"))
})

test_that("T2.2 when converting to character and na_char is specified", {
  y <- c(8.65, 8.75, NA, 9.85, 9.95)

  expect_equal(roundSAS(y, digits = 1, as_char = TRUE, na_char = "NE"), c("8.7", "8.8", "NE", "9.9", "10.0"))
})

test_that("T3.1 when converting to character and value is negative but rounds to 0", {
  y <- c(8.65, 8.75, -0.0000000000000000005, 9.85, 9.95)

  expect_equal(roundSAS(y, digits = 1, as_char = TRUE), c("8.7", "8.8", "0.0", "9.9", "10.0"))
})

#' @title getFileName
#'
#' @description This function returns the file path up until the program's
#'   name.
#'
#' @author Steven Haesendonckx, \email{shaesen2@@its.jnj.com}
#'
#' @return The file path up until the program's name.
#' @export
#' @examples
#' get_file_name()
get_file_name <- function() {
  ##### If run from MCD BATCH\
  file_name <- get_file_name_batch(commandArgs(trailingOnly = FALSE))
  if (!is.null(file_name)) {
    return(file_name)
  }

  #### If Run from Source
  ret <- base::sys.frame(1)$ofile # gets file path of sourced file

  if (!is.null(ret)) {
    return(stringr::str_replace(ret, "~", base::path.expand("~")))
  }

  ### If Run from rstudio server console
  if (rstudioapi::isAvailable()) {
    # Gets file path of current active Rstuido editor
    ret <- rstudioapi::getSourceEditorContext()$path
    if (!is.null(ret)) {
      return(stringr::str_replace(ret, "~", base::path.expand("~")))
    }
  }

  NULL
}

#' @noRd
get_file_name_batch <- function(cmd_args) {
  res <- NULL

  # Update to use logrx over timber
  timber_needle <- "logrx::axecute"
  timber_match <- base::grep(timber_needle, cmd_args)
  file_needle <- "--file="
  file_match <- base::grep(file_needle, cmd_args)


  if (any(timber_match)) {
    file_arg <- stringr::str_extract(
      cmd_args[timber_match],
      'file~\\+~\\=~\\+~"(.+?)"'
    )
    if (!is.na(file_arg)) {
      file_arg2 <- stringr::str_extract(file_arg, '"(.+?)"')
      res <- normalizePath(gsub("\"", "", file_arg2))
    } else {
      try(
        {
          res <- rlang::call_args(str2lang(cmd_args[timber_match]))$file
        },
        silent = TRUE
      )
    }
  } else if (any(file_match)) {
    res <- normalizePath(sub(file_needle, "", cmd_args[file_match]))
  }

  res
}

test_that("getFileNameBatch returns the expected results", {
  cmd_args1 <- c(
    "--arg1",
    "someThing",
    "-e",
    paste0(
      'logrx::axecute(file~+~=~+~"/asdf/programs/CopyOftest_demo.R",log_name',
      '="CopyOftest_demo.R.log",log_path="/wrkdir/output")'
    )
  )
  cmd_args2 <- c(
    "--arg1",
    "someThing",
    "-e",
    'logrx::axecute(file="/asdf/programs/CopyOftest_demo.R",log_name="CopyOftest_demo.R.log",log_path="/wrkdir/output")'
  )
  cmd_args3 <- c("--arg1", "someThing", "--file=/asdf/programs/CopyOftest_demo.R")

  skip_on_os(c("windows", "mac", "solaris"))

  expect_equal(suppressWarnings(get_file_name_batch(cmd_args1)), "/asdf/programs/CopyOftest_demo.R")

  expect_equal(suppressWarnings(get_file_name_batch(cmd_args2)), "/asdf/programs/CopyOftest_demo.R")

  expect_equal(suppressWarnings(get_file_name_batch(cmd_args3)), "/asdf/programs/CopyOftest_demo.R")
})

gentlg_single <- function(huxme = NULL,
                          tlf = "Table",
                          format = "rtf",
                          colspan = NULL,
                          idvars = NULL,
                          plotnames = NULL,
                          plotwidth = NULL,
                          plotheight = NULL,
                          wcol = 0.45,
                          orientation = "portrait",
                          opath = ".",
                          title_file = NULL,
                          file = NULL,
                          title = NULL,
                          footers = NULL,
                          print.hux = TRUE,
                          watermark = NULL,
                          colheader = NULL,
                          pagenum = FALSE,
                          bottom_borders = NULL,
                          border_fns = list(),
                          alignments = list(),
                          index_in_result = 1,
                          footers_one_row = FALSE) {
  assertthat::is.count(index_in_result)
  # check all the arguments being passed in except ...
  assertthat::assert_that(
    is.list(border_fns),
    all(vapply(border_fns, is.function, FUN.VALUE = logical(1)))
  )
  assertthat::assert_that(is.matrix(bottom_borders) || bottom_borders == "old_format")
  arglist <- list()
  args_to_chk <- names(formals())[names(formals()) != "..."]
  purrr::walk(args_to_chk, .f = {
    function(x) arglist[[x]] <<- eval(rlang::sym(x))
  })
  check_gentlg(arglist)
  if (identical(footers, NA) || length(footers) == 0) {
    footers <- NULL
  }

  if (!is.null(title_file)) {
    title_df <- readxl::read_excel(title_file,
      sheet = 1,
      range = readxl::cell_cols("A:C")
    )
    if (!all(c("TABLE ID", "IDENTIFIER", "TEXT") %in% names(title_df))) {
      stop("'title_file' file must have columns: 'TABLE ID',
           'IDENTIFIER', and 'TEXT'")
    }
    title_df <- title_df |>
      filter(`TABLE ID` == file)

    if (is.null(title) && nrow(title_df) > 0) {
      title <- title_df |>
        filter(str_detect(IDENTIFIER, regex("title", ignore_case = TRUE))) |>
        extract2("TEXT")
      if (length(title) == 0) {
        title <- NULL
      }
    }
    if (is.null(footers) && nrow(title_df) > 0) {
      footers <- title_df |>
        filter(str_detect(
          IDENTIFIER,
          regex("^footnote", ignore_case = TRUE)
        )) |>
        extract2("TEXT")
      if (length(footers) == 0) {
        footers <- NULL
      }
    }
  }

  if (!is.null(attr(huxme, "column_metadata")) && is.null(colspan)) {
    cm <- attr(huxme, "column_metadata")
    cm <- cm[, colSums(is.na(cm)) < nrow(cm)]
    if (any(str_detect(names(cm), "span"))) {
      colspan <- cm |>
        select(starts_with("span")) |>
        as.list() |>
        unname()
      colspan <- colspan[!purrr::map_lgl(colspan, ~ all(is.na(.)))]
      colspan <- purrr::map(colspan, ~ c("", replace_na_with_blank(.))) |>
        rev()
    }
  }
  if (!is.null(attr(huxme, "column_metadata")) && is.null(colheader)) {
    cm <- attr(huxme, "column_metadata")
    if ("decode" %in% names(cm)) {
      colheader <- cm |>
        select(decode) |>
        unlist() |>
        unname()
      colheader <- c("", colheader)
      names(colheader) <- c("label", paste0("col", seq_len(nrow(cm))))
    }
  }

  if (is.null(colheader)) {
    if (is_listing(tlf)) {
      null_labels <- purrr::map_lgl(map(huxme, ~ attr(., "label")), is.null)
      colheader <- names(huxme)
      colheader[!null_labels] <-
        unname(map_chr(huxme[!null_labels], ~ attr(., "label")))
    } else {
      colheader <- c("", names(huxme)[-1])
    }
  }

  wrote_png <- FALSE
  if (is_graph(
    tlf
  ) &&
    is.null(
      plotnames
    ) &&
    inherits(huxme, "ggplot")) {
    a_file <- "__tempggplot__"
    tmpdir <- base::tempdir()
    ggsave(
      filename = paste0(tmpdir, "/__tempggplot__"),
      plot = huxme,
      device = "png",
      width = plotwidth %||% NA,
      height = plotheight %||% NA
    )
    huxme <- NULL
    plotnames <- a_file
    wrote_png <- TRUE
  }

  if (is_graph(
    tlf
  ) &&
    is.null(
      plotwidth
    ) &&
    is.null(plotheight)) {
    png_d <- dim(readPNG(ifelse(
      wrote_png,
      paste0(tmpdir, "/__tempggplot__"), plotnames
    )))
    plotheight <- png_d[1]
    plotwidth <- png_d[2]
  }


  huxme <- data.frame(huxme, stringsAsFactors = FALSE, check.names = FALSE)

  # Check if the huxme is blank, this will error out in R < 4
  if (ncol(huxme) > 0) {
    huxme <- huxme |>
      select(!c(any_of("func"), ends_with("_ord")))
  }

  ###########################
  ### Validation of input ###
  ###########################

  formatcolumns <- c(
    "anbr", "roworder", "boldme", "indentme", "newrows", "newpage",
    "rowvar", "row_type", "nested_level", "group_level"
  )

  # exploratory
  # restrict to only display label and variables that start with col followed by numbers
  # Pro:
  #   This allows users to keep whatever working variables they like and not worry about them
  #   displaying in the output.
  #   We can drop the byvars argument.
  # Con:
  #   You have to rename columns to label or col1-coln


  if (is.null(huxme) && !is_graph(tlf)) {
    stop("huxme argument is required for tables and listings. Consult the documentation via ?gentlg.")
  }

  if (!("label" %in% colnames(huxme)) && is_table(tlf)) {
    stop("No column present with the name 'label'. Consult the documentation via ?gentlg")
  }

  if (sum(!(colnames(huxme) %in% c(formatcolumns, "label"))) < 1 && !is_graph(tlf)) {
    stop("A column with summary measures are missing. Consult the documentation via ?gentlg.")
  }

  if (!substr(tolower(tlf), 1, 1) %in% c("t", "l", "f", "g")) {
    stop("tlf argument not correctly specified. Consult the documentation via ?gentlg.")
  }

  if (is_graph(tlf) && is.null(plotnames)) {
    stop("plotnames argument not used. Consult the documentation via ?gentlg.")
  }

  if (!is.null(wcol) && !is_graph(tlf)) {
    if (
      (length(wcol) == 1 && (!is.numeric(wcol) || wcol > 1)) ||
        (length(wcol) > 1 && is_format_rtf(format) &&
          (
            !is.numeric(wcol) ||
              sum(!(colnames(huxme) %in% c(formatcolumns))) != length(wcol) ||
              sum(wcol) > 1
          ))
    ) {
      stop("wcol not defined properly. Consult the documentation via ?gentlg.")
    }
    if (is_format_html(format) && length(wcol) > 1) {
      warning(
        paste0(
          "Format = 'HTML'. Only the first argument of wcol will be used. ",
          "For more information, consult the documentation via ?gentlg."
        )
      )
    }
  }

  if (
    !is.null(colspan) &&
      substr(tolower(tlf), 1, 1) %in% c("t", "l") &&
      sum(lapply(colspan, length) == (sum(!(colnames(huxme) %in% c(formatcolumns))))) != length(colspan)
  ) {
    stop(
      paste0(
        "The length of each colspan argument needs to be equal to the amount of columns ",
        "in the output data frame. Consult the documentation via ?gentlg."
      )
    )
  }

  if (
    substr(tolower(tlf), 1, 1) %in% "l" && !is.na(idvars[1]) &&
      sum(!idvars %in% colnames(huxme)) > 0
  ) {
    stop("One of the idvars is not part of the column headers. Consult the documentation via ?gentlg.")
  }

  if (!substr(tolower(tlf), 1, 1) %in% "l" && !is.null(idvars)) {
    stop("idvars argument is not available for non-listing outputs. Consult the documentation via ?gentlg.")
  }

  ###########################
  ### Formatting values   ###
  ###########################

  if ("newrows" %in% colnames(huxme)) {
    huxme <- insert_empty_rows(huxme)
  }
  if ("boldme" %in% colnames(huxme) && length(which(huxme$boldme == 1)) > 0) {
    boldme <- which(huxme$boldme == 1)
  } else {
    boldme <- NULL
  }
  if (
    "indentme" %in% colnames(huxme) &&
      length(which(huxme$indentme == 0)) > 0
  ) {
    indentme0 <- which(huxme$indentme == 0)
  } else {
    indentme0 <- NULL
  }
  if (
    "indentme" %in% colnames(huxme) &&
      length(which(huxme$indentme == 1)) > 0
  ) {
    indentme1 <- which(huxme$indentme == 1)
  } else {
    indentme1 <- NULL
  }
  if (
    "indentme" %in% colnames(huxme) &&
      length(which(huxme$indentme == 2)) > 0
  ) {
    indentme2 <- which(huxme$indentme == 2)
  } else {
    indentme2 <- NULL
  }
  if (
    "indentme" %in% colnames(huxme) &&
      length(which(huxme$indentme == 3)) > 0
  ) {
    indentme3 <- which(huxme$indentme == 3)
  } else {
    indentme3 <- NULL
  }
  if (
    "indentme" %in% colnames(huxme) &&
      length(which(huxme$indentme == 4)) > 0
  ) {
    indentme4 <- which(huxme$indentme == 4)
  } else {
    indentme4 <- NULL
  }
  if (
    "indentme" %in% colnames(huxme) &&
      length(which(huxme$indentme == 5)) > 0
  ) {
    indentme5 <- which(huxme$indentme == 5)
  } else {
    indentme5 <- NULL
  }
  if (
    "indentme" %in% colnames(huxme) &&
      length(which(huxme$indentme == 6)) > 0
  ) {
    indentme6 <- which(huxme$indentme == 6)
  } else {
    indentme6 <- NULL
  }
  if (
    "newpage" %in% colnames(huxme) &&
      length(which(huxme$newpage == 1)) > 0
  ) {
    newpage <- which(huxme$newpage == 1)
  } else {
    newpage <- NULL
  }

  # display columns columns only
  if (sum(colnames(huxme) %in% formatcolumns) != 0) {
    huxme[, colnames(huxme) %in% formatcolumns] <- NULL
  }

  if (substr(tolower(tlf), 1, 1) == "t") {
    huxme <- huxme[, c(
      which(colnames(huxme) == "label"),
      which(!(colnames(huxme) %in% "label"))
    )]
  }

  #############################
  ###       Translate       ###
  #############################

  .to_html <- function(df) {
    list_df <- df |>
      as.list() |>
      lapply(gsub,
        pattern = "(\\{\\\\super)(.*?)(\\})",
        replacement = "<sup>\\2</sup>"
      ) |>
      lapply(gsub,
        pattern = "(\\{\\\\sub)(.*?)(\\})",
        replacement = "<sub>\\2</sub>"
      ) |>
      lapply(gsub, pattern = "\\\\n", replacement = "<br/>") |>
      lapply(gsub, pattern = "\\\\line", replacement = "<br/>")

    if (base::inherits(df, "data.frame")) {
      df <- list_df |>
        data.frame(stringsAsFactors = FALSE, check.names = FALSE)
    } else if (base::inherits(df, "character") && !is.null(df)) {
      df <- list_df |>
        unlist(use.names = FALSE)
    }

    df
  }

  if (is_format_html(format)) {
    huxme <- .to_html(huxme)
    if (!is.null(huxme) && dim(huxme)[1] > 0) huxme <- .to_html(huxme)
    if (!is.null(footers)) footers <- .to_html(footers)
    if (!is.null(colspan)) colspan <- .to_html(colspan)
    if (!is.null(title)) title <- .to_html(title)
    if (!is.null(colheader)) colheader <- .to_html(colheader)
    if (!is.null(idvars)) idvars <- .to_html(idvars)
  }

  #############################
  ###       Plotit!         ###
  #############################

  if (!is.null(plotnames) && tolower(substr(tlf, 1, 1)) %in% c("g", "f")) {
    pngrtfcode <- function(file = NULL, width = width, height = height) {
      # return a hexadecimal version of a file
      max_bytes <- 100000000 # maximum file size in bytes (~100MB)
      hexcode <- readBin(file,
        what = "raw", size = 1, signed = TRUE,
        endian = "little", n = max_bytes
      )
      paste0(
        "\\qc{\\pict\\pngblip\\picwgoal", round(width * 1440, 1),
        "\\pichgoal", round(height * 1440, 1), " ",
        paste0(hexcode, collapse = ""), "}"
      )
    }

    huxme <- data.frame(cbind(col1 = seq_along(plotnames)))

    for (i in seq_along(plotnames)) {
      if (is_format_rtf(format)) {
        huxme[i, ] <- paste0(
          "",
          pngrtfcode(
            ifelse(wrote_png, paste0(tmpdir, "/", plotnames[i]), plotnames[i]),
            width = plotwidth, height = plotheight
          ), "\\line"
        )
      } else {
        huxme[i, ] <- paste0(
          "<center>", "<img src=\"",
          ifelse(
            wrote_png,
            paste0(tmpdir, "/", plotnames[i]),
            plotnames[i]
          ),
          "\"", " alt=\"",
          file,
          "\" style=\"width:",
          round(plotwidth * 96 / 2.54, 0),
          "px; height:",
          round(plotheight * 96 / 2.54, 0), "px; \"> <br/>", "</center>"
        )
      }
    }
  }

  #############################
  ###       idvars          ###
  #############################

  if (tolower(substr(tlf, 1, 1)) %in% c("l")) {
    if (is.null(idvars)) {
      idvars <- colnames(huxme)
    } else if (!is.na(idvars[1])) {
      if (sum(idvars != colnames(huxme)[which(idvars %in% colnames(huxme))]) > 0) {
        idvars <- idvars[order(match(idvars, colnames(huxme)[which(idvars %in% colnames(huxme))]))]
      }
    }

    if (!is.na(idvars[1]) && dim(huxme)[1] >= 1) {
      for (i in rev(seq_along(idvars))) {
        less <- idvars[1:i]
        huxme[duplicated(huxme[, less]), less] <- ""
      }
    } else if (is.na(idvars[1]) || dim(huxme)[1] < 1) {
      huxme <- huxme
    }
  }

  #############################
  ###         No data?      ###
  #############################

  ### Empty dataset: If there is no data then display "No data to report"
  if (!(tolower(substr(tlf, 1, 1)) %in% c("f", "g"))) {
    if (dim(huxme)[1] < 1) {
      col_names <- colnames(huxme)
      huxme <- data.frame(lapply(huxme, as.character), stringsAsFactors = FALSE)
      huxme[nrow(huxme) + 1, 1] <- "No data to report"
      colnames(huxme) <- col_names
    }
  }

  #############################
  ### Orientation of output ###
  #############################
  if (is_format_rtf(format)) {
    if (tolower(orientation) == "portrait") {
      huxwidth <- 1.06352
    } else if (tolower(orientation) == "landscape") {
      huxwidth <- 1.47
    } else {
      stop("orientation can be only portrait or landscape")
    }
  } else if (is_format_html(format)) {
    if (tolower(orientation) == "portrait") {
      huxwidth <- 0.65 # 0.5
    } else if (tolower(orientation) == "landscape") {
      huxwidth <- 1 # 0.8
    } else {
      stop("orientation can be only portrait or landscape")
    }
  }

  #############################
  ###       Huxit!          ###
  #############################

  if (is_format_rtf(format)) {
    if (tolower(substr(tlf, 1, 1)) %in% c("t")) {
      ht <- huxtable::as_hux(huxme, add_colnames = TRUE) |>
        huxtable::set_width(value = huxwidth)
      if (ncol(ht) == length(colheader)) {
        ht[1, ] <- colheader
      } else {
        usethis::ui_warn(
          "Column header not used; {length(colheader)} column header provided, but data contain {ncol(ht)} columns"
        )
      }
      ht[1, 1] <- replace_lead_whitespaces_ind(ht[1, 1])
      ht[1, ] <- paste0("\\keepn\\trhdr ", ht[1, ]) # Make repeated treatments on each page
      formatindex <- 1
    } else if (tolower(substr(tlf, 1, 1)) %in% c("l")) {
      ht <- huxtable::as_hux(huxme, add_colnames = TRUE) |>
        huxtable::set_width(value = huxwidth)
      ht[1, ] <- colheader
      ht[1, 1] <- replace_lead_whitespaces_ind(ht[1, 1])
      ht[1, ] <- paste0("\\keepn\\trhdr ", ht[1, ]) # Make repeated treatments on each page
      formatindex <- 1
    } else if (tolower(substr(tlf, 1, 1)) %in% c("f", "g")) {
      ht <- huxtable::as_hux(huxme, add_colnames = FALSE) |>
        huxtable::set_width(value = huxwidth)
    } else {
      stop("tlf can have following character values: Table, Listing, Graph, Figure")
    }
  } else if (is_format_html(format)) {
    if (tolower(substr(tlf, 1, 1)) %in% c("t")) {
      ht <- huxtable::as_hux(huxme, add_colnames = TRUE) |>
        huxtable::set_width(value = huxwidth)
      if (ncol(ht) == length(colheader)) {
        ht[1, ] <- colheader
      } else {
        usethis::ui_warn(
          "Column header not used; {length(colheader)} column header provided, but data contain {ncol(ht)} columns"
        )
      }

      formatindex <- 1
    } else if (tolower(substr(tlf, 1, 1)) %in% c("l")) {
      ht <- huxtable::as_hux(huxme, add_colnames = TRUE) |>
        huxtable::set_width(value = huxwidth)
      ht[1, ] <- colheader
      formatindex <- 1
    } else if (tolower(substr(tlf, 1, 1)) %in% c("f", "g")) {
      ht <- huxtable::as_hux(huxme, add_colnames = FALSE) |>
        huxtable::set_width(value = huxwidth)
    } else {
      stop("tlf can have following character values: Table, Listing, Graph, Figure")
    }
  }

  ht <- ht |>
    huxtable::set_right_padding(value = getOption("tidytlg.right.padding")) |>
    huxtable::set_left_padding(value = getOption("tidytlg.left.padding"))

  #############################
  ###       Column Width    ###
  #############################

  if (is_format_rtf(format)) {
    if (length(wcol) == 1) {
      huxtable::col_width(ht) <- c(wcol, rep(
        (1 - wcol) / (ncol(ht) - 1),
        ncol(ht) - 1
      ))
    } else if (length(wcol) > 1) {
      huxtable::col_width(ht) <- wcol
    }
  } else if (is_format_html(format)) {
    if (!(tolower(substr(tlf, 1, 1)) %in% c("f", "g"))) {
      huxtable::col_width(ht) <- c(wcol[1], rep(
        (1 - wcol[1]) / (ncol(ht) - 1),
        ncol(ht) - 1
      ))
    }
  }

  #############################
  ###       Colspan         ###
  #############################

  if (!is.null(colspan) && !(tolower(substr(tlf, 1, 1)) %in% c("f", "g"))) {
    cnames <- colnames(ht)

    ### add row one by one to maintain huxtable structure
    if (is_format_rtf(format)) {
      for (i in rev(seq_along(colspan))) {
        colspan[[i]][1] <- replace_lead_whitespaces_ind(colspan[[i]][1])
        ht <- huxtable::insert_row(ht, paste0("\\keepn\\trhdr ", colspan[[i]]),
          after = 0, fill = ""
        ) |>
          huxtable::set_number_format(row = 1, col = seq_len(ncol(ht)), value = NA)
      }
    } else if (is_format_html(format)) {
      ### add row one by one to maintain huxtable structure
      for (i in rev(seq_along(colspan))) {
        ht <- huxtable::insert_row(ht, paste0(colspan[[i]]), after = 0)
      }
    }

    rle_colspan <- lapply(colspan, rle)

    for (j in seq_along(colspan)) {
      new <- data.frame(
        colspan = rep(
          rle_colspan[[j]]$values,
          rle_colspan[[j]]$lengths
        ),
        rle = rep(
          rle_colspan[[j]]$lengths,
          rle_colspan[[j]]$lengths
        )
      )
      ### remove only adjacent duplicates in cases such as A A _ A A
      new <- new[with(new, c(colspan[-1] != colspan[-nrow(new)], TRUE)), ]
      ### merge only for those with occurence > 1
      new <- new[which(new$rle > 1), ]

      if (dim(new)[1] != 0) {
        new$end <- as.numeric(row.names(new))
        new$start <- new$end + 1 - new$rle
        for (k in seq_len(dim(new)[1])) {
          ht <- huxtable::merge_cells(ht, j, new$start[k]:new$end[k])
        }
      }
    }

    colnames(ht) <- cnames
    formatindex <- formatindex + length(colspan)
  }

  #############################
  ###       Formatting      ###
  #############################

  ### font size
  if ((tolower(substr(tlf, 1, 1)) %in% "l")) {
    ht <- ht |>
      huxtable::set_font_size(value = getOption("tidytlg.fontsize.listing"))
  } else {
    ht <- ht |>
      huxtable::set_font_size(value = getOption("tidytlg.fontsize.table"))
  }

  if (!is_graph(tlf)) {
    if (is_format_rtf(format)) {
      ### bolding
      if (!is.null(boldme) && tolower(substr(tlf, 1, 1)) %in% c("t")) {
        ht <- huxtable::set_bold(ht, boldme + formatindex, "label",
          value = TRUE
        )
      }

      ### newpage
      if (!is.null(newpage)) {
        newpage <- newpage[order(newpage)]
        if (tolower(substr(tlf, 1, 1)) %in% c("l", "t")) {
          newpage <- (newpage + formatindex)
        }
        for (i in seq_along(newpage)) {
          ht[newpage[i], 1] <- paste0("\\pagebb ", ht[newpage[i], 1])
        }
      }
    } else if (is_format_html(format)) {
      ### bolding
      if (!is.null(boldme) && tolower(substr(tlf, 1, 1)) %in% c("t")) {
        huxtable::bold(ht[boldme + formatindex, "label"]) <- TRUE
      } else {
        # Sometimes gives warnings if bold me is null
        suppressWarnings(huxtable::bold(ht[boldme + formatindex, ]) <- TRUE)
      }
    }
  }

  ### Indentation
  #  The indent is set by (180/1440 = 0.125in) time the value of the indent variable,
  #  plus an additional hanging indent of (87/1440=.06in).
  #  (from LSAF macros)

  # make these options too?
  space <- 180
  hang <- 87
  px <- 17.76

  if (!is.null(indentme0)) {
    indentme0 <- indentme0[order(indentme0)]
    if (tolower(substr(tlf, 1, 1)) %in% c("l", "t")) {
      indentme0 <- (indentme0 + formatindex)
    }
    if (is_format_rtf(format)) {
      for (i in seq_along(indentme0)) {
        ht[indentme0[i], 1] <- glue::glue("\\intbl\\li{hang}\\fi-{hang} {ht[indentme0[i], 1]}")
      }
    } else if (is_format_html(format)) {
      for (i in seq_along(indentme0)) {
        ht[indentme0[i], 1] <- glue::glue(
          "<div style='text-indent: -{px}px; padding-left: {px}px'> {ht[indentme0[i], 1]}"
        )
      }
    }
  }

  if (!is.null(indentme1)) {
    indentme1 <- indentme1[order(indentme1)]
    if (tolower(substr(tlf, 1, 1)) %in% c("l", "t")) {
      indentme1 <- (indentme1 + formatindex)
    }
    if (is_format_rtf(format)) {
      for (i in seq_along(indentme1)) {
        ht[indentme1[i], 1] <- glue::glue("\\intbl\\li{space+hang}\\fi-{hang} {ht[indentme1[i], 1]}")
      }
    } else if (is_format_html(format)) {
      for (i in seq_along(indentme1)) {
        ht[indentme1[i], 1] <- glue::glue(
          "<div style='text-indent: -{px}px; padding-left: {px*2}px'> {ht[indentme1[i], 1]}"
        )
      }
    }
  }

  if (!is.null(indentme2)) {
    indentme2 <- indentme2[order(indentme2)]
    if (tolower(substr(tlf, 1, 1)) %in% c("l", "t")) {
      indentme2 <- (indentme2 + formatindex)
    }
    if (is_format_rtf(format)) {
      for (i in seq_along(indentme2)) {
        ht[indentme2[i], 1] <- glue::glue("\\intbl\\li{space*2+hang}\\fi-{hang} {ht[indentme2[i], 1]}")
      }
    } else if (is_format_html(format)) {
      for (i in seq_along(indentme2)) {
        ht[indentme2[i], 1] <- glue::glue(
          "<div style='text-indent: -{px}px; padding-left: {px*3}px'> {ht[indentme2[i], 1]}"
        )
      }
    }
  }

  if (!is.null(indentme3)) {
    indentme3 <- indentme3[order(indentme3)]
    if (tolower(substr(tlf, 1, 1)) %in% c("l", "t")) {
      indentme3 <- (indentme3 + formatindex)
    }
    if (is_format_rtf(format)) {
      for (i in seq_along(indentme3)) {
        ht[indentme3[i], 1] <- glue::glue("\\intbl\\li{space*3+hang}\\fi-{hang} {ht[indentme3[i], 1]}")
      }
    } else if (is_format_html(format)) {
      for (i in seq_along(indentme3)) {
        ht[indentme3[i], 1] <- glue::glue(
          "<div style='text-indent: -{px}px; padding-left: {px*4}px'> {ht[indentme3[i], 1]}"
        )
      }
    }
  }

  if (!is.null(indentme4)) {
    indentme4 <- indentme4[order(indentme4)]
    if (tolower(substr(tlf, 1, 1)) %in% c("l", "t")) {
      indentme4 <- (indentme4 + formatindex)
    }
    if (is_format_rtf(format)) {
      for (i in seq_along(indentme4)) {
        ht[indentme4[i], 1] <- glue::glue("\\intbl\\li{space*4+hang}\\fi-{hang} {ht[indentme4[i], 1]}")
      }
    } else if (is_format_html(format)) {
      for (i in seq_along(indentme4)) {
        ht[indentme4[i], 1] <- glue::glue(
          "<div style='text-indent: -{px}px; padding-left: {px*5}px'> {ht[indentme4[i], 1]}"
        )
      }
    }
  }

  if (!is.null(indentme5)) {
    indentme5 <- indentme5[order(indentme5)]
    if (tolower(substr(tlf, 1, 1)) %in% c("l", "t")) {
      indentme5 <- (indentme5 + formatindex)
    }
    if (is_format_rtf(format)) {
      for (i in seq_along(indentme5)) {
        ht[indentme5[i], 1] <- glue::glue("\\intbl\\li{space*5+hang}\\fi-{hang} {ht[indentme5[i], 1]}")
      }
    } else if (is_format_html(format)) {
      for (i in seq_along(indentme5)) {
        ht[indentme5[i], 1] <- glue::glue(
          "<div style='text-indent: -{px}px; padding-left: {px*6}px'> {ht[indentme5[i], 1]}"
        )
      }
    }
  }

  if (!is.null(indentme6)) {
    indentme6 <- indentme6[order(indentme6)]
    if (tolower(substr(tlf, 1, 1)) %in% c("l", "t")) {
      indentme6 <- (indentme6 + formatindex)
    }
    if (is_format_rtf(format)) {
      for (i in seq_along(indentme6)) {
        ht[indentme6[i], 1] <- glue::glue("\\intbl\\li{space*6+hang}\\fi-{hang} {ht[indentme6[i], 1]}")
      }
    } else if (is_format_html(format)) {
      for (i in seq_along(indentme6)) {
        ht[indentme6[i], 1] <- glue::glue(
          "<div style='text-indent: -{px}px; padding-left: {px*7}px'> {ht[indentme6[i], 1]}"
        )
      }
    }
  }

  #############################
  ###       Header          ###
  #############################

  add_header <- function(dsnin, header) {
    if (is_format_rtf(format)) {
      tmp <- t(huxtable::as_hux(c(header, rep("", ncol(dsnin) - 1)),
        add_colnames = FALSE
      ))
      huxtable::col_width(tmp) <- huxtable::col_width(dsnin)
      huxtable::width(tmp) <- huxtable::width(dsnin)
      huxtable::font_size(tmp) <- getOption("tidytlg.fontsize.title")
      huxtable::bold(tmp) <- TRUE
      ret <- huxtable::add_rows(dsnin, tmp, after = 0)
      ret <- ret[seq_len(nrow(ret)), ]
      ret
    } else {
      ht2 <- dsnin[1, ]
      ht2[1, ] <- ""
      dsnin <- rbind(dsnin, ht2)
      dsnin <- huxtable::add_footnote(dsnin,
        bquote(.(header)),
        number_format = NA,
        bold = TRUE,
        border = 0,
        font_size = getOption("tidytlg.fontsize.title")
      )
      dsnin <- rbind(dsnin[nrow(dsnin), ], dsnin[1:(nrow(dsnin) - 1), ])
      dsnin <- dsnin[seq_len(nrow(dsnin) - 1), ]
      dsnin
    }
  }

  if (is_format_rtf(format)) {
    # Used to prevent the Rplot.pdf from outputting in batch.
    if (dev.cur() == 1) {
      # If there is no graphics dev available. Create a NULL PDF device
      pdf(NULL)
      round(
        1440 / graphics::strwidth(
          expression(huxtable::bold(paste0(file, ":"))),
          family = "serif",
          font = getOption("tidytlg.fontsize.graph"),
          units = "in"
        ),
        0
      )
      # Close the device
      dev.off()
    } else {
      round(
        1440 / graphics::strwidth(
          expression(huxtable::bold(paste0(file, ":"))),
          family = "serif",
          font = getOption("tidytlg.fontsize.graph"),
          units = "in"
        ),
        0
      )
    }

    # Make repeated header on each page
    ht <- add_header(ht, paste0(
      "\\pnhang\\trhdr\\fi-1152", "\\li1152\\keepn",
      if (index_in_result == 1) "\\s15 " else "\\s16 ",
      file, ":\\tab", " ", title
    ))
  } else if (is_format_html(format)) {
    # Make repeated header on each page
    ht <- add_header(
      ht,
      paste0(
        '<div style = "text-indent: -36px; padding-left: 36px;"> ',
        file,
        ": &emsp; ",
        title,
        "</div> "
      )
    )
  }

  #############################
  ###      Table borders    ###
  #############################
  if (is_format_rtf(format)) {
    bordervalue <- ifelse(tlf %in% "l", getOption("tidytlg.fontsize.listing"),
      getOption("tidytlg.fontsize.table")
    ) / 10

    ht <- huxtable::set_top_border(ht, 1, value = bordervalue) |>
      huxtable::set_bottom_border(1, value = bordervalue) |>
      huxtable::set_bottom_border(nrow(ht), value = bordervalue)
  } else if (is_format_html(format)) {
    ht[1, ] <- paste0("<div style='border-top :1pt solid; border-bottom :1pt solid; '> ", ht[1, ])
  }
  #############################
  ###       Footers         ###
  #############################
  fontsize <- switch(tolower(substr(tlf, 1, 1)),
    "t" = getOption("tidytlg.fontsize.table.footnote"),
    "l" = getOption("tidytlg.fontsize.listing.footnote"),
    "g" = getOption("tidytlg.fontsize.graph.footnote"),
    "f" = getOption("tidytlg.fontsize.graph.footnote")
  )

  if (is_format_rtf(format)) {
    add_footer <- function(dsnin, footer, first = FALSE, size = fontsize) {
      if (first) {
        # In case hanging indent is required + hard enter: \\par\\pard\\pnhang\\fi-180\\li180
        dsnin <- huxtable::add_footnote(dsnin, paste0("\\line ", footer),
          number_format = NA, font_size = size,
          border = 0
        )
      } else {
        dsnin <- huxtable::add_footnote(dsnin, footer,
          number_format = NA,
          font_size = size, border = 0
        )
      }
      dsnin
    }
  } else if (is_format_html(format)) {
    add_footer <- function(dsnin, footer, first = FALSE, size = fontsize) {
      if (first) {
        # In case hanging indent is required + hard enter:
        # \\par\\pard\\pnhang\\fi-180\\li180

        dsnin <- huxtable::add_footnote(
          dsnin,
          paste0("<div style='border-top:1pt solid;'> ", "<br />", footer),
          number_format = NA,
          font_size = size,
          border = 0
        )
      } else {
        dsnin <- huxtable::add_footnote(dsnin, footer,
          number_format = NA,
          font_size = size, border = 0
        )
      }
      dsnin
    }
  }

  if (!is.null(footers)) {
    if (isTRUE(footers_one_row)) {
      if (is_format_rtf(format)) {
        footers <- paste(footers, collapse = " \\line ")
      } else {
        footers <- paste(footers, collapse = " <br /> ")
      }
    }
    ht <- add_footer(ht, footers[1], first = TRUE)

    if (length(footers) > 1) {
      for (i in 2:length(footers)) {
        ht <- add_footer(ht, footers[i], first = FALSE)
      }
    }
  }

  adjfilename <- stringr::str_replace_all(
    stringr::str_to_lower(file),
    "(-|_)", ""
  )


  ifelse(is.null(ht), ht <- ht,
    ifelse(getOption("tidytlg.add_datetime"),
      ht <- add_footer(ht, paste0(
        "[", adjfilename, ".", tolower(format), "]",
        paste0("[", get_file_name(), "] "),
        toupper(format(Sys.time(),
          format = "%d%b%Y, %H:%M"
        ))
      ),
      size = 8
      ),
      ht <- add_footer(ht, " ", size = 8)
    )
  )

  if (is_format_rtf(format)) {
    ht <- huxtable::set_top_border(ht, nrow(ht) - length(footers),
      value = huxtable::brdr(bordervalue,
        color = "black"
      )
    )
    ht <- huxtable::set_top_border(ht, nrow(ht),
      value = huxtable::brdr(bordervalue,
        color = "black"
      )
    )
  } else if (is_format_html(format)) {
    ht[nrow(ht), ] <- paste0(
      "<div style='border-bottom:1pt solid'> ",
      ht[nrow(ht), ]
    )
  }

  #############################
  ###       Alignments      ###
  #############################
  if (tolower(substr(tlf, 1, 1)) %in% c("t", "l")) {
    ht <- huxtable::set_valign(ht, seq_len(nrow(ht)), seq_len(ncol(ht)),
      value = "bottom"
    )
  }

  if (is_format_rtf(format)) {
    # Get the old format of bottom borders for backwards
    # compatibility.
    if (!is.matrix(bottom_borders)) {
      bottom_borders <- old_format(ht, colspan, colheader, tlf)
    }
    huxtable::left_padding(ht) <- 0
    huxtable::right_padding(ht) <- 0
    huxtable::bottom_padding(ht) <- 0
    huxtable::top_padding(ht) <- 0
    if (is_table(tlf) || is_listing(tlf)) {
      ht <- add_bottom_borders(ht, bottom_borders, border_fns)
    }
    if (is_table(tlf)) {
      ht <- huxtable::set_align(ht, 2:nrow(ht), 2:ncol(ht), "center")
    } else if (is_listing(tlf)) {
      ht <- huxtable::set_align(ht, 2:nrow(ht), 2:ncol(ht), "center")
      # Top center per JCEV-12. Only top align the first two rows, the title and
      # the column header, plus whatever colspans there are.
      ht <- huxtable::set_valign(
        ht, (2 + length(colspan)):nrow(ht),
        seq_len(ncol(ht)),
        "top"
      )
      ht <- huxtable::set_valign(
        ht, 1:(2 + length(colspan)),
        seq_len(ncol(ht)),
        "bottom"
      )
    } else if (is_graph(tlf)) {
      ht <- huxtable::set_align(ht, 2:nrow(ht), seq_len(ncol(ht)), "center")
    }
  } else if (is_format_html(format)) {
    if (is_table(tlf)) {
      ht <- huxtable::set_align(ht, 2:nrow(ht), 2:ncol(ht), "center")

      for (z in 2:(1 + formatindex)) {
        borderlineme <- 2:ncol(ht)
        ht[z, borderlineme] <- paste0(
          "<div style='border-bottom:1pt solid'> ",
          ht[z, borderlineme]
        )
      }
    } else if (is_listing(tlf)) {
      ht <- huxtable::set_align(ht, 2:nrow(ht), 2:ncol(ht), "center")

      for (z in 2:(1 + formatindex)) {
        borderlineme <- seq_len(ncol(ht))
        ht[z, borderlineme] <- paste0(
          "<div style='border-bottom:1pt solid'> ",
          ht[z, borderlineme]
        )
      }
    }
  }

  # Alignment first column + footers is left
  ht <- huxtable::set_align(ht, seq_len(nrow(ht)), 1, "left")
  if (getOption("tidytlg.add_datetime")) {
    ht <- huxtable::set_align(ht, nrow(ht), seq_len(ncol(ht)), "right")
  } # Alignment fileloc, datetime to the right

  # Custom alignments
  for (alignment in alignments) {
    ht <- huxtable::set_align(
      ht,
      alignment$row,
      alignment$col,
      alignment$value
    )
  }

  #############################
  ###       Escapeme!       ###
  #############################
  huxtable::escape_contents(ht) <- FALSE

  #############################
  ###       Returnme!       ###
  #############################
  # shiny needs to be able to save to object so return(ht) for both HTML and RTF
  # static rtf, just needs to run quick_rtf_jnj
  if (exists("a_file") && file.exists(a_file)) file.remove(a_file)
  list(ht = ht, colspan = colspan)
}

#' Bind a set of `tidytlg` tables together with formatting variables
#'
#' `bind_table` combines analysis results with formatting variables
#' (`indentme`, `newrows`, `newpage`) based on `by` variables
#' (`tablebyvar`, `rowbyvar`), such that appropriate formatting (indentation,
#' line break, page break) can be applied while creating the output.
#' It can also attach the column metadata attribute,
#' which will be automatically used in `gentlg` for creating output.
#'
#' @param ... (required) a set of `tidytlg` tables to bind together
#' @param colvar (required) treatment variable within `df` to use to summarize.
#'   Required if `add_count` is TRUE.
#' @param tablebyvar (optional) repeat entire table by variable within `df`.
#' @param rowbyvar (optional) any `rowbyvar` values used to create the table.
#' @param prefix (optional) text to prefix the values of `tablebyvar` with.
#' @param add_count (optional) Should a count be included in the `tablebyvar`?
#'   (default = `TRUE`)
#' @param add_format (optional) Should format be added to the output table?
#'   This is done using the `add_format` function. (default = `TRUE`)
#' @param column_metadata_file (optional) An excel file for `column_metadata`.
#'   Does not change the behavior of the function binds the column metadata
#'   for `gentlg`. If a `column_metadata` dataframe is passed in too,
#'   this is ignored.
#' @param column_metadata (optional) A dataframe containing the column metadata.
#'   This will be used in place of `column_metadata_file.`
#' @param tbltype (optional) A value used to subset the `column_metadata_file.`
#'
#' @return The `tidytlg` tables bound together reflecting the
#' `tablebyvars` used.
#' @export
#'
#' @examples
#' library(magrittr)
#'
#' # bind tables together
#' t1 <- cdisc_adsl %>%
#'   freq(
#'     colvar = "TRT01PN",
#'     rowvar = "ITTFL",
#'     statlist = statlist("n"),
#'     subset = ITTFL == "Y",
#'     rowtext = "Analysis set: ITT"
#'   )
#'
#' t2 <- cdisc_adsl %>%
#'   univar(
#'     colvar = "TRT01PN",
#'     rowvar = "AGE",
#'     decimal = 0,
#'     row_header = "Age, years"
#'   )
#'
#' bind_table(t1, t2)
#'
#' # bind tables together w/by groups
#' t1 <- cdisc_adsl %>%
#'   freq(
#'     colvar = "TRT01PN",
#'     rowvar = "ITTFL",
#'     rowbyvar = "SEX",
#'     statlist = statlist("n"),
#'     subset = ITTFL == "Y",
#'     rowtext = "Analysis set: ITT"
#'   )
#'
#' t2 <- cdisc_adsl %>%
#'   univar(
#'     colvar = "TRT01PN",
#'     rowvar = "AGE",
#'     rowbyvar = "SEX",
#'     decimal = 0,
#'     row_header = "Age, years"
#'   )
#'
#' bind_table(t1, t2, rowbyvar = "SEX")
#'
#' # bind tables together w/table by groups
#' t1 <- cdisc_adsl %>%
#'   freq(
#'     colvar = "TRT01PN",
#'     rowvar = "ITTFL",
#'     tablebyvar = "SEX",
#'     statlist = statlist("n"),
#'     subset = ITTFL == "Y",
#'     rowtext = "Analysis set: ITT"
#'   )
#'
#' t2 <- cdisc_adsl %>%
#'   univar(
#'     colvar = "TRT01PN",
#'     rowvar = "AGE",
#'     tablebyvar = "SEX",
#'     decimal = 0,
#'     row_header = "Age, years"
#'   )
#'
#' bind_table(t1, t2, tablebyvar = "SEX")
#'
#' # w/prefix
#' bind_table(t1, t2, tablebyvar = "SEX", prefix = "Gender: ")
#'
#' # w/counts
#' bind_table(t1, t2, tablebyvar = "SEX", add_count = TRUE, colvar = "TRT01PN")
bind_table <- function(...,
                       colvar = NULL,
                       tablebyvar = NULL,
                       rowbyvar = NULL,
                       prefix = NULL,
                       add_count = FALSE,
                       add_format = TRUE,
                       column_metadata_file = NULL,
                       column_metadata = NULL,
                       tbltype = NULL) {
  # Logic to unnest list if passed in generate_results
  dfs_ <- list(...)
  if (length(dfs_) == 1 && all(class(dfs_[[1]]) == "list")) {
    dfs_ <- dfs_[[1]]
  }

  # check all the arguments being passed in except ...
  arglist <- list()
  args_to_chk <- names(formals())[names(formals()) != "..."]
  purrr::walk(args_to_chk, .f = function(x) {
    arglist[[x]] <<- eval(rlang::sym(x))
  })
  check_bind_table(dfs_, arglist)

  # set up the environment for the iteration of anbr to happen in
  env <- new.env()
  if (is.null(tablebyvar)) {
    res <- map_dfr(dfs_, ~ add_anbr(.x, env = env)) %>%
      {
        if (add_format) {
          add_format(., tableby = tablebyvar, groupby = rowbyvar)
        } else {
          .
        }
      }
  } else {
    dfs <- purrr::map_dfr(dfs_, ~ add_rowtext_by(.x,
      tablebyvar = tablebyvar, # nolint indentation_linter
      env = env
    ))

    if (add_count) {
      first_freq <- min(which(purrr::map_chr(dfs_, first_class) ==
        "tidytlg.freq")) # nolint indentation_linter
      denoms_ <- attr(dfs_[[first_freq]], "denom")

      res <- dfs %>%
        nest(data_nest = -all_of(tablebyvar))

      if (is.null(colvar)) stop("bind_table is missing colvar")
      for (i in seq_len(nrow(res))) {
        cur_denoms_ <- get_tby_denoms(
          denoms_, tablebyvar,
          res[i, tablebyvar][[1]], colvar
        )
        res[i, "data_nest"] <- res[i, "data_nest"] %>%
          extract2(1) %>%
          extract2(1) %>%
          add_row(!!!as.list(cur_denoms_),
            label = paste0(prefix, res[i, tablebyvar][[1]]),
            row_type = "TABLE_BY_HEADER",
            .before = 1
          ) %>%
          list() %>%
          list()
      }
      res <- res %>%
        unnest(data_nest) %>%
        ungroup() %>%
        {
          if (add_format) {
            add_format(., tableby = tablebyvar, groupby = rowbyvar)
          } else {
            .
          }
        }
    } else {
      res <- dfs %>%
        nest(data_nest = -all_of(tablebyvar)) %>%
        rowwise() %>%
        mutate(data_nest = list(
          data_nest %>%
            add_row(
              label = paste0(prefix, !!sym(tablebyvar)),
              row_type = "TABLE_BY_HEADER", anbr = 0,
              .before = 1
            )
        )) %>%
        unnest(data_nest) %>%
        ungroup() %>%
        {
          if (add_format) {
            add_format(., tableby = tablebyvar, groupby = rowbyvar)
          } else {
            .
          }
        }
    }
  }

  if (!is.null(c(column_metadata_file, column_metadata)) && !is.null(tbltype)) {
    if (is.null(column_metadata)) {
      column_metadata <- readxl::read_excel(column_metadata_file, sheet = 1)
    }

    attr(res, "column_metadata") <- column_metadata %>%
      filter(tbltype == !!tbltype)
  }

  res
}

#' add_rowtext_by
#'
#' Adds in new rows with `label` equal to `rowtext` for each `tablebyvar` group
#'
#' @param df dataframe
#' @param tablebyvar df field that breaks apart table
#' @param env environment
#'
#' @return df with rowtext row header added
#' @noRd
add_rowtext_by <- function(df, tablebyvar, env) {
  if (any(df[[tablebyvar]] == "")) {
    rowtext <- df[df[[tablebyvar]] == "", "label"][[1]]
    df <- df %>%
      nest(data_nest = -all_of(tablebyvar)) %>%
      rowwise() %>%
      filter(!(!!sym(tablebyvar) == "")) %>%
      mutate(data_nest = list(data_nest %>%
        add_row(
          label = rowtext,
          row_type = "HEADER",
          .before = 1
        ))) %>%
      unnest(data_nest)
  }

  df <- df %>%
    add_anbr(env = env)

  if ("anbr" %in% names(df)) {
    df[is.na(df[["anbr"]]), "anbr"] <- unique(df[is.na(df[["anbr"]]), "anbr"])
  }

  df
}

#' add_anbr
#'
#' Adds or updates anbr counter field in `df` which comes from `anbr_counter`
#' variable in `env`
#'
#' @param df dataframe
#' @param env environment
#'
#' @return df with rowby row header added
#' @noRd
add_anbr <- function(df, env = parent.frame()) {
  # check if the counter variable exists in the specified env
  if (!exists("anbr_counter", envir = env)) {
    # set up anbr counter to be used later
    assign("anbr_counter", 0, envir = env)
  }

  # check if anbr has been added or if it's not a valid numeric
  if ("anbr" %in% names(df) &&
    !all(is.na(suppressWarnings(as.numeric(df[["anbr"]]))))) { # nolint indentation_linter
    # update counter to be the max anbr in the input df for future layers
    anbr_values <- df[["anbr"]]
    assign("anbr_counter",
      max(c(
        suppressWarnings(as.numeric(df[["anbr"]])),
        get("anbr_counter", envir = env) + 1
      ), na.rm = TRUE),
      envir = env
    )
    # return df
    df %>%
      select(-"anbr") %>%
      mutate(anbr = suppressWarnings(as.numeric(anbr_values)))
  } else {
    if ("anbr" %in% names(df)) {
      df <- df %>%
        select(-"anbr")
    }
    # get the value from the parent env
    anbr_value <- get("anbr_counter", envir = env) + 1
    # increment the anbr_counter
    assign("anbr_counter", anbr_value, envir = env)
    # return df with anbr added
    df %>%
      mutate(anbr = anbr_value)
  }
}

#' get_tby_denoms
#'
#' Filters _denoms for current tablebyvar in `cur_tby` var
#'
#' @param denoms_ denominator
#' @param tablebyvar repeat entire table by variable within df
#' @param cur_tby current by
#' @param colvar treatment variable within df to use to summarize
#'
#' @return `denoms_` filtered for tablebyvar
#' @noRd
get_tby_denoms <- function(denoms_, tablebyvar, cur_tby, colvar) {
  tmp <- denoms_ %>%
    filter(!!sym(paste0("denom_", tablebyvar)) == as.character(cur_tby))

  cur_denoms <- tmp %>%
    extract2("denom") %>%
    as.character()

  names(cur_denoms) <- tmp[[paste0("denom_", colvar)]]

  cur_denoms
}

#' Get Titles and Footnotes for all TLGs or one specific TLG
#'
#' @param df dataframe with three variables; table name, row identifier (`TITLE`
#'   or `FOOTNOTEn`), and title or footnote text to display.
#' @param tblid  character vector containing the table id, optional, used to
#'   subset `df` to a specific table (defaults to `tblid`).
#' @param idvar  character vector containing the variable in `df` that contains
#'   your table id.
#' @param identifier character vector containing the variable name in `df` that
#'   contains your record identifier (defaults to "identifier").
#' @param text  character vector containing the variable name in `df` that
#'   contains your title and footnote text (defaults to "text").
#'
#' @return list of length two, the first element contains the titles
#'   as a `tibble` and the second contains the footnotes as a list.
#' @export
#'
#' @examples
#' tblid <- "TSIDEM01"
#'
#' titles <- tibble::tribble(
#'   ~tblid, ~identifier, ~text,
#'   "TSIDEM01", "TITLE", "Demographics Example",
#'   "TSIDEM01", "FOOTNOTE1", "Example footnote."
#' )
#'
#' title_foot <- rmdpstitle(titles, tblid)
#'
#' title_foot[[1]]
#' title_foot[[2]]
rmdpstitle <-
  function(df,
           tblid,
           idvar = "tblid",
           identifier = "identifier",
           text = "text") {
    # function argument checks
    rlang::check_required(df)
    rlang::check_required(tblid)
    if (!(identifier %in% names(df))) {
      cli::cli_abort("{identifier} is not in {substitute(df)}")
    }
    if (!(text %in% names(df))) {
      cli::cli_abort("{text} is not in {substitute(df)}")
    }

    # split titles and footnotes into separate objects
    ttl <- df %>%
      dplyr::filter(.data[[identifier]] == "TITLE")

    foot <- df %>%
      dplyr::filter(.data[[identifier]] != "TITLE")

    if (!(tblid %in% unique(df[[idvar]]))) {
      cli::cli_abort("{tblid} is not in
                     {substitute(df)}${substitute(idvar)}")
    }

    ttl <- ttl %>%
      dplyr::filter_at(idvar, ~ . == tblid) %>%
      dplyr::select(.data[[text]])

    foot <- foot %>%
      dplyr::filter_at(idvar, ~ . == tblid) %>%
      dplyr::select(.data[[text]])

    # return NULL for titles and/or footnotes if none are found
    if (nrow(ttl) == 0) {
      ttl <- NULL
    }
    if (nrow(foot) == 0) {
      foot <- NULL
    } else {
      foot <- split(foot$text, seq_len(nrow(foot)))
    }

    return(list(ttl, foot))
  }

#' Generate nested count/percent for two or three levels
#'
#' This will call `freq()` multiple times and combine the levels together. This
#' is useful for adverse event and concomitant medications.
#'
#' @param df (required) dataframe containing the two levels to summarize
#' @param denom_df (optional) dataframe containing records to use as the
#'   denominator (default = `df`)
#' @param colvar (required) treatment variable within `df` to use to summarize
#' @param tablebyvar (optional) repeat entire table by variable within `df.`
#' @param rowvar (required) nested levels separated by a star, for example
#'   `AEBODSYS*AEDECOD`,  this can handle up to three levels.
#' @param rowbyvar (optional) repeat `rowvar` by variable within `df`
#' @param statlist (optional) count/percent type to return
#'   `(default = "n (x.x)")`
#' @param decimal (optional) decimal precision root level `(default = 1)`
#' @param cutoff (optional) numeric value used to cut the data to a percentage
#'   threshold, if any column meets the threshold the entire record is kept.
#' @param cutoff_stat (optional) The value to cutoff by, n or pct. (default =
#'   'pct')
#' @param subset (optional) An R expression that will be passed to a
#'   `dplyr::filter()` function to subset the data.frame
#' @param descending_by (optional) The column or columns to sort descending
#'   values by. Can also provide a named list to do ascending order. ex.
#'   `c("VarName1" = "asc", "VarName2" = "desc")` would sort by `VarName1` in
#'   ascending order and `VarName2` in descending order. If not provided, the
#'   columns will be sorted alphabetically.
#' @param display_missing (optional) Should the "missing" values be displayed?
#'   (default = `FALSE`)
#' @param rowtext (optional) A character vector used to rename the `label`
#'   column. If named, names will give the new level and values will be the
#'   replaced value. If unnamed, and the table has only one row, the `rowtext`
#'   will rename the `label` of the row.
#' @param row_header (optional) A character vector to be added to the table.
#' @param .keep (optional) Should the `rowbyvar` and `tablebyvar` be
#'   output in the table. If `FALSE`, `rowbyvar` will still be output
#'   in the `label` column. `(default = TRUE)`.
#' @param .ord Should the ordering columns be output with the table? This is
#'   useful if a table needs to be merged or reordered in any way after build.
#' @param ... (optional) Named arguments to be included as columns on the table.
#'
#' @return A dataframe of nested results by `colvar` and optional `tablebyvar`.
#'   There are a few additional variable sets added to support multiple
#'   requirements. \cr  \cr The level variables (`level1_`, `level2_`,
#'   `level3_`) will carry down the counts for each level to every record.  This
#'   allows for easy sorting of nested groups. \cr  \cr The header variables
#'   (`header1`, `header2`, `header3`) will flag the header for each level to
#'   ensure each level header is sorted to the top of the level. \cr  \cr The n
#'   variables ("n_*") provide a numeric variable containing frequency for each
#'   `colvar`.  This can be used to sort and filter records. \cr  \cr The pct
#'   variables ("pct_*") provide a numeric variable containing percentages for
#'   each `colvar`.  This can be used to sort and filter records.\cr  \cr
#'
#' @export
#'
#' @examples
#' adae <- data.frame(
#'   SITEID = c("100", "100", "100", "200", "200", "200"),
#'   USUBJID = c(
#'     "Demo1-101", "Demo1-102", "Demo1-103",
#'     "Demo1-104", "Demo1-105", "Demo1-106"
#'   ),
#'   AEBODSYS = c(
#'     "Cardiac disorders", "Cardiac disorders",
#'     "Respiratory, thoracic and mediastinal disorders",
#'     "Infections and infestations",
#'     "Skin and subcutaneous tissue disorders",
#'     "Infections and infestations"
#'   ),
#'   AEDECOD = c(
#'     "Arrhythmia supraventricular", "Cardiac failure",
#'     "Chronic obstructive pulmonary disease", "Pneumonia",
#'     "Pustular psoriasis", "Upper respiratory tract infection"
#'   ),
#'   colnbr = structure(
#'     c(1L, 2L, 3L, 1L, 2L, 3L),
#'     .Label = c("Active", "Placebo", "Comparator"),
#'     class = "factor"
#'   )
#' )
#'
#' # Frequency and percent for two levels of nesting
#' nested_freq(adae,
#'   colvar = "colnbr",
#'   rowvar = "AEBODSYS*AEDECOD",
#'   statlist = statlist("n (x.x%)")
#' )
#'
#' # Frequency and percent for three levels of nesting (for illustrative
#' # purpose)
#' nested_freq(adae,
#'   colvar = "colnbr",
#'   rowvar = "SITEID*AEBODSYS*AEDECOD",
#'   statlist = statlist("n (x.x%)")
#' )
#'
#' # Cut records where pct meets threshold for a any column
#' nested_freq(cdisc_adae,
#'   colvar = "TRTA",
#'   rowvar = "AEBODSYS*AEDECOD",
#'   statlist = statlist("n (x.x%)", distinct = TRUE),
#'   cutoff = 2,
#'   cutoff_stat = "n"
#' )
#'
#' # Cut records where pct meets threshold for a specific column
#' nested_freq(cdisc_adae,
#'   rowvar = "AEBODSYS*AEDECOD",
#'   colvar = "TRTAN",
#'   statlist = statlist("n (x.x%)", distinct = TRUE),
#'   cutoff = "54 >= 2",
#'   cutoff_stat = "n"
#' )
#'
#' # Frequency and percent for two levels of nesting and sort by descending
#' # active
#' nested_freq(adae,
#'   colvar = "colnbr",
#'   rowvar = "AEBODSYS*AEDECOD",
#'   statlist = statlist("n (x.x%)"),
#'   descending = "Active"
#' )
#'
#' # Below illustrates how make the same calls to nested_freq() as above, using
#' # table and # column metadata along with generate_results().
#'
#' column_metadata <- tibble::tribble(
#'   ~tbltype, ~coldef, ~decode,
#'   "type1", "1", "Placebo",
#'   "type1", "2", "Low",
#'   "type1", "3", "High"
#' )
#'
#' # Frequency and percent for two levels of nesting
#' table_metadata <- tibble::tribble(
#'   ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar, ~statlist,
#'   "1", "nested_freq", "cdisc_adae", "AEBODSYS*AEDECOD", "type1", "TRTP",
#'   statlist("n (x.x%)")
#' )
#' # generate_results(table_metadata,
#' # column_metadata_file = tidytlg_metadata(path)
#'
#'
#' # Frequency and percent for three levels of nesting (for illustrative purpose)
#' table_metadata <- tibble::tribble(
#'   ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar,
#'   ~statlist,
#'   "1", "nested_freq", "cdisc_adae", "SITEID*AEBODSYS*AEDECOD", "type1",
#'   "TRTP", statlist("n (x.x%)")
#' )
#' # Commented out because it takes too long
#' # generate_results(table_metadata, column_metadata)
#'
#' # Cut records where pct meets threshold for a any column
#' column_metadata <- tibble::tribble(
#'   ~tbltype, ~coldef, ~decode,
#'   "type2", "1", "Placebo",
#'   "type2", "2", "Active"
#' )
#' table_metadata <- tibble::tibble(
#'   anbr = "1", func = "nested_freq", df = "cdisc_adae",
#'   rowvar = "AEBODSYS*AEDECOD",
#'   tbltype = "type2", colvar = "TRTP", statlist = statlist("n (x.x%)"),
#'   dotdotdot = "cutoff = 5"
#' )
#' # generate_results(table_metadata,
#' # column_metadata_file = tidytlg_metadata(path)
#'
#' # Cut records where pct meets threshold for a specific column
#' table_metadata <- tibble::tibble(
#'   anbr = "1", func = "nested_freq", df = "cdisc_adae",
#'   rowvar = "AEBODSYS*AEDECOD",
#'   tbltype = "type2", colvar = "TRTP", statlist = statlist("n (x.x%)"),
#'   dotdotdot = "cutoff = 'col1 >= 5'"
#' )
#' # generate_results(table_metadata,
#' # column_metadata_file = tidytlg_metadata(path)
#'
#' # Frequency and percent for two levels of nesting and sort by descending col1
#' table_metadata <- tibble::tibble(
#'   anbr = "1", func = "nested_freq", df = "cdisc_adae",
#'   rowvar = "AEBODSYS*AEDECOD",
#'   tbltype = "type2", colvar = "TRTP", statlist = statlist("n (x.x%)"),
#'   dotdotdot = "descending = 'col1'"
#' )
#' # generate_results(table_metadata,
#' # column_metadata_file = tidytlg_metadata(path)
nested_freq <- function(df,
                        denom_df = df,
                        colvar = NULL,
                        tablebyvar = NULL,
                        rowvar = NULL,
                        rowbyvar = NULL,
                        statlist = getOption("tidytlg.nested_freq.statlist.default"),
                        decimal = 1,
                        cutoff = NULL,
                        cutoff_stat = "pct",
                        subset = TRUE,
                        descending_by = NULL,
                        display_missing = FALSE,
                        rowtext = NULL,
                        row_header = NULL,
                        .keep = TRUE,
                        .ord = FALSE,
                        ...) {
  subset <- rlang::enexprs(subset)
  if (length(subset) > 0 && is.character(subset[[1]])) {
    subset <- as.list(parse(text = subset))
  }

  # check all the arguments being passed in except ...
  arglist <- list()
  args_to_chk <- names(formals())[names(formals()) != "..."]
  purrr::walk(
    args_to_chk,
    .f = function(x) {
      arglist[[x]] <<- eval(rlang::sym(x))
    }
  )
  check_nested_freq(arglist)

  level <- stringr::str_split(rowvar, "\\s*\\*\\s*", simplify = TRUE)[1, ]

  attr(df, "nested") <- TRUE
  attr(df, "nested_level") <- level
  attr(df, "rowbyvar") <- rowbyvar

  # If the statlist has a denoms_by config, use that. Otherwise default to colvar,
  # tablebyvar, and rowbyvar
  denoms_by <- statlist[["denoms_by"]] %||% c(colvar, tablebyvar)

  if (getOption("tidytlg.denoms.message")) {
    message("denominator set using ", paste0(denoms_by, collapse = ", "),
      ". You can overwrite using the denom_by argument in statlist.\n",
      sep = ""
    )
  }
  colvar_cols <- levels(as.factor(df[[colvar]]))

  freqs <- df %>%
    # get necessary records
    filter(!!!subset) %>%
    # derive nested counts
    derive_nested(
      denom_df = denom_df,
      colvar = colvar,
      level = level,
      rowbyvar = rowbyvar,
      tablebyvar = tablebyvar,
      statlist = statlist,
      denoms_by = denoms_by,
      display_missing = display_missing
    ) %>%
    # remove any results that do not meet table criteria
    drop_nested(level, cutoff, colvar, level, tablebyvar, cutoff_stat, rowbyvar) %>%
    round_freq(decimal) %>%
    arrange_freq(statlist) %>%
    row_header_merge(
      colvar = colvar,
      rowvar = tail(level, 1),
      rowbyvar = NULL,
      tablebyvar = tablebyvar,
      row_header = row_header
    ) %>%
    # pivot table to final orientation
    pivot_freq(
      colvar,
      tail(level, 1),
      c(rowbyvar, head(level, -1)),
      tablebyvar,
      statlist,
      rowtext = rowtext,
      row_header = row_header,
      display_missing = FALSE,
      nested = TRUE,
      .keep = .keep,
      .ord = .ord,
      ... = ...
    ) %>%
    sort_freq(level, descending_by, tablebyvar, rowbyvar, colvar_cols, colvar)

  attr(freqs, "colvar") <- colvar
  attr(freqs, "tablebyvar") <- tablebyvar
  attr(freqs, "rowvar") <- rowvar
  attr(freqs, "statlist") <- statlist
  attr(freqs, "cutoff") <- cutoff
  attr(freqs, "cutoff_stat") <- cutoff_stat
  attr(freqs, "descending_by") <- descending_by
  attr(freqs, "rowtext") <- rowtext
  attr(freqs, "row_header") <- row_header
  attr(freqs, "subset") <- subset
  attr(freqs, "denoms_by") <- denoms_by

  structure(freqs,
    class = c("tidytlg.nested_freq", "tidytlg.freq", class(freqs))
  )
}

#' Derive nested frequency
#'
#' @return A long dataframe with n, denom, N, and pct for each combination
#' @noRd
derive_nested <- function(df, denom_df, colvar, level, rowbyvar, tablebyvar,
                          statlist, descending_by, denoms_by, display_missing) {
  # Get long dataframes for the "top" level
  level1_df <- df %>%
    derive_freq(
      denom_df = denom_df,
      colvar = colvar,
      rowbyvar = rowbyvar,
      tablebyvar = tablebyvar,
      rowvar = level[1],
      statlist = statlist,
      nested = TRUE,
      rowtext = NULL,
      has_subset = FALSE,
      denoms_by = denoms_by,
      display_missing = display_missing
    ) %>%
    mutate(
      nested_level = 0
    )


  # Get derivations for second level
  level2_df <- df %>%
    derive_freq(
      denom_df = denom_df,
      colvar = colvar,
      rowbyvar = c(level[1], rowbyvar),
      tablebyvar = tablebyvar,
      rowvar = level[2],
      statlist = statlist,
      nested = TRUE,
      rowtext = NULL,
      has_subset = FALSE,
      denoms_by = denoms_by,
      display_missing = display_missing
    ) %>%
    mutate(
      nested_level = 1
    )

  if (length(level) == 3) {
    # If there are three levels, make a third level
    level3_df <- df %>%
      derive_freq(
        denom_df = denom_df,
        colvar = colvar,
        rowbyvar = c(level[1], level[2], rowbyvar),
        tablebyvar = tablebyvar,
        rowvar = level[3],
        statlist = statlist,
        nested = TRUE,
        rowtext = NULL,
        has_subset = FALSE,
        denoms_by = denoms_by,
        display_missing = display_missing
      ) %>%
      mutate(
        nested_level = 2
      )

    # In the top level dataframe, make a new variable named level[2], and level[3],
    # that are the values of whatever is in level[1]. When the merge happens the
    # higher levels will have values to help row bind.
    level1_df <- level1_df %>%
      mutate(
        !!level[2] := !!sym(level[1]),
        !!level[3] := !!sym(level[1])
      )
    level2_df <- level2_df %>%
      mutate(
        !!level[3] := !!sym(level[2])
      )
  } else {
    level3_df <- NULL

    level1_df <- level1_df %>%
      mutate(
        !!level[2] := !!sym(level[1])
      )
  }

  # Bind the other data.frames together
  bind_rows(level1_df, level2_df, level3_df) %>%
    # The complete is different for nested because the levels should only include
    # The combinations present in the data.
    complete(nesting(!!!syms(c(level, rowbyvar))), !!!syms(c(tablebyvar, colvar)),
      fill = list(n = 0, N = 0, denom = 0, pct = 0)
    )
}

#' Drop values for a nested frequency
#'
#' Calls drop_freq but also additional logic if the lowest rowvar doesn't have
#' any values to remove all upper values.
#'
#' @return Long dataframe with values dropped
#' @noRd
drop_nested <-
  function(df,
           level,
           cutoff,
           colvar,
           rowvar,
           tablebyvar,
           cutoff_stat,
           rowbyvar) {
    # Use the normal steps for dropping values
    drop_freq(
      df = df,
      cutoff = cutoff,
      # tail(level, 1) is always the lowest level
      rowvar = tail(level, 1),
      colvar = colvar,
      cutoff_stat = cutoff_stat,
      # Treat the top levels as a tablebyvar so values that span higher
      # level are calculated properly.
      drop_by = c(tablebyvar, head(level, -1), rowbyvar)
    ) %>%
      # nest the table by any tablebyvars and the top levels
      nest(nest_treatment = -any_of(c(tablebyvar, head(level, -1)))) %>%
      # Update the nested tables to filter out any where n is 0 for all treatment
      # groups
      mutate(
        nest_treatment = map(.data[["nest_treatment"]],
          ~ group_by(.x, !!!syms(tail(level, 1))) %>%
            filter(any(n > 0)),
          .preserve = FALSE
        )
      ) %>%
      rowwise() %>%
      # If that removed all of the treatments, drop it
      filter(nrow(nest_treatment) > 0) %>%
      unnest(nest_treatment) %>%
      # In certain cases the top level rowvar variables will persist even if there
      # are no subordinate values below them. This nests the top levels(level1, or
      # level1 and level2 if a three variable nest) and drops the ones that don't
      # have subordinate items.
      # -head(level, 1) returns everything but the top levels for nesting
      nest(nested_level = -any_of(c(head(level, 1), tablebyvar))) %>%
      rowwise() %>%
      # If there are only 'length(unique(df[[colvar]]))' number
      # of entries in the nested df, we know that the entries that are there are the
      # top level and contain subordinate items, so remove them
      filter(nrow(nested_level) > length(unique(df[[colvar]]))) %>%
      unnest(cols = nested_level)
  }

adsl <- cdisc_adsl %>% filter(TRT01A != "")

test_that("nested_freq can handle subset argument", {
  expected <- tibble::tribble(
    ~label,
    ~Placebo,
    ~Xanomeline.High.Dose,
    ~Xanomeline.Low.Dose,
    ~row_type,
    ~nested_level,
    ~group_level,
    ~SEX,
    "F",
    "2 (40.0)",
    "3 (60.0)",
    "1 (20.0)",
    "NESTED",
    0,
    0,
    "F",
    "WHITE",
    "2 (40.0)",
    "3 (60.0)",
    "1 (20.0)",
    "NESTED",
    1,
    0,
    "F",
    "M",
    "3 (60.0)",
    "2 (40.0)",
    "4 (80.0)",
    "NESTED",
    0,
    0,
    "M",
    "WHITE",
    "3 (60.0)",
    "2 (40.0)",
    "4 (80.0)",
    "NESTED",
    1,
    0,
    "M"
  )

  expected$SEX <- factor(expected$SEX)

  expect_equal(
    nested_freq(adsl, rowvar = "SEX*RACE", colvar = "TRT01A", subset = ITTFL == "Y"), expected,
    ignore_attr = TRUE
  )
})

test_that("nested_freq can handle row_header", {
  expected <- tibble::tribble(
    ~label,
    ~Placebo,
    ~Xanomeline.High.Dose,
    ~Xanomeline.Low.Dose,
    ~row_type,
    ~nested_level,
    ~group_level,
    ~SEX,
    "Geo, country",
    "",
    "",
    "",
    "HEADER",
    NA,
    0,
    "F",
    "F",
    "2 (40.0)",
    "3 (60.0)",
    "1 (20.0)",
    "NESTED",
    0,
    0,
    "F",
    "WHITE",
    "2 (40.0)",
    "3 (60.0)",
    "1 (20.0)",
    "NESTED",
    1,
    0,
    "F",
    "M",
    "3 (60.0)",
    "2 (40.0)",
    "4 (80.0)",
    "NESTED",
    0,
    0,
    "M",
    "WHITE",
    "3 (60.0)",
    "2 (40.0)",
    "4 (80.0)",
    "NESTED",
    1,
    0,
    "M"
  )

  expected$SEX <- factor(expected$SEX)

  expect_equal(
    nested_freq(adsl, rowvar = "SEX*RACE", colvar = "TRT01A", row_header = "Geo, country"), expected,
    ignore_attr = TRUE
  )
})

#' `tidytlg`: A package for producing tables, listings,
#'  and graphs (`TLGs`) using `tidyverse` packages.
#'
#' The `tidytlg` package provide a set of function allowing you to produce TLGs
#' using table metadata.
#'
#' @section `tidytlg` Options:
#'
#' \itemize{ \item{`tidytlg.row_type.levels` - The values of row_type and their
#' respective levels. Default: c("TABLE_BY_HEADER", "BY_HEADER1", "BY_HEADER2",
#' "BY_HEADER3", "BY_HEADER4", "BY_HEADER5", "HEADER", "N", "VALUE", "NESTED")}
#' \item{`tidytlg.fontsize.title` - Font size for titles in points. Default: 10}
#' \item{`tidytlg.fontsize.table.footnote` - Font size for footnotes in points.
#' For tables. Default: 9} \item{`tidytlg.fontsize.listing.footnote` - Font size
#' for footnotes in points. For listings. Default: 8}
#' \item{`tidytlg.fontsize.graph.footnote` - Font size for footnotes in points.
#' For graphs. Default: 8} \item{`tidytlg.fontsize.table` - Font size for tables
#' in points. Default: 9} \item{`tidytlg.fontsize.graph` - Font size for graphs in
#' points. Default: 10} \item{`tidytlg.fontsize.listing` - Font size for listings
#' in points. Default: 8} \item{`tidytlg.right.padding` - Amount of right padding
#' for cells in points. Default: 1} \item{`tidytlg.left.padding` - Amount of left
#' padding for cells in points. Default: 1}
#' \item{`tidytlg.stat_labels` - Labels
#' to be used for each summary statistic in the output table.  Will need to have
#' all available statistics if this option is changed!}
#' \item{`tidytlg.precision.extra` - Value to be added on for each summary
#' statistic when calculating precision.  This will be used to make different
#' statistics have different precision as needed}
#' \item{`tidytlg.nested_freq.statlist.default` - Default `statlist` object for
#' nested_freq tables. Default: `statlist("n (x.x)")`}
#' \item{`tidytlg.univar.statlist.default` - Default `statlist` object
#' for `univar` tables.
#' Default: `statlist(c("N", "MEANSD", "MEDIAN", "RANGE", "IQRANGE"))`}
#' \item{`tidytlg.freq.statlist.default` - Default `statlist` object
#' for frequency tables. Default: `statlist("n (x.x)")`}
#' \item{`tidytlg.stat_labels` - A `data.frame` controlling how the stats
#' are labeled in a `univar` table. See 'Default Stat Labels' section
#' for defaults.}
#' \item{`tidytlg.denoms.message` - A `boolean`, should a message print
#' detailing what the denominators are?}
#' }
#'
#' @section Default Stat Labels:
#'
#' | **stat** | **label** |
#' | --- | --- |
#' | `N` | N |
#' | `SUM` | Sum |
#' | `MEAN` | Mean |
#' | `GeoMEAN` | Geometric Mean |
#' | `SD` | Std. Deviation |
#' | `SE` | Std. Error |
#' | `GSD` | Geometric Std. Deviation |
#' | `GSE` | Geometric Std. Error |
#' | `CV` | CV |
#' | `MEANSD` | Mean (SD) |
#' | `MEANSE` | Mean (SE) |
#' | `MEDIAN` | Median |
#' | `MIN` | Minimum |
#' | `MAX` | Maximum |
#' | `RANGE` | Range |
#' | `Q1` | First quartile |
#' | `Q3` | Third quartile |
#' | `IQRANGE` | IQ range |
#' | `MEDRANGE` | Median (Range) |
#' | `MEDIQRANGE` | Median (Q1; Q3) |
#' | `MEAN_CI` | Mean (95% C.I.) |
#' | `GeoMEAN_CI` | Geometric Mean (95% C.I.) |
#'
#' @section Default Precision Extra:
#'
#' | **stat** | **extra** |
#' | --- | --- |
#' |`N` | 0 |
#' |`SUM`| 0 |
#' |`MEAN`| 1 |
#' |`GeoMEAN`| 1 |
#' |`SD`| 2 |
#' |`SE`| 2 |
#' |`GSD`| 2 |
#' |`GSE`| 2 |
#' |`CV`| 1 |
#' |`MEDIAN`| 1 |
#' |`MIN`| 0 |
#' |`MAX`| 0 |
#' |`Q1`| 1 |
#' |`Q3`| 1 |
#' |`LCL_MEAN`| 2 |
#' |`UCL_MEAN`| 2 |
#' |`LCL_GeoMEAN`| 2 |
#' |`UCL_GeoMEAN`| 2 |
#'
#' @section Updating Options:
#' \itemize{
#'   \item{For a single session, an option can be changed by
#'   `option(<optionToChange> = <NewValue>)`}
#'   \item{To change an option for a single projects across sessions in that
#'   projects, place the options update in the `.Rprofile` in that project
#'   directory.}
#'   \item{To change an option for a user across all sessions, place the options
#'   update in the `.Rprofile` file in the users home directory.}
#'   \item{To change an option for all users in an R environment, place the
#'   options update in the `.Rprofile.site` file in the R home directory.}
#' }
#'
#'
#'
#' @name tidytlg
#' @keywords internal
"_PACKAGE"

utils::globalVariables(c(
  "anbr", "label", "rowvar", "rowcondition", "row_type",
  "rowtext", "tableby", "by", "func", "colvar", "header_order",
  "df", "filter_to_df", "statlist", "denom_df", "precision",
  "tbltype", "dotdotdot",
  ".", "name", "alpha", "value", "N", "pct", "denom", "results",
  "n_ord", "n_ord2", "n_ord3", "data_nest", "nested_treatment",
  "nested_level", "nest_treatment", "stat", "extra", "prec",
  "IDENTIFIER", "TABLE ID", "decode", "colnbr", "newrows", "el",
  "sorter", "coldef", "V1"
))

## usethis namespace: start
#' @importFrom rlang enexprs is_call expr sym %||% exprs syms as_string is_named
#'   eval_tidy enexpr as_name .data :=
#' @importFrom tidyr pivot_wider pivot_longer complete nesting nest unnest
#'   replace_na crossing fill
#' @importFrom dplyr select bind_rows ungroup mutate rowwise filter summarise
#'   bind_cols
#' @importFrom dplyr group_by distinct n left_join rename relocate across
#'   if_else
#' @importFrom dplyr arrange case_when desc cur_group_id inner_join rename_with
#'   row_number
#' @importFrom dplyr group_by_at slice mutate_if
#' @importFrom stringr str_detect str_remove str_replace str_split regex
#'   str_to_lower
#' @importFrom dplyr all_of any_of everything starts_with ends_with
#' @importFrom forcats fct_expand fct_recode fct_relevel
#' @importFrom magrittr %>% extract2
#' @importFrom purrr map walk map_dfr pmap discard compact map_lgl imap map_chr
#' @importFrom tibble tibble tribble add_row tibble_row
#' @importFrom methods formalArgs
#' @importFrom stats setNames lag
#' @importFrom utils head modifyList tail
#' @importFrom huxtable everywhere
#' @importFrom crayon bold underline
#' @importFrom readxl read_excel
#' @importFrom cellranger cell_cols
#' @importFrom png readPNG
#' @importFrom ggplot2 ggsave
#' @importFrom grDevices dev.cur dev.off pdf
#' @importFrom rstudioapi getSourceEditorContext isAvailable
## usethis namespace: end
NULL


.onLoad <- function(libname, pkgname) {
  op <- options()
  op_tidytlg <- list(
    tidytlg.row_type.levels =
      c(
        "TABLE_BY_HEADER",
        "BY_HEADER1", "BY_HEADER2", "BY_HEADER3", "BY_HEADER4", "BY_HEADER5",
        "HEADER",
        "N", "VALUE", "NESTED"
      ),
    tidytlg.fontsize.title = 10,
    tidytlg.fontsize.table.footnote = 9,
    tidytlg.fontsize.listing.footnote = 8,
    tidytlg.fontsize.graph.footnote = 8,
    tidytlg.fontsize.table = 9,
    tidytlg.fontsize.graph = 10,
    tidytlg.fontsize.listing = 8,
    tidytlg.right.padding = 1,
    tidytlg.left.padding = 1,
    tidytlg.stat_labels = tibble::tribble(
      ~stat,           ~label,
      "N",             "N",
      "SUM",           "Sum",
      "MEAN",          "Mean",
      "GeoMEAN",       "Geometric Mean",
      "SD",            "Std. Dev.",
      "SE",            "Std. Error",
      "GSD",           "Geometric Std. Dev.",
      "GSE",           "Geometric Std. Error",
      "CV",            "CV (%)",
      "MEANSD",        "Mean (SD)",
      "MEANSE",        "Mean (SE)",
      "MEDIAN",        "Median",
      "MIN",           "Minimum",
      "MAX",           "Maximum",
      "RANGE",         "Range",
      "Q1",            "First quartile",
      "Q3",            "Third quartile",
      "IQRANGE",       "IQ range",
      "MEDRANGE",      "Median (Range)",
      "MEDIQRANGE",    "Median (Q1; Q3)",
      "MEAN_CI",       "Mean (95% C.I.)",
      "GeoMEAN_CI",    "Geometric Mean (95% C.I.)"
    ),
    tidytlg.precision.extra = tibble::tribble(
      ~stat,           ~extra,
      "N",             0,
      "SUM",           0,
      "MEAN",          1,
      "GeoMEAN",       1,
      "SD",            2,
      "SE",            2,
      "GSD",           2,
      "GSE",           2,
      "CV",            1,
      "MEDIAN",        1,
      "MIN",           0,
      "MAX",           0,
      "Q1",            1,
      "Q3",            1,
      "LCL_MEAN",      2,
      "UCL_MEAN",      2,
      "LCL_GeoMEAN",   2,
      "UCL_GeoMEAN",   2
    ),
    tidytlg.univar.statlist.default =
      statlist(c("N", "MEANSD", "MEDIAN", "RANGE", "IQRANGE")),
    tidytlg.freq.statlist.default = statlist(c("n (x.x)")),
    tidytlg.nested_freq.statlist.default = statlist(c("n (x.x)")),
    tidytlg.denoms.message = FALSE,
    # Do not document. This is just used for testing
    tidytlg.add_datetime = TRUE
  )

  toset <- !(names(op_tidytlg) %in% names(op))
  if (any(toset)) options(op_tidytlg[toset])

  invisible()
}

#' Add indentation variable to the results dataframe
#'
#' Add the `indentme` variable to your results data. This drives the number of
#' indents for the row label text (e.g. 0, 1, 2, etc.).
#'
#' @details
#'
#' The `group_level` variable, which is added to the results dataframe by `freq()`
#' and `univar()` calls, is needed to define indentation when by variables are
#' used for summary.
#'
#' The `nested_level` variable, which is added to the results dataframe by
#' `nested_freq()`, is needed to define indentation for each level of nesting.
#'
#' Both of these are added to the default indentation which is driven by
#' `row_type`.
#'
#' | row_type          | default indentation |
#' | ----------------- |:-------------------:|
#' | TABLE_BY_HEADER   | 0                   |
#' | BY_HEADER\[1-9\]  | 0                   |
#' | HEADER            | 0                   |
#' | N                 | 1                   |
#' | VALUE             | 2                   |
#' | NESTED            | 0                   |
#'
#' @md
#'
#' @param df dataframe of results that contains `row_type` and `label`
#' and the optional `nested_level` and `group_level` variables.
#'
#' @return dataframe with the `indentme` variable added.
#' @export
#'
#' @examples
#' df <- tibble::tibble(
#'   row_type = c(
#'     "TABLE_BY_HEADER", "HEADER",
#'     "BY_HEADER1", "N", "VALUE", "COUNTS", "UNIVAR", "NESTED", "NESTED"
#'   ),
#'   nested_level = c(NA, NA, NA, NA, NA, NA, NA, 1, 2),
#'   group_level = c(0, 0, 0, 0, 0, 0, 0, 0, 0),
#'   label = c(NA, NA, NA, NA, NA, "N", NA, NA, NA),
#'   by = c(NA, NA, NA, NA, NA, NA, NA, NA, NA),
#'   tableby = c(NA, NA, NA, NA, NA, NA, NA, NA, NA)
#' )
#' add_indent(df)
add_indent <- function(df) {
  # set default list of variables to be removed
  remove_vars <- c("nested_level", "group_level", "value_add")

  # check if variables are already present in df, if not, add and set defaults
  if (!("nested_level" %in% names(df))) {
    df <- df %>%
      dplyr::mutate(nested_level = 0)
  }
  if (!("group_level" %in% names(df))) {
    df <- df %>%
      dplyr::mutate(group_level = 0)
  }

  # define amount to indent VALUE rows based on if a N row exists in the group
  # based on anbr
  if (!("value_add" %in% names(df))) {
    if ("anbr" %in% names(df)) {
      df <- df %>%
        group_by(anbr) %>%
        dplyr::mutate(value_add = 1 + max(as.numeric(row_type == "N")))
    } else {
      df <- df %>%
        dplyr::mutate(value_add = 1 + max(as.numeric(row_type == "N")))
    }
  }
  # if any table by headers, add one indentation to everything
  add_tableby_to_default <- any(df[["row_type"]] == "TABLE_BY_HEADER")

  df %>%
    dplyr::mutate(
      indentme =
        dplyr::case_when(
          row_type == "TABLE_BY_HEADER" ~ 0,
          grepl("BY_HEADER[0-9]", row_type) ~ 0 + add_tableby_to_default
            + group_level, # nolint indentation_linter
          row_type == "HEADER" ~ 0 + add_tableby_to_default + group_level,
          row_type == "N" ~ 1 + add_tableby_to_default + group_level,
          row_type == "VALUE" ~ value_add + add_tableby_to_default
            + group_level, # nolint indentation_linter
          row_type == "NESTED" ~ nested_level + add_tableby_to_default
            + group_level, # nolint indentation_linter
          TRUE ~ 0
        )
    ) %>%
    select(-any_of(remove_vars))
}

is_listing <- function(type) {
  substr(tolower(type), 1, 1) == "l"
}

is_table <- function(type) {
  substr(tolower(type), 1, 1) == "t"
}

is_graph <- function(type) {
  substr(tolower(type), 1, 1) %in% c("g", "f")
}

is_format_rtf <- function(format) {
  tolower(format) == "rtf"
}

is_format_html <- function(format) {
  tolower(format) == "html"
}

#' `ADSL` data created from subsetting the `CDISC` `ADSL` with 15 subjects (5 subjects in each arm)
#'
#' @source CDISC SDTM/ADAM Pilot Project.
#'
#' @format A data frame with 15 rows and 49 variables:
#' \describe{
#'  \item{`STUDYID`}{Study Identifier}
#'  \item{`USUBJID`}{Unique Subject Identifier}
#'  \item{`SUBJID`}{Subject Identifier for the Study}
#'  \item{`SITEID`}{Study Site Identifier}
#'  \item{`SITEGR1`}{Pooled Site Group 1}
#'  \item{`ARM`}{Description of Planned Arm}
#'  \item{`TRT01P`}{Planned Treatment for Period 01}
#'  \item{`TRT01PN`}{Planned Treatment for Period 01 (N)}
#'  \item{`TRT01A`}{Actual Treatment for Period 01}
#'  \item{`TRT01AN`}{Actual Treatment for Period 01 (N)}
#'  \item{`TRTSDT`}{Date of First Exposure to Treatment}
#'  \item{`TRTEDT`}{Date of Last Exposure to Treatment}
#'  \item{`TRTDUR`}{Duration of Treatment (days)}
#'  \item{`AVGDD`}{Avg Daily Dose (as planned)}
#'  \item{`CUMDOSE`}{Cumulative Dose (as planned)}
#'  \item{`AGE`}{Age}
#'  \item{`AGEGR1`}{Pooled Age Group 1}
#'  \item{`AGEGR1N`}{Pooled Age Group 1 (N)}
#'  \item{`AGEU`}{Age Units}
#'  \item{`RACE`}{Race}
#'  \item{`RACEN`}{Race (N)}
#'  \item{`SEX`}{Sex}
#'  \item{`ETHNIC`}{Ethnicity}
#'  \item{`SAFFL`}{Safety Population Flag}
#'  \item{`ITTFL`}{Intent-To-Treat Population Flag}
#'  \item{`EFFFL`}{Efficacy Population Flag}
#'  \item{`COMP8FL`}{Finishers of Week 8 Population Flag}
#'  \item{`COMP16FL`}{Finishers of Week 16 Population Flag}
#'  \item{`COMP24FL`}{Finishers of Week 24 Population Flag}
#'  \item{`DISCONFL`}{Did the Subject Discontinue the Study?}
#'  \item{`DSRAEFL`}{Discontinued due to `AE`?}
#'  \item{`DTHFL`}{Subject Died?}
#'  \item{`BMIBL`}{Baseline BMI (kg/m^2)}
#'  \item{`BMIBLGR1`}{Pooled Baseline BMI Group 1}
#'  \item{`HEIGHTBL`}{Baseline Height (cm)}
#'  \item{`WEIGHTBL`}{Baseline Weight (kg)}
#'  \item{`EDUCLVL`}{Years of Education}
#'  \item{`DISONSDT`}{Date of Onset of Disease}
#'  \item{`DURDIS`}{Duration of Disease (Months)}
#'  \item{`DURDSGR1`}{Pooled Disease Duration Group 1}
#'  \item{`VISIT1DT`}{Date of Visit 1}
#'  \item{`RFSTDTC`}{Subject Reference Start Date/Time}
#'  \item{`RFENDTC`}{Subject Reference End Date/Time}
#'  \item{`VISNUMEN`}{End of Treatment Visit (Visit 12 or Early Term.)}
#'  \item{`RFENDT`}{Date of Discontinuation/Completion}
#'  \item{`DCDECOD`}{Standardized Disposition Term}
#'  \item{`EOSSTT`}{End of Study Status}
#'  \item{`DCREASCD`}{Reason for Discontinuation}
#'  \item{`MMSETOT`}{`MMSE` Total}
#'
#' }
#' @keywords datasets CDISC adsl
#' @name cdisc adsl
"cdisc_adsl"

#'  `ADAE` data created from subsetting the `CDISC` `ADAE` dataset
#'
#' @source CDISC SDTM/ADAM Pilot Project.
#'
#' @format A data frame with 84 rows and 55 variables:
#' \describe{
#'  \item{`STUDYID`}{Study Identifier}
#'  \item{`SITEID`}{Study Site Identifier}
#'  \item{`USUBJID`}{Unique Subject Identifier}
#'  \item{`SUBJID`}{Subject Identifier for the Study}
#'  \item{`TRTA`}{Actual Treatment}
#'  \item{`TRTAN`}{Actual Treatment (N)}
#'  \item{`AGE`}{Age}
#'  \item{`AGEGR1`}{Pooled Age Group 1}
#'  \item{`AGEGR1N`}{Pooled Age Group 1 (N)}
#'  \item{`RACE`}{Race}
#'  \item{`RACEN`}{Race (N)}
#'  \item{`SEX`}{Sex}
#'  \item{`SAFFL`}{Safety Population Flag}
#'  \item{`TRTSDT`}{Date of First Exposure to Treatment}
#'  \item{`TRTEDT`}{Date of Last Exposure to Treatment}
#'  \item{`ASTDT`}{Analysis Start Date}
#'  \item{`ASTDTF`}{Analysis Start Date Imputation Flag}
#'  \item{`ASTDY`}{Analysis Start Relative Day}
#'  \item{`AENDT`}{Analysis End Date}
#'  \item{`AENDY`}{Analysis End Relative Day}
#'  \item{`ADURN`}{`AE` Duration (N)}
#'  \item{`ADURU`}{`AE` Duration Units}
#'  \item{`AETERM`}{Reported Term for the Adverse Event}
#'  \item{`AELLT`}{Lowest Level Term}
#'  \item{`AELLTCD`}{Lowest Level Term Code}
#'  \item{`AEDECOD`}{Dictionary-Derived Term}
#'  \item{`AEPTCD`}{Preferred Term Code}
#'  \item{`AEHLT`}{High Level Term}
#'  \item{`AEHLTCD`}{High Level Term Code}
#'  \item{`AEHLGT`}{High Level Group Term}
#'  \item{`AEHLGTCD`}{High Level Group Term Code}
#'  \item{`AEBODSYS`}{Body System or Organ Class}
#'  \item{`AESOC`}{Primary System Organ Class}
#'  \item{`AESOCCD`}{Primary System Organ Class Code}
#'  \item{`AESEV`}{Severity/Intensity}
#'  \item{`AESER`}{Serious Event}
#'  \item{`AESCAN`}{Involves Cancer}
#'  \item{`AESCONG`}{Congenital Anomaly or Birth Defect}
#'  \item{`AESDISAB`}{Persist or `Signif` Disability/Incapacity}
#'  \item{`AESDTH`}{Results in Death}
#'  \item{`AESHOSP`}{Requires or Prolongs Hospitalization}
#'  \item{`AESLIFE`}{Is Life Threatening}
#'  \item{`AESOD`}{Occurred with Overdose}
#'  \item{`AEREL`}{Causality}
#'  \item{`AEACN`}{Action Taken with Study Treatment}
#'  \item{`AEOUT`}{Outcome of Adverse Event}
#'  \item{`AESEQ`}{Sequence Number}
#'  \item{`TRTEMFL`}{Treatment Emergent Analysis Flag}
#'  \item{`AOCCFL`}{First Occurrence of Any `AE` Flag}
#'  \item{`AOCCSFL`}{First Occurrence of `SOC` Flag}
#'  \item{`AOCCPFL`}{First Occurrence of Preferred Term Flag}
#'  \item{`AOCC02FL`}{First Occurrence 02 Flag for Serious}
#'  \item{`AOCC03FL`}{First Occurrence 03 Flag for Serious `SOC`}
#'  \item{`AOCC04FL`}{First Occurrence 04 Flag for Serious `PT`}
#'  \item{`CQ01NAM`}{Customized Query 01 Name}
#'  \item{`AOCC01FL`}{First Occurrence 01 Flag for `CQ01`}
#' }
#' @keywords datasets CDISC adae
#' @name cdisc adae
"cdisc_adae"

#' `ADLB` data created from subsetting the `CDISC` `ADLB` dataset
#'
#' @source CDISC SDTM/ADAM Pilot Project.
#'
#' @format A data frame with 2154 rows and 46 variables:
#' \describe{
#'  \item{`STUDYID`}{Study Identifier}
#'  \item{`SUBJID`}{Subject Identifier for the Study}
#'  \item{`USUBJID`}{Unique Subject Identifier}
#'  \item{`TRTA`}{Actual Treatment}
#'  \item{`TRTAN`}{Actual Treatment (N)}
#'  \item{`TRTSDT`}{Date of First Exposure to Treatment}
#'  \item{`TRTEDT`}{Date of Last Exposure to Treatment}
#'  \item{`AGE`}{Age}
#'  \item{`AGEGR1`}{Pooled Age Group 1}
#'  \item{`AGEGR1N`}{Pooled Age Group 1 (N)}
#'  \item{`RACE`}{Race}
#'  \item{`RACEN`}{Race (N)}
#'  \item{`SEX`}{Sex}
#'  \item{`COMP24FL`}{Finishers of Week 24 Population Flag}
#'  \item{`DSRAEFL`}{Discontinued due to `AE`?}
#'  \item{`SAFFL`}{Safety Population Flag}
#'  \item{`AVISIT`}{Analysis Visit}
#'  \item{`AVISITN`}{Analysis Visit (N)}
#'  \item{`ADY`}{Analysis Relative Day}
#'  \item{`ADT`}{Analysis Date}
#'  \item{`VISIT`}{Visit Name}
#'  \item{`VISITNUM`}{Visit Number}
#'  \item{`PARAM`}{Parameter}
#'  \item{`PARAMCD`}{Parameter Code}
#'  \item{`PARAMN`}{Parameter (N)}
#'  \item{`PARCAT1`}{Parameter Category 1}
#'  \item{`AVAL`}{Analysis Value}
#'  \item{`BASE`}{Baseline Value}
#'  \item{`CHG`}{Change from Baseline}
#'  \item{`A1LO`}{Analysis Range 1 Lower Limit}
#'  \item{`A1HI`}{Analysis Range 1 Upper Limit}
#'  \item{`R2A1LO`}{Ratio to Analysis Range 1 Lower Limit}
#'  \item{`R2A1HI`}{Ratio to Analysis Range 1 Upper Limit}
#'  \item{`BR2A1LO`}{Base Ratio to Analysis Range 1 Lower Limit}
#'  \item{`BR2A1HI`}{Base Ratio to Analysis Range 1 Upper Limit}
#'  \item{`ANL01FL`}{Analysis 01 - Special Interest Flag}
#'  \item{`ALBTRVAL`}{Amount Threshold Range}
#'  \item{`ANRIND`}{Analysis Reference Range Indicator}
#'  \item{`BNRIND`}{Baseline Reference Range Indicator}
#'  \item{`ABLFL`}{Baseline Record Flag}
#'  \item{`AENTMTFL`}{Last value in treatment visit}
#'  \item{`LBSEQ`}{Sequence Number}
#'  \item{`LBNRIND`}{Reference Range Indicator}
#'  \item{`LBSTRESN`}{Numeric Result/Finding in Standard Units}
#' }
#' @keywords datasets CDISC adlb
#' @name cdisc adlb
"cdisc_adlb"


#' `ADVS` data created from subsetting the `CDISC` `ADVS` dataset
#'
#' @source CDISC SDTM/ADAM Pilot Project.
#'
#' @format A data frame with 1938 rows and 35 variables:
#' \describe{
#'  \item{`STUDYID`}{Study Identifier}
#'  \item{`SITEID`}{Study Site Identifier}
#'  \item{`USUBJID`}{Unique Subject Identifier}
#'  \item{`AGE`}{Age}
#'  \item{`AGEGR1`}{Pooled Age Group 1}
#'  \item{`AGEGR1N`}{Pooled Age Group 1 (N)}
#'  \item{`RACE`}{Race}
#'  \item{`RACEN`}{Race (N)}
#'  \item{`SEX`}{Sex}
#'  \item{`SAFFL`}{Safety Population Flag}
#'  \item{`TRTSDT`}{Date of First Exposure to Treatment}
#'  \item{`TRTEDT`}{Date of Last Exposure to Treatment}
#'  \item{`TRTP`}{Planned  Treatment}
#'  \item{`TRTPN`}{Planned  Treatment (N)}
#'  \item{`TRTA`}{Actual Treatment}
#'  \item{`TRTAN`}{Actual Treatment (N)}
#'  \item{`PARAMCD`}{Parameter Code}
#'  \item{`PARAM`}{Parameter}
#'  \item{`PARAMN`}{Parameter (N)}
#'  \item{`ADT`}{Analysis Date}
#'  \item{`ADY`}{Analysis Relative Day}
#'  \item{`ATPTN`}{Analysis Timepoint (N)}
#'  \item{`ATPT`}{Analysis Timepoint}
#'  \item{`AVISIT`}{Analysis Visit}
#'  \item{`AVISITN`}{Analysis Visit (N)}
#'  \item{`AVAL`}{Analysis Value}
#'  \item{`BASE`}{Baseline Value}
#'  \item{`BASETYPE`}{Baseline Value}
#'  \item{`CHG`}{Change from Baseline}
#'  \item{`PCHG`}{Percent Change from Baseline}
#'  \item{`VISITNUM`}{Visit Number}
#'  \item{`VISIT`}{Visit Name}
#'  \item{`VSSEQ`}{Sequence Number}
#'  \item{`ANL01FL`}{Analysis 01 - Special Interest Flag}
#'  \item{`ABLFL`}{Baseline Record Flag}
#' }
#' @keywords datasets CDISC advs
#' @name cdisc advs
"cdisc_advs"


#' Metadata describing the data, functions and arguments needed to produce your
#' results.
#'
#' @format A data frame with one row per function call and 16 variables:
#' \describe{
#'   \item{`func`}{name of the function you wish to call}
#'   \item{`df`}{data frame to pass to the function call}
#'   \item{`subset`}{filter `df` records, this is passed directly to filter,
#'   ex. `"AESER == 'Y'"`}
#'   \item{`rowvar`}{variable being summarized that will pass to the function
#'   call}
#'   \item{`rowtext`}{row label text to display in the table}
#'   \item{`row_header`}{header text to display above row summary}
#'   \item{`statlist`}{list of statistics in the analysis, see individual
#'   functions for what is available per function (e.g. `"N, n (x.x)"`)}
#'   \item{`colvar`}{variable used to determine the columns of the table}
#'   \item{`decimal`}{decimal precision}
#'   \item{`rowbyvar`}{repeat `rowvar` summary by this variable/s, comma separated
#'   for multiple (e.g. `"ETHNIC, AGEGR1"`)}
#'   \item{`tablebyvar`}{repeat the entire table summary by this variable/s,
#'   comma separated for multiple (e.g. `"ETHNIC, AGEGR1"`)}
#'   \item{`denom_df`}{used to set denominators if `df` does not contain
#'   all required records}
#' }
#' @name table_metadata
"table_metadata"


#' Metadata describing table column layouts
#'
#' This is used by `tlgsetup` to prepare you input data to support
#' the desired column layout.
#'
#' @format A data frame with one row per column for each table type
#' and six variables:
#' \describe{
#'   \item{`tbltype`}{identifier used to group a table column layout}
#'   \item{`coldef`}{distinct variable values used, typically numeric
#'   and typically a treatment or main effect variable, think `TRT01PN`}
#'   \item{`decode`}{decode of `coldef` that will display as a column header
#'   in the table}
#'   \item{`span1`}{spanning header to display across multiple columns}
#'   \item{`span2`}{spanning header to display across multiple columns,
#'   second level}
#'   \item{`span3`}{spanning header to display across multiple columns,
#'   third level}
#' }
#'
#' @name column_metadata
"column_metadata"

#' Generate Results using Table and Column Metadata
#'
#' @param table_metadata a data frame containing table metadata (see
#'   ?table_metadata for details)
#' @param column_metadata_file An excel file with the data
#'   for `column_metadata`.
#'   The file is read in with `readxl::read_excel()`. Should not be used with
#'   `column_metadata` argument. Results in a data frame containing the column
#'   metadata that is passed to `tlgsetup` (see `tlgsetup()` for details). If a
#'   `column_metadata` data frame is passed in too, this is ignored.
#' @param column_metadata A data frame containing the column metadata. This will
#'   be used in place of `column_metadata_file`.
#' @param env environment to find data frame specified in the table metadata
#'   (defaults to parent environment).
#' @param tbltype If used, this will be used to subset
#'   the `column_metadata` based on the `tbltype` column.
#' @param add_count Passed to `bind_table()` should counts be added for
#'   `tablebyvars`?
#'
#' @return dataframe of results
#' @export
generate_results <-
  function(table_metadata,
           column_metadata_file = NULL,
           column_metadata = NULL,
           env = parent.frame(),
           tbltype = NULL,
           add_count = FALSE) {
    if (is.null(table_metadata)) {
      stop("Argument table_metadata to function generate_results is required
         but no value has been supplied",
        call. = FALSE
      )
    }

    if (!("anbr" %in% names(table_metadata))) {
      table_metadata$anbr <- seq_len(nrow(table_metadata))
    }

    # check incoming metadata
    arglist <- list()
    args_to_chk <- names(formals())[names(formals()) != "..."]
    purrr::walk(
      args_to_chk,
      .f = function(x) {
        arglist[[x]] <<- eval(rlang::sym(x))
      }
    )
    check_generate_results(arglist)

    if (is.null(column_metadata) && !is.null(column_metadata_file)) {
      column_metadata <- read_excel(column_metadata_file, sheet = 1) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U2190", "\U2190")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U2192", "\U2192")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U2264", "\U2264")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U2265", "\U2265")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U2260", "\U2260")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U00b1", "\U00b1")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U03b1", "\U03b1")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U03b2", "\U03b2")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U03bc", "\U03bc")
        }) %>%
        mutate_if(is.character, function(x) {
          x %>% stringr::str_replace_all("\\\\U00ab", "\U00ab")
        })
    } else if (
      is.null(column_metadata_file) &&
        is.null(column_metadata)
    ) {
      stop(
        "Either column_metadata_file or column_metadata must be supplied
      to generate_results"
      )
    }

    chunks <- split(table_metadata, as.numeric(rownames(table_metadata)))

    # pass each record to the funtion and return one set of results for each list
    # element
    result_list <- purrr::map(seq_len(length(chunks)), function(i) {
      do_tidytlg(chunks[[i]],
        column_metadata = column_metadata,
        column_metadata_file = column_metadata_file,
        tbltype = tbltype,
        env = env
      )
    })

    tablebys <- unique(unlist(purrr::map(chunks, ~ extract2(., "tablebyvar"))))

    res <- bind_table(result_list,
      tablebyvar = tablebys,
      add_count = add_count,
      colvar = "colnbr",
      add_format = FALSE
    ) %>%
      add_format(tableby = tablebys)

    attr(res, "column_metadata") <- column_metadata %>%
      # Per JCEV-16: Any non breaking space(utf8 160) should be collapsed into space
      # (utf8 32)
      mutate(
        tbltype = gsub("\u00A0", " ", tbltype)
      ) %>%
      filter(tbltype == !!tbltype)

    structure(
      res,
      class = c("tidytlg.metadata_table", class(res))
    )
  }

do_tidytlg <-
  function(x,
           column_metadata,
           column_metadata_file,
           tbltype,
           env = parent.frame()) {
    # The dfs are characters, so this takes them and turns them into the actual
    # data.frames
    x <- as.list(x)
    x <- x[!purrr::map_lgl(x, is.na)]
    if ("statlist" %in% names(x)) {
      st <- if (class(x$statlist)[1] == "list") x$statlist[[1]] else x$statlist
    }
    x <- purrr::map(x[names(x) != "statlist"], unlist)
    if (exists("st")) x$statlist <- st
    x$df <- get(x$df, envir = env) %>%
      tlgsetup(
        var = x$colvar,
        column_metadata = column_metadata,
        column_metadata_file = column_metadata_file,
        tbltype = tbltype
      )
    if (!is.null(x$denom_df) && exists(x$denom_df, envir = env)) {
      x$denom_df <- get(x$denom_df, envir = env) %>%
        tlgsetup(
          var = x$colvar,
          column_metadata = column_metadata,
          column_metadata_file = column_metadata_file,
          tbltype = tbltype
        )
    }

    x$colvar <- "colnbr"
    # If a value is a function argument, pass it into the function. Otherwise
    # append it to the output table
    do.call(x$func, x)
  }

#' @title Adds bottom borders to a `huxtable`
#'
#' @details
#' Adds bottom borders to a `huxtable` based on
#' a matrix indicating where the borders should be put.
#'
#' @details
#' This function is responsible for adding bottom borders to a `huxtable` object.
#' It supports borders spanning multiple columns and borders that are under neighbouring,
#' single cells (or merged cells), but separate (see examples).
#'
#' This feature has limitations. Mainly, it does not support both versions of the borders
#' (continuous and separate) on the same line. In such a case, the borders in the resulting
#' RTF look misaligned.
#'
#' @section `border_matrix` details:
#' You mark where the bottom borders should go in the table by passing a matrix.
#' The matrix has to have the same number of columns as the passed `huxtable`
#' and the number of rows lower by one than the passed `huxtable`. Each cell
#' in `border_matrix` corresponds to a cell in `huxtable` (starting from the first row).
#'
#' Internally, the function adds the first row of 0s to `border_matrix` before the execution.
#' At that point, `border_matrix`'s dimensions match `ht`'s dimensions.
#'
#' Table:
#' | foo | bar |
#' | --- | --- |
#' | `baz` | `bim` |
#'
#' A border matrix:
#' | 1 | 1 |
#' | - | - |
#' | 0 | 0 |
#'
#' The above border matrix puts a bottom border across the entire first row
#' and no borders in the second row.
#'
#' A border matrix:
#' | 1 | 2 |
#' | - | - |
#' | 0 | 0 |
#'
#' The above border matrix puts one border under the first cell in the first row;
#' and another border (separate from the first one) under the second cell
#' in the first row. The second row stays without any borders.
#'
#' @section
#' Functions transforming the border matrix:
#'
#' The below functions can be passed to [gentlg()]'s
#' `border_fns` argument to modify how `gentlg` renders
#' the borders under the cells.
#'
#' Border functions:
#' * [no_borders()]
#' * [spanning_borders()]
#' * [col_borders()]
#' * [single_border()]
#' * [row_border()]
#'
#' `border_fns` will accept your own, custom functions as long as
#' they adhere to the format.
#' All the functions passed to `border_fns` need to accept two arguments:
#' * the first - the printed `huxtable` object,
#' * the second - a border matrix.
#'
#' They also must return a matrix interpreted the same way as `border_matrix`
#' passed to `add_bottom_borders` or [gentlg()].
#'
#' @param ht `huxtable` A `huxtable` object
#' @param border_matrix (optional) `matrix` A matrix indicating where to add the bottom
#' borders. If `NULL`, then no borders are added.
#' @param transform_fns (optional) `list` of `function` A list of functions applied to the
#' `border_matrix`. The functions have to accept two arguments:
#' 1. The `huxtable`.
#' 1. The `border_matrix` with dimensions matching `huxtable`.
#'
#' The functions in the list are applied sequentially to `border_matrix`.
#'
#' @return A `huxtable` with added borders.
#'
#' @examples
#' border_matrix <- matrix(c(1, 1, 2, 0, 1, 1, 0, 0, 0), nrow = 3, ncol = 3)
#' ht <- huxtable::as_huxtable(
#'   data.frame(a = c(1, 2, 3), b = c("a", "b", "c"), c = c(TRUE, FALSE, TRUE))
#' )
#' # By default adds no borders
#' add_bottom_borders(ht, border_matrix)
#' # Adds spanning borders under cells with text in the second row
#' add_bottom_borders(ht, transform_fns = list(spanning_borders(2)))
#' # Adds spanning borders under cells with text in the second row and a border
#' # under a cell in row 3 and column 3
#' add_bottom_borders(ht, transform_fns = list(spanning_borders(2), single_border(3, 3)))
#'
#' final <- data.frame(
#'   label = c(
#'     "Overall", "Safety Analysis Set",
#'     "Any Adverse event{\\super a}", "- Serious Adverse Event"
#'   ),
#'   Drug_A = c("", "40", "10 (25%)", "0"),
#'   Drug_B = c("", "40", "10 (25%)", "0"),
#'   anbr = c(1, 2, 3, 4),
#'   roworder = c(1, 1, 1, 1),
#'   boldme = c(1, 0, 0, 0),
#'   newrows = c(0, 0, 1, 0),
#'   indentme = c(0, 0, 0, 1),
#'   newpage = c(0, 0, 0, 0)
#' )
#' # Add spanning bottom borders under the cells in the first row
#' gentlg(
#'   huxme = final,
#'   wcol = c(0.70, 0.15, 0.15),
#'   file = "TSFAEX",
#'   colheader = c("", "Drug A", "Drug B"),
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   ),
#'   border_fns = list(no_borders, spanning_borders(1))
#' )
#'
#' # Tables with no bottom borders
#' gentlg(
#'   huxme = final,
#'   wcol = c(0.70, 0.15, 0.15),
#'   file = "TSFAEX",
#'   colheader = c("", "Drug A", "Drug B"),
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   ),
#'   border_fns = list(no_borders)
#' )
#'
#' # Tables with a border under cell in the 3nd row and 3rd column,
#' # and borders under cells in the first row
#' gentlg(
#'   huxme = final,
#'   wcol = c(0.70, 0.15, 0.15),
#'   file = "TSFAEX",
#'   colheader = c("", "Drug A", "Drug B"),
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   ),
#'   border_fns = list(no_borders, spanning_borders(1), single_border(3, 3))
#' )
#'
#' # We discourage, but you can pass the border matrix directly
#' mat <- matrix(rep(0, 8 * 3), ncol = 3, nrow = 8)
#' mat[3, 3] <- 1
#' gentlg(
#'   huxme = final,
#'   wcol = c(0.70, 0.15, 0.15),
#'   file = "TSFAEX",
#'   colheader = c("", "Drug A", "Drug B"),
#'   title = "This is Amazing Demonstration 1",
#'   footers = c(
#'     "Note: For demonstrative purposes only",
#'     "{\\super a} Subjects are counted once for any given event."
#'   ),
#'   bottom_borders = mat, # The same as a single border under 3nd row and 3rd column
#'   border_fns = list()
#' )
#'
#' # clean up.
#' file.remove("tsfaex.rtf")
#'
#' @export
add_bottom_borders <- function(ht, border_matrix = no_borders(ht), transform_fns = list()) {
  if (is.null(border_matrix) && is.null(transform_fns)) {
    return(ht)
  } else if (is.null(border_matrix)) {
    border_matrix <- no_borders(ht)
  }
  for (transform_fn in transform_fns) {
    border_matrix <- transform_fn(ht, border_matrix)
  }
  border_matrix <- rbind(rep(0, ncol(border_matrix)), border_matrix)

  for (row in seq_len(nrow(border_matrix))) {
    # The below enforces an "all or nothing" behaviour on using
    # one of the border styles, because mixing them in the
    # scope of a row gives bad results - the borders
    # are misaligned and of slightly different width
    if (should_use_internal_borders(border_matrix[row, ])) {
      ht <- huxtable::map_contents(
        ht = ht,
        row = row,
        fn = function(ht, rows, cols, current) {
          res <- current
          res[border_matrix[rows, ] != 0] <- sprintf("%s %s", r"{\brdrb\brdrs}", res[border_matrix[rows, ] != 0])
          res
        }
      )
      huxtable::right_padding(ht)[row, border_matrix[row, ] != 0] <- 3
      # This is in fact interpreted as left padding by Word...
      huxtable::top_padding(ht)[row, border_matrix[row, ] != 0] <- 3
      huxtable::top_padding(ht)[row, 1] <- 0
    } else {
      huxtable::bottom_border(ht)[row, border_matrix[row, ] != 0] <- huxtable::brdr(thickness = 0.9, style = "solid")
    }
  }
  ht
}

should_use_internal_borders <- function(row) {
  col_seq <- seq_len(length(row))
  for (col in col_seq[-1]) {
    if (row[col] != 0 && row[col - 1] != 0 && row[col - 1] != row[col]) {
      return(TRUE)
    }
  }
  FALSE
}

#' Removes all borders from the table
#'
#' @param ht `huxtable` object.
#' @param matrix `matrix` of bottom borders. Ignored. Included for the sake
#' of compatibility with the interface of all border mutating functions.
#' @export
#' @family border_functions
no_borders <- function(ht, matrix = NULL) {
  if (is.null(ht)) {
    return(NULL)
  }
  ht_dims <- dim(ht)
  matrix(rep(0, (ht_dims[1] - 1) * ht_dims[2]), nrow = ht_dims[1] - 1, ncol = ht_dims[2])
}

#' Adds borders under cells in a row, excluding the first column.
#'
#' Adds borders under cells that are not empty in a given row,
#' omitting the first column of the row.
#' The borders do not touch each other - they are separate.
#'
#' @param row `numeric` the row of the table
#' @param cols `numeric` the columns of the row to consider
#' @export
#' @family border_functions
spanning_borders <- function(row, cols = c(-1)) {
  function(ht, matrix) {
    last_num <- matrix[row][1]
    r <- trimws(gsub("\\\\keepn\\\\trhdr", "", ht[row + 1, ]))
    for (col in seq_len(length(r))[cols]) {
      if (r[col] != "") {
        if (col == 1 || (r[col - 1] != r[col])) {
          last_num <- last_num + 1
        }
        matrix[row, col] <- last_num
      }
    }
    matrix
  }
}

#' Adds borders under cells in a column
#'
#' @param col `numeric` the column of the table
#' @param rows `numeric` the range of rows to include
#' @export
#' @family border_functions
col_borders <- function(col, rows) {
  rows <- rows + 1
  function(ht, matrix) {
    for (row in rows) {
      matrix[row, col] <- 1
    }
    matrix
  }
}

#' Adds a border under a cell
#'
#' @param row `numeric` the row of the cell
#' @param col `numeric` the column of the cell
#' @export
#' @family border_functions
single_border <- function(row, col) {
  function(ht, matrix) {
    border <- 1
    if (col > 1) {
      border <- border + matrix[row, col - 1]
    }
    if (col < ncol(matrix)) {
      border <- border + matrix[row, col + 1]
    }
    matrix[row, col] <- border
    matrix
  }
}

#' Adds a continuous bottom border under a row
#'
#' @param row `numeric` the row of the table
#' @export
#'
#' @family border_functions
row_border <- function(row) {
  function(ht, matrix) {
    matrix[row, ] <- 1
    matrix
  }
}

#' Adds bottom borders according to the old formatting
#'
#' This function is vectorized over its arguments.
#'
#' @param ht the `hux` object passed to [gentlg()]
#' @param colspan `colspan` argument to [gentlg()]
#' @param colheader `colheader` argument to [gentlg()]
#' @param tlf `character` type of the output
#' @return a bottom border matrix for use with [add_bottom_borders()] or `NULL`
#' if `ht` is `NULL`
#'
#' @keywords internal
old_format <- function(ht, colspan, colheader, tlf) {
  if (is.null(ht)) {
    return(NULL)
  }

  border_matrix <- no_borders(ht)

  # Attach borders to colspan and colheader rows
  colspan_length <- length(colspan)
  if (is_listing(tlf)) {
    for (row in 1:(1 + colspan_length)) {
      border_matrix <- spanning_borders(
        row,
        cols = seq_len(ncol(border_matrix))
      )(ht, border_matrix)
    }
  } else {
    for (row in 1:(1 + colspan_length)) {
      border_matrix <- spanning_borders(
        row,
        cols = c(-1)
      )(ht, border_matrix)
    }
  }

  border_matrix
}

# This file includes utility functions from the huxtable package for creating rtf output
#' @references \url{https://github.com/hughjonesd/huxtable}
#'

# merge header updates rtf output to add merging tags to the title row
#' merge_header
#'
#' Merge the columns in the header row
#'
#' @param result The current RTF output
#'
#' @return Returns the RTF output but with the Title rows merged
#' @noRd
merge_header <- function(result) {
  result_sectioned <- result %>%
    stringr::str_split("\\\\row") %>%
    base::unlist() # breakes apart by section
  result_sectioned[1] <- result_sectioned[1] %>%
    stringr::str_replace_all("\\\\clbrdrt", "\\\\clmrg\\\\clbrdrt") %>%
    stringr::str_replace("\\\\clmrg\\\\clbrdrt", "\\\\clmgf\\\\clbrdrt")
  base::paste0(result_sectioned, collapse = "\\row")
}

#' pad_header
#'
#' Adds padding to `colvar` row to add line break between `colvar` headers
#'
#' @param result The current RTF output
#' @param nheader Number of headers which is equal to `colspan` + 1
#' @param header_pad a list of row numbers to add pading to columns
#'
#' @return Returns the RTF output but with padding added to colvar column
#' @noRd
pad_header <- function(result, nheader, header_pad) {
  result_sectioned <- result %>%
    stringr::str_split("\\\\row") %>%
    base::unlist() # breakes apart by section

  section_selection <- header_pad[header_pad %in% 2:(nheader + 1)]

  result_sectioned[section_selection] <- result_sectioned[section_selection] %>%
    stringr::str_replace_all("\\\\cellx", "\\\\clpadt67\\\\clpadft3\\\\clpadr67\\\\clpadfr3\\\\cellx") %>%
    stringr::str_replace(
      "\\\\clpadt67\\\\clpadft3\\\\clpadr67\\\\clpadfr3\\\\cellx", "\\\\clpadr67\\\\clpadfr3\\\\cellx"
    )

  base::paste0(result_sectioned, collapse = "\\row")
}

ncharw <- function(x) nchar(x, type = "width")

blank_where <- function(text, cond) {
  stopifnot(length(text) == length(cond))
  text[cond] <- ""
  text
}

str_rep <- function(x, times) {
  mapply(function(s, t) paste0(rep(s, t), collapse = ""), x, times)
}

# pinched from HMS. Registers the method or sets a hook to
# register it on load of other package
register_s3_method <- function(pkg, generic, class = "huxtable") {
  assertthat::assert_that(
    assertthat::is.string(pkg),
    assertthat::is.string(generic)
  )
  fun <- get(paste0(generic, ".", class), envir = parent.frame())

  if (pkg %in% loadedNamespaces()) {
    registerS3method(generic, class, fun, envir = asNamespace(pkg))
  }
  setHook(packageEvent(pkg, "onLoad"), function(...) {
    registerS3method(generic, class, fun, envir = asNamespace(pkg))
  })
}

utf8_to_rtf <- function(mx) {
  utf8_codes <- function(x) utf8ToInt(enc2utf8(x))

  rtf_encode <- function(x) {
    code <- utf8_codes(x)
    x <- strsplit(x, split = "")[[1]]
    x[code > 127L & code <= 32767L] <- code[code > 127L & code <= 32767L]
    x[code > 32767L] <- code[code > 32767L] - 65536L
    x[code > 127L] <- paste0("\\u", x[code > 127L], "?")
    paste0(x, collapse = "")
  }

  needs_conv <- vapply(
    c(mx), function(x) any(utf8_codes(x) > 127L),
    logical(1)
  )
  mx[needs_conv] <- vapply(
    mx[needs_conv], rtf_encode,
    character(1)
  )

  mx
}


# return character matrix of formatted contents, suitably escaped
clean_contents <- function(ht,
                           type = c(
                             "latex", "html", "screen", "markdown",
                             "word", "excel", "rtf"
                           ),
                           ...) {
  type <- match.arg(type)
  contents <- as.matrix(as.data.frame(ht))

  for (col in seq_len(ncol(contents))) {
    for (row in seq_len(nrow(contents))) {
      cell <- contents[row, col]
      num_fmt <- huxtable::number_format(ht)[[row, col]]
      cell <- format_numbers(cell, num_fmt)
      if (is.na(cell)) cell <- huxtable::na_string(ht)[row, col]
      contents[row, col] <- as.character(cell)
    }
  }
  contents[is.na(contents)] <- huxtable::na_string(ht)

  for (col in seq_len(ncol(contents))) {
    if (type %in% c("latex", "html", "rtf")) {
      to_esc <- huxtable::escape_contents(ht)[, col]
      contents[to_esc, col] <- huxtable::sanitize(contents[to_esc, col], type)
    }
    # has to be after sanitization because we add &nbsp; for HTML
    # later we can just use align for this:
    pad_chars <- 0
    align_pad <- ncharw(huxtable::align(ht)[, col]) == 1
    pad_chars[align_pad] <- huxtable::align(ht)[align_pad, col]
    contents[, col] <- decimal_pad(contents[, col], pad_chars, type)
  }

  if (type == "rtf") {
    contents <- utf8_to_rtf(contents)
  }

  contents
}

format_color <- function(r_color, default = "white") {
  r_color[is.na(r_color)] <- default
  apply(grDevices::col2rgb(r_color), 2, paste0, collapse = ", ")
}

get_visible_borders <- function(ht) {
  dc <- display_cells(ht)

  # a vertical border is hidden, if it is shadowed by a cell to its left
  vert_borders <- attr(ht, "lr_borders")$thickness
  left_shadowed <- dc[dc$display_col < dc$col, ]
  left_shadowed <- as.matrix(left_shadowed[c("row", "col")])
  vert_borders[left_shadowed] <- 0

  # a horizontal border is hidden, if it is shadowed by a cell above it
  horiz_borders <- attr(ht, "tb_borders")$thickness
  top_shadowed <- dc[dc$display_row < dc$row, ]
  top_shadowed <- as.matrix(top_shadowed[c("row", "col")])
  horiz_borders[top_shadowed] <- 0

  list(vert = vert_borders, horiz = horiz_borders)
}

# returns two rows(+1),cols(+1) arrays of border colors.
# right and top borders have priority.
# A border of 0 can still have a color.
collapsed_border_colors <- function(ht) {
  result <- do_collapse(ht, get_all_border_colors, default = NA)
  result$vert <- result$right
  result$vert[is.na(result$right)] <- result$left[is.na(result$right)]
  result$horiz <- result$bottom
  result$horiz[is.na(result$bottom)] <- result$top[is.na(result$bottom)]

  result[c("vert", "horiz")]
}

# returns two rows(+1),cols(+1) arrays of border styles.
# Non-"solid" styles have priority;
# if two styles are non-"solid" then right and top has priority
# A border of 0 can still have a style.
collapsed_border_styles <- function(ht) {
  result <- do_collapse(ht, get_all_border_styles, default = "solid")
  result$vert <- result$right
  result$vert[result$right == "solid"] <- result$left[result$right == "solid"]
  result$horiz <- result$bottom
  result$horiz[result$bottom == "solid"] <- result$top[result$bottom == "solid"]

  result[c("vert", "horiz")]
}

do_collapse <- function(ht, prop_fun, default) {
  res <- list()
  res$top <- res$left <- res$right <- res$bottom <- matrix(
    default, nrow(ht),
    ncol(ht)
  )
  dc <- display_cells(ht, all = TRUE)
  # provides large speedup:
  dc <- as.matrix(dc[, c(
    "row", "col", "display_row", "display_col", "end_row",
    "end_col"
  )])
  dc_idx <- dc[, c("display_row", "display_col"), drop = FALSE]
  dc_map <- matrix(seq_len(nrow(ht) * ncol(ht)), nrow(ht), ncol(ht))
  dc_map <- dc_map[dc_idx]

  at <- list()
  at$left <- dc[, "col"] == dc[, "display_col"]
  at$right <- dc[, "col"] == dc[, "end_col"]
  at$top <- dc[, "row"] == dc[, "display_row"]
  at$bottom <- dc[, "row"] == dc[, "end_row"]

  properties <- prop_fun(ht)
  for (side in names(at)) {
    at_side <- at[[side]]
    res[[side]][at_side] <- properties[[side]][dc_map][at_side]
  }

  res$left <- cbind(res$left, default)
  res$right <- cbind(default, res$right)
  res$top <- rbind(res$top, default)
  res$bottom <- rbind(default, res$bottom)

  res
}

# Format numeral generics
numeral_formatter <- function(x) {
  UseMethod("numeral_formatter")
}

#' @export
numeral_formatter.default <- function(x) {
  stop("Unrecognized number_format. Please use a number, string or function.")
}

# If we are a function then return output from the function
#' @export
numeral_formatter.function <- function(x) {
  x
}

#' @export
numeral_formatter.character <- function(x) {
  function(numeral) sprintf(x, numeral)
}

#' @export
numeral_formatter.numeric <- function(x) {
  function(numeral) formatC(round(numeral, x), format = "f", digits = x)
}


# find each numeric substring, and replace it:
format_numbers <- function(string, num_fmt) {
  if (is.na(string)) {
    return(NA_character_)
  }

  # ! is.function avoids a warning if num_fmt is a function:
  if (!is.function(num_fmt) && is.na(num_fmt)) {
    return(string)
  }

  format_numeral <- numeral_formatter(num_fmt)
  # Breakdown:
  # -?                    optional minus sign
  # [0-9]*                followed by any number of digits
  # \\.?                  optionally followed by a decimal
  # [0-9]+                which may also be followed by any number of digits
  # ([eE]-?[0-9]+)?       optionally including e or E as in scientific notation
  #                       along with (optionally) a sign preceding the digits
  #                       specifying the level of the exponent.
  stringr::str_replace_all(
    string, "-?[0-9]*\\.?[0-9]+([eE][+-]?[0-9]+)?",
    function(x) format_numeral(as.numeric(x))
  )
}


decimal_pad <- function(col, pad_chars, type) {
  # where pad_chars is NA we do not pad
  orig_col <- col
  na_pad <- is.na(pad_chars)
  col <- col[!na_pad]
  pad_chars <- pad_chars[!na_pad]
  if (length(col) == 0) {
    return(orig_col)
  }

  find_pos <- function(string, char) {
    regex <- gregexpr(char, string, fixed = TRUE)[[1]]
    regex[length(regex)]
  }
  pos <- mapply(find_pos, col, pad_chars)
  nchars <- nchar(col, type = "width")
  # take the biggest distance from the decimal point
  pos[pos == -1L] <- nchars[pos == -1L] + 1
  chars_after__ <- nchars - pos

  pad_n_spaces <- max(chars_after__) - chars_after__
  pad_char <- switch(type,
    "html"   = "&nbsp;",
    "latex"  = "~",
    "screen" = "\u00a0", # screen non-breaking space
    "rtf"    = "\\~",
    " "
  )
  col <- paste0(col, str_rep(pad_char, pad_n_spaces))

  orig_col[!na_pad] <- col
  orig_col
}


check_positive_dims <- function(ht) {
  if (nrow(ht) < 1) {
    warning("huxtable has zero rows")
    return(FALSE)
  }
  if (ncol(ht) < 1) {
    warning("huxtable has zero columns")
    return(FALSE)
  }

  TRUE
}


# return data frame mapping real cell positions to cells displayed.
# cells, including those shadowed by others.
# data frame is ordered by row then column, i.e. the same as 1-based
# indexing into a matrix
# columns are row, col (of real cell);
# shadowed if cell is covered by another, the "display cell"; if not,
# it is its own "display cell";
# display_row, display_col, rowspan, colspan, end_row, end_col of the
# display cell.
display_cells <- function(ht, all = TRUE,
                          new_rowspan = huxtable::rowspan(ht),
                          new_colspan = huxtable::colspan(ht)) {
  rowspan <- new_rowspan
  colspan <- new_colspan
  display_row <- end_row <- row <- row(ht)
  display_col <- end_col <- col <- col(ht)
  displayers <- rowspan > 1 | colspan > 1
  touched <- shadowed <- matrix(FALSE, nrow(ht), ncol(ht))
  for (idx in which(displayers)) {
    rr <- row[idx]
    cc <- col[idx]
    end_r <- rr + rowspan[idx] - 1
    end_c <- cc + colspan[idx] - 1
    da_rows <- seq(rr, end_r)
    da_cols <- seq(cc, end_c)
    if (any(touched[da_rows, da_cols])) {
      cli::cli_abort(c(
        "Overlapping multirow/multicolumn cells in",
        " [{da_rows}, {da_cols}] of huxtable"
      ))
    }
    display_row[da_rows, da_cols] <- rr
    display_col[da_rows, da_cols] <- cc
    rowspan[da_rows, da_cols] <- rowspan[idx]
    colspan[da_rows, da_cols] <- colspan[idx]
    end_row[da_rows, da_cols] <- end_r
    end_col[da_rows, da_cols] <- end_c
    shadowed[da_rows, da_cols] <- TRUE
    touched[da_rows, da_cols] <- TRUE
    shadowed[rr, cc] <- FALSE
  }

  dcells <- data.frame(
    row         = c(row),
    col         = c(col),
    rowspan     = c(rowspan),
    colspan     = c(colspan),
    display_row = c(display_row),
    display_col = c(display_col),
    shadowed    = c(shadowed),
    end_row     = c(end_row),
    end_col     = c(end_col)
  )
  if (!all) dcells <- dcells[!dcells$shadowed, ]

  return(dcells)
}


get_caption_hpos <- function(ht) {
  hpos <- sub(".*(left|center|right)", "\\1", huxtable::caption_pos(ht))
  if (!hpos %in% c("left", "center", "right")) hpos <- position_no_wrap(ht)

  hpos
}

position_no_wrap <- function(ht) {
  switch(huxtable::position(ht),
    "wrapleft"  = "left",
    "wrapright" = "right",
    huxtable::position(ht)
  )
}

real_align <- function(ht) {
  # align(ht) can be e.g. "." for aligning on a decimal point
  al <- huxtable::align(ht)
  al[!al %in% c("left", "center", "right")] <- "right"

  al
}

smart_hux_from_df <- function(dfr) {
  col_nchars <- sapply(dfr, function(col) {
    max(nchar(as.character(col),
      type = "width"
    ))
  })

  ht <- huxtable::as_hux(dfr, add_colnames = TRUE, autoformat = TRUE)

  huxtable::wrap(ht)[-1, col_nchars > 15] <- TRUE
  width <- sum(col_nchars) / 90
  huxtable::width(ht) <- min(1, max(0.2, width))

  ht
}

# get_set
get_all_borders <- function(ht, row, col, drop = TRUE) {
  list(
    left   = huxtable::left_border(ht)$thickness[row, col, drop = drop],
    right  = huxtable::right_border(ht)$thickness[row, col, drop = drop],
    top    = huxtable::top_border(ht)$thickness[row, col, drop = drop],
    bottom = huxtable::bottom_border(ht)$thickness[row, col, drop = drop]
  )
}

get_all_border_colors <- function(ht, row, col, drop = TRUE) {
  list(
    left   = huxtable::left_border_color(ht)[row, col, drop = drop],
    right  = huxtable::right_border_color(ht)[row, col, drop = drop],
    top    = huxtable::top_border_color(ht)[row, col, drop = drop],
    bottom = huxtable::bottom_border_color(ht)[row, col, drop = drop]
  )
}

get_all_border_styles <- function(ht, row, col, drop = TRUE) {
  list(
    left   = huxtable::left_border_style(ht)[row, col, drop = drop],
    right  = huxtable::right_border_style(ht)[row, col, drop = drop],
    top    = huxtable::top_border_style(ht)[row, col, drop = drop],
    bottom = huxtable::bottom_border_style(ht)[row, col, drop = drop]
  )
}

get_all_padding <- function(ht, row, col, drop = TRUE) {
  list(
    left   = huxtable::left_padding(ht)[row, col, drop = drop],
    right  = huxtable::right_padding(ht)[row, col, drop = drop],
    top    = huxtable::top_padding(ht)[row, col, drop = drop],
    bottom = huxtable::bottom_padding(ht)[row, col, drop = drop]
  )
}

rtf_fc_tables <- function(..., extra_fonts = "Times",
                          extra_colors = character(0)) {
  hts <- list(...)
  assertthat::assert_that(all(sapply(hts, huxtable::is_huxtable)))

  fonts <- unlist(lapply(hts, function(ht) huxtable::font(ht)))
  fonts <- unique(c(extra_fonts, fonts))
  fonts <- stats::na.omit(fonts)

  colors <- unlist(lapply(hts, function(ht) {
    c(
      huxtable::text_color(ht), huxtable::background_color(ht),
      unlist(collapsed_border_colors(ht))
    )
  }))
  colors <- unique(c(extra_colors, colors))
  colors <- stats::na.omit(colors)

  result <- list()
  result$fonts <- fonts
  result$colors <- colors
  class(result) <- "rtfFCTables"

  result
}

custom_to_rtf <- function(ht, fc_tables = rtf_fc_tables(ht), watermark,
                          nheader, tlf, header_pad = NULL, ...) {
  assertthat::assert_that(inherits(fc_tables, "rtfFCTables"))
  color_index <- function(color) {
    res <- match(color, fc_tables$colors)
    if (any(is.na(res) & !is.na(color))) {
      warning(
        "Color not found in color table.\n",
        "(Did you change colors after calling `rtf_fc_tables`?)"
      )
    }
    res
  }

  cb <- get_visible_borders(ht)
  cbc <- collapsed_border_colors(ht)
  cbs <- collapsed_border_styles(ht)
  bgc <- huxtable::background_color(ht)
  tc <- huxtable::text_color(ht)

  ## MAKE CELLX DEFINITIONS ----
  left_merge <- ifelse(huxtable::colspan(ht) > 1, "\\clmgf", "")
  top_merge <- ifelse(huxtable::rowspan(ht) > 1, "\\clvmgf", "")
  dc <- display_cells(ht, all = TRUE)
  right_merge <- ifelse(dc$col > dc$display_col, "\\clmrg", "")
  bottom_merge <- ifelse(dc$row > dc$display_row, "\\clvmrg", "")
  merge_def <- paste0(left_merge, top_merge, right_merge, bottom_merge)

  bdr_width_vert <- paste0("\\brdrw", cb$vert * 20)
  bdr_width_horiz <- paste0("\\brdrw", cb$horiz * 20)
  bdr_style_map <- c(
    "solid"  = "\\brdrs",
    "double" = "\\brdrdb",
    "dashed" = "\\brdrdash",
    "dotted" = "\\brdrdot"
  )
  bdr_style_vert <- bdr_style_map[cbs$vert]
  bdr_style_horiz <- bdr_style_map[cbs$horiz]
  bdr_color_vert <- sprintf("\\brdrcf%d", color_index(cbc$vert))
  bdr_color_horiz <- sprintf("\\brdrcf%d", color_index(cbc$horiz))
  bdr_color_vert <- blank_where(bdr_color_vert, is.na(cbc$vert))
  bdr_color_horiz <- blank_where(bdr_color_horiz, is.na(cbc$horiz))

  # these are matrices (horiz = nr+1 * nc, vert = nr * nc+1).
  # For cell (i, j), top and left are i, j; right is i,
  # j+1; bottom is i+1,j; in respective matrices
  bdr_def_vert <- paste0(bdr_style_vert, bdr_width_vert, bdr_color_vert)
  bdr_def_horiz <- paste0(bdr_style_horiz, bdr_width_horiz, bdr_color_horiz)
  dim(bdr_def_vert) <- dim(cb$vert)
  dim(bdr_def_horiz) <- dim(cb$horiz)

  bdr_def_left <- bdr_def_vert[, -ncol(bdr_def_vert), drop = FALSE]
  bdr_def_right <- bdr_def_vert[, -1, drop = FALSE]
  bdr_def_top <- bdr_def_horiz[-nrow(bdr_def_horiz), , drop = FALSE]
  bdr_def_bottom <- bdr_def_horiz[-1, , drop = FALSE]

  bdr_def_left <- blank_where(bdr_def_left, cb$vert[
    , -ncol(cb$vert),
    drop = FALSE
  ] == 0)
  bdr_def_right <- blank_where(bdr_def_right, cb$vert[, -1, drop = FALSE] == 0)
  bdr_def_top <- blank_where(bdr_def_top, cb$horiz[
    -nrow(cb$horiz), ,
    drop = FALSE
  ] == 0)
  bdr_def_bottom <- blank_where(bdr_def_bottom, cb$horiz[
    -1, ,
    drop = FALSE
  ] == 0)

  bdr_def_left <- paste0("\\clbrdrl", bdr_def_left)
  bdr_def_right <- paste0("\\clbrdrr", bdr_def_right)
  bdr_def_top <- paste0("\\clbrdrt", bdr_def_top)
  bdr_def_bottom <- paste0("\\clbrdrb", bdr_def_bottom)
  bdr_def <- paste0(bdr_def_top, bdr_def_left, bdr_def_bottom, bdr_def_right)

  bg_def <- sprintf("\\clcbpat%d", color_index(bgc))
  bg_def <- blank_where(bg_def, is.na(bgc))

  valign_map <- c(
    top = "\\clvertalt", middle = "\\clvertalc",
    bottom = "\\clvertalb"
  )
  valign_def <- valign_map[huxtable::valign(ht)]
  # also handles rotation:
  valign_def[huxtable::rotation(ht) == 90] <- "\\cltxbtlr"
  valign_def[huxtable::rotation(ht) == 270] <- "\\cltxtbrl"

  wrap_def <- ifelse(huxtable::wrap(ht), "", "\\clNoWrap")

  pad_def <- sprintf(
    "\\clpadfl3\\clpadl%d \\clpadft3\\clpadt%d \\clpadfb3\\clpadb%d \\clpadfr3\\clpadr%d ",
    huxtable::left_padding(ht) * 20,
    huxtable::top_padding(ht) * 20,
    huxtable::bottom_padding(ht) * 20,
    huxtable::right_padding(ht) * 20
  )

  table_width <- huxtable::width(ht)
  col_width <- huxtable::col_width(ht)

  col_width <- if (is.numeric(col_width)) {
    col_width
  } else if (all(grepl("pt", col_width))) {
    as.numeric(sub("((\\d|\\.)+).*", "\\1", col_width)) * 20
  } else {
    if (!all(is.na(col_width))) {
      warning("to_rtf can only handle numeric or \"pt\" col_width")
    }
    rep(1 / ncol(ht), ncol(ht))
  }

  if (!is.numeric(table_width)) {
    warning("to_rtf can only handle numeric table width")
    table_width <- huxtable::get_default_properties("width")[[1]]
  }
  text_width_twips <- 6 * 72 * 20
  col_width <- col_width * text_width_twips * table_width
  # \cellx specifies the position of the RH cell edge:
  right_edges <- ceiling(cumsum(col_width))

  cellx_def <- sprintf("\\cellx%d ", right_edges)

  # cellx_def has to go along rows:
  cellx <- paste0(
    merge_def, bdr_def, bg_def, valign_def, wrap_def, pad_def,
    rep(cellx_def, each = nrow(ht))
  )

  dim(cellx) <- dim(ht)

  ## MAKE CELL CONTENTS ----
  cc <- clean_contents(ht, type = "rtf")
  ## removed brackets "{}"
  cells <- paste0("", cc, "")
  cells[huxtable::bold(ht)] <- paste0(
    "\\b ", cells[huxtable::bold(ht)],
    "\\b0"
  )
  cells[huxtable::italic(ht)] <- paste0(
    "\\i ", cells[huxtable::italic(ht)],
    "\\i0"
  )
  fs <- ceiling(huxtable::font_size(ht) * 2) # "half-points", must be integer
  ## removed "{}", put space at the end (so that indentation works)
  cells[!is.na(fs)] <- paste0(
    "\\fs", fs[!is.na(fs)], " ", cells[!is.na(fs)],
    " "
  )
  cells[!is.na(tc)] <- paste0(
    "{\\cf", match(tc[!is.na(tc)], fc_tables$colors), " ",
    cells[!is.na(tc)], "}"
  )

  ft <- huxtable::font(ht)
  findex <- match(ft[!is.na(ft)], fc_tables$fonts) - 1
  if (any(is.na(findex))) {
    warning(
      "Font not found in font table.\n",
      "(Did you change a font after calling `rtf_fc_table`?)"
    )
  }
  cells[!is.na(ft)] <- paste0("{\\f", findex, " ", cells[!is.na(ft)], "}")

  align_map <- c("left" = "\\ql", "center" = "\\qc", "right" = "\\qr")
  cells <- paste0(align_map[real_align(ht)], cells)
  cells <- paste0("\\pard\\intbl", cells, "\\cell")
  dim(cells) <- dim(ht)

  if (!is.null(watermark)) {
    watermark_bf <-
      paste0(
        "{{\\shp{\\*\\shpinst\\shpleft0\\shptop0\\shpright13921\\shpbottom",
        "2320\\shpfhdr0\\shpbxcolumn\\shpbxignore\\shpbypara\\shpbyignore\\shpwr3
    {\\sp{\\sn gtextUNICODE}   {\\sv "
      )
    watermark_af <- "}}
    {\\sp{\\sn gtextSize}      {\\sv 65536}}
    {\\sp{\\sn gtextFont}      {\\sv Calibri}}
    {\\sp{\\sn gtextFStretch}  {\\sv 1}}
    {\\sp{\\sn shapeType}      {\\sv 136}}
    {\\sp{\\sn fFlipH}         {\\sv 0}}
    {\\sp{\\sn fFlipV}         {\\sv 0}}
    {\\sp{\\sn rotation}       {\\sv 20643840}}
    {\\sp{\\sn fGtext}         {\\sv 1}}
    {\\sp{\\sn fillColor}      {\\sv 12632256}}
    {\\sp{\\sn fillOpacity}    {\\sv 15000}}
    {\\sp{\\sn fLine}          {\\sv 0}}
    {\\sp{\\sn wzName}         {\\sv PowerPlusWaterMarkObject357476642}}
    {\\sp{\\sn posh}           {\\sv 2}}
    {\\sp{\\sn posrelh}        {\\sv 0}}
    {\\sp{\\sn posv}           {\\sv 2}}
    {\\sp{\\sn posrelv}        {\\sv 0}}
    {\\sp{\\sn dhgt}           {\\sv 251659264}}
    {\\sp{\\sn fLayoutInCell}  {\\sv 0}}
    {\\sp{\\sn fBehindDocument}{\\sv 0}}
    }}}"
    # paste the rtf string for watermark behind the "\cell" on the first row of
    # last column
    cells[1, ncol(cells)] <- paste0(
      cells[1, ncol(cells)],
      paste0(
        watermark_bf,
        watermark,
        watermark_af
      )
    )
  }
  ## CREATE ROWS ----
  cellx_rows <- apply(cellx, 1, paste0, collapse = "\n")
  cell_content_rows <- apply(cells, 1, paste0, collapse = "\n")

  row_align_map <- c(
    "left"      = "\\trql ",
    "center"    = "\\trqc ",
    "right"     = "\\trqr ",
    "wrapleft"  = "\\trql \\dfrmtxtx480 ",
    "wrapright" = "\\trqr \\dfrmtxtx480 "
  )
  row_align <- row_align_map[huxtable::position(ht)]

  rh <- huxtable::row_height(ht)
  table_height <- huxtable::height(ht)
  row_heights <- ""
  if (any(!is.na(rh)) || !is.na(table_height)) {
    if (!is.numeric(rh) && !all(is.na(rh))) {
      warning("to_rtf can only handle numeric row_height.")
    }
    if (!is.numeric(table_height) && !is.na(table_height)) {
      warning(
        "to_rtf can only handle numeric table height."
      )
    }
    if (!is.numeric(table_height) || is.na(table_height)) table_height <- 0.33
    page_height <- 10 * 72 * 20 # 10 inches in twips
    if (any(is.na(as.numeric(rh)))) rh <- rep(1 / nrow(ht), nrow(ht))
    rh <- ceiling(rh * page_height * table_height)
    row_heights <- sprintf("\\trrh%d ", rh)
  }

  rows <- paste0(
    "{\n\\trowd\n", row_align, row_heights, cellx_rows,
    cell_content_rows, "\n\\row\n}\n"
  )

  ## CAPTION ----
  caption <- huxtable::caption(ht)
  cap_align <- align_map[get_caption_hpos(ht)]
  caption_par <- if (is.na(caption)) {
    ""
  } else {
    sprintf("{\\pard %s {%s} \\par}", cap_align, caption)
  }

  ## PASTE EVERYTHING TOGETHER ----
  result <- paste(rows, collapse = "\n")
  result <- if (grepl("top", huxtable::caption_pos(ht))) {
    paste(caption_par, result, sep = "\n")
  } else {
    paste(
      result, caption_par,
      sep = "\n"
    )
  }
  attr(result, "fc_tables") <- fc_tables

  result <- merge_header(result)
  if (is.null(header_pad)) {
    header_pad <- 2:(nheader + 1)
  } else {
    header_pad <- header_pad + 1
  }
  if (is_table(tlf) && !is.null(header_pad)) {
    result <- pad_header(result, nheader, header_pad)
  }
  return(result)
}

print_rtf_01 <- function(ht, fc_tables = rtf_fc_tables(ht),
                         watermark, nheader, header_pad, tlf, ...) {
  cat(custom_to_rtf(ht, fc_tables, watermark, nheader, header_pad, tlf, ...))
}


huxtableize <- function(obj_list, borders) {
  lapply(obj_list, function(obj) {
    if (!inherits(obj, "huxtable")) {
      obj <- huxtable::as_huxtable(obj)
      obj <- huxtable::set_all_borders(obj, borders)
    }
    obj
  })
}


confirm <- function(file) {
  if (!interactive()) stop("Please specify a `file` argument for non-interactive use of quick_xxx functions.")
  if (file.exists(file)) {
    answer <- readline(paste0("File '", file, "' already exists. Overwrite? [yN]"))
    if (!answer %in% c("y", "Y")) stop("OK, stopping")
  }
  file
}

auto_open <- function(path) {
  sysname <- Sys.info()["sysname"]
  switch(sysname,
    Darwin  = system2("open", path),
    Windows = system2("start", path),
    Linux   = system2("xdg-open", path),
    warning("Could not determine OS to open document automatically")
  )
}

#' @title quick_rtf_jnj
#' @aliases {quick_rtf}
#'
#' @description Write a series of `huxtables` to a RTF document.
#' Please refer to `quick_rtf` of the `huxtable` package for more information.
#'   Adjusted from [huxtable::quick_rtf()].
#'
#' @seealso \code{\link{gentlg}} \code{\link[huxtable]{quick_rtf}}
#'
#' @author Steven Haesendonckx {shaesen2@@its.jnj.com}
#' @author Pelagia Alexandra Papadopoulou {ppapadop@@its.jnj.com}
#' @author Jiaqi Song {JSong67@@its.jnj.com}
#'
#' @inheritParams huxtable::quick_rtf
#' @inheritParams gentlg
#' @param hts list of `huxtables` to print.
#' @param portrait String: "portrait" or "landscape". Default is portrait.
#' @param mode Permissions to apply to file (default to 770)
#' @param debug logical to turn on `browser()`, defaults to `FALSE`.
#' @param nheader ('numeric' vector) number of header rows.
#' @noRd
#'
#' @references \url{https://github.com/hughjonesd/huxtable}
quick_rtf_jnj <- function(hts,
                          file = confirm("huxtable-output.rtf"),
                          borders = 0.4,
                          open = FALSE,
                          portrait = TRUE,
                          pagenum = FALSE,
                          watermark = NULL,
                          mode = "0770",
                          debug = FALSE,
                          nheader = c(1),
                          tlf = "Table",
                          header_pad = TRUE) {
  assertthat::assert_that(inherits(hts, "list"))
  assertthat::assert_that(assertthat::not_empty(hts))
  for (ht in hts) {
    assertthat::assert_that(inherits(ht, "huxtable"))
  }
  assertthat::assert_that(assertthat::is.number(borders))
  assertthat::assert_that(assertthat::is.flag(open))
  force(file)
  hts <- huxtableize(hts, borders)

  portrait_t <- paste0(
    "{\\rtf1\\ansi\\deff0\\portrait\\paperw12240\\paperh15840\\margl1440\\margr1440\\",
    "margt1440\\margb1440\\headery1440\\footery1440{\\stylesheet{\\ql \\li0\\ri0\\wid",
    "ctlpar\\wrapdefault\\faauto\\adjustright\\rin0\\lin0\\itap0 \\rtlch\\fcs1 \\af0",
    "\\afs20\\alang1025 \\ltrch\\fcs0 \\fs20\\lang9\\langfe3081\\loch\\f0\\hich\\af0\\",
    "dbch\\af31505\\cgrid\\langnp9\\langfenp3081 \\snext0 \\sqformat \\spriority0 Normal;}",
    "{\\s15\\ql \\fi-1152\\li1152\\ri0\\keepn\\widctlpar\\tx1152\\wrapdefault\\faauto\\",
    "rin0\\lin1152\\itap0 \\rtlch\\fcs1 \\af0\\afs18\\alang1025 \\ltrch\\fcs0 \\b\\fs20",
    "\\lang1033\\langfe1033\\loch\\f0\\hich\\af0\\dbch\\af31505\\cgrid\\langnp1033\\",
    "langfenp1033 \\sbasedon0 \\snext0 \\sqformat caption;}{\\s16 \\ql \\fi-1152\\li1152",
    "\\ri0\\keepn\\widctlpar\\tx1152\\wrapdefault\\faauto\\rin0\\lin1152\\itap0 \\rtlch",
    "\\fcs1 \\af0\\afs18\\alang1025 \\ltrch\\fcs0 \\b\\fs20\\lang1033\\langfe1033\\loch",
    "\\f0\\hich\\af0\\dbch\\af31505\\cgrid\\langnp1033\\langfenp1033 \\sbasedon0 \\snext0 ",
    "\\sqformat;}}\n"
  )
  portrait_f <- paste0(
    "{\\rtf1\\ansi\\deff0\\landscape\\paperw15840\\paperh12240\\margl1440\\margr1440",
    "\\margt1440\\margb1440\\headery1440\\footery1440{\\stylesheet{\\ql \\li0\\ri0\\",
    "widctlpar\\wrapdefault\\faauto\\adjustright\\rin0\\lin0\\itap0 \\rtlch\\fcs1 \\",
    "af0\\afs20\\alang1025 \\ltrch\\fcs0 \\fs20\\lang9\\langfe3081\\loch\\f0\\hich\\",
    "af0\\dbch\\af31505\\cgrid\\langnp9\\langfenp3081 \\snext0 \\sqformat \\",
    "spriority0 Normal;}{\\s15\\ql \\fi-1152\\li1152\\ri0\\keepn\\widctlpar\\tx1152\\",
    "wrapdefault\\faauto\\rin0\\lin1152\\itap0 \\rtlch\\fcs1 \\af0\\afs18\\alang1025 \\",
    "ltrch\\fcs0 \\b\\fs20\\lang1033\\langfe1033\\loch\\f0\\hich\\af0\\dbch\\af31505\\",
    "cgrid\\langnp1033\\langfenp1033 \\sbasedon0 \\snext0 \\sqformat caption;}{\\s16 \\",
    "ql \\fi-1152\\li1152\\ri0\\keepn\\widctlpar\\tx1152\\wrapdefault\\faauto\\rin0\\",
    "lin1152\\itap0 \\rtlch\\fcs1 \\af0\\afs18\\alang1025 \\ltrch\\fcs0 \\b\\fs20\\",
    "lang1033\\langfe1033\\loch\\f0\\hich\\af0\\dbch\\af31505\\cgrid\\langnp1033\\",
    "langfenp1033 \\sbasedon0 \\snext0 \\sqformat;}}\n"
  )
  pagenum_t <- paste(
    "\\par {\\footer\\pard\\sb240\\qr\\fs16{\\insrsid2691151 ",
    ifelse(substr(tolower(tlf), 1, 1) == "t", "Table", "Listing"),
    paste0(
      " Page }{\\field{\\*\\fldinst {\\insrsid2691151 PAGE }}{\\fldrslt {\\",
      "insrsid26911511}}}{\\insrsid2691151  of }{\\field{\\*\\fldinst {",
      "\\insrsid2691151  NUMPAGES }} {\\fldrslt {\\insrsid112265262}}}{\\insrsid2691151 \\",
      "par }}"
    ),
    collapse = "",
    sep = ""
  )

  header <- ifelse(portrait, portrait_t, portrait_f)
  rtf_hts <- mapply(
    function(ht, nheader, watermark, pagenum, header_pad) {
      rtf <- custom_to_rtf(
        ht,
        watermark = watermark,
        nheader = nheader,
        header_pad = header_pad,
        tlf = tlf
      )
      rtf <- sprintf("%s%s", rtf, ifelse(pagenum, pagenum_t, ""))
    },
    hts,
    nheader,
    watermark,
    pagenum,
    header_pad
  )

  tables <- paste0(rtf_hts, collapse = "\\page\n\\v\\par\\v0")
  file_contents <- sprintf(
    "%s\n%s\n}",
    header,
    tables
  )

  file_contents <- remove_zero_padding(file_contents)

  sink(file)
  cat(file_contents)
  sink()

  # update permissions
  Sys.chmod(file, mode, use_umask = FALSE)

  if (open) auto_open(file)
  invisible(NULL)
}

#' Add the formatting variables of `indentme`, `newrows`, `newpage`, and `roworder` to
#' the results dataframe
#'
#' @param df (required) dataframe of results and must contain the `anbr` variable.
#' @param tableby (optional) character vector containing table by variables.
#' @param groupby (optional) character vector containing group by variables.
#' @param .keep (optional) should `tableby` and `groupby` variables be kept in the
#'   final dataframe. (default = `FALSE`).
#'
#' @return dataframe with the formatting variables `indentme`, `newrows`, `newpage`, and `roworder` added.
#' @importFrom purrr walk
#' @importFrom rlang sym
#' @export
#'
#' @examples
#' df <- tibble::tibble(
#'   row_type =
#'     c(
#'       "TABLE_BY_HEADER", "HEADER", "BY_HEADER1", "N", "VALUE",
#'       "COUNTS", "UNIVAR", "NESTED", "NESTED"
#'     ),
#'   nested_level = c(NA, NA, NA, NA, NA, NA, NA, 1, 2),
#'   group_level = c(0, 0, 0, 0, 0, 0, 0, 0, 0),
#'   label = c(NA, NA, NA, NA, NA, "N", NA, NA, NA),
#'   by = c(NA, NA, NA, NA, NA, NA, NA, NA, NA),
#'   tableby = c(NA, NA, NA, NA, NA, NA, NA, NA, NA),
#'   anbr = c(1:9)
#' )
#' add_format(df)
add_format <- function(df, tableby = NULL, groupby = NULL, .keep = FALSE) {
  # check all the arguments being passed in except ...
  arglist <- list()
  args_to_chk <- names(formals())[names(formals()) != "..."]
  walk(args_to_chk, .f = function(x) arglist[[x]] <<- eval(sym(x)))
  check_add_format(arglist)

  df2 <- df |>
    add_indent() |>
    add_newrows(tableby = tableby, groupby = groupby) |>
    add_newpage()

  if (is.null(c(tableby, groupby)) || .keep == TRUE) {
    df2
  } else {
    df2 |>
      select(-all_of(c(tableby, groupby)))
  }
}

# Function that sets when a new page is to be added
# A new page is added when row_type is TABLE_BY_HEADER and not the first row
# but when first row instead adds a new row
#' add_newpage
#'
#' @param df dataframe to add newpage col to
#'
#' @return df with the newpage cols row added and newrows updated
#' @noRd
add_newpage <- function(df) {
  if (any(df[["row_type"]] == "TABLE_BY_HEADER")) {
    df %>%
      dplyr::mutate(
        newpage = case_when(
          row_type == "TABLE_BY_HEADER" &
            dplyr::row_number() > 1 ~ 1,
          TRUE ~ 0
        ),
        newrows = case_when(
          row_type == "TABLE_BY_HEADER" &
            dplyr::row_number() == 1 ~ 1,
          row_type == "TABLE_BY_HEADER" &
            dplyr::row_number() > 1 ~ 0,
          TRUE ~ newrows
        )
      )
  } else {
    df %>%
      mutate(newpage = 0)
  }
}

#' Replace NA with ""
#'
#' Used to swap in "" for by variables so the headers sort correctly to the top
#'
#' @param x variable to check for NA and replace with "".
#'
#' @return x with NAs replaced with "". Factors will add ""
#' as the first level.
#' @export
#'
#' @examples
#' replace_na_with_blank(c("a", "b", NA))
#'
#' replace_na_with_blank(factor(c("a", "b", NA), levels = c("a", "b")))
replace_na_with_blank <- function(x) {
  if (sum(is.na(x)) > 0) {
    if (is.factor(x)) {
      x <- forcats::fct_relevel(forcats::fct_expand(x, ""), "", after = 0)
    }

    tidyr::replace_na(x, "")
  } else {
    x
  }
}

#' @title roxygen2_data
#' @description Creates the Roxygen2 documentation for a given data frame.
#'
#' @author Steven Haesendonckx {shaesen2@@its.jnj.com}
#'
#' @param df data frame.
#' @param study study name.
#' @param pkg package path.
#'
#' @noRd
#'
#' @return Roxygen2 documentation for the data frame.
roxygen2_data <- function(df, study = NULL, pkg = "~/tidytlg") {
  name <- deparse(substitute(df))

  title <- paste0("#' @title ", name)

  descr <- paste0("#' @description ", name, " copied from ", study)
  src <-
    "#' @source Synthetic data on S drive, general compound."

  fmt <-
    paste0(
      "#' @format A data frame with ",
      nrow(df),
      " rows and ",
      ncol(df),
      " variables:"
    )
  itemize <- data.frame(
    colname = colnames(df),
    label = purrr::map_chr(seq_along(colnames(df)), function(x) {
      attr(df[[x]], "label")
    }),
    stringsAsFactors = FALSE
  )
  item <-
    paste0("#'  \\item{", itemize$colname, "}{", itemize$label, "}")
  item <- c("#' \\describe{", item, "#' }")
  seealso <-
    paste0(
      "#' @seealso ",
      paste0("\\code{\\link{", noquote(gsub(
        ".rda", "", list.files(path = paste0(pkg, "/data"))
      )), "}}", collapse = " ")
    )
  key <-
    paste0(
      "#' @keywords datasets synthetic ",
      tolower(stringr::word(gsub("_", " ", name), 1))
    )
  atname <- paste0("#' @name ", tolower(name))
  examp <-
    c(
      "#' @examples",
      paste0("#'  data(\"", deparse(substitute(df)), "\")"),
      paste0("\"", deparse(substitute(df)), "\"")
    )

  description <-
    c(
      title,
      "#'",
      descr,
      src,
      "#'",
      fmt,
      item,
      seealso,
      key,
      atname,
      examp,
      ""
    )

  return(writeLines(description))
}

#' Convert character variable to a factor based off it's numeric variable
#' counterpart.
#'
#' @param df data frame.
#' @param c_var character variable within the data frame.
#' @param n_var numeric variable counter part within the data frame to control
#'   the levels.
#'
#' @return A factor.
#' @export
#'
#' @examples
#' df <- tibble::tribble(
#'   ~TRT01P, ~TRT01PN,
#'   "Placebo", 1,
#'   "Low Dose", 2,
#'   "High Dose", 3
#' )
#'
#' # alphabetical order
#' dplyr::arrange(df, TRT01P)
#'
#' # change to factor with char2factor
#' df$TRT01P <- char2factor(df, "TRT01P", "TRT01PN")
#'
#' # factor order
#' dplyr::arrange(df, TRT01P)
char2factor <- function(df, c_var, n_var) {
  lbl <- attr(df[[c_var]], "label")
  new_variable <- forcats::fct_reorder(as.factor(df[[c_var]]), df[[n_var]])
  attr(new_variable, "label") <- lbl
  new_variable
}


#' add group_level variable for indentation
#' @noRd
add_group_level <- function(df, rowbyvar) {
  # assign default value
  df$group_level <- 0
  if (length(rowbyvar) > 1) {
    # if more than one rowbyvar exists iterate over adding group_level
    for (var in rowbyvar) {
      current_level <- match(var, rowbyvar) - 1
      df$group_level <- ifelse(as.character(df[[var]]) != "",
        current_level, df$group_level
      )
    }
  }
  df
}

#' Workhorse for finding a file in a working directory
#'
#' @param base_path Base path to start looking for a file
#' @param name File name sans extention
#' @param ext Vector of possible extensions
#' @noRd
find_file <- function(base_path, name, ext) {
  if (!dir.exists(base_path)) {
    stop(paste("Directory:", base_path, "does not exist"))
  }
}

#' Helper functions for returning files used in `gentlg`
#'
#' @param path Working directory of the project
#'
#' @return A character vector to the requested file.
#'
#' @export
#'
#' @rdname files
tidytlg_titles <- function(path) {
  list.files(path,
    pattern = "(^titles\\.xl)", full.names = TRUE,
    ignore.case = TRUE
  )
}

#' @rdname files
#' @export
tidytlg_metadata <- function(path) {
  list.files(path,
    pattern = "(^column_metadata\\.xl)", full.names = TRUE,
    ignore.case = TRUE
  )
}

first_class <- function(.) {
  class(.)[1]
}

#' @title Utility function requirements
#' @section Last updated by:
#' Nicholas Masel
#' @section Last update date:
#' 13-JAN-2020
#' @section Category:
#' Helper

library(dplyr)

# Requirements replace_na_with_blank ---------------------------------------------------------------
#' T1. Returns vector with ...
#' T1.1 no changes if no NA.
#' T1.2 blanks in place of NA for character.
#' T1.3 blanks in place of NA for factor.
#' T1.4 blank added as a factor level for factor.

test_that("T1.1 no changes if no NA.", {
  v <- paste(rep("test"), 1:3)

  expect_equal(replace_na_with_blank(v), c("test 1", "test 2", "test 3"))
})

test_that("T1.2 blanks in place of NA for character.", {
  v <- c(NA, "test 1", "test 2", "test 3")

  expect_equal(replace_na_with_blank(v), c("", "test 1", "test 2", "test 3"))
})

test_that("T1.3 blanks in place of NA for factor.", {
  v <- factor(c(NA, "test 1", "test 2", "test 3"), levels = c("test 1", "test 2", "test 3"))

  expect_equal(as.character(replace_na_with_blank(v)), c("", "test 1", "test 2", "test 3"))
})

test_that("T1.4 NA added as a factor level for factor.", {
  v <- factor(c(NA, "test 1", "test 2", "test 3"), levels = c("test 1", "test 2", "test 3"))

  expect_equal(levels(replace_na_with_blank(v)), c("", "test 1", "test 2", "test 3"))
})

# Requirements char2factor ---------------------------------------------------------------
#' T1. Returns factor with ...
#' T1.1 label preserved.
#' T1.2 levels based off the numeric variable.

df_char2factor <- tibble::tribble(~TRT01P, ~TRT01PN, "Placebo", 1, "Low Dose", 2, "High Dose", 3)

attr(df_char2factor$TRT01P, "label") <- "Planned Treatment"

test_that("T1.1 label preserved.", {
  expect_equal(attr(char2factor(df_char2factor, "TRT01P", "TRT01PN"), "label"), "Planned Treatment")
})

test_that("T1.2 levels based off the numeric variable.", {
  expect_equal(levels(char2factor(df_char2factor, "TRT01P", "TRT01PN")), c("Placebo", "Low Dose", "High Dose"))
})

---
title: "Manipulating `tbl` data frame"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Manipulating `tbl` data frame}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

The `tbl` data frame is the main input to the `gentlg` function for creating the RTF or HTML outputs.
This vignette will show you the `tbl` structure and how the `tbl` variables are in action for
rendering the RTF or HTML outputs.

### Basic `tbl` data frame

The basic variables of `tbl` includes `label`, `col1`, `col2`, `...`, `coln`, where

* `label`: row text displayed on the first column of the table
* `col1`: statistic results displayed on the second column of the table
* `col2`: statistic results displayed on the third column of the table.

The example below shows you a very basic `tbl` data frame, and how this `tbl` is transformed
through the `gentlg` function call to create the HTML output.

```{r, message=FALSE}
library(dplyr)
library(tidytlg)

tbl <- tibble::tribble(
  ~label, ~col1, ~col2, ~col3,
  "Analysis Set: ITT", "86", "84", "84",
  "Age (Years)", NA, NA, NA,
  "N", "86", "84", "84"
)

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO1",
  title = "Basic tbl without formatting",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)
```

## Formatting table

The above example does not have any formatting in actions on the table. To enable formatting, additional variables will
need to be created in the `tbl` data frame.

### Add new row

For inserting a blank line prior to the second row (i.e. `Age (Years)`), we will need to add the `newrows` variable with `value = 1`.

```{r}
tbl <- tbl %>%
  mutate(newrows = case_when(
    label == "Age (Years)" ~ 1,
    TRUE ~ 0
  ))

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO2",
  title = "Adding the variable of newrows",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)
```

### Add indentation

For adding one indentation to the `N` row, we will need to add the `indentme`
variable with `value = 1`, which indicates one indentation (two will result in two indentation, and so on).

```{r}
tbl <- tbl %>%
  mutate(indentme = case_when(
    label == "N" ~ 1,
    TRUE ~ 0
  ))

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO3",
  title = "Adding the variable of indentme",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)
```

### Enable bold font

To enable bold font on the row of `Age (Years)`, the `boldme` variable with `value = 1` will need to be added.

```{r}
tbl <- tbl %>%
  mutate(boldme = case_when(
    label == "Age (Years)" ~ 1,
    TRUE ~ 0
  ))

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO4",
  title = "Adding the variable of boldme",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)
```

There is another formatting variable called `newpage`, where assigning `newpage = 1` will start a new page in the output.

## Formatting functions

Besides manually adding the formatting variables to `tbl`, we have developed several formatting
functions to facilitate the formatting process:

* `add_newrows`: add `newrows` variable to the `tbl` based on `row_type`
* `add_indent`: add `indentme` variable to the `tbl` based on `row_type`
* `add_newpage`: add `newpage` variable to the `tbl` based on `row_type`.

The `row_type` variable is created in the `tbl` by calling the `freq`, `nested_freq`, and `univar` functions,
which is used by the above functions for setting up the formatting variables. The `tbl` example below is
obtained by calling the `univar` function for summarizing the age statistics in the `CDISC` `ADSL` dataset.
After calling the `univar` function, we create the `anbr` (analysis number) variable as the identifier of this `tbl` chunk.

```{r}
tbl <- cdisc_adsl %>%
  univar(
    colvar = "TRT01PN",
    rowvar = "AGE",
    statlist = statlist(c("N", "MEANSD")),
    decimal = 0,
    row_header = "Age (Years)"
  ) %>%
  mutate(anbr = "01")

knitr::kable(tbl)
```

The `add_format` function, which incorporates all 3 formatting functions above (`add_indent`, `add_newrows`, `add_newpage`),
can then be applied to the `tbl` for creating the formatting variables.

```{r}
tbl <- cdisc_adsl %>%
  univar(
    colvar = "TRT01PN",
    rowvar = "AGE",
    statlist = statlist(c("N", "MEANSD")),
    decimal = 0,
    row_header = "Age (Years)"
  ) %>%
  mutate(anbr = "01") %>%
  add_format()

knitr::kable(tbl)

# render tbl
gentlg(
  huxme = tbl,
  tlf = "Table",
  format = "HTML",
  orientation = "landscape",
  opath = ".",
  file = "DEMO5",
  title = "Using row_type to set up indentation",
  colheader = c("", "Placebo", "Active 1", "Active 2"),
  print.hux = FALSE,
  wcol = .30
)
```

The default indentation for each `row_type` is shown below.

| `row_type`        | default indentation |
| ----------------- |:-------------------:|
| TABLE_BY_HEADER   | 0                   |
| BY_HEADER[1-9]    | 0                   |
| HEADER            | 0                   |
| N                 | 1                   |
| VALUE             | 2                   |
| NESTED            | 0                   |

---
title: "Get Started"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get Started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "figures"
)

working_dir <- tempdir()
```

## Overview of TLG Programming

`tidytlg` provides a framework of creating TLG outputs for clinical study report. The TLG programming workflow includes the following steps:

* Prep environment: set up the R environment for the I/O paths.
* Process data: filter analysis data, perform data manipulation (e.g. convert character variable to factor), and define column variable.
* Generate results: create analysis rows of summary statistics (for tables) or plots (for graphs).
* Output results: output analysis results in designated format such as `rtf` or `html`.

We will illustrate the above steps by creating a demographic table first, and then follow by examples of creating listing and graph.

## Prep environment

To set up the R environment, you can set the path objects of the input folder and output folder
consistently for all TLG programs. The analysis datasets and other required inputs such as the titles file and column metadata
file are placed in the input folder, while the output folder will be used to store the output files.
The [`envsetup`](https://pharmaverse.github.io/envsetup/main/) package can be used to set up the R environment for TLG programming.

### titles and footnotes

The information for titles and footnotes for each TLG can be stored in an excel file called `titles.xls` (see below snapshot),
which will be used later to create the outputs.

```{r, echo=FALSE, out.width = "600px"}
knitr::include_graphics("titles.PNG")
```

### Column metadata

Column metadata provides the column structure of the table layout and includes the following variables:

* `tbltype`: identifier used to group a table column layout
* `coldef`: distinct variable values used, typically numeric and typically a treatment variable, think `TRT01PN`
* `decode`: decode of `coldef` that will display as a column header in the table
* `span1`: spanning header to display across multiple columns (the lowest level)
* `span2`: spanning header to display across multiple columns, second level
* `span3`: spanning header to display across multiple columns, third level

Please see below for a snapshot of `column_metadata.xlsx`.

```{r, echo=FALSE, out.width = "500px"}
knitr::include_graphics("column_metadata.PNG")
```

Different types of column layouts identified by different `tbltype` can be stored in an excel file called `column_metadata.xlsx`.
Within each `tbltype`, the `coldef` variable defines the order of the column based on the column variable used for
creating the output (typically the numeric treatment variable, `TRT01PN`, is used as the column variable).
For example, there are 3 columns for `tbltype` = "type1" in the above snapshot and the column layout is
defined as follows: the first column of summary statistics represents the treatment group of `TRT01PN = 0` with
the column header of `Placebo` defined by `decode`, the second and third columns represent the `Low Dose` and `High Dose`
groups respectively with the spanning header of `Xanomeline` defined by the `span1` variable.

Users can also include the column that is derived from combination of individual columns. For example, the `tbltype` of
`type3` include the fourth column of combined `Low Dose` and `High Dose` as well as the fifth column of total group.
Please see below for the snapshot of column headers defined by `type3`.

```{r, echo=FALSE, out.width = "600px", out.height = "60px"}
knitr::include_graphics("column_header.PNG")
```

We will use the `adsl` data from the [PHUSE Test Data Factory](https://github.com/phuse-org/TestDataFactory/tree/main/Updated/TDF_ADaM)
to illustrate the creation of a demographic table.

```{r, message=FALSE}
# Prep Environment -------------------------------------------------------------
library(dplyr)
library(haven)
library(tidytlg)

# read adsl from PhUSE test data factory
testdata <- "https://github.com/phuse-org/TestDataFactory/raw/refs/heads/main/Updated/TDF_ADaM/adsl.xpt"
adsl <- read_xpt(url(testdata))
```

## Process data

Before generating analysis summary, the analysis data need to be processed first as shown in the code below.

```{r, message=FALSE}
# Process Data -----------------------------------------------------------------
adsl <- adsl %>%
  filter(ITTFL == "Y") %>%
  mutate(
    SEX = factor(
      SEX,
      levels = c("M", "F", "U"), labels = c("Male", "Female", "Unknown")
    )
  ) %>%
  tlgsetup(
    var = "TRT01PN",
    column_metadata_file = system.file(
      "extdata/column_metadata.xlsx",
      package = "tidytlg"
    ),
    tbltype = "type3"
  )
```

The above code perform the tasks below:

* filtering analysis population
* convert the `SEX` variable from character type to factor type. So all the factor levels of `SEX` will be displayed in
  the analysis summary even there are no records for the factor level of "Unknown".
* create the column variable, `colnbr`, through the `tlgsetup` function call: `tlgsetup`
  is using the numeric treatment variable (e.g. `TRT01PN`) to match with `coldef` in column metadata defined
  by `column_metadata_file` and `tbltype` to create the column variable, `colnbr`, in `adsl` for reflecting
  the column layout. Please see the `vignette("tlgsetup")` for more details. The column variable, `colnbr`,
  will be used in the subsequent analysis function calls for creating analysis results.

If you need multiple analysis datasets for creating TLG, `tlgsetup` will need to be applied to each dataset.
Therefore, you will have a consistent column variable of `colnbr` for creating analysis summary.

## Generate results

`tidytlg` provides 3 functions, `univar`, `freq`, and `nested_freq`, to generate analysis summary of
descriptive statistics (univariate statistics and count (percentages)). For more details, please see the frequency
analysis `vignette("freq")` and the univariate statistical analysis `vignette("univar")`.

```{r, message=FALSE}
# Generate Results -------------------------------------------------------------

## Analysis set row
t1 <- adsl %>%
  freq(
    colvar = "colnbr",
    rowvar = "ITTFL",
    statlist = statlist("n"),
    subset = ITTFL == "Y",
    rowtext = "Analysis set: ITT"
  )

## Univariate summary for AGE
t2 <- adsl %>%
  univar(
    colvar = "colnbr",
    rowvar = "AGE",
    statlist = statlist(c("N", "MEANSD", "MEDIAN", "RANGE", "IQRANGE")),
    decimal = 0,
    row_header = "Age, years"
  )

## Count (percentages) for SEX
t3 <- adsl %>%
  freq(
    colvar = "colnbr",
    rowvar = "SEX",
    statlist = statlist(c("N", "n (x.x%)")),
    row_header = "Gender"
  )
```

The above function calls generate the requested analysis rows for the table output sequentially and store the results in individual
objects (i.e. `t1`, `t2`, `t3`). The next step is to combine analysis results into a single `tbl` dataframe through the `bind_table` function call.

```{r}
# Format Results ---------------------------------------------------------------

tbl <- bind_table(t1, t2, t3,
  column_metadata_file = system.file(
    "extdata/column_metadata.xlsx",
    package = "tidytlg"
  ),
  tbltype = "type3"
)
```

The above `bind_table` function call performs the following tasks:

* bind the analysis rows from `t1`, `t2`, `t3` into `tbl`
* add formatting variables (`indentme`, `newrows`, `newpage`), which will be used in the `gentlg`
  function call below for creating the output.
* attach the column metadata specified by `column_metadata_file` and `tbltype` as an attribute of the `tbl`.
  So the column header and the spanning headers (i.e. `decode`, `span1`, `span2`, `span3`) defined
  in the column metadata can be used automatically in the `gentlg` function call.

## Output results

The `tbl` data frame is the main input to the `gentlg` function for creating the RTF/HTML outputs.

The basic structure of `tbl` includes `label`, `col1`, `col2`, `...`, `coln`, where:

* `label`: row text displayed on the first column of the table
* `col1`: statistic results displayed on the second column of the table
* `col2`: statistic results displayed on the third column of the table.

All other columns contain formatting instructions to create the RTF/HTML outputs. For tweaking the formatting variables to customize
the table layout, please see the `vignette("tbl_manipulation")` for more details.

```{r}
knitr::kable(tbl)
```

The `gentlg` function call below will create the RTF output using the `tblid` as the file name in the folder
defined by the `opath` argument. Please ensure that the `titles.xls` file contains the records of titles
and footnotes for the specified `tblid`.

```{r}
tblid <- "Table01"

gentlg(
  huxme = tbl,
  opath = file.path(working_dir),
  file = tblid,
  orientation = "landscape",
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)[[1]]
```

To create the html output, users need to specify the `format` argument as "HTML" and `print.hux` argument as `FALSE` in the `gentlg` call.

```{r}
gentlg(
  huxme = tbl,
  format = "HTML",
  print.hux = FALSE,
  file = tblid,
  orientation = "landscape",
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)[[1]]
```

Users can also include superscripts, subscripts, or line breaks via `UNICODE`. Please see the `vignette("symbols")` for more details.
Besides using `univar`, `freq`, and `nested_freq` functions to create the `tbl` dataframe, users can use other R packages to create
analysis results and perform data wrangling to fit the `tbl` structure, which can be passed into
the `gentlg` function call for generating the desired outputs.

## Listing programming

The above workflow can also be used to create listings. Users need to prepare the data and assign it to `tbl`.
In the `gentlg` function, users need to pay attention to:

* specify the `tlf` argument to `Listing` (i.e. `tlf = "Listing"`)
* specify the `idvars` argument for identifying variables (such as treatment variable and `USUBJID`)
  where repeated values will be removed
* specify the `colheader` (column header) argument; if not specified, the column labels will be
  used as the column headers. For the below example, if `colheader` argument is not specified, some column headers
  will use variable names since these columns are newly created without labels.
* user has the option to control the column width by passing a vector of column width to the `wcol` argument.
  Please ensure that the length of the column width vector is the same as the number of columns
  in your data. For the example below, there are 8 columns in the data and users can specify
  customized column width such as `c(0.15, 0.10, 0.05, 0.15, 0.20, 0.15, 0.05, 0.05)`
  for the `wcol` argument to create the RTF output. However, for the HTML output shown here,
  the `wcol` argument can only take a single number as the column width and apply to every column.

```{r}
# Prep Environment -------------------------------------------------------------
library(dplyr)
library(haven)
library(tidytlg)

adsl <- cdisc_adsl
adae <- cdisc_adae

# Process Data -----------------------------------------------------------------
adsl <- adsl %>%
  filter(SAFFL == "Y") %>%
  select(USUBJID, SAFFL, TRT01AN, TRT01A)

adae <- adae %>%
  filter(SAFFL == "Y" & TRTEMFL == "Y") %>%
  mutate(
    BSPT = paste(AEBODSYS, "[", AEDECOD, "]"),
    SAEFL = if_else(AESER == "Y", "Yes", "No"),
    DTHFL = if_else(AEOUT == "FATAL", "Yes", "No")
  ) %>%
  select(USUBJID, ASTDY, TRTA, BSPT, AETERM, SAEFL, DTHFL)

tbl <- inner_join(adsl, adae, by = "USUBJID") %>%
  arrange(TRT01AN, USUBJID, ASTDY) %>%
  select(TRT01A, USUBJID, ASTDY, TRTA, BSPT, AETERM, SAEFL, DTHFL) %>%
  filter(USUBJID %in% c("01-701-1015", "01-701-1023"))

# Output Results ---------------------------------------------------------------
gentlg(
  huxme = tbl,
  tlf = "l",
  format = "HTML",
  print.hux = FALSE,
  orientation = "landscape",
  file = "Listing01",
  title = "Listing of Adverse Events",
  idvars = c("TRT01A", "USUBJID"),
  wcol = 0.15,
  colheader = c(
    "Treatment Group",
    "Subject ID",
    "Study Day of AE",
    "Treatment Period",
    "Body System [Preferred Term]",
    "Verbatim Term",
    "Serious",
    "Fatal"
  )
)[[1]]
```

## Graph programming

To create the graph output, `tidytlg` provides a framework of integrating the PNG file with titles and footnotes for producing the RTF or HTML output.

In the `gentlg` function, users need to:

* specify the `tlf` argument to `g` for graph
* specify the `plotnames` argument with the full path of the PNG file
* define the `plotwidth` and `plotheight`: it's advised here that users keep the aspect ratio of plot
  width and height approximately the same as the PNG image.

The code below will create the RTF output of the plot.

```{r}
# Prep Environment -------------------------------------------------------------
library(dplyr)
library(haven)
library(ggplot2)
library(tidytlg)

# read adsl from PhUSE test data factory
testdata <- "https://github.com/phuse-org/TestDataFactory/raw/refs/heads/main/Updated/TDF_ADaM/adsl.xpt"
adsl <- read_xpt(url(testdata))

tblid <- "Graph01"

# Process Data -------------------------------------------------------
adsl <- adsl %>%
  filter(ITTFL == "Y") %>%
  select(USUBJID, ITTFL, TRT01PN, TRT01P, AGE, SEX, HEIGHTBL, WEIGHTBL) %>%
  mutate(SEX = factor(SEX, levels = c("M", "F"), labels = c("Male", "Female")))

# Generate Results -------------------------------------------------------------

plot <- ggplot(data = adsl, aes(x = HEIGHTBL, y = WEIGHTBL)) +
  geom_point() +
  labs(
    x = "Baseline Height (cm)",
    y = "Baseline Weight (kg)"
  ) +
  facet_wrap(~SEX, nrow = 1)

# create png file
png(
  file.path(working_dir, paste0(tblid, ".png")),
  width = 2800, height = 1300, res = 300, type = "cairo"
)

plot

dev.off()

# Output Results ---------------------------------------------------------------

gentlg(
  tlf = "g",
  plotnames = file.path(
    system.file("extdata", package = "tidytlg"), paste0(tblid, ".png")
  ),
  plotwidth = 10,
  plotheight = 5,
  orientation = "landscape",
  opath = file.path(working_dir), ,
  file = tblid,
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)
```

```{r, echo=FALSE, out.width = "750px"}
knitr::include_graphics("graph01.PNG")
```

## Metadata method

Besides building the table section-by-section as shown above, we can use the table metadata approach
as an efficient alternative for generating outputs. Table metadata is a data frame describing the data, functions and arguments
needed to produce your table results. The table metadata shown below can be used to create the same table output
as above. Each row in the table metadata describes how a `tbl` chunk will be created by the
function defined in the `func` column. The rest of the columns defines the arguments (i.e. `df`,
`colvar`, `rowvar`, `statlist`, `rowtext`, `row_header`) that will be passed into the function.

Once table metadata is defined, users just need to call the `generate_results` function with the column metadata
define in the `column_metadata_file` and `tbltype` arguments to create the `tbl` dataframe.
In the processing data step, users don't need to call `tlgsetp`, since `tlgsetup` is embedded
within the `generate_results` function. That's why we need to specify the column metadata in the `generate_results` call.

```{r}
library(dplyr)
library(haven)
library(tidytlg)

# read adsl from PhUSE test data factory
testdata <- "https://github.com/phuse-org/TestDataFactory/raw/refs/heads/main/Updated/TDF_ADaM/adsl.xpt"
adsl <- read_xpt(url(testdata))

# Process data
adsl <- adsl %>%
  filter(ITTFL == "Y") %>%
  mutate(
    SEX = factor(
      SEX,
      levels = c("M", "F", "U"), labels = c("Male", "Female", "Unknown")
    )
  )

# define table metadata
table_metadata <- tibble::tribble(
  ~func, ~df, ~rowvar, ~decimal, ~rowtext, ~row_header, ~statlist, ~subset,
  "freq",
  "adsl", "ITTFL", NA, "Analysis set: ITT", NA, statlist("n"), "ITTFL == 'Y'",
  "univar", "adsl", "AGE", 0, NA, "Age (Years)", NA, NA,
  "freq", "adsl", "SEX", NA, NA, "Gender", statlist(c("N", "n (x.x%)")), NA
) %>%
  mutate(colvar = "TRT01PN")

# Generate results
tbl <- generate_results(table_metadata,
  column_metadata_file = system.file(
    "extdata/column_metadata.xlsx",
    package = "tidytlg"
  ),
  tbltype = "type3"
)

# Output results
tblid <- "Table01"

gentlg(
  huxme = tbl,
  format = "HTML",
  print.hux = FALSE,
  file = tblid,
  orientation = "landscape",
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)[[1]]
```

## By processing

There are two types of by-processing that `tidytlg` functions can provide:

* `rowbyvar`: split the summary statistics of `rowvar` by other variable(s) specified in `rowbyvar`.
  Please see the by processing section of frequency analysis `vignette("freq")` for further details.
  The `rowbyvar` argument can also be used in the `univar` function. A typical use case is
  to summarize the lab values by parameter and analysis visit, where we can call the `univar`
  function with `rowvar = AVAL` and `rowbyvar = c("PARAM","AVISIT")`.
  It is advised in this use case to turn on the `.ord` argument (i.e. `.ord = TRUE`)
  in the `univar` function call. So the numeric sorting columns associated with
  the `by` variables (`PARAM_ord` and `AVISIT_ord`) can be created and used for sorting
  the interleaved summary results of `AVAL` and `CHG`.
* `tablebyvar`: the argument of `tablebyvar` is designed to facilitate the sub-group analysis,
  which repeats a table summary by the sub-group variable. A typical use case is the summary of
  demographics table by country. For creating the sub-group analysis version of the same table, we just
  need to add the argument of `tablebyvar` with the sub-group variable in each function call.

The code below provides an example of summarizing age and race by gender using `tablebyvar`.

```{r, message=FALSE}
library(dplyr)
library(haven)
library(tidytlg)

# read adsl from PhUSE test data factory
testdata <- "https://github.com/phuse-org/TestDataFactory/raw/refs/heads/main/Updated/TDF_ADaM/adsl.xpt"
adsl <- read_xpt(url(testdata))

# Process data
adsl <- adsl %>%
  filter(ITTFL == "Y") %>%
  mutate(SEX = factor(SEX, levels = c("M", "F"), labels = c("Male", "Female")))

# define table metadata
table_metadata <- tibble::tribble(
  ~func, ~df, ~rowvar, ~decimal, ~rowtext,
  ~row_header, ~statlist, ~subset, ~tablebyvar,
  "univar", "adsl", "AGE", 0, NA, "Age (Years)", NA, NA, "SEX",
  "freq", "adsl", "RACE", NA, NA,
  "Race", statlist(c("N", "n (x.x%)")), NA, "SEX"
) %>%
  mutate(colvar = "TRT01PN")

# Generate results
tbl <- generate_results(table_metadata,
  column_metadata_file = system.file(
    "extdata/column_metadata.xlsx",
    package = "tidytlg"
  ),
  tbltype = "type3"
)

# Output results
tblid <- "Table01"

gentlg(
  huxme = tbl,
  format = "HTML",
  print.hux = FALSE,
  file = tblid,
  orientation = "landscape",
  title_file = system.file("extdata/titles.xls", package = "tidytlg")
)[[1]]
```

In summary, `rowbyvar` is used to create the by-variable summary for one `rowvar` in a single function call.
To perform sub-group analysis, users need to specify `tablebyvar` in every function calls except the analysis population row.

---
title: "Tips & Tricks:  Including Symbols, Superscripts, Subscripts & Line Breaks"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tips & Tricks:  Including Symbols, Superscripts, Subscripts & Line Breaks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Symbols

Symbols can be added to the label column via `UNICODE`.  Here is a quick reference to commonly
used symbols.


```{r setup, echo = FALSE}
library(huxtable)

opath <- "."

df <- tibble::tribble(
  ~Symbol, ~`Textual Description`, ~Unicode,
  "\u2190", "Left arrow", "\\u2190",
  "\u2192", "Right arrow", "\\u2192",
  "\u2264", "Less-than or equal to", "\\u2264",
  "\u2265", "Greater-than or equal to", "\\u2265",
  "\u2260", "Not equal to", "\\u2260",
  "\u00b1", "Plus-minus sign", "\\u00b1",
  "\u03b1", "Alpha", "\\u03b1",
  "\u03b2", "Beta", "\\u03b2",
  "\u03bc", "Mu", "\\u03bc",
  "\u00ab", "Non-breaking space", "\\u00ab"
)
df

# quick_rtf(hux(df),
#           file = "./test.rtf")
#
#
```

Here is an example call to `tidytlg::gentlg()` that will add the symbols
to the label column.

```{r eval = FALSE}
df <- tibble::tibble(
  label = c("\u2264", "\u2265"),
  col1 = c("100", "200")
)

tidytlg::gentlg(df,
  file = "demo"
)
```


# Superscripts and Subscripts

Superscripts and Subscripts can be added to the label column via `UNICODE`.

```{r eval = FALSE}
df <- tibble::tibble(
  label = c(
    "This is a superscript a{\\super a}",
    "This is a subscript b{\\sub b}"
  ),
  col1 = c("100", "200")
)

tidytlg::gentlg(df,
  file = "demo"
)
```


Superscripts and Subscripts can be added to the footnotes via `UNICODE` as well.

```{r eval = FALSE}
df <- tibble::tibble(
  label = c(
    "This is a superscript a{\\super a}",
    "This is a subscript b{\\sub b}"
  ),
  col1 = c("100", "200")
)

tidytlg::gentlg(df,
  file = "demo",
  footers = "This is a footnote superscript{\\super a}"
)
```

# Inline RTF Line Breaks

Sometimes you need add a line break into your RTF.  Inserting `'\\\\\\n'` into your string will add your
line break for you.


```{r eval = FALSE}
df <- tibble::tibble(
  label = c("Bodysystem \\\n Preferred Term"),
  col1 = c("100")
)

tidytlg::gentlg(df,
  file = "demo"
)
```

If you need a line break followed by a tab, just add in `'\\\\li180'`.

```{r eval = FALSE}
df <- tibble::tibble(
  label = c("Bodysystem\\\n\\li180Preferred Term"),
  col1 = c("100")
)

tidytlg::gentlg(df,
  file = "demo"
)
```

---
title: "Column Metadata & How tidytlg Sets Up Your Data Using `tlgsetup`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Column Metadata & How tidytlg Sets Up Your Data Using `tlgsetup`}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, message=FALSE}
library(dplyr)
library(tidytlg)
```

The `tlgsetup` helper function was created to support generating different column structures for our outputs.  This allows us
to specify column types once that can be used across multiple outputs.  This metadata can be hardcoded as well
(please see the code below), or pulled from another source like a dataframe, file or database.

Here we see an example of the column metadata which defines our columns, how to combine columns, column
labels and spanning headers across columns.

```{r}
column_metadata <-
  tibble::tribble(
    ~tbltype, ~coldef, ~decode,                ~span1,
    "type1",  "0",     "Placebo",              "",
    "type1",  "54",    "Low Dose",             "Xanomeline",
    "type1",  "81",    "High Dose",            "Xanomeline",
    "type1",  "54+81", "Total Xanomeline",     ""
  )
column_metadata
```

In this example, we would summarize `Placebo`, `Xanomeline Low Dose`, and `Xanomeline High Dose`
which are already available in the data. The helper function will add the observations to the data for the
`Total Xanomeline` column based off of `coldef`.  So for our `adsl` data,
we will add a spanning header for `Xanomeline` and a total `Xanomeline` column which is a
combination of the `Xanomeline Low Dose` and `Xanomeline High Dose`.

In addition, the helper function will add the factor variable `colnbr` which is used as our new
column summary variable.  Note that our column summary variable has been converted to a factor.  This is required and allows
us to define column order as well as label in one variable.

Let's read in `adsl` and check the dimensions.

```{r}
data("cdisc_adsl")
adsl <- cdisc_adsl %>%
  filter(ITTFL == "Y") %>%
  select(USUBJID, TRT01PN, TRT01P, ITTFL, SEX, RACE, AGE)
glimpse(adsl)
```

```{r echo = FALSE}
paste0("Dimensions prior to the tlgsetup call are ", dim(adsl)[1], " rows and ", dim(adsl)[2], " columns.")
```

Now let's pass the `adsl` data through our `tlgsetup` function to add observations to support the
`type1` column structure and check out the dimensions.

```{r warning=FALSE}
setup_table <- tlgsetup(adsl,
  var = "TRT01PN",
  column_metadata = column_metadata
)
glimpse(setup_table)
```

```{r echo = FALSE}
paste0(
  "Dimensions after to the rmtsetup call are ", dim(setup_table)[1], " rows and ", dim(setup_table)[2], " columns."
)
```

Here we see we have added two new variables, `colnbr` and `tbltype`, which will now be used as our treatment variable when generating our results.

If we take a look at the observation counts for `colnbr`, we see the 168 records added to the data support `Total Xanomeline` as we expected!
```{r}
setup_table %>%
  group_by(colnbr) %>%
  count()
```

---
title: "Frequency Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Frequency Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(dplyr)
library(tidytlg)
```

## Introduction

The `freq` and `nested_freq` functions are called to produce categorical-type summary statistics (i.e.
counts and percentages) for character variables. These functions can be used to create all
different types of categorical summary statistics tables.

### Counts: `statlist = statlist("n")`

There are many uses for `freq`, the first one we'll show is to create the first
row of analysis set summary of population counts in the output.  In the following table the `subset` argument
is used to create the one row summary with the row heading specified in the `rowtext` argument.

```{r}
tbl <- cdisc_adsl %>%
  freq(
    colvar = "TRT01PN",
    rowvar = "ITTFL",
    statlist = statlist("n"),
    subset = ITTFL == "Y",
    rowtext = "Analysis set: ITT"
  )

knitr::kable(tbl)
```

### Big N row, counts and percentages: `statlist = statlist(c("N", "n (x.x%)"))`

A typical call of using the `freq` function is to create the `tbl` chunk with:

-   the row of big N indicating number of subjects having values in the character variable for each treatment group
-   summary of counts and percentages for each category in the character variable such as age groups, gender, race, ethnicity, etc.

```{r}
tbl <- cdisc_adsl %>%
  freq(
    colvar = "TRT01PN",
    rowvar = "SEX",
    statlist = statlist(c("N", "n (x.x%)")),
    row_header = "Sex"
  )

knitr::kable(tbl)
```


### Using factor to order category summary

In the above table, `F` appears first and then `M` is shown. This is because
`SEX` is a character variable and alphabetical sorting is applied to the summarized results. We can convert
`SEX` to a factor variable with the customized order and labels for enabling the user-defined sorting.

```{r}
tbl <- cdisc_adsl %>%
  mutate(
    SEX = factor(SEX, levels = c("M", "F"), labels = c("Male", "Female"))
  ) %>%
  freq(
    colvar = "TRT01PN",
    rowvar = "SEX",
    statlist = statlist(c("N", "n (x.x%)")),
    row_header = "Sex"
  )

knitr::kable(tbl)
```

### Counts, denominators, and percentages: `statlist = statlist(c("n/N (x.x%)"))`

In some outputs, the table mock-up may require showing the percentage denominator,
which can be done by specifying `statlist = statlist("n/N (x.x%)")`.

```{r}
tbl <- cdisc_adsl %>%
  mutate(SEX = factor(
    SEX,
    levels = c("M", "F"), labels = c("Male", "Female")
  )) %>%
  freq(
    colvar = "TRT01PN",
    rowvar = "SEX",
    statlist = statlist("n/N (x.x%)"),
    row_header = "Sex"
  )

knitr::kable(tbl)
```


### By processing: `statlist = statlist(c("N", "n (x.x%)"))`

By-processing splits the summary statistics by another character variable that can be specified in the argument of
`rowbyvar` or `tablebyvar`. For the example shown below, age group categories are summarized by
the `SEX` variable. In this scenario, the denominator should also be split by `SEX` in
addition to `TRT01PN`, which can be done by `statlist(c("N","n (x.x%)"), denoms_by = c("SEX", "TRT01PN"))`.

```{r}
tbl <- cdisc_adsl %>%
  mutate(SEX = factor(
    SEX,
    levels = c("M", "F"), labels = c("Male", "Female")
  )) %>%
  freq(
    colvar = "TRT01PN",
    rowbyvar = "SEX",
    rowvar = "AGEGR1",
    statlist = statlist(c("N", "n (x.x%)"), denoms_by = c("SEX", "TRT01PN")),
    row_header = "Age group"
  )

knitr::kable(tbl)
```

By default, the denominators are calculated by using `colvar`, `tablebyvar`, and `rowbyvar`.
The above `freq` function call will also produce the same results without specifying the `denoms_by`
argument inside the `statlist` function.

### Padding factor levels on and off

When using `rowbyvar` to create by-processing summaries, some levels of the `rowvar` may
have zero records as shown in the example below.

```{r}
tbl <- cdisc_adsl %>%
  freq(
    colvar = "TRT01PN",
    rowbyvar = "ETHNIC",
    rowvar = "RACE",
    statlist = statlist(c("N", "n (x.x%)")),
    row_header = "Race"
  )

knitr::kable(tbl)
```

To remove the zero record rows and create the data driven summary, users can specify
the `pad = FALSE` in the `freq` function.

```{r}
tbl <- cdisc_adsl %>%
  freq(
    colvar = "TRT01PN",
    rowbyvar = "ETHNIC",
    rowvar = "RACE",
    statlist = statlist(c("N", "n (x.x%)")),
    row_header = "Race",
    pad = FALSE
  )

knitr::kable(tbl)
```

### Denominator Dataframe

When using frequency analysis you do not need to always use your main dataframe to calculate your denominators.
For example, an adverse event table may use `ADSL` as the denominator dataframe instead of `ADAE` even
though the counts are coming from `ADAE.`  The example below shows a table counting `AEDECOD` but using `ADSL` to calculate the denominators.

```{r}
adae <- cdisc_adae %>%
  rename(TRT01AN = TRTAN)

tbl <- adae %>%
  freq(
    denom_df = cdisc_adsl,
    colvar = "TRT01AN",
    rowvar = "AEDECOD",
    descending_by = "81"
  )

knitr::kable(head(tbl, 10))
```

### By processing and denominators: `statlist = statlist(c("n (x.x)"), denoms_by = "TRT01AN")`

When using by variables such as `tablebyvar` and `rowbyvar` the denominators may need to be changed
to work correctly.  This also works in conjunction with the `denom_df` argument.  By default, the denominators are
calculated by using `colvar`, `tablebyvar`, and `rowbyvar`.  This works well if you are
using the same dataframe for counts and denominators but this is not always the case. In the following example we
are doing a similar table as above but using `AESEV` as a `rowbyvar`.  In this example we
don't want our denominators by `AESEV` since that variable is not in `ADSL`, which is where our
denominators are coming from.  To change the variables by which our denominators are calculated by we use the `denoms_by`
argument to the `statlist` function.  Below you can see that we are using only
our `colvar` of `TRT01PN` as our `denoms_by`.

```{r}
tbl <- adae %>%
  freq(
    denom_df = cdisc_adsl,
    colvar = "TRT01AN",
    rowvar = "AEDECOD",
    rowbyvar = "AESEV",
    statlist = statlist(c("n (x.x)"), denoms_by = "TRT01AN")
  )

knitr::kable(head(tbl, 10))
```

### Available `statlist` formats for frequency summary

We have shown the common use cases of calling different variants of the `statlist` for frequency summaries. The table below
describes all available options to be specified for the `statlist` in `freq`.

+--------------+-----------------------------------------------------+
| `Statlist`     | Description                                         |
+==============+:===================================================:+
| `n`            | count                                               |
+--------------+-----------------------------------------------------+
| `n (x.x)`      | count (percentage without %)                        |
+--------------+-----------------------------------------------------+
| `n (x.x%)`     | count (percentage with %)                           |
+--------------+-----------------------------------------------------+
| `n/N`          | count/denominator                                   |
+--------------+-----------------------------------------------------+
| `n/N (x.x)`    | count/denominator (percentage without %)            |
+--------------+-----------------------------------------------------+
| `n/N (x.x%)`   | count/denominator (percentage with %)               |
+--------------+-----------------------------------------------------+

To learn more about using the `statlist` function for freq analysis, please type `?statlist` in your console.

## Nested Frequency Analysis

A major portion of Adverse Events (`AE`) summary tables require summarizing number of subjects with treatment-emergent
adverse events by system organ class and preferred term, which is in a nested structure and needs additional processing
on top of the `freq` function. Therefore, we developed the `nested_freq` function to address the nested structure (counts within counts):

* `rowvar`: we can specify nested levels separated by `*` (e.g. `AEBODSYS*AEDECOD`); this can be expanded to three levels.
* `descending_by`: the name of the column for sorting in descending frequency order.
* `cutoff_stat`: the value to cutoff by, `n` (count) or `pct` (percentage); default = `pct`.
* `cutoff`: numeric value of the percentage/count threshold in any treatment group for cutting the data to be presented;
  for example, `cutoff = 1.0` means to only keep the preferred term rows with percentages `>= 1%` when `cutoff_stat = 'pct'`.

In the example below, we will show you how to use these arguments in the `nested_freq` function
call for creating the `AE` summary table by `AEBODSYS` and `AEDECOD`.

```{r, message=FALSE}
adae <- cdisc_adae %>%
  filter(SAFFL == "Y", TRTEMFL == "Y") %>%
  filter(AEBODSYS %in% c(
    "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS",
    "SKIN AND SUBCUTANEOUS TISSUE DISORDERS"
  )) %>%
  rename(TRT01AN = TRTAN)

adsl <- cdisc_adsl %>%
  filter(SAFFL == "Y")
```

For illustration purpose, we subset the `adae` data to only contain records in the 2 categories of
system organ class: GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS, SKIN AND SUBCUTANEOUS TISSUE DISORDERS. So the table output
is not too long and will be easier to visualize. In addition, we would like to sort the output
by the active drug group `(TRT01AN = 81)` with descending frequency. Therefore, we specify `descending_by = "81"`.

```{r}
tbl <- nested_freq(adae,
  denom_df = adsl,
  colvar = "TRT01AN",
  rowvar = "AEBODSYS*AEDECOD",
  statlist = statlist("n (x.x%)"),
  descending_by = "81",
  row_header = "System organ class \\\n Preferred term"
)

knitr::kable(tbl)
```

As shown in the output above, the most frequent system organ class is **GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS**,
followed by **SKIN AND SUBCUTANEOUS TISSUE DISORDERS**. Within each system organ class, the preferred terms are
also sorted by descending frequency. When there is a tie in the counts, the preferred terms are sorted alphabetically.

The cutoff feature is controlled by two arguments: `cutoff` and `cutoff_stat`. If we want
to remove the rows of preferred terms with only one count in any treatment columns, we can specify
`cutoff = 2` and `cutoff_stat = "n"` in the `nested_freq` call below.

```{r}
tbl <- nested_freq(adae,
  denom_df = adsl,
  colvar = "TRT01AN",
  rowvar = "AEBODSYS*AEDECOD",
  statlist = statlist("n (x.x%)"),
  descending_by = "81",
  cutoff = 2,
  cutoff_stat = "n",
  row_header = "System organ class \\\n Preferred term"
)

knitr::kable(tbl)
```

The same cutoff results can also be achieved by specifying the cutoff percentage: `cutoff = 25` and `cutoff_stat = "pct"`.
In our example, we only have 5 subjects in each arm and one subject count is equal to 20%,
and so we need more than 20% as the cutoff.

For only keeping the preferred terms with at least 2 counts in the active arm of `TRT01AN = 81`
(i.e. not considering the other arms), we can specify `cutoff = "81 >= 2"` and `cutoff_stat = "n"`.


```{r}
tbl <- nested_freq(adae,
  denom_df = adsl,
  colvar = "TRT01AN",
  rowvar = "AEBODSYS*AEDECOD",
  statlist = statlist("n (x.x%)"),
  descending_by = "81",
  cutoff = "81 >= 2",
  cutoff_stat = "n",
  row_header = "System organ class \\\n Preferred term"
)

knitr::kable(tbl)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freq.R
\name{freq}
\alias{freq}
\title{Frequency counts and percentages}
\usage{
freq(
  df,
  denom_df = df,
  colvar = NULL,
  tablebyvar = NULL,
  rowvar = NULL,
  rowbyvar = NULL,
  statlist = getOption("tidytlg.freq.statlist.default"),
  decimal = 1,
  nested = FALSE,
  cutoff = NULL,
  cutoff_stat = "pct",
  subset = TRUE,
  descending_by = NULL,
  display_missing = FALSE,
  rowtext = NULL,
  row_header = NULL,
  .keep = TRUE,
  .ord = FALSE,
  pad = TRUE,
  ...
)
}
\arguments{
\item{df}{(required) dataframe containing records to summarize by treatment.}

\item{denom_df}{(optional) dataframe used for population based denominators
(default = \code{df}).}

\item{colvar}{(required) treatment variable within \code{df} to use to summarize}

\item{tablebyvar}{(optional) repeat entire table by variable within \code{df}}

\item{rowvar}{(required) character vector of variables to
summarize within the dataframe.}

\item{rowbyvar}{(optional) repeat \code{rowvar} by variable within \code{df}}

\item{statlist}{(optional) \code{statlist} object of stats to keep of length
1 or 2 specifying list of statistics and format desired
(e.g \code{statlist(c("N", "n (x.x\\\%)"))}) (default = \code{statlist(c("n (x.x)"))}).}

\item{decimal}{(optional) decimal precision root level default (default = 1).}

\item{nested}{(optional) INTERNAL USE ONLY. The default should
not be changed. Switch on when this function is called by
\code{nested_freq()} so we will not include the by variables as
part of the group denominators (default = \code{FALSE}).}

\item{cutoff}{(optional) percentage cutoff threshold. This can be passed as a
numeric cutoff, in that case any rows with greater than or equal to that
cutoff will be preserved, others will be dropped. To specify a single column
to define the cutoff logic, pass a character value of the form
\verb{<colName> >= <value>} and only that column will be used.}

\item{cutoff_stat}{(optional) The value to cutoff by, \code{n} or \code{pct.}
(default = \code{'pct'}). Can be done with multiple columns
by adding \code{&} or \code{|} ex. \code{col1 >= val1 & col2 >= val2}.}

\item{subset}{(optional) An R expression that will be passed to a
\code{\link[dplyr:filter]{dplyr::filter()}} function to subset the \code{data.frame}.
This is performed on the numerator before any other
derivations. Denominators must be preprocessed and passed
through using \code{denom_df}.}

\item{descending_by}{(optional) The column or columns to sort descending
counts. Can also provide a named list to do ascending order ex.
\code{c("VarName1" = "asc", "VarName2" = "desc")} would sort by \code{VarName1} in
ascending order and \code{VarName2} in descending order.
In case of a tie in count or \code{descending_by} not provided,
the columns will be sorted alphabetically.}

\item{display_missing}{(optional) Should the "missing" values be displayed?
If missing values are displayed, denominators will include missing values.
(default = \code{FALSE}).}

\item{rowtext}{(optional) A character vector used to rename the \code{label}
column. If named, names will give the new level and values will be the
replaced value. If unnamed, and the table has only one row, the \code{rowtext}
will rename the label of the row. If the \code{rowtext} is unnamed,
the table has no rows, and there is a subset,
the table will be populated with zeros and the label will be the only row.}

\item{row_header}{(optional) A character vector to be added to the table.}

\item{.keep}{(optional) Should the \code{rowbyvar} and \code{tablebyvar} be
output in the table. If \code{FALSE}, \code{rowbyvar} will still be
output in the \code{label} column. (Default = \code{TRUE}).}

\item{.ord}{Should the ordering columns be output with the table? This is
useful if a table needs to be merged or reordered in any way after build.}

\item{pad}{(optional) A boolean that controls if levels with zero records
should be included in the final table. (default = \code{TRUE}).}

\item{...}{(optional) Named arguments to be included as columns on the table.}
}
\value{
A dataframe of results
}
\description{
Frequency counts and percentages for a variable by treatment and/or group.
}
\section{Sorting a 'freq' table}{


By default, a frequency table is sorted based on the factor level of the
\code{rowvar} variable. If the \code{rowvar} variable isn't a factor, it will be
sorted alphabetically. This behavior can be modified in two ways, the first
is the \code{char2factor()} function that offers a interface for discretization a
variable based on a numeric variable, like \code{VISITN}. The second is based on
the \code{descending_by} argument which will sort based on counts on a variable.
}

\examples{
adsl <- data.frame(
  USUBJID = c("DEMO-101", "DEMO-102", "DEMO-103"),
  RACE = c("WHITE", "BLACK", "ASIAN"),
  SEX = c("F", "M", "F"),
  colnbr = factor(c("Placebo", "Low", "High"))
)

# Unique subject count of a single variable
freq(adsl,
  colvar = "colnbr",
  rowvar = "RACE",
  statlist = statlist("n")
)

# Unique subject count and percent of a single variable
freq(adsl,
  colvar = "colnbr",
  rowvar = "RACE",
  statlist = statlist(c("N", "n (x.x\%)"))
)

# Unique subject count of a variable by another variable
freq(adsl,
  colvar = "colnbr",
  rowvar = "RACE",
  rowbyvar = "SEX",
  statlist = statlist("n")
)

# Unique subject count of a variable by another variable using colvar and
# group to define the denominator
freq(adsl,
  colvar = "colnbr",
  rowvar = "RACE",
  rowbyvar = "SEX",
  statlist = statlist("n (x.x\%)", denoms_by = c("colnbr", "SEX"))
)

# Cut records where count meets threshold for any column
freq(cdisc_adsl,
  rowvar = "ETHNIC",
  colvar = "TRT01P",
  statlist = statlist("n (x.x\%)"),
  cutoff = "5",
  cutoff_stat = "n"
)

# Cut records where count meets threshold for a specific column
freq(cdisc_adsl,
  rowvar = "ETHNIC",
  colvar = "TRT01P",
  statlist = statlist("n (x.x\%)"),
  cutoff = "Placebo >= 3",
  cutoff_stat = "n"
)

# Below illustrates how to make the same calls to freq() as above, using
# table and column metadata.

# Unique subject count of a single variable
table_metadata <- tibble::tribble(
  ~anbr, ~func, ~df, ~rowvar, ~statlist, ~colvar,
  1, "freq", "cdisc_adsl", "ETHNIC", statlist("n"), "TRT01PN"
)

generate_results(table_metadata,
  column_metadata = column_metadata,
  tbltype = "type1"
)

# Unique subject count and percent of a single variable
table_metadata <- tibble::tribble(
  ~anbr, ~func, ~df, ~rowvar, ~statlist, ~colvar,
  "1", "freq", "cdisc_adsl", "ETHNIC", statlist(c("N", "n (x.x\%)")), "TRT01PN"
)

generate_results(table_metadata,
  column_metadata = column_metadata,
  tbltype = "type1"
)

# Cut records where count meets threshold for any column
table_metadata <- tibble::tibble(
  anbr = "1", func = "freq", df = "cdisc_adsl", rowvar = "ETHNIC",
  statlist = statlist("n (x.x\%)"), colvar = "TRT01PN", cutoff = 5,
  cutoff_stat = "n"
)

generate_results(table_metadata,
  column_metadata = column_metadata,
  tbltype = "type1"
)

# Cut records where count meets threshold for a specific column
table_metadata <- tibble::tibble(
  anbr = 1, func = "freq", df = "cdisc_adsl", rowvar = "ETHNIC",
  statlist = statlist("n (x.x\%)"), colvar = "TRT01PN",
  cutoff = "col1 >= 3", cutoff_stat = "n"
)

generate_results(table_metadata,
  column_metadata = column_metadata,
  tbltype = "type1"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmdpstitle.R
\name{rmdpstitle}
\alias{rmdpstitle}
\title{Get Titles and Footnotes for all TLGs or one specific TLG}
\usage{
rmdpstitle(
  df,
  tblid,
  idvar = "tblid",
  identifier = "identifier",
  text = "text"
)
}
\arguments{
\item{df}{dataframe with three variables; table name, row identifier (\code{TITLE}
or \code{FOOTNOTEn}), and title or footnote text to display.}

\item{tblid}{character vector containing the table id, optional, used to
subset \code{df} to a specific table (defaults to \code{tblid}).}

\item{idvar}{character vector containing the variable in \code{df} that contains
your table id.}

\item{identifier}{character vector containing the variable name in \code{df} that
contains your record identifier (defaults to "identifier").}

\item{text}{character vector containing the variable name in \code{df} that
contains your title and footnote text (defaults to "text").}
}
\value{
list of length two, the first element contains the titles
as a \code{tibble} and the second contains the footnotes as a list.
}
\description{
Get Titles and Footnotes for all TLGs or one specific TLG
}
\examples{
tblid <- "TSIDEM01"

titles <- tibble::tribble(
  ~tblid, ~identifier, ~text,
  "TSIDEM01", "TITLE", "Demographics Example",
  "TSIDEM01", "FOOTNOTE1", "Example footnote."
)

title_foot <- rmdpstitle(titles, tblid)

title_foot[[1]]
title_foot[[2]]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bind_table.R
\name{bind_table}
\alias{bind_table}
\title{Bind a set of \code{tidytlg} tables together with formatting variables}
\usage{
bind_table(
  ...,
  colvar = NULL,
  tablebyvar = NULL,
  rowbyvar = NULL,
  prefix = NULL,
  add_count = FALSE,
  add_format = TRUE,
  column_metadata_file = NULL,
  column_metadata = NULL,
  tbltype = NULL
)
}
\arguments{
\item{...}{(required) a set of \code{tidytlg} tables to bind together}

\item{colvar}{(required) treatment variable within \code{df} to use to summarize.
Required if \code{add_count} is TRUE.}

\item{tablebyvar}{(optional) repeat entire table by variable within \code{df}.}

\item{rowbyvar}{(optional) any \code{rowbyvar} values used to create the table.}

\item{prefix}{(optional) text to prefix the values of \code{tablebyvar} with.}

\item{add_count}{(optional) Should a count be included in the \code{tablebyvar}?
(default = \code{TRUE})}

\item{add_format}{(optional) Should format be added to the output table?
This is done using the \code{add_format} function. (default = \code{TRUE})}

\item{column_metadata_file}{(optional) An excel file for \code{column_metadata}.
Does not change the behavior of the function binds the column metadata
for \code{gentlg}. If a \code{column_metadata} dataframe is passed in too,
this is ignored.}

\item{column_metadata}{(optional) A dataframe containing the column metadata.
This will be used in place of \code{column_metadata_file.}}

\item{tbltype}{(optional) A value used to subset the \code{column_metadata_file.}}
}
\value{
The \code{tidytlg} tables bound together reflecting the
\code{tablebyvars} used.
}
\description{
\code{bind_table} combines analysis results with formatting variables
(\code{indentme}, \code{newrows}, \code{newpage}) based on \code{by} variables
(\code{tablebyvar}, \code{rowbyvar}), such that appropriate formatting (indentation,
line break, page break) can be applied while creating the output.
It can also attach the column metadata attribute,
which will be automatically used in \code{gentlg} for creating output.
}
\examples{
library(magrittr)

# bind tables together
t1 <- cdisc_adsl \%>\%
  freq(
    colvar = "TRT01PN",
    rowvar = "ITTFL",
    statlist = statlist("n"),
    subset = ITTFL == "Y",
    rowtext = "Analysis set: ITT"
  )

t2 <- cdisc_adsl \%>\%
  univar(
    colvar = "TRT01PN",
    rowvar = "AGE",
    decimal = 0,
    row_header = "Age, years"
  )

bind_table(t1, t2)

# bind tables together w/by groups
t1 <- cdisc_adsl \%>\%
  freq(
    colvar = "TRT01PN",
    rowvar = "ITTFL",
    rowbyvar = "SEX",
    statlist = statlist("n"),
    subset = ITTFL == "Y",
    rowtext = "Analysis set: ITT"
  )

t2 <- cdisc_adsl \%>\%
  univar(
    colvar = "TRT01PN",
    rowvar = "AGE",
    rowbyvar = "SEX",
    decimal = 0,
    row_header = "Age, years"
  )

bind_table(t1, t2, rowbyvar = "SEX")

# bind tables together w/table by groups
t1 <- cdisc_adsl \%>\%
  freq(
    colvar = "TRT01PN",
    rowvar = "ITTFL",
    tablebyvar = "SEX",
    statlist = statlist("n"),
    subset = ITTFL == "Y",
    rowtext = "Analysis set: ITT"
  )

t2 <- cdisc_adsl \%>\%
  univar(
    colvar = "TRT01PN",
    rowvar = "AGE",
    tablebyvar = "SEX",
    decimal = 0,
    row_header = "Age, years"
  )

bind_table(t1, t2, tablebyvar = "SEX")

# w/prefix
bind_table(t1, t2, tablebyvar = "SEX", prefix = "Gender: ")

# w/counts
bind_table(t1, t2, tablebyvar = "SEX", add_count = TRUE, colvar = "TRT01PN")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{replace_na_with_blank}
\alias{replace_na_with_blank}
\title{Replace NA with ""}
\usage{
replace_na_with_blank(x)
}
\arguments{
\item{x}{variable to check for NA and replace with "".}
}
\value{
x with NAs replaced with "". Factors will add ""
as the first level.
}
\description{
Used to swap in "" for by variables so the headers sort correctly to the top
}
\examples{
replace_na_with_blank(c("a", "b", NA))

replace_na_with_blank(factor(c("a", "b", NA), levels = c("a", "b")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statlist.R
\name{statlist}
\alias{statlist}
\title{Create a \code{statlist} interface for a table}
\usage{
statlist(stats, ...)
}
\arguments{
\item{stats}{(required) A character vector of statistics to display in the
table.}

\item{...}{(optional) Additional configuration for stats. See sections below
for allowable arguments.}
}
\value{
A \code{statlist} object that can be passed in the \code{statlist} argument of
\code{freq}, \code{nested_freq}, or \code{univar}.
}
\description{
The \code{statlist} is the interface for the presentation of data in a \code{tidytlg}
table.
}
\section{\code{Statlists} for \code{freq()} and \code{nested_freq()}}{

\code{freq()} \code{statlists} can be composed of \code{n} (count), \code{N} (denominator), and
\code{x.x} (percentage, formatted with or without a percent sign).
Denominators will include missing values if the 'display_missing'
argument is \code{TRUE}, otherwise they will be excluded. They can be arranged
in the following ways:
\itemize{
\item{\code{n}}
\item{\code{n/N}}
\item{\code{n (x.x)}}
\item{\verb{n (x.x\%)}}
\item{\code{n/N (x.x)}}
\item{\verb{n/N (x.x\%)}}
}

The following other configurations are supported:
\itemize{
\item{\code{denoms_by} - Controls what groupings of variables should define the
denominator. Variables should be passed as a quoted vector}
\item{\code{distinct} - A boolean value. Should the numerator reflect distinct
\code{USUBJIDs} or event counts. Defaults to \code{TRUE} which captures distinct
subjects.}
\item{\code{distinct_by} - A character value used to select the variable that
should be used to "distinct" the frequency tables. Defaults to \code{USUBJID}.}
\item{\code{zero_denom} - The string to display when there are no records found
in an entire denominator group. Defaults to \code{-}}
\item{\code{zero_n} - The string to display when there are no records found for
a numerator. Defaults to \code{0}.}
}
}

\section{\code{Statlists} for \code{univar} \code{statlists}}{

\itemize{
\item{\code{N}}
\item{\code{SUM}}
\item{\code{MEAN}}
\item{\code{GeoMEAN}}
\item{\code{SD}}
\item{\code{SE}}
\item{\code{CV}}
\item{\code{GSD}}
\item{\code{GSE}}
\item{\code{MEANSD}}
\item{\code{MEANSE}}
\item{\code{MEDIAN}}
\item{\code{MIN}}
\item{\code{MAX}}
\item{\code{RANGE}}
\item{\code{Q1}}
\item{\code{Q3}}
\item{\code{IQRANGE}}
\item{\code{MEDRANGE}}
\item{\code{MEDIQRANGE}}
\item{\code{MEAN_CI}}
\item{\code{GeoMEAN_CI}}
}

where \code{GeoMEAN}: Geometric Mean, \code{CV}: Coefficient of Variation,
\code{GSD}: Geometric standard deviation, \code{GSE}: Geometric standard error,
\code{MEAN_CI}: Mean (95\% C.I.), \code{GeoMEAN_CI}: Geometric Mean (95\% C.I.).
In calculating geometric statistics, if there
are zero values in the inputs, zero values will be excluded before
calculating geometric statistics.
}

\examples{

freq(
  mtcars,
  colvar = "gear",
  rowvar = "cyl",
  rowbyvar = "am",
  statlist = statlist("n/N (x.x)",
    distinct = FALSE,
    denoms_by = c("gear", "am"),
    zero_denom = "_0_"
  )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{tidytlg_titles}
\alias{tidytlg_titles}
\alias{tidytlg_metadata}
\title{Helper functions for returning files used in \code{gentlg}}
\usage{
tidytlg_titles(path)

tidytlg_metadata(path)
}
\arguments{
\item{path}{Working directory of the project}
}
\value{
A character vector to the requested file.
}
\description{
Helper functions for returning files used in \code{gentlg}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_results.R
\name{generate_results}
\alias{generate_results}
\title{Generate Results using Table and Column Metadata}
\usage{
generate_results(
  table_metadata,
  column_metadata_file = NULL,
  column_metadata = NULL,
  env = parent.frame(),
  tbltype = NULL,
  add_count = FALSE
)
}
\arguments{
\item{table_metadata}{a data frame containing table metadata (see
?table_metadata for details)}

\item{column_metadata_file}{An excel file with the data
for \code{column_metadata}.
The file is read in with \code{readxl::read_excel()}. Should not be used with
\code{column_metadata} argument. Results in a data frame containing the column
metadata that is passed to \code{tlgsetup} (see \code{tlgsetup()} for details). If a
\code{column_metadata} data frame is passed in too, this is ignored.}

\item{column_metadata}{A data frame containing the column metadata. This will
be used in place of \code{column_metadata_file}.}

\item{env}{environment to find data frame specified in the table metadata
(defaults to parent environment).}

\item{tbltype}{If used, this will be used to subset
the \code{column_metadata} based on the \code{tbltype} column.}

\item{add_count}{Passed to \code{bind_table()} should counts be added for
\code{tablebyvars}?}
}
\value{
dataframe of results
}
\description{
Generate Results using Table and Column Metadata
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_bottom_borders.R
\name{row_border}
\alias{row_border}
\title{Adds a continuous bottom border under a row}
\usage{
row_border(row)
}
\arguments{
\item{row}{\code{numeric} the row of the table}
}
\description{
Adds a continuous bottom border under a row
}
\seealso{
Other border_functions: 
\code{\link{col_borders}()},
\code{\link{no_borders}()},
\code{\link{single_border}()},
\code{\link{spanning_borders}()}
}
\concept{border_functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tlgsetup.R
\name{tlgsetup}
\alias{tlgsetup}
\title{Setup data to support the specified column type}
\usage{
tlgsetup(
  df,
  var,
  column_metadata_file = NULL,
  column_metadata = NULL,
  tbltype = NULL
)
}
\arguments{
\item{df}{dataframe of records for analysis.}

\item{var}{character vector that identifies the numeric column/treatment
variable.}

\item{column_metadata_file}{A file containing the column metadata. Read in
with \code{readxl::read_excel()}. If a \code{column_metadata} dataframe is passed in
too, this is ignored.}

\item{column_metadata}{A dataframe containing the column metadata. This will
be used in place of \code{column_metadata_file}.}

\item{tbltype}{A value used to subset the \code{column_metadata}, both this
and the file requirements are needed to bind the data to the table.}
}
\value{
dataframe with observations added to support the column type as well
as the factor variable \code{colnbr} which is used as our new column summary
variable. Regardless of if a \code{coldef} exists in data,
the column will exist in the table.
}
\description{
\code{tlgsetup} is useful for preprocessing total columns and columns composed of
other columns. \code{tlgsetup} is called internally by \code{generate_results()}
and can be run manually for custom tables.
}
\examples{
df <-
  tibble::tribble(
    ~TRT01AN, ~USUBJID,
    0, "A",
    54, "B",
    81, "C"
  )

tlgsetup(df, "TRT01AN", column_metadata = column_metadata)

# Using a dataframe of column metadata
column_metadata <-
  tibble::tribble(
    ~tbltype, ~coldef, ~decode,                ~span1,
    "type1",  "0",     "Placebo",              "",
    "type1",  "54",    "Low Dose",             "Xanomeline",
    "type1",  "81",    "High Dose",            "Xanomeline",
    "type1",  "54+81", "Total Xanomeline",     ""
  )

tlgsetup(df, "TRT01AN", column_metadata = column_metadata)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insert_empty_rows.R
\name{insert_empty_rows}
\alias{insert_empty_rows}
\title{Inserts empty rows into a data frame}
\usage{
insert_empty_rows(huxme, newrows = huxme$newrows)
}
\arguments{
\item{huxme}{\code{data.frame} The input data frame.}

\item{newrows}{\code{integer} A numeric vector specifying where the new rows
should be inserted.}
}
\value{
A data frame with added new empty rows.
}
\description{
Inserts empty rows into a data frame
}
\details{
\code{gentlg} allows for formatting the input table based on formatting columns
(see \code{\link[=gentlg]{gentlg()}}). One of the formatting mechanisms is empty row insertion.
This function inserts the empty rows based on the \code{newrows} column in the
data frame. The new rows are inserted before the rows with value \code{1} in the
\code{newrows} column.
}
\examples{
df <- iris[1:10, ]
df <- as.data.frame(apply(df, 2, as.character))
df$newrows <- c(0, 1, 0, 1, 1, 0, 0, 0, 0, 0)
insert_empty_rows(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cdisc adlb}
\alias{cdisc adlb}
\alias{cdisc_adlb}
\title{\code{ADLB} data created from subsetting the \code{CDISC} \code{ADLB} dataset}
\format{
A data frame with 2154 rows and 46 variables:
\describe{
\item{\code{STUDYID}}{Study Identifier}
\item{\code{SUBJID}}{Subject Identifier for the Study}
\item{\code{USUBJID}}{Unique Subject Identifier}
\item{\code{TRTA}}{Actual Treatment}
\item{\code{TRTAN}}{Actual Treatment (N)}
\item{\code{TRTSDT}}{Date of First Exposure to Treatment}
\item{\code{TRTEDT}}{Date of Last Exposure to Treatment}
\item{\code{AGE}}{Age}
\item{\code{AGEGR1}}{Pooled Age Group 1}
\item{\code{AGEGR1N}}{Pooled Age Group 1 (N)}
\item{\code{RACE}}{Race}
\item{\code{RACEN}}{Race (N)}
\item{\code{SEX}}{Sex}
\item{\code{COMP24FL}}{Finishers of Week 24 Population Flag}
\item{\code{DSRAEFL}}{Discontinued due to \code{AE}?}
\item{\code{SAFFL}}{Safety Population Flag}
\item{\code{AVISIT}}{Analysis Visit}
\item{\code{AVISITN}}{Analysis Visit (N)}
\item{\code{ADY}}{Analysis Relative Day}
\item{\code{ADT}}{Analysis Date}
\item{\code{VISIT}}{Visit Name}
\item{\code{VISITNUM}}{Visit Number}
\item{\code{PARAM}}{Parameter}
\item{\code{PARAMCD}}{Parameter Code}
\item{\code{PARAMN}}{Parameter (N)}
\item{\code{PARCAT1}}{Parameter Category 1}
\item{\code{AVAL}}{Analysis Value}
\item{\code{BASE}}{Baseline Value}
\item{\code{CHG}}{Change from Baseline}
\item{\code{A1LO}}{Analysis Range 1 Lower Limit}
\item{\code{A1HI}}{Analysis Range 1 Upper Limit}
\item{\code{R2A1LO}}{Ratio to Analysis Range 1 Lower Limit}
\item{\code{R2A1HI}}{Ratio to Analysis Range 1 Upper Limit}
\item{\code{BR2A1LO}}{Base Ratio to Analysis Range 1 Lower Limit}
\item{\code{BR2A1HI}}{Base Ratio to Analysis Range 1 Upper Limit}
\item{\code{ANL01FL}}{Analysis 01 - Special Interest Flag}
\item{\code{ALBTRVAL}}{Amount Threshold Range}
\item{\code{ANRIND}}{Analysis Reference Range Indicator}
\item{\code{BNRIND}}{Baseline Reference Range Indicator}
\item{\code{ABLFL}}{Baseline Record Flag}
\item{\code{AENTMTFL}}{Last value in treatment visit}
\item{\code{LBSEQ}}{Sequence Number}
\item{\code{LBNRIND}}{Reference Range Indicator}
\item{\code{LBSTRESN}}{Numeric Result/Finding in Standard Units}
}
}
\source{
CDISC SDTM/ADAM Pilot Project.
}
\usage{
cdisc_adlb
}
\description{
\code{ADLB} data created from subsetting the \code{CDISC} \code{ADLB} dataset
}
\keyword{CDISC}
\keyword{adlb}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_bottom_borders.R
\name{old_format}
\alias{old_format}
\title{Adds bottom borders according to the old formatting}
\usage{
old_format(ht, colspan, colheader, tlf)
}
\arguments{
\item{ht}{the \code{hux} object passed to \code{\link[=gentlg]{gentlg()}}}

\item{colspan}{\code{colspan} argument to \code{\link[=gentlg]{gentlg()}}}

\item{colheader}{\code{colheader} argument to \code{\link[=gentlg]{gentlg()}}}

\item{tlf}{\code{character} type of the output}
}
\value{
a bottom border matrix for use with \code{\link[=add_bottom_borders]{add_bottom_borders()}} or \code{NULL}
if \code{ht} is \code{NULL}
}
\description{
This function is vectorized over its arguments.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_zero_padding.R
\name{remove_zero_padding}
\alias{remove_zero_padding}
\title{Removes no-op padding from an RTF markup}
\usage{
remove_zero_padding(rtf)
}
\arguments{
\item{rtf}{\code{character(1)} RTF markup.}
}
\value{
\code{character(1)} RTF markup with removed zero padding
}
\description{
Removes no-op padding from an RTF markup
}
\details{
Removes no-op padding, which is padding that has a thickness of zero
from an RTF markup.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replace_lead_whitespaces_ind.R
\name{replace_lead_whitespaces_ind}
\alias{replace_lead_whitespaces_ind}
\title{Replace leading whitespaces with left indentation RTF markup}
\usage{
replace_lead_whitespaces_ind(x)
}
\arguments{
\item{x}{\code{character(1)} a string to replace the leading whitespaces.}
}
\value{
\code{character(1)} RTF markup with leading whitespaces replaced.
}
\description{
Replace leading whitespaces with left indentation RTF markup
}
\details{
The following function receives a string 'x' and returns a modified string
where every leading whitespace is replaced with 90 twips (0.0625 inches) left indentation RTF markup.
If the input does not start with a whitespace, the string is returned as is.
}
\examples{
tidytlg:::replace_lead_whitespaces_ind("    this is x")
# [1] "\intbl\li360\fi0 this is x"
tidytlg:::replace_lead_whitespaces_ind("this is x")
# [1] "this is x"

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_bottom_borders.R
\name{spanning_borders}
\alias{spanning_borders}
\title{Adds borders under cells in a row, excluding the first column.}
\usage{
spanning_borders(row, cols = c(-1))
}
\arguments{
\item{row}{\code{numeric} the row of the table}

\item{cols}{\code{numeric} the columns of the row to consider}
}
\description{
Adds borders under cells that are not empty in a given row,
omitting the first column of the row.
The borders do not touch each other - they are separate.
}
\seealso{
Other border_functions: 
\code{\link{col_borders}()},
\code{\link{no_borders}()},
\code{\link{row_border}()},
\code{\link{single_border}()}
}
\concept{border_functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nested_freq.R
\name{nested_freq}
\alias{nested_freq}
\title{Generate nested count/percent for two or three levels}
\usage{
nested_freq(
  df,
  denom_df = df,
  colvar = NULL,
  tablebyvar = NULL,
  rowvar = NULL,
  rowbyvar = NULL,
  statlist = getOption("tidytlg.nested_freq.statlist.default"),
  decimal = 1,
  cutoff = NULL,
  cutoff_stat = "pct",
  subset = TRUE,
  descending_by = NULL,
  display_missing = FALSE,
  rowtext = NULL,
  row_header = NULL,
  .keep = TRUE,
  .ord = FALSE,
  ...
)
}
\arguments{
\item{df}{(required) dataframe containing the two levels to summarize}

\item{denom_df}{(optional) dataframe containing records to use as the
denominator (default = \code{df})}

\item{colvar}{(required) treatment variable within \code{df} to use to summarize}

\item{tablebyvar}{(optional) repeat entire table by variable within \code{df.}}

\item{rowvar}{(required) nested levels separated by a star, for example
\code{AEBODSYS*AEDECOD},  this can handle up to three levels.}

\item{rowbyvar}{(optional) repeat \code{rowvar} by variable within \code{df}}

\item{statlist}{(optional) count/percent type to return
\code{(default = "n (x.x)")}}

\item{decimal}{(optional) decimal precision root level \code{(default = 1)}}

\item{cutoff}{(optional) numeric value used to cut the data to a percentage
threshold, if any column meets the threshold the entire record is kept.}

\item{cutoff_stat}{(optional) The value to cutoff by, n or pct. (default =
'pct')}

\item{subset}{(optional) An R expression that will be passed to a
\code{dplyr::filter()} function to subset the data.frame}

\item{descending_by}{(optional) The column or columns to sort descending
values by. Can also provide a named list to do ascending order. ex.
\code{c("VarName1" = "asc", "VarName2" = "desc")} would sort by \code{VarName1} in
ascending order and \code{VarName2} in descending order. If not provided, the
columns will be sorted alphabetically.}

\item{display_missing}{(optional) Should the "missing" values be displayed?
(default = \code{FALSE})}

\item{rowtext}{(optional) A character vector used to rename the \code{label}
column. If named, names will give the new level and values will be the
replaced value. If unnamed, and the table has only one row, the \code{rowtext}
will rename the \code{label} of the row.}

\item{row_header}{(optional) A character vector to be added to the table.}

\item{.keep}{(optional) Should the \code{rowbyvar} and \code{tablebyvar} be
output in the table. If \code{FALSE}, \code{rowbyvar} will still be output
in the \code{label} column. \code{(default = TRUE)}.}

\item{.ord}{Should the ordering columns be output with the table? This is
useful if a table needs to be merged or reordered in any way after build.}

\item{...}{(optional) Named arguments to be included as columns on the table.}
}
\value{
A dataframe of nested results by \code{colvar} and optional \code{tablebyvar}.
There are a few additional variable sets added to support multiple
requirements. \cr  \cr The level variables (\code{level1_}, \code{level2_},
\code{level3_}) will carry down the counts for each level to every record.  This
allows for easy sorting of nested groups. \cr  \cr The header variables
(\code{header1}, \code{header2}, \code{header3}) will flag the header for each level to
ensure each level header is sorted to the top of the level. \cr  \cr The n
variables ("n_\emph{") provide a numeric variable containing frequency for each
\code{colvar}.  This can be used to sort and filter records. \cr  \cr The pct
variables ("pct_}") provide a numeric variable containing percentages for
each \code{colvar}.  This can be used to sort and filter records.\cr  \cr
}
\description{
This will call \code{freq()} multiple times and combine the levels together. This
is useful for adverse event and concomitant medications.
}
\examples{
adae <- data.frame(
  SITEID = c("100", "100", "100", "200", "200", "200"),
  USUBJID = c(
    "Demo1-101", "Demo1-102", "Demo1-103",
    "Demo1-104", "Demo1-105", "Demo1-106"
  ),
  AEBODSYS = c(
    "Cardiac disorders", "Cardiac disorders",
    "Respiratory, thoracic and mediastinal disorders",
    "Infections and infestations",
    "Skin and subcutaneous tissue disorders",
    "Infections and infestations"
  ),
  AEDECOD = c(
    "Arrhythmia supraventricular", "Cardiac failure",
    "Chronic obstructive pulmonary disease", "Pneumonia",
    "Pustular psoriasis", "Upper respiratory tract infection"
  ),
  colnbr = structure(
    c(1L, 2L, 3L, 1L, 2L, 3L),
    .Label = c("Active", "Placebo", "Comparator"),
    class = "factor"
  )
)

# Frequency and percent for two levels of nesting
nested_freq(adae,
  colvar = "colnbr",
  rowvar = "AEBODSYS*AEDECOD",
  statlist = statlist("n (x.x\%)")
)

# Frequency and percent for three levels of nesting (for illustrative
# purpose)
nested_freq(adae,
  colvar = "colnbr",
  rowvar = "SITEID*AEBODSYS*AEDECOD",
  statlist = statlist("n (x.x\%)")
)

# Cut records where pct meets threshold for a any column
nested_freq(cdisc_adae,
  colvar = "TRTA",
  rowvar = "AEBODSYS*AEDECOD",
  statlist = statlist("n (x.x\%)", distinct = TRUE),
  cutoff = 2,
  cutoff_stat = "n"
)

# Cut records where pct meets threshold for a specific column
nested_freq(cdisc_adae,
  rowvar = "AEBODSYS*AEDECOD",
  colvar = "TRTAN",
  statlist = statlist("n (x.x\%)", distinct = TRUE),
  cutoff = "54 >= 2",
  cutoff_stat = "n"
)

# Frequency and percent for two levels of nesting and sort by descending
# active
nested_freq(adae,
  colvar = "colnbr",
  rowvar = "AEBODSYS*AEDECOD",
  statlist = statlist("n (x.x\%)"),
  descending = "Active"
)

# Below illustrates how make the same calls to nested_freq() as above, using
# table and # column metadata along with generate_results().

column_metadata <- tibble::tribble(
  ~tbltype, ~coldef, ~decode,
  "type1", "1", "Placebo",
  "type1", "2", "Low",
  "type1", "3", "High"
)

# Frequency and percent for two levels of nesting
table_metadata <- tibble::tribble(
  ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar, ~statlist,
  "1", "nested_freq", "cdisc_adae", "AEBODSYS*AEDECOD", "type1", "TRTP",
  statlist("n (x.x\%)")
)
# generate_results(table_metadata,
# column_metadata_file = tidytlg_metadata(path)


# Frequency and percent for three levels of nesting (for illustrative purpose)
table_metadata <- tibble::tribble(
  ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar,
  ~statlist,
  "1", "nested_freq", "cdisc_adae", "SITEID*AEBODSYS*AEDECOD", "type1",
  "TRTP", statlist("n (x.x\%)")
)
# Commented out because it takes too long
# generate_results(table_metadata, column_metadata)

# Cut records where pct meets threshold for a any column
column_metadata <- tibble::tribble(
  ~tbltype, ~coldef, ~decode,
  "type2", "1", "Placebo",
  "type2", "2", "Active"
)
table_metadata <- tibble::tibble(
  anbr = "1", func = "nested_freq", df = "cdisc_adae",
  rowvar = "AEBODSYS*AEDECOD",
  tbltype = "type2", colvar = "TRTP", statlist = statlist("n (x.x\%)"),
  dotdotdot = "cutoff = 5"
)
# generate_results(table_metadata,
# column_metadata_file = tidytlg_metadata(path)

# Cut records where pct meets threshold for a specific column
table_metadata <- tibble::tibble(
  anbr = "1", func = "nested_freq", df = "cdisc_adae",
  rowvar = "AEBODSYS*AEDECOD",
  tbltype = "type2", colvar = "TRTP", statlist = statlist("n (x.x\%)"),
  dotdotdot = "cutoff = 'col1 >= 5'"
)
# generate_results(table_metadata,
# column_metadata_file = tidytlg_metadata(path)

# Frequency and percent for two levels of nesting and sort by descending col1
table_metadata <- tibble::tibble(
  anbr = "1", func = "nested_freq", df = "cdisc_adae",
  rowvar = "AEBODSYS*AEDECOD",
  tbltype = "type2", colvar = "TRTP", statlist = statlist("n (x.x\%)"),
  dotdotdot = "descending = 'col1'"
)
# generate_results(table_metadata,
# column_metadata_file = tidytlg_metadata(path)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cdisc advs}
\alias{cdisc advs}
\alias{cdisc_advs}
\title{\code{ADVS} data created from subsetting the \code{CDISC} \code{ADVS} dataset}
\format{
A data frame with 1938 rows and 35 variables:
\describe{
\item{\code{STUDYID}}{Study Identifier}
\item{\code{SITEID}}{Study Site Identifier}
\item{\code{USUBJID}}{Unique Subject Identifier}
\item{\code{AGE}}{Age}
\item{\code{AGEGR1}}{Pooled Age Group 1}
\item{\code{AGEGR1N}}{Pooled Age Group 1 (N)}
\item{\code{RACE}}{Race}
\item{\code{RACEN}}{Race (N)}
\item{\code{SEX}}{Sex}
\item{\code{SAFFL}}{Safety Population Flag}
\item{\code{TRTSDT}}{Date of First Exposure to Treatment}
\item{\code{TRTEDT}}{Date of Last Exposure to Treatment}
\item{\code{TRTP}}{Planned  Treatment}
\item{\code{TRTPN}}{Planned  Treatment (N)}
\item{\code{TRTA}}{Actual Treatment}
\item{\code{TRTAN}}{Actual Treatment (N)}
\item{\code{PARAMCD}}{Parameter Code}
\item{\code{PARAM}}{Parameter}
\item{\code{PARAMN}}{Parameter (N)}
\item{\code{ADT}}{Analysis Date}
\item{\code{ADY}}{Analysis Relative Day}
\item{\code{ATPTN}}{Analysis Timepoint (N)}
\item{\code{ATPT}}{Analysis Timepoint}
\item{\code{AVISIT}}{Analysis Visit}
\item{\code{AVISITN}}{Analysis Visit (N)}
\item{\code{AVAL}}{Analysis Value}
\item{\code{BASE}}{Baseline Value}
\item{\code{BASETYPE}}{Baseline Value}
\item{\code{CHG}}{Change from Baseline}
\item{\code{PCHG}}{Percent Change from Baseline}
\item{\code{VISITNUM}}{Visit Number}
\item{\code{VISIT}}{Visit Name}
\item{\code{VSSEQ}}{Sequence Number}
\item{\code{ANL01FL}}{Analysis 01 - Special Interest Flag}
\item{\code{ABLFL}}{Baseline Record Flag}
}
}
\source{
CDISC SDTM/ADAM Pilot Project.
}
\usage{
cdisc_advs
}
\description{
\code{ADVS} data created from subsetting the \code{CDISC} \code{ADVS} dataset
}
\keyword{CDISC}
\keyword{advs}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidytlg-package.R
\docType{package}
\name{tidytlg}
\alias{tidytlg-package}
\alias{tidytlg}
\title{\code{tidytlg}: A package for producing tables, listings,
and graphs (\code{TLGs}) using \code{tidyverse} packages.}
\description{
The \code{tidytlg} package provide a set of function allowing you to produce TLGs
using table metadata.
}
\section{\code{tidytlg} Options}{


\itemize{ \item{\code{tidytlg.row_type.levels} - The values of row_type and their
respective levels. Default: c("TABLE_BY_HEADER", "BY_HEADER1", "BY_HEADER2",
"BY_HEADER3", "BY_HEADER4", "BY_HEADER5", "HEADER", "N", "VALUE", "NESTED")}
\item{\code{tidytlg.fontsize.title} - Font size for titles in points. Default: 10}
\item{\code{tidytlg.fontsize.table.footnote} - Font size for footnotes in points.
For tables. Default: 9} \item{\code{tidytlg.fontsize.listing.footnote} - Font size
for footnotes in points. For listings. Default: 8}
\item{\code{tidytlg.fontsize.graph.footnote} - Font size for footnotes in points.
For graphs. Default: 8} \item{\code{tidytlg.fontsize.table} - Font size for tables
in points. Default: 9} \item{\code{tidytlg.fontsize.graph} - Font size for graphs in
points. Default: 10} \item{\code{tidytlg.fontsize.listing} - Font size for listings
in points. Default: 8} \item{\code{tidytlg.right.padding} - Amount of right padding
for cells in points. Default: 1} \item{\code{tidytlg.left.padding} - Amount of left
padding for cells in points. Default: 1}
\item{\code{tidytlg.stat_labels} - Labels
to be used for each summary statistic in the output table.  Will need to have
all available statistics if this option is changed!}
\item{\code{tidytlg.precision.extra} - Value to be added on for each summary
statistic when calculating precision.  This will be used to make different
statistics have different precision as needed}
\item{\code{tidytlg.nested_freq.statlist.default} - Default \code{statlist} object for
nested_freq tables. Default: \code{statlist("n (x.x)")}}
\item{\code{tidytlg.univar.statlist.default} - Default \code{statlist} object
for \code{univar} tables.
Default: \code{statlist(c("N", "MEANSD", "MEDIAN", "RANGE", "IQRANGE"))}}
\item{\code{tidytlg.freq.statlist.default} - Default \code{statlist} object
for frequency tables. Default: \code{statlist("n (x.x)")}}
\item{\code{tidytlg.stat_labels} - A \code{data.frame} controlling how the stats
are labeled in a \code{univar} table. See 'Default Stat Labels' section
for defaults.}
\item{\code{tidytlg.denoms.message} - A \code{boolean}, should a message print
detailing what the denominators are?}
}
}

\section{Default Stat Labels}{
\tabular{ll}{
   \strong{stat} \tab \strong{label} \cr
   \code{N} \tab N \cr
   \code{SUM} \tab Sum \cr
   \code{MEAN} \tab Mean \cr
   \code{GeoMEAN} \tab Geometric Mean \cr
   \code{SD} \tab Std. Deviation \cr
   \code{SE} \tab Std. Error \cr
   \code{GSD} \tab Geometric Std. Deviation \cr
   \code{GSE} \tab Geometric Std. Error \cr
   \code{CV} \tab CV \cr
   \code{MEANSD} \tab Mean (SD) \cr
   \code{MEANSE} \tab Mean (SE) \cr
   \code{MEDIAN} \tab Median \cr
   \code{MIN} \tab Minimum \cr
   \code{MAX} \tab Maximum \cr
   \code{RANGE} \tab Range \cr
   \code{Q1} \tab First quartile \cr
   \code{Q3} \tab Third quartile \cr
   \code{IQRANGE} \tab IQ range \cr
   \code{MEDRANGE} \tab Median (Range) \cr
   \code{MEDIQRANGE} \tab Median (Q1; Q3) \cr
   \code{MEAN_CI} \tab Mean (95\% C.I.) \cr
   \code{GeoMEAN_CI} \tab Geometric Mean (95\% C.I.) \cr
}
}

\section{Default Precision Extra}{
\tabular{ll}{
   \strong{stat} \tab \strong{extra} \cr
   \code{N} \tab 0 \cr
   \code{SUM} \tab 0 \cr
   \code{MEAN} \tab 1 \cr
   \code{GeoMEAN} \tab 1 \cr
   \code{SD} \tab 2 \cr
   \code{SE} \tab 2 \cr
   \code{GSD} \tab 2 \cr
   \code{GSE} \tab 2 \cr
   \code{CV} \tab 1 \cr
   \code{MEDIAN} \tab 1 \cr
   \code{MIN} \tab 0 \cr
   \code{MAX} \tab 0 \cr
   \code{Q1} \tab 1 \cr
   \code{Q3} \tab 1 \cr
   \code{LCL_MEAN} \tab 2 \cr
   \code{UCL_MEAN} \tab 2 \cr
   \code{LCL_GeoMEAN} \tab 2 \cr
   \code{UCL_GeoMEAN} \tab 2 \cr
}
}

\section{Updating Options}{

\itemize{
\item{For a single session, an option can be changed by
\verb{option(<optionToChange> = <NewValue>)}}
\item{To change an option for a single projects across sessions in that
projects, place the options update in the \code{.Rprofile} in that project
directory.}
\item{To change an option for a user across all sessions, place the options
update in the \code{.Rprofile} file in the users home directory.}
\item{To change an option for all users in an R environment, place the
options update in the \code{.Rprofile.site} file in the R home directory.}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://pharmaverse.github.io/tidytlg/main/}
  \item \url{https://github.com/pharmaverse/tidytlg}
  \item Report bugs at \url{https://github.com/pharmaverse/tidytlg/issues}
}

}
\author{
\strong{Maintainer}: Konrad Pagacz \email{kpagacz@its.jnj.com}

Authors:
\itemize{
  \item Nicholas Masel \email{nmasel@its.jnj.com}
  \item Steven Haesendonckx \email{shaesen2@its.jnj.com}
  \item Pelagia Alexandra Papadopoulou \email{ppapadop@its.jnj.com}
  \item Sheng-Wei Wang \email{swang69@its.jnj.com}
  \item Eli Miller \email{eli.miller@atorusresearch.com} (\href{https://orcid.org/0000-0002-2127-9456}{ORCID})
  \item Nathan Kosiba \email{nkosiba@its.jnj.com} (\href{https://orcid.org/0000-0001-5359-4234}{ORCID})
  \item Aidan Ceney \email{aceney@its.jnj.com} (\href{https://orcid.org/0000-0001-8313-487X}{ORCID})
}

Other contributors:
\itemize{
  \item Janssen R&D [copyright holder, funder]
  \item David Hugh-Jones (Author of included 'huxtable' library) [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_bottom_borders.R
\name{col_borders}
\alias{col_borders}
\title{Adds borders under cells in a column}
\usage{
col_borders(col, rows)
}
\arguments{
\item{col}{\code{numeric} the column of the table}

\item{rows}{\code{numeric} the range of rows to include}
}
\description{
Adds borders under cells in a column
}
\seealso{
Other border_functions: 
\code{\link{no_borders}()},
\code{\link{row_border}()},
\code{\link{single_border}()},
\code{\link{spanning_borders}()}
}
\concept{border_functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{table_metadata}
\alias{table_metadata}
\title{Metadata describing the data, functions and arguments needed to produce your
results.}
\format{
A data frame with one row per function call and 16 variables:
\describe{
\item{\code{func}}{name of the function you wish to call}
\item{\code{df}}{data frame to pass to the function call}
\item{\code{subset}}{filter \code{df} records, this is passed directly to filter,
ex. \code{"AESER == 'Y'"}}
\item{\code{rowvar}}{variable being summarized that will pass to the function
call}
\item{\code{rowtext}}{row label text to display in the table}
\item{\code{row_header}}{header text to display above row summary}
\item{\code{statlist}}{list of statistics in the analysis, see individual
functions for what is available per function (e.g. \code{"N, n (x.x)"})}
\item{\code{colvar}}{variable used to determine the columns of the table}
\item{\code{decimal}}{decimal precision}
\item{\code{rowbyvar}}{repeat \code{rowvar} summary by this variable/s, comma separated
for multiple (e.g. \code{"ETHNIC, AGEGR1"})}
\item{\code{tablebyvar}}{repeat the entire table summary by this variable/s,
comma separated for multiple (e.g. \code{"ETHNIC, AGEGR1"})}
\item{\code{denom_df}}{used to set denominators if \code{df} does not contain
all required records}
}
}
\usage{
table_metadata
}
\description{
Metadata describing the data, functions and arguments needed to produce your
results.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roundSAS.R
\name{roundSAS}
\alias{roundSAS}
\title{SAS rounding in R}
\usage{
roundSAS(x, digits = 0, as_char = FALSE, na_char = NULL)
}
\arguments{
\item{x}{Numeric vector.}

\item{digits}{An integer specifying the number of decimal places to be
displayed after rounding. Default is 0.}

\item{as_char}{logical value indicating conversion of rounded numerical
vector to character vector; default is \code{FALSE}.}

\item{na_char}{A character string indicating missing value; if not specified,
\code{"NA"} is created.}
}
\value{
character vector of rounded values
}
\description{
\code{roundSAS} is an alternative rounding function, ensuring that decimals equal or
bigger than 5 are rounded upwards to the nearest number and returned as
character vector.
}
\details{
At the midpoint of a decimal place (e.g. 0.5, 1.5), the round function in R
rounds to the nearest even number (i.e. 0.5 is rounded to 0; 1.5 is rounded
to 2), whereas SAS rounds to the nearest number (i.e. 0.5 is rounded to 1;
1.5 is rounded to 2). The \code{roundSAS} function is an alternative rounding
function for R that ensures rounding to the nearest number, as done in SAS.
\code{roundSAS} comes from this Stack Overflow
\href{https://stackoverflow.com/questions/12688717/round-up-from-5}{post}.
}
\examples{
### input data vector with midpoint decimals
x <- c(-2.5, -1.5, -0.5, 0.5, 1.5, 2.5)

### rounds to integer
roundSAS(x, digits = 0)

### input data vector with a missing value
y <- c(8.65, 8.75, NA, 9.85, 9.95)

### rounds to tenths and label the missing value with "NE"
roundSAS(y, digits = 1, as_char = TRUE, na_char = "NE")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_bottom_borders.R
\name{add_bottom_borders}
\alias{add_bottom_borders}
\title{Adds bottom borders to a \code{huxtable}}
\usage{
add_bottom_borders(ht, border_matrix = no_borders(ht), transform_fns = list())
}
\arguments{
\item{ht}{\code{huxtable} A \code{huxtable} object}

\item{border_matrix}{(optional) \code{matrix} A matrix indicating where to add the bottom
borders. If \code{NULL}, then no borders are added.}

\item{transform_fns}{(optional) \code{list} of \code{function} A list of functions applied to the
\code{border_matrix}. The functions have to accept two arguments:
\enumerate{
\item The \code{huxtable}.
\item The \code{border_matrix} with dimensions matching \code{huxtable}.
}

The functions in the list are applied sequentially to \code{border_matrix}.}
}
\value{
A \code{huxtable} with added borders.
}
\description{
Adds bottom borders to a \code{huxtable}
}
\details{
Adds bottom borders to a \code{huxtable} based on
a matrix indicating where the borders should be put.

This function is responsible for adding bottom borders to a \code{huxtable} object.
It supports borders spanning multiple columns and borders that are under neighbouring,
single cells (or merged cells), but separate (see examples).

This feature has limitations. Mainly, it does not support both versions of the borders
(continuous and separate) on the same line. In such a case, the borders in the resulting
RTF look misaligned.
}
\section{\code{border_matrix} details}{

You mark where the bottom borders should go in the table by passing a matrix.
The matrix has to have the same number of columns as the passed \code{huxtable}
and the number of rows lower by one than the passed \code{huxtable}. Each cell
in \code{border_matrix} corresponds to a cell in \code{huxtable} (starting from the first row).

Internally, the function adds the first row of 0s to \code{border_matrix} before the execution.
At that point, \code{border_matrix}'s dimensions match \code{ht}'s dimensions.

Table:\tabular{ll}{
   foo \tab bar \cr
   \code{baz} \tab \code{bim} \cr
}


A border matrix:\tabular{ll}{
   1 \tab 1 \cr
   0 \tab 0 \cr
}


The above border matrix puts a bottom border across the entire first row
and no borders in the second row.

A border matrix:\tabular{ll}{
   1 \tab 2 \cr
   0 \tab 0 \cr
}


The above border matrix puts one border under the first cell in the first row;
and another border (separate from the first one) under the second cell
in the first row. The second row stays without any borders.
}

\section{Functions transforming the border matrix}{


The below functions can be passed to \code{\link[=gentlg]{gentlg()}}'s
\code{border_fns} argument to modify how \code{gentlg} renders
the borders under the cells.

Border functions:
\itemize{
\item \code{\link[=no_borders]{no_borders()}}
\item \code{\link[=spanning_borders]{spanning_borders()}}
\item \code{\link[=col_borders]{col_borders()}}
\item \code{\link[=single_border]{single_border()}}
\item \code{\link[=row_border]{row_border()}}
}

\code{border_fns} will accept your own, custom functions as long as
they adhere to the format.
All the functions passed to \code{border_fns} need to accept two arguments:
\itemize{
\item the first - the printed \code{huxtable} object,
\item the second - a border matrix.
}

They also must return a matrix interpreted the same way as \code{border_matrix}
passed to \code{add_bottom_borders} or \code{\link[=gentlg]{gentlg()}}.
}

\examples{
border_matrix <- matrix(c(1, 1, 2, 0, 1, 1, 0, 0, 0), nrow = 3, ncol = 3)
ht <- huxtable::as_huxtable(
  data.frame(a = c(1, 2, 3), b = c("a", "b", "c"), c = c(TRUE, FALSE, TRUE))
)
# By default adds no borders
add_bottom_borders(ht, border_matrix)
# Adds spanning borders under cells with text in the second row
add_bottom_borders(ht, transform_fns = list(spanning_borders(2)))
# Adds spanning borders under cells with text in the second row and a border
# under a cell in row 3 and column 3
add_bottom_borders(ht, transform_fns = list(spanning_borders(2), single_border(3, 3)))

final <- data.frame(
  label = c(
    "Overall", "Safety Analysis Set",
    "Any Adverse event{\\\\super a}", "- Serious Adverse Event"
  ),
  Drug_A = c("", "40", "10 (25\%)", "0"),
  Drug_B = c("", "40", "10 (25\%)", "0"),
  anbr = c(1, 2, 3, 4),
  roworder = c(1, 1, 1, 1),
  boldme = c(1, 0, 0, 0),
  newrows = c(0, 0, 1, 0),
  indentme = c(0, 0, 0, 1),
  newpage = c(0, 0, 0, 0)
)
# Add spanning bottom borders under the cells in the first row
gentlg(
  huxme = final,
  wcol = c(0.70, 0.15, 0.15),
  file = "TSFAEX",
  colheader = c("", "Drug A", "Drug B"),
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  ),
  border_fns = list(no_borders, spanning_borders(1))
)

# Tables with no bottom borders
gentlg(
  huxme = final,
  wcol = c(0.70, 0.15, 0.15),
  file = "TSFAEX",
  colheader = c("", "Drug A", "Drug B"),
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  ),
  border_fns = list(no_borders)
)

# Tables with a border under cell in the 3nd row and 3rd column,
# and borders under cells in the first row
gentlg(
  huxme = final,
  wcol = c(0.70, 0.15, 0.15),
  file = "TSFAEX",
  colheader = c("", "Drug A", "Drug B"),
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  ),
  border_fns = list(no_borders, spanning_borders(1), single_border(3, 3))
)

# We discourage, but you can pass the border matrix directly
mat <- matrix(rep(0, 8 * 3), ncol = 3, nrow = 8)
mat[3, 3] <- 1
gentlg(
  huxme = final,
  wcol = c(0.70, 0.15, 0.15),
  file = "TSFAEX",
  colheader = c("", "Drug A", "Drug B"),
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  ),
  bottom_borders = mat, # The same as a single border under 3nd row and 3rd column
  border_fns = list()
)

# clean up.
file.remove("tsfaex.rtf")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cdisc adae}
\alias{cdisc adae}
\alias{cdisc_adae}
\title{\code{ADAE} data created from subsetting the \code{CDISC} \code{ADAE} dataset}
\format{
A data frame with 84 rows and 55 variables:
\describe{
\item{\code{STUDYID}}{Study Identifier}
\item{\code{SITEID}}{Study Site Identifier}
\item{\code{USUBJID}}{Unique Subject Identifier}
\item{\code{SUBJID}}{Subject Identifier for the Study}
\item{\code{TRTA}}{Actual Treatment}
\item{\code{TRTAN}}{Actual Treatment (N)}
\item{\code{AGE}}{Age}
\item{\code{AGEGR1}}{Pooled Age Group 1}
\item{\code{AGEGR1N}}{Pooled Age Group 1 (N)}
\item{\code{RACE}}{Race}
\item{\code{RACEN}}{Race (N)}
\item{\code{SEX}}{Sex}
\item{\code{SAFFL}}{Safety Population Flag}
\item{\code{TRTSDT}}{Date of First Exposure to Treatment}
\item{\code{TRTEDT}}{Date of Last Exposure to Treatment}
\item{\code{ASTDT}}{Analysis Start Date}
\item{\code{ASTDTF}}{Analysis Start Date Imputation Flag}
\item{\code{ASTDY}}{Analysis Start Relative Day}
\item{\code{AENDT}}{Analysis End Date}
\item{\code{AENDY}}{Analysis End Relative Day}
\item{\code{ADURN}}{\code{AE} Duration (N)}
\item{\code{ADURU}}{\code{AE} Duration Units}
\item{\code{AETERM}}{Reported Term for the Adverse Event}
\item{\code{AELLT}}{Lowest Level Term}
\item{\code{AELLTCD}}{Lowest Level Term Code}
\item{\code{AEDECOD}}{Dictionary-Derived Term}
\item{\code{AEPTCD}}{Preferred Term Code}
\item{\code{AEHLT}}{High Level Term}
\item{\code{AEHLTCD}}{High Level Term Code}
\item{\code{AEHLGT}}{High Level Group Term}
\item{\code{AEHLGTCD}}{High Level Group Term Code}
\item{\code{AEBODSYS}}{Body System or Organ Class}
\item{\code{AESOC}}{Primary System Organ Class}
\item{\code{AESOCCD}}{Primary System Organ Class Code}
\item{\code{AESEV}}{Severity/Intensity}
\item{\code{AESER}}{Serious Event}
\item{\code{AESCAN}}{Involves Cancer}
\item{\code{AESCONG}}{Congenital Anomaly or Birth Defect}
\item{\code{AESDISAB}}{Persist or \code{Signif} Disability/Incapacity}
\item{\code{AESDTH}}{Results in Death}
\item{\code{AESHOSP}}{Requires or Prolongs Hospitalization}
\item{\code{AESLIFE}}{Is Life Threatening}
\item{\code{AESOD}}{Occurred with Overdose}
\item{\code{AEREL}}{Causality}
\item{\code{AEACN}}{Action Taken with Study Treatment}
\item{\code{AEOUT}}{Outcome of Adverse Event}
\item{\code{AESEQ}}{Sequence Number}
\item{\code{TRTEMFL}}{Treatment Emergent Analysis Flag}
\item{\code{AOCCFL}}{First Occurrence of Any \code{AE} Flag}
\item{\code{AOCCSFL}}{First Occurrence of \code{SOC} Flag}
\item{\code{AOCCPFL}}{First Occurrence of Preferred Term Flag}
\item{\code{AOCC02FL}}{First Occurrence 02 Flag for Serious}
\item{\code{AOCC03FL}}{First Occurrence 03 Flag for Serious \code{SOC}}
\item{\code{AOCC04FL}}{First Occurrence 04 Flag for Serious \code{PT}}
\item{\code{CQ01NAM}}{Customized Query 01 Name}
\item{\code{AOCC01FL}}{First Occurrence 01 Flag for \code{CQ01}}
}
}
\source{
CDISC SDTM/ADAM Pilot Project.
}
\usage{
cdisc_adae
}
\description{
\code{ADAE} data created from subsetting the \code{CDISC} \code{ADAE} dataset
}
\keyword{CDISC}
\keyword{adae}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{char2factor}
\alias{char2factor}
\title{Convert character variable to a factor based off it's numeric variable
counterpart.}
\usage{
char2factor(df, c_var, n_var)
}
\arguments{
\item{df}{data frame.}

\item{c_var}{character variable within the data frame.}

\item{n_var}{numeric variable counter part within the data frame to control
the levels.}
}
\value{
A factor.
}
\description{
Convert character variable to a factor based off it's numeric variable
counterpart.
}
\examples{
df <- tibble::tribble(
  ~TRT01P, ~TRT01PN,
  "Placebo", 1,
  "Low Dose", 2,
  "High Dose", 3
)

# alphabetical order
dplyr::arrange(df, TRT01P)

# change to factor with char2factor
df$TRT01P <- char2factor(df, "TRT01P", "TRT01PN")

# factor order
dplyr::arrange(df, TRT01P)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_newrows.R
\name{add_newrows}
\alias{add_newrows}
\title{Add the \code{newrows} variable to the results dataframe.}
\usage{
add_newrows(df, tableby = NULL, groupby = NULL)
}
\arguments{
\item{df}{dataframe of results. must contain the \code{anbr} variable that is
added by \code{\link[=add_format]{add_format()}}.}

\item{tableby}{character vector containing table by variables used to
generate the results.}

\item{groupby}{character vector containing group by variables used to
generate the results.}
}
\value{
dataframe with the variable \code{newrows} and \code{roworder} added.
\code{newrows} is used by \code{gentlg} to insert line breaks.
}
\description{
The \code{newrows} variable is used by \code{gentlg()} to define when to add a blank row
to the output. Data will be grouped by \code{anbr} and the variables passed into
the \code{tableby} and \code{groupby} parameters. \code{newrows} will be set to 1
for the first record in each group, except for the first row in the data.
The first row will always be set to 0.
}
\examples{
# Example showing how newrows is set to one for each new anbr except
# the first
tbl <-
  structure(
    list(
      rowvar = c("RANDFL", "AGE", "AGE", "AGE", "AGE", "AGE"),
      anbr = c(1L, 2L, 2L, 2L, 2L, 2L),
      label = c(
        "Analysis set: Subjects Randomized", "Age (Years)", "N",
        "Mean (SD)", "Range", "IQ Range"
      ),
      row_type = c("COUNT", "UNIVAR", "UNIVAR", "UNIVAR", "UNIVAR", "UNIVAR")
    ),
    row.names = c(NA, -6L),
    class = c("tbl_df", "tbl", "data.frame")
  )

add_newrows(tbl)

# Example of use when you have results summarized by one or more variables
tbl2 <- tibble::tribble(
  ~anbr, ~SEX, ~label, ~row_type,
  "01", "F", "Sex : F", "TABLE_BY_HEADER",
  "01", "F", "<65", "VALUE",
  "01", "F", "65-80", "VALUE",
  "01", "F", ">80", "VALUE",
  "01", "M", "Sex : M", "TABLE_BY_HEADER",
  "01", "M", "<65", "VALUE",
  "01", "M", "65-80", "VALUE",
  "01", "M", ">80", "VALUE"
)

add_newrows(tbl2, tableby = "SEX")

tbl3 <- tibble::tribble(
  ~anbr, ~SEX, ~ETHNIC, ~label, ~row_type,
  "01", "F", NA, "Sex : F", "TABLE_BY_HEADER",
  "01", "F", "HISPANIC OR LATINO", "HISPANIC OR LATINO", "BY_HEADER1",
  "01", "F", "HISPANIC OR LATINO", "<65", "VALUE",
  "01", "F", "HISPANIC OR LATINO", ">80", "VALUE",
  "01", "F", "HISPANIC OR LATINO", "65-80", "VALUE",
  "01", "F", "NOT HISPANIC OR LATINO", "NOT HISPANIC OR LATINO", "BY_HEADER1",
  "01", "F", "NOT HISPANIC OR LATINO", "<65", "VALUE",
  "01", "F", "NOT HISPANIC OR LATINO", "65-80", "VALUE",
  "01", "F", "NOT HISPANIC OR LATINO", ">80", "VALUE",
  "01", "M", NA, "Sex : M", "TABLE_BY_HEADER",
  "01", "M", "HISPANIC OR LATINO", "HISPANIC OR LATINO", "BY_HEADER1",
  "01", "M", "HISPANIC OR LATINO", "<65", "VALUE",
  "01", "M", "HISPANIC OR LATINO", "65-80", "VALUE",
  "01", "M", "HISPANIC OR LATINO", ">80", "VALUE",
  "01", "M", "NOT HISPANIC OR LATINO", "NOT HISPANIC OR LATINO", "BY_HEADER1",
  "01", "M", "NOT HISPANIC OR LATINO", "<65", "VALUE",
  "01", "M", "NOT HISPANIC OR LATINO", "65-80", "VALUE",
  "01", "M", "NOT HISPANIC OR LATINO", ">80", "VALUE"
)

add_newrows(tbl3, tableby = "SEX", groupby = "ETHNIC")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFileName.R
\name{get_file_name}
\alias{get_file_name}
\title{getFileName}
\usage{
get_file_name()
}
\value{
The file path up until the program's name.
}
\description{
This function returns the file path up until the program's
name.
}
\examples{
get_file_name()
}
\author{
Steven Haesendonckx, \email{shaesen2@its.jnj.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_bottom_borders.R
\name{single_border}
\alias{single_border}
\title{Adds a border under a cell}
\usage{
single_border(row, col)
}
\arguments{
\item{row}{\code{numeric} the row of the cell}

\item{col}{\code{numeric} the column of the cell}
}
\description{
Adds a border under a cell
}
\seealso{
Other border_functions: 
\code{\link{col_borders}()},
\code{\link{no_borders}()},
\code{\link{row_border}()},
\code{\link{spanning_borders}()}
}
\concept{border_functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_bottom_borders.R
\name{no_borders}
\alias{no_borders}
\title{Removes all borders from the table}
\usage{
no_borders(ht, matrix = NULL)
}
\arguments{
\item{ht}{\code{huxtable} object.}

\item{matrix}{\code{matrix} of bottom borders. Ignored. Included for the sake
of compatibility with the interface of all border mutating functions.}
}
\description{
Removes all borders from the table
}
\seealso{
Other border_functions: 
\code{\link{col_borders}()},
\code{\link{row_border}()},
\code{\link{single_border}()},
\code{\link{spanning_borders}()}
}
\concept{border_functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_indent.R
\name{add_indent}
\alias{add_indent}
\title{Add indentation variable to the results dataframe}
\usage{
add_indent(df)
}
\arguments{
\item{df}{dataframe of results that contains \code{row_type} and \code{label}
and the optional \code{nested_level} and \code{group_level} variables.}
}
\value{
dataframe with the \code{indentme} variable added.
}
\description{
Add the \code{indentme} variable to your results data. This drives the number of
indents for the row label text (e.g. 0, 1, 2, etc.).
}
\details{
The \code{group_level} variable, which is added to the results dataframe by \code{freq()}
and \code{univar()} calls, is needed to define indentation when by variables are
used for summary.

The \code{nested_level} variable, which is added to the results dataframe by
\code{nested_freq()}, is needed to define indentation for each level of nesting.

Both of these are added to the default indentation which is driven by
\code{row_type}.\tabular{lc}{
   row_type \tab default indentation \cr
   TABLE_BY_HEADER \tab 0 \cr
   BY_HEADER[1-9] \tab 0 \cr
   HEADER \tab 0 \cr
   N \tab 1 \cr
   VALUE \tab 2 \cr
   NESTED \tab 0 \cr
}
}
\examples{
df <- tibble::tibble(
  row_type = c(
    "TABLE_BY_HEADER", "HEADER",
    "BY_HEADER1", "N", "VALUE", "COUNTS", "UNIVAR", "NESTED", "NESTED"
  ),
  nested_level = c(NA, NA, NA, NA, NA, NA, NA, 1, 2),
  group_level = c(0, 0, 0, 0, 0, 0, 0, 0, 0),
  label = c(NA, NA, NA, NA, NA, "N", NA, NA, NA),
  by = c(NA, NA, NA, NA, NA, NA, NA, NA, NA),
  tableby = c(NA, NA, NA, NA, NA, NA, NA, NA, NA)
)
add_indent(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gentlg.R
\name{gentlg}
\alias{gentlg}
\title{Output a \code{tidytlg} table}
\usage{
gentlg(
  huxme = NULL,
  tlf = "Table",
  format = "rtf",
  colspan = NULL,
  idvars = NULL,
  plotnames = NULL,
  plotwidth = NULL,
  plotheight = NULL,
  wcol = 0.45,
  orientation = "portrait",
  opath = ".",
  title_file = NULL,
  file = NULL,
  title = NULL,
  footers = NULL,
  print.hux = TRUE,
  watermark = NULL,
  colheader = NULL,
  pagenum = FALSE,
  bottom_borders = "old_format",
  border_fns = list(),
  alignments = list(),
  footers_one_row = FALSE
)
}
\arguments{
\item{huxme}{(optional) For tables and listings, A list of input dataframes
containing all columns of interest. For graphs, either \code{NULL} or
a list of \code{ggplot} objects. Vectorized.}

\item{tlf}{(optional) String, representing the output choice. Choices are
\code{"Table"} \code{"Listing"} \code{"Figure"}. Abbreviations are allowed e.g. \code{"T"} for Table.
Strings can be either upper- or lowercase. Vectorized. (Default = \code{"Table"})}

\item{format}{(optional) String, representing the output format. Choices are
\code{"rtf"} and \code{"html"}. Strings can be either upper- or lowercase.(Default = \code{"rtf"})}

\item{colspan}{(optional) A list of character vectors representing the
spanning headers to be used for the table or listing. The first vector
represents the top spanning header, etc. Each vector should have a length
equal to the number of columns in the output data frame. A spanning header
is identified through the use of the same column name in adjacent elements.
Vectorized.}

\item{idvars}{(optional) Character vector defining the columns of a listing
where repeated values should be removed recursively. If \code{NULL} then
all column names are used in the algorithm. If \code{NA}, then the listing remains
as is.}

\item{plotnames}{(optional) Character vector containing the names of the PNG
files, with their extension to be incorporated for figure outputs.
The PNG files need to be located in the path defined by the
parameter \code{opath}.}

\item{plotwidth}{(optional) Numerical value that indicates the plot width in
cm for figure outputs. (Default = 6)}

\item{plotheight}{(optional) Numerical value that indicates the plot height
in cm for figure outputs. (Default = 5)}

\item{wcol}{(optional) Can be one of:
\itemize{
\item a single numeric value that represents the width of the first column
\item a numeric vector, specifying the widths of all columns in the final table or listing
\item a list of numeric vectors (applicable when \code{huxme} is a list). Each element
can specify the widths of all columns or the width of the first column only\cr
}

When a single numerical value is used, this will be taken as the column width
for the first column. The other columns will be equally spaced across the
remainder of the available space. Alternatively, a vector can be used to
represent the widths of all columns in the final output. The order of the
arguments needs to correspond to the order of the columns in the \code{huxme}
dataset, that are not part of the formatting algorithms
(e.g. \code{anbr}, \code{roworder}, \code{newpage}, \code{newrow},
\code{indentme}, \code{boldme}, \code{by_value}, \code{by_order}).
The sum of the widths in the vector needs to be less or equal to one. When
\code{format="HTML"} \code{wcol} can take only one value, the width
of the first column. (Default = 0.45).}

\item{orientation}{(optional) String: "portrait" or "landscape".
(Default = "portrait")}

\item{opath}{(optional) File path pointing to the output files
(including PNG files for graphs). (Default = ".").}

\item{title_file}{An Excel file that will be read in
with \code{readxl::read_excel()} to be used as the \code{title} and \code{footers} argument.
The use of \code{title} or \code{footers} will override the values passed by this
argument. The file should be either an \code{xls} or \code{xlsx} file with the columns
\verb{TABLE ID}, \code{IDENTIFIER}, and \code{TEXT}. The file will be read in, subset to
where the \code{tblid} matches the \code{tlf} argument, and identifiers with 'title' or
'footnote' will be used to populate the table.}

\item{file}{(required) String. Output identifier.
File name will be adjusted to be lowercase and have \code{-} and \verb{_} removed,
this will not affect table title.}

\item{title}{(required) String. Title of the output. Vectorized.}

\item{footers}{(optional) Character vector, containing strings of footnotes
to be included. Vectorized.}

\item{print.hux}{(optional) Logical, indicating whether the output should be
printed to RTF \code{('format' = "rtf")} or displayed
as HTML \code{('format' = "HTML")}.
(Default = \code{TRUE}). Note that RTF is written using \code{quick_rtf_jnj()}
function and that the HTML is displayed via the \code{huxtable::print_html}
function.}

\item{watermark}{(optional) String containing the desired watermark for
RTF outputs. Vectorized.}

\item{colheader}{(optional) Character vector that contains the column labels
for a table or listing. Default uses the column labels of \code{huxme}.
Vectorized.}

\item{pagenum}{(optional) Logical. When true page numbers are added on the
right side of the footer section in the format page \code{x/y}.
Vectorized. (Default = \code{FALSE}).}

\item{bottom_borders}{(optional) Matrix or \code{"old_format"}.
A matrix indicating where to add the bottom
borders. Vectorized. See \code{\link[=add_bottom_borders]{add_bottom_borders()}} for more information.
If \code{"old_format"}, then borders are added to the \code{colspan} and \code{colheader}
rows. (Default = "old_format").}

\item{border_fns}{(optional) List. A list of functions that transform
the matrix passed to \code{bottom_borders}. Vectorized. See
\code{\link[=add_bottom_borders]{add_bottom_borders()}} for more information.}

\item{alignments}{(optional) List of named lists. Vectorized.
(Default = \code{list()}) Used to specify individual column or cell alignments.
Each named list contains \code{row}, \code{col}, and \code{value}, which are passed to
\code{\link[huxtable:align]{huxtable::set_align()}} to set the alignments.}

\item{footers_one_row}{(optional) Logical. Whether to export the footers
as a single table row (Default = \code{FALSE}).}
}
\value{
A list of formatted \code{huxtables} with desired
properties for output to an RTF or HTML.
}
\description{
Generate and output a \code{huxtable} with desired properties
During this function call, the \code{huxtable} can be written to an RTF or
displayed in HTML. \code{gentlg} is vectorized, see parameter descriptions
to learn for which arguments.
}
\section{\code{Huxme} Details}{

For tables and listings, formatting of the output can be dictated through the
formatting columns
(\code{newrows}, \code{indentme}, \code{boldme}, \code{newpage}), present in the input dataframe.
The final \code{huxtable} will display all columns of the input dataframe, except
any recognized formatting or sorting columns.
For tables, the algorithm uses the column \code{label} as first column.
The remaining columns are treated as summary columns.
For graphs, you can pass a \code{ggplot} object directly
into \code{huxme} and \code{gentlg} will save a PNG with with \code{ggplot2::ggsave()}
and output an RTF.
}

\examples{

final <- data.frame(
  label = c(
    "Overall", "Safety Analysis Set",
    "Any Adverse event{\\\\super a}", "- Serious Adverse Event"
  ),
  Drug_A = c("", "40", "10 (25\%)", "0"),
  Drug_B = c("", "40", "10 (25\%)", "0"),
  anbr = c(1, 2, 3, 4),
  roworder = c(1, 1, 1, 1),
  boldme = c(1, 0, 0, 0),
  newrows = c(0, 0, 1, 0),
  indentme = c(0, 0, 0, 1),
  newpage = c(0, 0, 0, 0)
)

# Produce output in rtf format
gentlg(
  huxme = final,
  wcol = c(0.70, 0.15, 0.15),
  file = "TSFAEX",
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  )
)

# Pass in column headers instead of using variable name
gentlg(
  huxme = final,
  wcol = c(0.70, 0.15, 0.15),
  file = "TSFAEX",
  colheader = c("", "Drug A", "Drug B"),
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  )
)

# Add spanning bottom borders under the cells in the second row
gentlg(
  huxme = final,
  wcol = c(0.70, 0.15, 0.15),
  file = "TSFAEX",
  colheader = c("", "Drug A", "Drug B"),
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  ),
  border_fns = list(spanning_borders(2))
)

# Use a watermark
gentlg(
  huxme = final,
  wcol = c(0.70, 0.15, 0.15),
  file = "TSFAEX",
  colheader = c("", "Drug A", "Drug B"),
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  ),
  watermark = "Confidential"
)

# Set alignments
gentlg(
  huxme = final,
  file = "TSFAEX",
  alignments = list(
    # Align the second column to the left
    list(row = 1:7, col = 2, value = "left"),

    # Align cell "Drug: B" to the right
    list(row = 2, col = 3, value = "right")
  )
)

final_2 <- data.frame(
  label = c(
    "Overall", "Safety Analysis Set",
    "Any Adverse event{\\\\super a}", "- Serious Adverse Event"
  ),
  Drug_A = c("", "40", "10 (25\%)", "0"),
  Drug_B = c("", "40", "10 (25\%)", "0")
)

gentlg(
  huxme = list(final_2, final_2),
  wcol = list(c(0.70, 0.15, 0.15), c(0.5)),
  file = "TSFAEX",
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  )
)

# Produce output in HTML format
hux <- gentlg(
  huxme = final,
  file = "TSFAEX",
  colheader = c("", "Drug A", "Drug B"),
  title = "This is Amazing Demonstration 1",
  footers = c(
    "Note: For demonstrative purposes only",
    "{\\\\super a} Subjects are counted once for any given event."
  ),
  watermark = "Confidential",
  format = "HTML",
  print.hux = FALSE
)

# Export to HTML page
huxtable::quick_html(hux, file = "TSFAEX.html", open = FALSE)

# clean up.
file.remove("TSFAEX.html", "tsfaex.rtf")
}
\references{
\url{https://github.com/hughjonesd/huxtable}
}
\author{
Steven Haesendonckx \href{mailto:shaesen2@its.jnj.com}{shaesen2@its.jnj.com}

Pelagia Alexandra Papadopoulou \href{mailto:ppapadop@its.jnj.com}{ppapadop@its.jnj.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/univar.R
\name{univar}
\alias{univar}
\title{Descriptive statistics}
\usage{
univar(
  df,
  colvar = NULL,
  tablebyvar = NULL,
  rowvar = NULL,
  rowbyvar = NULL,
  statlist = getOption("tidytlg.univar.statlist.default"),
  decimal = 1,
  precisionby = NULL,
  precisionon = NULL,
  wide = FALSE,
  alpha = 0.05,
  rowtext = NULL,
  row_header = NULL,
  .keep = TRUE,
  .ord = FALSE,
  ...
)
}
\arguments{
\item{df}{(required) dataframe containing records to summarize by treatment.}

\item{colvar}{(required) character vector of the treatment variable within
the dataframe.}

\item{tablebyvar}{(optional) repeat entire table by variable within \code{df}.}

\item{rowvar}{(required) character vector of variable to summarize within the
dataframe.}

\item{rowbyvar}{(optional) repeat \code{rowvar} by variable within \code{df}.}

\item{statlist}{(optional) \code{statlist} object of stats to keep (default =
\code{statlist(c("N", "MEANSD", "MEDIAN", "RANGE", "IQRANGE"))}).}

\item{decimal}{(optional) decimal precision root level, when using
\code{presisionby} this will be used as the base decimal cap (default = 1).}

\item{precisionby}{(optional) vector of by variable(s) to use when
calculating parameter based precision.}

\item{precisionon}{(optional) variable to use when calculating parameter
based precision. If \code{precisionby} is specified but not \code{precisionon}
this will default to \code{rowvar}.}

\item{wide}{(optional) logical indicating to convert labels to column and
columns to labels (default = \code{FALSE}).}

\item{alpha}{(optional) alpha level for 2-sided confidence interval (default
= 0.05).}

\item{rowtext}{(optional) A text string to replace the \code{label} value on the
table. Useful for tables with a single row.}

\item{row_header}{(optional) A row to add as a header for the table.}

\item{.keep}{(optional) Should the \code{rowbyvar} and \code{tablebyvar} be
output in the table. If \code{FALSE}, \code{rowbyvar} will still be output
in the \code{label} column. (default = \code{TRUE}).}

\item{.ord}{Should the ordering columns be output with the table? This is
useful if a table needs to be merged or reordered in any way after build.}

\item{...}{(optional) Named arguments to be included as columns on the table.}
}
\value{
dataframe of results
}
\description{
Univariate statistics for a variables by treatment and/or group.
}
\examples{
adsl <-
  structure(
    list(
      USUBJID = c(
        "DEMO-101", "DEMO-102", "DEMO-103", "DEMO-104",
        "DEMO-105", "DEMO-106"
      ),
      AGE = c(59, 51, 57, 65, 21, 80),
      SEX = c("F", "M", "F", "M", "F", "M"),
      WEIGHTBL = c(83.6, 75, 84, 90, 65, 70),
      colnbr = structure(
        c(1L, 3L, 2L, 2L, 3L, 1L),
        .Label = c("Placebo", "Low", "High"),
        class = "factor"
      )
    ),
    row.names = c(NA, 6L),
    class = "data.frame"
  )

# N, Mean(SD), Median, Range, IQ Range for a rowvar by colvar
univar(adsl,
  colvar = "colnbr",
  rowvar = "AGE"
)

# N and Mean for a rowvar by colvar
univar(adsl,
  colvar   = "colnbr",
  rowvar   = "AGE",
  statlist = statlist(c("N", "MEAN"))
)

# N and Mean for a rowvar by colvar and a by variable
univar(adsl,
  colvar   = "colnbr",
  rowvar   = "AGE",
  rowbyvar = "SEX",
  statlist = statlist(c("N", "MEAN"))
)

# Below illustrates how make the same calls to univar() as above, using table
# and column metadata # along with generate_results().

column_metadata <- tibble::tribble(
  ~tbltype, ~coldef, ~decode,
  "type1", "0", "Placebo",
  "type1", "54", "Low",
  "type1", "81", "High"
)

# N, Mean(SD), Median, Range, IQ Range for a rowvar by colvar
table_metadata <- tibble::tribble(
  ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar,
  "1", "univar", "cdisc_adae", "AGE", "type1", "TRTA"
)

generate_results(table_metadata,
  column_metadata = column_metadata,
  tbltype = "type1"
)


# N and Mean for a rowvar by colvar
table_metadata <- tibble::tribble(
  ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar, ~statlist,
  "1", "univar", "cdisc_adae", "AGE", "type1", "TRTA",
  statlist(c("N", "MEAN"))
)

generate_results(table_metadata,
  column_metadata = column_metadata,
  tbltype = "type1"
)


# N and Mean for a rowvar by colvar and a by variable
table_metadata <- tibble::tribble(
  ~anbr, ~func, ~df, ~rowvar, ~tbltype, ~colvar, ~statlist, ~by,
  "1", "univar", "cdisc_adae", "AGE", "type1", "TRTA",
  statlist(c("N", "MEAN")), "SEX"
)

generate_results(table_metadata,
  column_metadata = column_metadata,
  tbltype = "type1"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spanning_headers.R
\name{spanning_headers}
\alias{spanning_headers}
\title{Spanning headers for outputs}
\usage{
spanning_headers(column_metadata)
}
\arguments{
\item{column_metadata}{dataframe containing the column metadata that is
passed to \code{tlgsetup()} (see \code{tlgsetup()} for details)}
}
\value{
List of character vectors containing column headers for an output.
}
\description{
This will create the list object to be passed to \code{gentlg()} You can create as
many spanning headers as you like, just add variables prefixed with span to
the column metadata.
}
\examples{
column_metadata <-
  tibble::tribble(
    ~tbltype, ~coldef, ~decode,                ~span1,
    "type1",  "0",     "Placebo",              "",
    "type1",  "54",    "Low Dose",             "Xanomeline",
    "type1",  "81",    "High Dose",            "Xanomeline",
    "type1",  "54+81", "Total Xanomeline",     ""
  )

spanning_headers(column_metadata)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cdisc adsl}
\alias{cdisc adsl}
\alias{cdisc_adsl}
\title{\code{ADSL} data created from subsetting the \code{CDISC} \code{ADSL} with 15 subjects (5 subjects in each arm)}
\format{
A data frame with 15 rows and 49 variables:
\describe{
\item{\code{STUDYID}}{Study Identifier}
\item{\code{USUBJID}}{Unique Subject Identifier}
\item{\code{SUBJID}}{Subject Identifier for the Study}
\item{\code{SITEID}}{Study Site Identifier}
\item{\code{SITEGR1}}{Pooled Site Group 1}
\item{\code{ARM}}{Description of Planned Arm}
\item{\code{TRT01P}}{Planned Treatment for Period 01}
\item{\code{TRT01PN}}{Planned Treatment for Period 01 (N)}
\item{\code{TRT01A}}{Actual Treatment for Period 01}
\item{\code{TRT01AN}}{Actual Treatment for Period 01 (N)}
\item{\code{TRTSDT}}{Date of First Exposure to Treatment}
\item{\code{TRTEDT}}{Date of Last Exposure to Treatment}
\item{\code{TRTDUR}}{Duration of Treatment (days)}
\item{\code{AVGDD}}{Avg Daily Dose (as planned)}
\item{\code{CUMDOSE}}{Cumulative Dose (as planned)}
\item{\code{AGE}}{Age}
\item{\code{AGEGR1}}{Pooled Age Group 1}
\item{\code{AGEGR1N}}{Pooled Age Group 1 (N)}
\item{\code{AGEU}}{Age Units}
\item{\code{RACE}}{Race}
\item{\code{RACEN}}{Race (N)}
\item{\code{SEX}}{Sex}
\item{\code{ETHNIC}}{Ethnicity}
\item{\code{SAFFL}}{Safety Population Flag}
\item{\code{ITTFL}}{Intent-To-Treat Population Flag}
\item{\code{EFFFL}}{Efficacy Population Flag}
\item{\code{COMP8FL}}{Finishers of Week 8 Population Flag}
\item{\code{COMP16FL}}{Finishers of Week 16 Population Flag}
\item{\code{COMP24FL}}{Finishers of Week 24 Population Flag}
\item{\code{DISCONFL}}{Did the Subject Discontinue the Study?}
\item{\code{DSRAEFL}}{Discontinued due to \code{AE}?}
\item{\code{DTHFL}}{Subject Died?}
\item{\code{BMIBL}}{Baseline BMI (kg/m^2)}
\item{\code{BMIBLGR1}}{Pooled Baseline BMI Group 1}
\item{\code{HEIGHTBL}}{Baseline Height (cm)}
\item{\code{WEIGHTBL}}{Baseline Weight (kg)}
\item{\code{EDUCLVL}}{Years of Education}
\item{\code{DISONSDT}}{Date of Onset of Disease}
\item{\code{DURDIS}}{Duration of Disease (Months)}
\item{\code{DURDSGR1}}{Pooled Disease Duration Group 1}
\item{\code{VISIT1DT}}{Date of Visit 1}
\item{\code{RFSTDTC}}{Subject Reference Start Date/Time}
\item{\code{RFENDTC}}{Subject Reference End Date/Time}
\item{\code{VISNUMEN}}{End of Treatment Visit (Visit 12 or Early Term.)}
\item{\code{RFENDT}}{Date of Discontinuation/Completion}
\item{\code{DCDECOD}}{Standardized Disposition Term}
\item{\code{EOSSTT}}{End of Study Status}
\item{\code{DCREASCD}}{Reason for Discontinuation}
\item{\code{MMSETOT}}{\code{MMSE} Total}

}
}
\source{
CDISC SDTM/ADAM Pilot Project.
}
\usage{
cdisc_adsl
}
\description{
\code{ADSL} data created from subsetting the \code{CDISC} \code{ADSL} with 15 subjects (5 subjects in each arm)
}
\keyword{CDISC}
\keyword{adsl}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{column_metadata}
\alias{column_metadata}
\title{Metadata describing table column layouts}
\format{
A data frame with one row per column for each table type
and six variables:
\describe{
\item{\code{tbltype}}{identifier used to group a table column layout}
\item{\code{coldef}}{distinct variable values used, typically numeric
and typically a treatment or main effect variable, think \code{TRT01PN}}
\item{\code{decode}}{decode of \code{coldef} that will display as a column header
in the table}
\item{\code{span1}}{spanning header to display across multiple columns}
\item{\code{span2}}{spanning header to display across multiple columns,
second level}
\item{\code{span3}}{spanning header to display across multiple columns,
third level}
}
}
\usage{
column_metadata
}
\description{
This is used by \code{tlgsetup} to prepare you input data to support
the desired column layout.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_format.R
\name{add_format}
\alias{add_format}
\title{Add the formatting variables of \code{indentme}, \code{newrows}, \code{newpage}, and \code{roworder} to
the results dataframe}
\usage{
add_format(df, tableby = NULL, groupby = NULL, .keep = FALSE)
}
\arguments{
\item{df}{(required) dataframe of results and must contain the \code{anbr} variable.}

\item{tableby}{(optional) character vector containing table by variables.}

\item{groupby}{(optional) character vector containing group by variables.}

\item{.keep}{(optional) should \code{tableby} and \code{groupby} variables be kept in the
final dataframe. (default = \code{FALSE}).}
}
\value{
dataframe with the formatting variables \code{indentme}, \code{newrows}, \code{newpage}, and \code{roworder} added.
}
\description{
Add the formatting variables of \code{indentme}, \code{newrows}, \code{newpage}, and \code{roworder} to
the results dataframe
}
\examples{
df <- tibble::tibble(
  row_type =
    c(
      "TABLE_BY_HEADER", "HEADER", "BY_HEADER1", "N", "VALUE",
      "COUNTS", "UNIVAR", "NESTED", "NESTED"
    ),
  nested_level = c(NA, NA, NA, NA, NA, NA, NA, 1, 2),
  group_level = c(0, 0, 0, 0, 0, 0, 0, 0, 0),
  label = c(NA, NA, NA, NA, NA, "N", NA, NA, NA),
  by = c(NA, NA, NA, NA, NA, NA, NA, NA, NA),
  tableby = c(NA, NA, NA, NA, NA, NA, NA, NA, NA),
  anbr = c(1:9)
)
add_format(df)
}
